{************************************************************************}
{  This Unit was modified to be interfaced in IBObjects
   by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com
   }
{  This software is distributed under the License is distributed on      }
{    an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either              }
{    express or implied. See the ORIGINAL Copyright                      }


{************************************************************************}
{       ORIGINAL Copyright                                               }
{************************************************************************}
{                                                                        }
{       Borland Delphi Visual Component Library                          }
{       InterBase Express core components                                }
{                                                                        }
{       Copyright (c) 1998-2001 Borland Software Corporation             }
{                                                                        }
{    InterBase Express is based in part on the product                   }
{    Free IB Components, written by Gregory H. Deatz for                 }
{    Hoagland, Longo, Moran, Dunst & Doukas Company.                     }
{    Free IB Components is used under license.                           }
{                                                                        }
{    The contents of this file are subject to the InterBase              }
{    Public License Version 1.0 (the "License"); you may not             }
{    use this file except in compliance with the License. You may obtain }
{    a copy of the License at http://www.borland.com/interbase/IPL.html  }
{    Software distributed under the License is distributed on            }
{    an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either              }
{    express or implied. See the License for the specific language       }
{    governing rights and limitations under the License.                 }
{    The Original Code was created by InterBase Software Corporation     }
{       and its successors.                                              }
{    Portions created by Borland Software Corporation are Copyright      }
{       (C) Borland Software Corporation. All Rights Reserved.           }
{    Contributor(s): Jeff Overcash                                       }
{                                                                        }
{************************************************************************}
unit IBOConnectionInfo;

interface

uses
  SysUtils, Classes, IB_Components, IB_Header, IB_Session;

type

  TIB_ConnectionInfo = class(TComponent)
  protected
    FIBLoaded: Boolean;
    FDatabase: TIB_Connection;
    FUserNames   : TStringList;
    FBackoutCount: TStringList;
    FDeleteCount: TStringList;
    FExpungeCount: TStringList;
    FInsertCount: TStringList;
    FPurgeCount: TStringList;
    FReadIdxCount: TStringList;
    FReadSeqCount: TStringList;
    FUpdateCount: TStringList;
    function GetAllocation: LongInt;
    function GetBaseLevel: LongInt;
    function GetDBFileName: String;
    function GetDBSiteName: String;
    function GetDBImplementationNo: LongInt;
    function GetDBImplementationClass: LongInt;
    function GetNoReserve: LongInt;
    function GetODSMinorVersion: LongInt;
    function GetODSMajorVersion: LongInt;
    function GetPageSize: LongInt;
    function GetVersion: String;
    function GetCurrentMemory: LongInt;
    function GetForcedWrites: LongInt;
    function GetMaxMemory: LongInt;
    function GetNumBuffers: LongInt;
    function GetSweepInterval: LongInt;
    function GetUserNames: TStringList;
    function GetFetches: LongInt;
    function GetMarks: LongInt;
    function GetReads: LongInt;
    function GetWrites: LongInt;
    function GetBackoutCount: TStringList;
    function GetDeleteCount: TStringList;
    function GetExpungeCount: TStringList;
    function GetInsertCount: TStringList;
    function GetPurgeCount: TStringList;
    function GetReadIdxCount: TStringList;
    function GetReadSeqCount: TStringList;
    function GetUpdateCount: TStringList;
    function GetOperationCounts(DBInfoCommand: Integer; FOperation: TStringList): TStringList;
    function GetReadOnly: LongInt;
    function GetStringDatabaseInfo(DatabaseInfoCommand: Integer): String;
    function GetDBSQLDialect: LongInt;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Call(ErrCode: ISC_STATUS; RaiseError: Boolean): ISC_STATUS;
    function GetLongDatabaseInfo(DatabaseInfoCommand: Integer): LongInt;
    property Allocation: LongInt read GetAllocation;
    property BaseLevel: LongInt read GetBaseLevel;
    property DBFileName: String read GetDBFileName;
    property DBSiteName: String read GetDBSiteName;
    property DBImplementationNo: LongInt read GetDBImplementationNo;
    property DBImplementationClass: LongInt read GetDBImplementationClass;
    property NoReserve: LongInt read GetNoReserve;
    property ODSMinorVersion: LongInt read GetODSMinorVersion;
    property ODSMajorVersion: LongInt read GetODSMajorVersion;
    property PageSize: LongInt read GetPageSize;
    property Version: String read GetVersion;
    property CurrentMemory: LongInt read GetCurrentMemory;
    property ForcedWrites: LongInt read GetForcedWrites;
    property MaxMemory: LongInt read GetMaxMemory;
    property NumBuffers: LongInt read GetNumBuffers;
    property SweepInterval: LongInt read GetSweepInterval;
    property UserNames: TStringList read GetUserNames;
    property Fetches: LongInt read GetFetches;
    property Marks: LongInt read GetMarks;
    property Reads: LongInt read GetReads;
    property Writes: LongInt read GetWrites;
    property BackoutCount: TStringList read GetBackoutCount;
    property DeleteCount: TStringList read GetDeleteCount;
    property ExpungeCount: TStringList read GetExpungeCount;
    property InsertCount: TStringList read GetInsertCount;
    property PurgeCount: TStringList read GetPurgeCount;
    property ReadIdxCount: TStringList read GetReadIdxCount;
    property ReadSeqCount: TStringList read GetReadSeqCount;
    property UpdateCount: TStringList read GetUpdateCount;
    property DBSQLDialect : LongInt read GetDBSQLDialect;
    property ReadOnly: LongInt read GetReadOnly;
  published
    property Database: TIB_Connection read FDatabase write FDatabase;
  end;

implementation

{ TIBDatabaseInfo }

function _Session: TIB_Session;
begin
  Result := TIB_Session.DefaultSession;
end;


constructor TIB_ConnectionInfo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FIBLoaded := False;
//  CheckIBLoaded;
  FIBLoaded := True;
  FUserNames := TStringList.Create;
  FBackoutCount                        := nil;
  FDeleteCount                         := nil;
  FExpungeCount                        := nil;
  FInsertCount                         := nil;
  FPurgeCount                          := nil;
  FReadIdxCount                        := nil;
  FReadSeqCount                        := nil;
  FUpdateCount                         := nil;
end;

destructor TIB_ConnectionInfo.Destroy;
begin
  if FIBLoaded then
  begin
    FUserNames.Free;
    FBackoutCount.Free;
    FDeleteCount.Free;
    FExpungeCount.Free;
    FInsertCount.Free;
    FPurgeCount.Free;
    FReadIdxCount.Free;
    FReadSeqCount.Free;
    FUpdateCount.Free;
  end;
  inherited Destroy;
end;

const
  IBLocalBufferLength = 512;
  IBBigLocalBufferLength = IBLocalBufferLength * 2;
  IBHugeLocalBufferLength = IBBigLocalBufferLength * 20;
(*
procedure IBDataBaseError;
var
  sqlcode: LongInt;
  IBErrorCode: LongInt;
  local_buffer: array[0..IBHugeLocalBufferLength - 1] of char;
  usr_msg: string;
  status_vector: PISC_STATUS;
//  IBDataBaseErrorMessages: TIBDataBaseErrorMessages;
begin
  usr_msg := '';

  { Get a local reference to the status vector.
    Get a local copy of the IBDataBaseErrorMessages options.
    Get the SQL error code }
  status_vector := StatusVector;
  IBErrorCode := StatusVectorArray[1];
  IBDataBaseErrorMessages := GetIBDataBaseErrorMessages;
  sqlcode := isc_sqlcode(status_vector);

  if (ShowSQLCode in IBDataBaseErrorMessages) then
    usr_msg := usr_msg + 'SQLCODE: ' + IntToStr(sqlcode); {do not localize}
  Exclude(IBDataBaseErrorMessages, ShowSQLMessage);
  if (ShowSQLMessage in IBDataBaseErrorMessages) then
  begin
    isc_sql_interprete(sqlcode, local_buffer, IBLocalBufferLength);
    if (ShowSQLCode in IBDataBaseErrorMessages) then
      usr_msg := usr_msg + CRLF;
    usr_msg := usr_msg + string(local_buffer);
  end;

  if (ShowIBMessage in IBDataBaseErrorMessages) then
  begin
    if (ShowSQLCode in IBDataBaseErrorMessages) or
       (ShowSQLMessage in IBDataBaseErrorMessages) then
      usr_msg := usr_msg + CRLF;
    while (isc_interprete(local_buffer, @status_vector) > 0) do
    begin
      if (usr_msg <> '') and (usr_msg[Length(usr_msg)] <> LF) then
        usr_msg := usr_msg + CRLF;
      usr_msg := usr_msg + string(local_buffer);
    end;
  end;
  if (usr_msg <> '') and (usr_msg[Length(usr_msg)] = '.') then
    Delete(usr_msg, Length(usr_msg), 1);
  if sqlcode <> -551 then
    raise EIBInterBaseError.Create(sqlcode, IBErrorCode, usr_msg)
  else
    raise EIBInterBaseRoleError.Create(sqlcode, IBErrorCode, usr_msg)
end;
*)

function TIB_ConnectionInfo.Call(ErrCode: ISC_STATUS;
  RaiseError: Boolean): ISC_STATUS;
begin
  result := ErrCode;
  if RaiseError and (ErrCode > 0) then
//    IBDataBaseError;
end;

function TIB_ConnectionInfo.GetAllocation: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_allocation);
end;

function TIB_ConnectionInfo.GetBaseLevel: LongInt;
var
  local_buffer: array[0..IBLocalBufferLength - 1] of Char;
  DatabaseInfoCommand: Char;

begin
  DatabaseInfoCommand := Char(isc_info_base_level);
  Call(isc_database_info(@_Session.status, @FDatabase.dbHandle, 1, @DatabaseInfoCommand,
                         IBLocalBufferLength, local_buffer), True);
  result :=isc_vax_integer(@local_buffer[4], 1);
end;

function TIB_ConnectionInfo.GetDBFileName: String;
var
  local_buffer: array[0..IBLocalBufferLength - 1] of Char;
  DatabaseInfoCommand: Char;
begin
  DatabaseInfoCommand := Char(isc_info_db_id);
  Call(isc_database_info(@_Session.status, @FDatabase.dbHandle, 1, @DatabaseInfoCommand,
                         IBLocalBufferLength, local_buffer), True);
  local_buffer[5 + Integer(local_buffer[4])] := #0;
  result := String(PChar(@local_buffer[5]));
end;

function TIB_ConnectionInfo.GetDBSiteName: String;
var
  local_buffer: array[0..IBBigLocalBufferLength - 1] of Char;
  p: PChar;
  DatabaseInfoCommand: Char;
begin
  DatabaseInfoCommand := Char(isc_info_db_id);
  Call(isc_database_info(@_Session.status, @FDatabase.dbHandle, 1, @DatabaseInfoCommand,
                        IBLocalBufferLength, local_buffer), True);
  p := @local_buffer[5 + Integer(local_buffer[4])]; { DBSiteName Length }
  p := p + Ord(p^) + 1;                         { End of DBSiteName }
  p^ := #0;                                     { Null it }
  result := String(PChar(@local_buffer[6 + Integer(local_buffer[4])]));
end;

function TIB_ConnectionInfo.GetDBImplementationNo: LongInt;
var
  local_buffer: array[0..IBLocalBufferLength - 1] of Char;
  DatabaseInfoCommand: Char;
begin
  DatabaseInfoCommand := Char(isc_info_implementation);
  Call(isc_database_info(@_Session.status, @FDatabase.dbHandle, 1, @DatabaseInfoCommand,
                        IBLocalBufferLength, local_buffer), True);
  result :=isc_vax_integer(@local_buffer[3], 1);
end;

function TIB_ConnectionInfo.GetDBImplementationClass: LongInt;
var
  local_buffer: array[0..IBLocalBufferLength - 1] of Char;
  DatabaseInfoCommand: Char;
begin
  DatabaseInfoCommand := Char(isc_info_implementation);
  Call(isc_database_info(@_Session.status, @FDatabase.dbHandle, 1, @DatabaseInfoCommand,
                         IBLocalBufferLength, local_buffer), True);
  result :=isc_vax_integer(@local_buffer[4], 1);
end;

function TIB_ConnectionInfo.GetNoReserve: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_no_reserve);
end;

function TIB_ConnectionInfo.GetODSMinorVersion: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_ods_minor_version);
end;

function TIB_ConnectionInfo.GetODSMajorVersion: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_ods_version);
end;

function TIB_ConnectionInfo.GetPageSize: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_page_size);
end;

function TIB_ConnectionInfo.GetVersion: String;
var
  local_buffer: array[0..IBBigLocalBufferLength - 1] of Char;
  DatabaseInfoCommand: Char;
begin
  DatabaseInfoCommand := Char(isc_info_version);
  Call(isc_database_info(@_Session.status, @FDatabase.dbHandle, 1, @DatabaseInfoCommand,
                        IBBigLocalBufferLength, local_buffer), True);
  local_buffer[5 + Integer(local_buffer[4])] := #0;
  result := String(PChar(@local_buffer[5]));
end;

function TIB_ConnectionInfo.GetCurrentMemory: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_current_memory);
end;

function TIB_ConnectionInfo.GetForcedWrites: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_forced_writes);
end;

function TIB_ConnectionInfo.GetMaxMemory: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_max_memory);
end;

function TIB_ConnectionInfo.GetNumBuffers: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_num_buffers);
end;

function TIB_ConnectionInfo.GetSweepInterval: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_sweep_interval);
end;

function TIB_ConnectionInfo.GetUserNames: TStringList;
var
  local_buffer: array[0..IBHugeLocalBufferLength - 1] of Char;
  temp_buffer: array[0..IBLocalBufferLength - 2] of Char;
  DatabaseInfoCommand: Char;
  i, user_length: Integer;
begin
  result := FUserNames;
  DatabaseInfoCommand := Char(isc_info_user_names);
  Call(isc_database_info(@_Session.status, @FDatabase.dbHandle, 1, @DatabaseInfoCommand,
                        IBHugeLocalBufferLength, local_buffer), True);
  FUserNames.Clear;
  i := 0;
  while local_buffer[i] = Char(isc_info_user_names) do
  begin
    Inc(i, 3); { skip "isc_info_user_names byte" & two unknown bytes of structure (see below) }
    user_length := LongInt(local_buffer[i]);
    Inc(i,1);
    Move(local_buffer[i], temp_buffer[0], user_length);
    Inc(i, user_length);
    temp_buffer[user_length] := #0;
    FUserNames.Add(String(temp_buffer));
  end;
end;

function TIB_ConnectionInfo.GetFetches: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_fetches);
end;

function TIB_ConnectionInfo.GetMarks: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_marks);
end;

function TIB_ConnectionInfo.GetReads: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_reads);
end;

function TIB_ConnectionInfo.GetWrites: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_writes);
end;

function TIB_ConnectionInfo.GetOperationCounts(DBInfoCommand: Integer; FOperation: TStringList): TStringList;
var
  local_buffer: array[0..IBHugeLocalBufferLength - 1] of Char;
  DatabaseInfoCommand: Char;
  i, qtd_tables, id_table, qtd_operations: Integer;
begin
  if FOperation = nil then FOperation := TStringList.Create;
  result := FOperation;
  DatabaseInfoCommand := Char(DBInfoCommand);
  Call(isc_database_info(@_Session.status, @FDatabase.dbHandle, 1, @DatabaseInfoCommand,
                         IBHugeLocalBufferLength, local_buffer), True);
  FOperation.Clear;
  { 1. 1 byte specifying the item type requested (e.g., isc_info_insert_count).
    2. 2 bytes telling how many bytes compose the subsequent value pairs.
    3. A pair of values for each table in the database on wich the requested
      type of operation has occurred since the database was last attached.
    Each pair consists of:
    1. 2 bytes specifying the table ID.
    2. 4 bytes listing the number of operations (e.g., inserts) done on that table.
  }
  qtd_tables := trunc(isc_vax_integer(@local_buffer[1],2)/6);
  for i := 0 to qtd_tables - 1 do
  begin
    id_table :=isc_vax_integer(@local_buffer[3+(i*6)],2);
    qtd_operations :=isc_vax_integer(@local_buffer[5+(i*6)],4);
    FOperation.Add(IntToStr(id_table)+'='+IntToStr(qtd_operations));
  end;
end;

function TIB_ConnectionInfo.GetBackoutCount: TStringList;
begin
  result := GetOperationCounts(isc_info_backout_count,FBackoutCount);
end;

function TIB_ConnectionInfo.GetDeleteCount: TStringList;
begin
  result := GetOperationCounts(isc_info_delete_count,FDeleteCount);
end;

function TIB_ConnectionInfo.GetExpungeCount: TStringList;
begin
  result := GetOperationCounts(isc_info_expunge_count,FExpungeCount);
end;

function TIB_ConnectionInfo.GetInsertCount: TStringList;
begin
  result := GetOperationCounts(isc_info_insert_count,FInsertCount);
end;

function TIB_ConnectionInfo.GetPurgeCount: TStringList;
begin
  result := GetOperationCounts(isc_info_purge_count,FPurgeCount);
end;

function TIB_ConnectionInfo.GetReadIdxCount: TStringList;
begin
  result := GetOperationCounts(isc_info_read_idx_count,FReadIdxCount);
end;

function TIB_ConnectionInfo.GetReadSeqCount: TStringList;
begin
  result := GetOperationCounts(isc_info_read_seq_count,FReadSeqCount);
end;

function TIB_ConnectionInfo.GetUpdateCount: TStringList;
begin
  result := GetOperationCounts(isc_info_update_count,FUpdateCount);
end;

function TIB_ConnectionInfo.GetReadOnly: LongInt;
begin
  result := GetLongDatabaseInfo(isc_info_db_read_only);
end;

function TIB_ConnectionInfo.GetLongDatabaseInfo(DatabaseInfoCommand: Integer): LongInt;
var
  local_buffer: array[0..IBLocalBufferLength - 1] of Char;
  length: Integer;
  _DatabaseInfoCommand: Char;
begin
  _DatabaseInfoCommand := Char(DatabaseInfoCommand);
  Call(isc_database_info(@_Session.status, @FDatabase.dbHandle, 1, @_DatabaseInfoCommand,
                         IBLocalBufferLength, local_buffer), True);
  length :=isc_vax_integer(@local_buffer[1], 2);
  result :=isc_vax_integer(@local_buffer[3], length);
end;

function TIB_ConnectionInfo.GetStringDatabaseInfo(DatabaseInfoCommand: Integer): String;
var
  local_buffer: array[0..IBBigLocalBufferLength - 1] of Char;
  _DatabaseInfoCommand: Char;
begin
  _DatabaseInfoCommand := Char(DatabaseInfoCommand);
  Call(isc_database_info(@_Session.status, @FDatabase.dbHandle, 1, @_DatabaseInfoCommand,
                         IBBigLocalBufferLength, local_buffer), True);
  local_buffer[4 + Integer(local_buffer[3])] := #0;
  result := String(PChar(@local_buffer[4]));
end;


function TIB_ConnectionInfo.GetDBSQLDialect: Integer;
var
  local_buffer: array[0..IBLocalBufferLength - 1] of Char;
  length: Integer;
  DatabaseInfoCommand: Char;
begin
  DatabaseInfoCommand := Char(isc_info_db_SQL_Dialect);
  Call(isc_database_info(@_Session.status, @FDatabase.dbHandle, 1, @DatabaseInfoCommand,
                       IBLocalBufferLength, local_buffer), True);
  if (local_buffer[0] <> Char(isc_info_db_SQL_dialect)) then
    result := 1
  else begin
    length :=isc_vax_integer(@local_buffer[1], 2);
    result :=isc_vax_integer(@local_buffer[3], length);
  end;
end;


end.
