{************************************************************************}
{  This Unit was created to by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com
}
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }

{  This software is distributed under the License is distributed on      }
{    an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either              }
{    express or implied. See the ORIGINAL Copyright                      }

unit FrameFieldsList;

interface

uses 
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Buttons, StdCtrls, Types;

type
  TFrameIndexList = class(TFrame)
    lbCols: TListBox;
    SpeedButtonAdd: TSpeedButton;
    SpeedButtonRemove: TSpeedButton;
    lvIdxColumns: TListBox;
    Panel1: TPanel;
    procedure FrameResize(Sender: TObject);
    procedure SpeedButtonAddClick(Sender: TObject);
    procedure SpeedButtonRemoveClick(Sender: TObject);
    procedure lvIdxColumnsDblClick(Sender: TObject);
    procedure lbColsDblClick(Sender: TObject);
  private
    FOnChange: TNotifyEvent;
    procedure SetOnChange(const Value: TNotifyEvent);
    function Present(f: string): boolean;
    { Private declarations }
  public
    { Public declarations }
    Fields:TStringDynArray;
    procedure RestoreFields;
    procedure ResetFields;
    procedure AddFields(lFields : TStringDynArray);
    procedure IncludeFields(lFields : TStringDynArray);
    procedure ExcludeFields(lFields: TStringDynArray);
    procedure ExcludeField(lField: String);
    procedure IncludeField(lField: String);
  published
    Property OnChange:TNotifyEvent read FOnChange write SetOnChange;
  end;

implementation

{$R *.dfm}

procedure TFrameIndexList.FrameResize(Sender: TObject);
var t:integer;
begin
  lbCols.Width := (ClientWidth - Panel1.Width) div 2;
  t := Panel1.Height - (SpeedButtonAdd.Height + SpeedButtonRemove.Height + 2);
  t := t div 2;
  SpeedButtonAdd.Top := t;
  SpeedButtonRemove.Top := t + 2 + SpeedButtonAdd.Height;
end;

procedure TFrameIndexList.Restorefields;
var n:integer;
begin
  lbCols.Items.Clear;
  for n:=Low(Fields) to high(Fields) do
  if lvIdxColumns.Items.IndexOf(Fields[n])<0 then
  begin
    lbCols.Items.Add(Fields[n]);
  end;
end;

procedure TFrameIndexList.SpeedButtonAddClick(Sender: TObject);
begin
  if lbCols.ItemIndex<> -1 then
  begin
    lvIdxColumns.Items.Add(lbCols.Items[lbCols.ItemIndex]);
    lbCols.Items.Delete(lbCols.ItemIndex);
    if Assigned(FOnChange) then
    try
      FOnChange(self);
    except
    end;
  end;
end;

procedure TFrameIndexList.SpeedButtonRemoveClick(Sender: TObject);
begin
  if lvIdxColumns.ItemIndex<> -1 then
  begin
    lvIdxColumns.Items.Delete(lvIdxColumns.ItemIndex);
    Restorefields;
    if Assigned(FOnChange) then
    try
      FOnChange(self);
    except
    end;
  end;
end;

procedure TFrameIndexList.lvIdxColumnsDblClick(Sender: TObject);
begin
  SpeedButtonRemoveClick(Sender);
end;

procedure TFrameIndexList.lbColsDblClick(Sender: TObject);
begin
  SpeedButtonAddClick(Sender);
end;


procedure TFrameIndexList.ResetFields;
begin
  lvIdxColumns.Clear;
  RestoreFields;
end;

procedure TFrameIndexList.AddFields(lFields: TStringDynArray);
var n:integer;
begin
  if Assigned(lFields) then
  begin
    SetLength(Fields,Length(lFields));
    for n := low(lFields) to High(lFields) do
    begin
      Fields[n] := lFields[n];
    end;
  end
  else
    SetLength(Fields,0);
  Restorefields;
end;

procedure TFrameIndexList.IncludeFields(lFields: TStringDynArray);
var n:integer;

begin
  for n := low(lFields) to High(lFields) do
  begin
    if Present(lFields[n]) then
    begin
      lvIdxColumns.Items.Add(lFields[n]);
    end;
  end;
  Restorefields;
end;

procedure TFrameIndexList.ExcludeFields(lFields: TStringDynArray);
var n,idx:integer;

begin
  for n := low(lFields) to High(lFields) do
  begin
    idx := lvIdxColumns.Items.IndexOf(lFields[n]);
    if idx >= 0 then
    begin
      lvIdxColumns.Items.Delete(idx);
    end;
  end;
  Restorefields;
end;

function TFrameIndexList.Present(f:string):boolean;
var n:integer;
begin
  result := false;
  for n := low(fields) to High(Fields) do
  begin
    if f = fields[n] then
    begin
      result := true;
      break;
    end;
  end;
end;


procedure TFrameIndexList.SetOnChange(const Value: TNotifyEvent);
begin
  FOnChange := Value;
end;

procedure TFrameIndexList.ExcludeField(lField: String);
var idx:integer;
begin
  idx := lvIdxColumns.Items.IndexOf(lField);
  if idx >= 0 then
  begin
    lvIdxColumns.Items.Delete(idx);
  end;
  Restorefields;
end;

procedure TFrameIndexList.IncludeField(lField: String);
begin
  if Present(lField) then
  begin
    lvIdxColumns.Items.Add(lField);
  end;
  Restorefields;
end;

end.
