{******************************************************************}
{  This Unit was created to by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.
}

{
15.October.2003, Lorenzo:
- Fixed set new value to generator

25.December.2003, Lorenzo:
- Fixed Name extraction on Dialect 3 generator names
}

unit FrameGeneratorsU;

interface

uses
{$IFDEF LINUX}
  QControls, QForms, QDialogs, QComCtrls, QStdCtrls, QMenus,
  Types, QTypes, QImgList, QExtCtrls,
  QStdActns, QActnList, SynEdit, SynEditPrint, IBCXDummies,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Controls, Forms, Dialogs, ComCtrls, StdCtrls,
  Menus, ImgList, StdActns, ActnList, Graphics, Spin,Grids, DBGrids,
{$ENDIF}
  SysUtils, Variants, Classes,
  DB, DBClient, dmuNode, ExtCtrls, SynEdit,
  SynDBEdit, IBOExtract;

type
  TFrameGenerators = class(TFrame)
    DBGrid1: TDBGrid;
    ClientDataSet1: TClientDataSet;
    ClientDataSet1Name: TStringField;
    ClientDataSet1Value: TLargeintField;
    DataSource1: TDataSource;
    PopupMenu1: TPopupMenu;
    NewGenerator1: TMenuItem;
    EditGenerator1: TMenuItem;
    DropGenerator1: TMenuItem;
    ClientDataSet1Metadata: TMemoField;
    DBSynEdit1: TDBSynEdit;
    Splitter1: TSplitter;
    procedure ClientDataSet1BeforeInsert(DataSet: TDataSet);
    procedure ClientDataSet1AfterPost(DataSet: TDataSet);
    procedure ClientDataSet1AfterCancel(DataSet: TDataSet);
    procedure ClientDataSet1AfterDelete(DataSet: TDataSet);
    procedure ClientDataSet1BeforeDelete(DataSet: TDataSet);
    procedure ClientDataSet1AfterInsert(DataSet: TDataSet);
    procedure ClientDataSet1BeforePost(DataSet: TDataSet);
    procedure NewGenerator1Click(Sender: TObject);
    procedure EditGenerator1Click(Sender: TObject);
    procedure DropGenerator1Click(Sender: TObject);
  protected
    fLoading:boolean;
    fInsert:boolean;
    CompileErrors:string;
  private
    FDataModule: TDmN;
    FAutoEdit: boolean;
    procedure DoResetValue(const generator: string; Value: int64);
    procedure DoDeleteGenerator(const generator: string);
    function DoCreateGenerator(const generator: string;
      const Value: int64): boolean;
    procedure SetDataModule(const Value: TDmN);
    procedure SetAutoEdit(const Value: boolean);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    property DataModule:TDmN read FDataModule write SetDataModule;
    procedure LoadObjects;
    property AutoEdit:boolean read FAutoEdit write SetAutoEdit;
    procedure ParentResize;
  end;

implementation

uses frmuMain, IBOAdminUtils, frmuMessage, IB_Schema, IBOCGlobal, dmuMain;

{$R *.dfm}

function TFrameGenerators.DoCreateGenerator(const generator:string; const Value:int64): boolean;
var error,
    lsql,
    gn:String;
begin
  Result := False;
  if Assigned(DataModule) then
  begin
    gn := generator;
    lsql := 'create generator ' +QuoteIdentifier(DataModule.Connection.SQLDialect,gn);
    Result := DataModule.ExecuteDDL(lsql,error);
    CompileErrors := error;
    if result then
    begin
      DataModule.IBOExtract.ExtractObject(eoGenerator, gn);
      if ClientDataSet1.State in dsEditModes then
        ClientDataSet1Metadata.AsString := DataModule.IBOExtract.Items.Text;
      if (Value>0) then
      begin
        lsql := 'set generator ' +
                QuoteIdentifier(DataModule.Connection.SQLDialect,gn)
                + ' to ' + IntToStr(value);
        DataModule.ExecuteDDL(lsql,Error);
        CompileErrors := error;
      end;
    end;
  end;
end;

procedure TFrameGenerators.DoResetValue(const generator:string; Value:int64);
var error :string;
begin
  if fInsert then
  begin
    if DoCreateGenerator(generator,Value) then
      frmMain.ObjectListRefresh(DataModule, NODE_GENERATOR)
    else
    begin
      DisplayMsg(ERR_DDL_EXECUTION,CompileErrors);
      Abort;
    end
  end
  else
  begin
    if DataModule.ExecuteSql('set generator ' + QuoteIdentifier(DataModule.Connection.SQLDialect,generator) + ' to ' + IntToStr(Value) + ';',Error) then
    begin
      frmMain.ObjectListRefresh(DataModule, NODE_GENERATOR);
    end
    else
    begin
      DisplayMsg(ERR_DDL_EXECUTION,Error);
      Abort;
    end
  end;
end;

procedure TFrameGenerators.DoDeleteGenerator(const generator:string);
var
  error:String;
begin
    if DataModule.DropObject(NODE_GENERATORS ,generator, error) then
    begin
      frmMain.ObjectListRefresh(DataModule, NODE_GENERATORS);
    end
    else
    begin
      if error <> '' then
      begin
        DisplayMsg(ERR_DDL_EXECUTION,Error);
        DataModule.RollbackDDL;
      end;
      Abort;
    end;
end;

procedure TFrameGenerators.ClientDataSet1BeforeInsert(DataSet: TDataSet);
begin
  if floading then exit;
  DBGrid1.Columns[0].ReadOnly := false;
end;

procedure TFrameGenerators.ClientDataSet1AfterPost(DataSet: TDataSet);
begin
  if floading then exit;
  fInsert := false;
  DBGrid1.Columns[0].ReadOnly := true;
end;

procedure TFrameGenerators.ClientDataSet1AfterCancel(DataSet: TDataSet);
begin
  if floading then exit;
  fInsert := false;
  DBGrid1.Columns[0].ReadOnly := true;
end;

procedure TFrameGenerators.ClientDataSet1AfterDelete(DataSet: TDataSet);
begin
  if floading then exit;
  DBGrid1.Columns[0].ReadOnly := true;
end;

procedure TFrameGenerators.ClientDataSet1BeforeDelete(DataSet: TDataSet);
begin
  if floading then exit;
  DoDeleteGenerator(ClientDataSet1Name.AsString)
end;

procedure TFrameGenerators.LoadObjects;
var
  Names:TStringList;
  n:integer;
  gn:string;
begin
  floading := true;

  Names:=TStringList.Create;
  if not ClientDataSet1.Active then ClientDataSet1.Open;
  ClientDataSet1.DisableControls;
  try
    ClientDataSet1.EmptyDataSet;
    SchemaGeneratorNames(DataModule.Connection,DataModule.Transaction,
                         gAppSettings[SYSTEM_DATA],Names);
    for n:= 0 to names.Count -1 do
    try
      ClientDataSet1.Append;
      gn := RemoveQuote(names[n]);
      ClientDataSet1Name.AsString := gn;
      ClientDataSet1Value.AsLargeInt := DataModule.Connection.Gen_ID(names[n],0);
      DataModule.IBOExtract.ExtractObject(eoGenerator, gn);
      ClientDataSet1Metadata.AsString := Trim(DataModule.IBOExtract.Items.Text);
      ClientDataSet1.Post;
    except
      ClientDataSet1.Cancel;
    end;
  finally
    names.Free;
    floading := false;
    ClientDataSet1.EnableControls;
    ClientDataSet1.First;
  end;
end;


procedure TFrameGenerators.ClientDataSet1AfterInsert(DataSet: TDataSet);
begin
  fInsert := not floading;
end;

procedure TFrameGenerators.ClientDataSet1BeforePost(DataSet: TDataSet);
begin
  if floading then exit;
  if gAppSettings[FORCE_UPPERCASE_IDENTIFIER] then
    ClientDataSet1Name.AsString := UpperCase(ClientDataSet1Name.AsString);
  if fInsert then
  begin
    DoCreateGenerator(ClientDataSet1Name.AsString,ClientDataSet1Value.AsLargeInt);
  end
  else
    DoResetValue(ClientDataSet1Name.AsString,ClientDataSet1Value.AsLargeInt);
end;

constructor TFrameGenerators.Create(aOwner: Tcomponent);
begin
  inherited;
  ClientDataSet1.CreateDataSet;
  fInsert := False;
  if not ClientDataSet1.Active then ClientDataSet1.Open;
end;

procedure TFrameGenerators.SetDataModule(const Value: TDmN);
begin
  FDataModule := Value;
  if FDataModule = nil then
    ClientDataSet1.Close;
end;

procedure TFrameGenerators.SetAutoEdit(const Value: boolean);
begin
  FAutoEdit := Value;
  DataSource1.AutoEdit := FAutoEdit;
end;

procedure TFrameGenerators.NewGenerator1Click(Sender: TObject);
begin
  if Assigned(FDataModule) then
  begin
    if not ClientDataSet1.Active then ClientDataSet1.Open;
    ClientDataSet1.Insert;
  end;
end;

procedure TFrameGenerators.EditGenerator1Click(Sender: TObject);
begin
  if Assigned(FDataModule) then
  begin
    if not ClientDataSet1.Active then ClientDataSet1.Open;
    ClientDataSet1.Edit;
  end;
end;

procedure TFrameGenerators.DropGenerator1Click(Sender: TObject);
begin
  if Assigned(FDataModule) then
  begin
    if not ClientDataSet1.Active then
      ClientDataSet1.Delete;
  end;
end;

procedure TFrameGenerators.ParentResize;
begin
  Splitter1.Height := 3;
  DBSynEdit1.Height := 50;
  DBGrid1.Columns[1].Width := 90;
  if dgIndicator in DBGrid1.Options then
    DBGrid1.Columns[0].Width := DBGrid1.ClientWidth - 90 - 24
  else
    DBGrid1.Columns[0].Width := DBGrid1.ClientWidth - 90;
end;

end.
