{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.
  See the ORIGINAL Copyright in source files
                                                                         }
{ 
 * The contents of this file are subject to the InterBase Public License 
 * Version 1.0 (the "License"); you may not use this file except in 
 * compliance with the License. 
 * 
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html. 
 * 
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific language governing rights and limitations 
 * under the License.  The Original Code was created by Inprise 
 * Corporation and its predecessors. 
 * 
 * Portions created by Inprise Corporation are Copyright (C) Inprise 
 * Corporation. All Rights Reserved. 
 * 
 * Contributor(s):  Krzysztof Golko. 
} 
 
// IBOConsole Persistentce Interface 
// Version 0.97.0 
 
unit IBOCPersistent; 
{ 
Next steps in evoulution of the persistent storage will be 
removing of Forms to avoid mixing visual and non-visual stuff and 
replacing TAppSettings by more general class, apart from these 
the code will be made more logical and uniform across different 
storage mediums (things like exceptions and section naming need to be unified) 
 
I've got acceptable ideas to redesign persistence stuff using interfaces 
} 
interface 
 
uses 
{$IFDEF MSWINDOWS} 
  Forms, 
{$ENDIF} 
{$IFDEF LINUX} 
  QForms, 
{$ENDIF} 
  Types, SysUtils, Classes, IBOServices, IBOCPersistentTypes, IB_Session;
 
const 
  EPACCESSSERVERALIAS = 'Server alias: %s. inaccessible'; 
  EPACCESSDATABASEALIAS = 'Database alias: %s on server %s inaccessible.'; 
  EPACCESSBACKUPALIAS = 'Backup alias: %s on server %s inaccessible.'; 
  EPACCESSSECTION = 'Section %s inaccessible.'; 
 
 
type 
{ Most of the procedures might raise an exception, eg when there's no a key in the registry 
  these situation are considered abnormal } 
 
  EIBCPersistent = class(Exception); 
 
type 
  // temporary this type is defined here 
  TibcServerProps = record 
    ServerName: string; 
    UserName: string; 
    Description: string; 
    Protocol: TIB_Protocol;
    Created: TDateTime;
    Accessed: TDateTime; 
  end; 
 
  TibcDatabaseProps = record 
    DatabaseFiles: string; 
    UserName: string; 
    Role: string; 
    CaseSensitiveRole: boolean; 
    CharacterSet: string; 
    Created: TDateTime; 
    Accessed: TDateTime;
//lm    PatchBlankCharset:boolean;
  end; 
 
  // under dev 
  TibcBackupProps = record 
    BackupFiles: string; 
    DatabaseAlias: string; 
    ServerAlias: string; 
    Created: TDateTime; 
    Accessed: TDateTime; 
  end; 
 
const 
  sectSettings = 'SETTINGS'; 
  sectServers = 'SERVERS'; 
  sectDatabases = 'DATABASES'; 
  sectBackupFiles = 'BACKUP FILES'; 
  sectTools = 'TOOLS'; 
 
type
  TibcPersistentInfo = class
  protected 
    procedure GetSetting(var Setting: TPersistentSetting); virtual; abstract;
    procedure StoreSetting(Setting: TPersistentSetting); virtual; abstract; 
    // to replace #13#10 with ';' 
    function DatabaseFilesToString(DatabaseFiles: string): string; virtual; 
    function StringToDatabaseFiles(DatabaseFilesString: string): string; virtual; 
  public 
    procedure Update; virtual; abstract; 
    // preferences 
    procedure GetSettings(var Settings: TPersistentSettings); virtual; abstract; 
    procedure StoreSettings(Settings: TPersistentSettings); virtual; abstract;
    procedure GetSectionSettings(SectionStrings: array of string; var Settings: TPersistentSettings; MustExist: boolean = false); virtual; abstract; 
    procedure StoreSectionSettings(SectionStrings: array of string; Settings: TPersistentSettings); virtual; abstract; 
    // form settings - obsolete, to be moved to IBCPersistent utils 
    procedure GetFormSettings(AForm: TForm; var Extra: word; Id: string); virtual; abstract;
    procedure StoreFormSettings(AForm: TForm; Extra: word; Id: string); virtual; abstract; 
    // server aliases 
    function ServerAliasExists(Alias: string): boolean; virtual; abstract; 
    procedure DeleteServerAlias(Alias: string); virtual; abstract; 
    procedure RenameServerAlias(SrcAlias, DestAlias: string); virtual; abstract;
    procedure GetServerAliases(Aliases: TStrings); virtual; abstract;
    procedure GetServerProps(Alias: string; var ServerProps: TibcServerProps); virtual; abstract;
    procedure StoreServerProps(Alias: string; ServerProps: TibcServerProps); virtual; abstract;
    // database aliases 
    function DatabaseAliasExists(ServerAlias, DatabaseAlias: string): boolean; virtual; abstract; 
    procedure DeleteDatabaseAlias(ServerAlias, DatabaseAlias: string); virtual; abstract; 
    procedure RenameDatabaseAlias(ServerAlias, SrcAlias, DestAlias: string); virtual; abstract; 
    procedure GetDatabaseAliases(ServerAlias: string; Aliases: TStrings); virtual; abstract; 
    procedure GetDatabaseProps(ServerAlias, DatabaseAlias: string; var DatabaseProps: TibcDatabaseProps); virtual; abstract;
    procedure StoreDatabaseProps(ServerAlias, DatabaseAlias: string; var DatabaseProps: TibcDatabaseProps); virtual; abstract;
    // backup aliases 
    function BackupAliasExists(ServerAlias, BackupAlias: string): boolean; virtual; abstract; 
    procedure DeleteBackupAlias(ServerAlias, BackupAlias: string); virtual; abstract; 
    procedure RenameBackupAlias(ServerAlias, SrcAlias, DestAlias: string); virtual; abstract;
    procedure GetBackupAliases(ServerAlias: string; Aliases: TStrings); virtual; abstract;
    procedure GetBackupAliasProps(ServerAlias, BackupAlias: string; var BackupProps: TibcBackupProps); virtual; abstract;
    procedure StoreBackupAliasProps(ServerAlias, BackupAlias: string; var BackupProps: TibcBackupProps); virtual; abstract;
    // external Apps/Tools
    procedure GetExternalApps(ExternalApps: TStrings); virtual; abstract;
    procedure GetExternalApp(Pos: integer; var Title, AppPath, WorkDir, Params: string); virtual; abstract;
    procedure AddExternalApp(Title, AppPath, WorkDir, Params: string); virtual; abstract;
    procedure StoreExternalApp(Pos: integer; Title, AppPath, WorkDir, Params: string);  virtual; abstract;
    procedure DeleteExternalApp(Pos: integer);  virtual; abstract;
    procedure Assign(Source : TibcPersistentInfo); virtual;
  end;

procedure SetFormStatePosition(AForm: TCustomForm; var _Left, _Top, _Width, _Height: integer; const _State: TWindowState);

implementation

const
  LineEnd = sLineBreak;
  LineDelim = ';';


procedure SetFormStatePosition(AForm: TCustomForm; var _Left, _Top, _Width, _Height: integer;
  const _State: TWindowState);
var
  R,Ro:TRect;
  x,y:integer;
begin
  if (_State = wsNormal) or (_State = wsMaximized) then
    AForm.WindowState := _State;
  R := Bounds(_Left, _Top, _Width, _Height);
  if Types.IntersectRect(Ro, Screen.DesktopRect, R)
     and PtInRect(Screen.DesktopRect,R.BottomRight)
  then
  begin
    AForm.BoundsRect := R;
    AForm.WindowState := _State;
  end
  else
  begin
    if _Width > Screen.Width then  _Width:= Screen.Width;
    x := (Screen.Width - _Width) div 2;
    if x < 0 then x:=0;
    if _Height > Screen.Height then _Height:=Screen.Height;
    y := (Screen.Height - _Height) div 2;
    if y < 0 then y:=0;
    R := Bounds(x, y, _Width, _Height);
    AForm.BoundsRect := R;
    AForm.WindowState := _State;
  end;
end;


  { TibcPersistentInfo }
procedure TibcPersistentInfo.Assign(Source: TibcPersistentInfo);
var lSettings: TPersistentSettings;
    a,d:TStringList;
    n,n1:integer;
var DatabaseProps: TibcDatabaseProps;
    ServerProps: TibcServerProps;
    BackupProps: TibcBackupProps;
    Title, AppPath, WorkDir, Params: string;
begin
  lSettings := TPersistentSettings.Create(0);
  source.GetSettings(lSettings);
  StoreSettings(lSettings);
  a:=TStringList.Create;
  d:=TStringList.Create;
  source.GetServerAliases(a);
  for n:=0 to a.Count-1 do
  begin
    source.GetServerProps(a[n],ServerProps);
    StoreServerProps(a[n],ServerProps);
    source.GetDatabaseAliases(a[n],d);
    for n1:=0 to d.Count-1 do
    begin
      source.GetDatabaseProps(a[n],d[n1], DatabaseProps);
      StoreDatabaseProps(a[n],d[n1],DatabaseProps);
    end;
    source.GetBackupAliases(a[n],d);
    for n1:=0 to d.Count-1 do
    begin
      source.GetBackupAliasProps(a[n],d[n1], BackupProps);
      StoreBackupAliasProps(a[n],d[n1],BackupProps);
    end;
  end;

  source.GetExternalApps(a);
  for n:=0 to a.Count-1 do
  begin
    source.GetExternalApp(n,Title, AppPath, WorkDir, Params);
    AddExternalApp(Title, AppPath, WorkDir, Params);
    //procedure StoreExternalApp(Pos: integer; Title, AppPath, WorkDir, Params: string);  virtual; abstract;
  end;
end;

function TibcPersistentInfo.DatabaseFilesToString(DatabaseFiles: string): string;
var
  N: byte;
begin
  N := Pos(LineEnd, DatabaseFiles);
  while N > 0 do
  begin 
    Delete(DatabaseFiles, N, Length(LineEnd)); 
    Insert(LineDelim, DatabaseFiles, N); 
    N := Pos(LineEnd, DatabaseFiles); 
  end; 
  Result := DatabaseFiles; 
end; 
 
function TibcPersistentInfo.StringToDatabaseFiles(DatabaseFilesString: string): string; 
var 
  N: byte; 
begin 
  N := Pos(LineDelim, DatabaseFilesString); 
  while N > 0 do 
  begin 
    Delete(DatabaseFilesString, N, Length(LineDelim)); 
    Insert(LineEnd, DatabaseFilesString, N); 
    N := Pos(LineDelim, DatabaseFilesString); 
  end; 
  Result := DatabaseFilesString; 
end; 

end. 
