{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 *
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 *
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 *
 * Contributor(s):  Krzysztof Golko.
}
{************************************************************************
  This Unit was modified to be interfaced in IBObjects
   by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed under the License is distributed on
    an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
    express or implied. See the ORIGINAL Copyright

8.june.2003, Lorenzo:
- Adjusted out of screen positioning of windows

}


unit IBOCPersistentIniFile;
interface
uses
{$IFDEF MSWINDOWS}
Forms,
{$ENDIF}
{$IFDEF LINUX}
 QForms,
 {$ENDIF}
 SysUtils, Classes, Variants, IniFiles, IBOCPersistentTypes, IBOCPersistent, Types, IB_Session;

type
  TibcIniPersistentInfo = class(TibcPersistentInfo)
  private
    FIni: TMemIniFile;
  protected
    function FormatSection(Sections: array of string): string;
    procedure GetSectionSetting(Section: string; var Setting: TPersistentSetting); virtual;
    procedure StoreSectionSetting(Section: string; Setting: TPersistentSetting); virtual;
    procedure GetSetting(var Setting: TPersistentSetting); override;
    procedure StoreSetting(Setting: TPersistentSetting); override;
    procedure DoGetExternalApp(Section: string; var AppPath, WorkDir, Params: string);
    procedure DoStoreExternalApp(Section: string; AppPath, WorkDir, Params: string);
  public
    constructor Create(const FileName: string);
    destructor Destroy; override;
    procedure Update; override;
    procedure GetFormSettings(AForm: TForm; var Extra: word; Id: string); override;
    procedure StoreFormSettings(AForm: TForm; Extra: word; Id: string); override;
    procedure GetSettings(var Settings: TPersistentSettings); override;
    procedure StoreSettings(Settings: TPersistentSettings); override;
    procedure GetSectionSettings(SectionStrings: array of string; var Settings: TPersistentSettings; MustExist: boolean = false); override;
    procedure StoreSectionSettings(SectionStrings: array of string; Settings: TPersistentSettings); override;
    function ServerAliasExists(Alias: string): boolean; override;
    procedure DeleteServerAlias(Alias: string); override;
    procedure RenameServerAlias(SrcAlias, DestAlias: string); override;
    procedure GetServerAliases(Aliases: TStrings); override;
    procedure GetServerProps(Alias: string; var ServerProps: TibcServerProps); override;
    procedure StoreServerProps(Alias: string; ServerProps: TibcServerProps); override;
    // database aliases
    function DatabaseAliasExists(ServerAlias, DatabaseAlias: string): boolean; override;
    procedure DeleteDatabaseAlias(ServerAlias, DatabaseAlias: string); override;
    procedure RenameDatabaseAlias(ServerAlias, SrcAlias, DestAlias: string); override;
    procedure GetDatabaseAliases(ServerAlias: string; Aliases: TStrings); override;
    procedure GetDatabaseProps(ServerAlias, DatabaseAlias: string; var DatabaseProps: TibcDatabaseProps); override;
    procedure StoreDatabaseProps(ServerAlias, DatabaseAlias: string; var DatabaseProps: TibcDatabaseProps); override;
    // backup aliases

    function BackupAliasExists(ServerAlias, BackupAlias: string): boolean; override;
    procedure DeleteBackupAlias(ServerAlias, BackupAlias: string); override;
    procedure RenameBackupAlias(ServerAlias, SrcAlias, DestAlias: string); override;
    procedure GetBackupAliases(ServerAlias: string; Aliases: TStrings); override;
    procedure GetBackupAliasProps(ServerAlias, BackupAlias: string; var BackupProps: TibcBackupProps); override;
    procedure StoreBackupAliasProps(ServerAlias, BackupAlias: string; var BackupProps: TibcBackupProps); override;
    // external Apps/Tools
    procedure GetExternalApps(ExternalApps: TStrings); override;
    procedure GetExternalApp(Pos: integer; var Title, AppPath, WorkDir, Params: string); override;
    procedure AddExternalApp(Title, AppPath, WorkDir, Params: string); override;
    procedure StoreExternalApp(Pos: integer; Title, AppPath, WorkDir, Params: string); override;
    procedure DeleteExternalApp(Pos: integer); override;
  end;

implementation uses
  zluUtility, IBOServices;
const
  pkeyServerName = 'ServerName';
  pkeyUserName = 'UserName';
  pkeyDescription = 'Description';
  pkeyProtocol = 'Protocol';
  pkeyDatabaseFiles = 'DatabaseFiles';
  pkeyRole = 'Role';
  pkeyCaseSensitiveRole = 'CaseSensitiveRole';
  pkeyCharacterSet = 'CharacterSet';
  pkeyBackupFiles = 'BackupFiles';
  pkeyBackupSourceServer = 'SourceDBServer';
  pkeyBackupSourceDatabase = 'SourceDBAlias';
  pkeyAccessed = 'Accessed';
  pkeyCreated = 'Created';
  pkeyPatchBlankCharset = 'PatchBlankCharset';

constructor TibcIniPersistentInfo.Create(const FileName: string);
begin
  // the name of the ini file is yet to be decided
  // advice welcome
  FIni := TMemIniFile.Create(FileName);
end;

destructor TibcIniPersistentInfo.Destroy;
begin
  FIni.Free;
  inherited;
end;

procedure TibcIniPersistentInfo.Update;
begin
  try
    FIni.UpdateFile;
  except
    // persistence exception to be raised here
    raise;
  end;
end;

 function TibcIniPersistentInfo.FormatSection(Sections: array of string): string;
var
  I: integer;
begin
  Result := Sections[0];
  for I := 1 to Length(Sections) - 1 do
    Result := Result + '/' + Sections[I];
end;

procedure TibcIniPersistentInfo.GetSectionSetting(Section: string; var Setting: TPersistentSetting);
begin
  case (VarType(Setting.Value) and varTypeMask) of
    varSmallint: Setting.Value := FIni.ReadInteger(Section, Setting.Name, Setting.Value);
    varInteger: Setting.Value := FIni.ReadInteger(Section, Setting.Name, Setting.Value);
    varBoolean: Setting.Value := FIni.ReadBool(Section, Setting.Name, Setting.Value);
    varString: Setting.Value := FIni.ReadString(Section, Setting.Name, Setting.Value);
  end;
end;

procedure TibcIniPersistentInfo.StoreSectionSetting(Section: string; Setting: TPersistentSetting);
begin
  case (VarType(Setting.Value) and varTypeMask) of
    varSmallint: FIni.WriteInteger(Section, Setting.Name, Setting.Value);
    varInteger: FIni.WriteInteger(Section, Setting.Name, Setting.Value);
    varBoolean: FIni.WriteBool(Section, Setting.Name, Setting.Value);
    varString: FIni.WriteString(Section, Setting.Name, Setting.Value);
  end;
end;

procedure TibcIniPersistentInfo.GetSetting(var Setting: TPersistentSetting);
begin
  GetSectionSetting(sectSETTINGS, Setting);
end;

procedure TibcIniPersistentInfo.StoreSetting(Setting: TPersistentSetting);
begin
  StoreSectionSetting(sectSETTINGS, Setting);
end;

procedure TibcIniPersistentInfo.GetFormSettings(AForm: TForm; var Extra: word; Id: string);
var
  _Left, _Top, _Width, _Height: integer;
  _State: TWindowState;
begin
  _Left := FIni.ReadInteger(Id, 'Left', AForm.Left);
  _Top := FIni.ReadInteger(Id, 'Top', AForm.Top);
  _Width := FIni.ReadInteger(Id, 'Width', AForm.Width);
  _Height := FIni.ReadInteger(Id, 'Height', AForm.Height);
  _State := TWindowState(FIni.ReadInteger(Id, 'State', integer(wsNormal)));
  SetFormStatePosition(AForm,_Left, _Top, _Width, _Height,_State);
end;

procedure TibcIniPersistentInfo.StoreFormSettings(AForm: TForm; Extra: word; Id: string);
begin
  FIni.WriteInteger(Id, 'Left', AForm.Left);
  FIni.WriteInteger(Id, 'Top', AForm.Top);
  FIni.WriteInteger(Id, 'Width', AForm.Width);
  FIni.WriteInteger(Id, 'Height', AForm.Height);
  FIni.WriteInteger(Id, 'State', integer(AForm.WindowState));
  FIni.WriteInteger(Id, 'Extra', Extra);
end;
// preferences

procedure TibcIniPersistentInfo.GetSettings(var Settings: TPersistentSettings);
var
  i: integer;
  TempSetting: TPersistentSetting;
begin
  for i := 0 to Settings.Count - 1 do
  begin
    TempSetting := Settings.Settings[i];
    GetSetting(TempSetting);
    Settings.Settings[i] := TempSetting;
  end;
end;

procedure TibcIniPersistentInfo.StoreSettings(Settings: TPersistentSettings);
var
  i: integer;
begin
  for i := 0 to Settings.Count - 1 do
    StoreSetting(Settings.Settings[i]);
end;

procedure TibcIniPersistentInfo.GetSectionSettings(SectionStrings: array of string; var Settings: TPersistentSettings; MustExist: boolean = false);
var
  TempSetting: TPersistentSetting;
  Section: string;
  i: integer;
begin
  Section := '';
  for i := 0 to Length(SectionStrings) - 1 do
  begin
    if Section <> '' then
      Section := Section + '\';
    Section := Section + SectionStrings[i];
  end;
  // exception if section doesn't exist
  for i := 0 to Settings.Count - 1 do
  begin
    TempSetting := Settings.Settings[i];
    GetSectionSetting(Section, TempSetting);
    Settings.Settings[i] := TempSetting;
  end;
end;

procedure TibcIniPersistentInfo.StoreSectionSettings(SectionStrings: array of string; Settings: TPersistentSettings);
var
  Section: string;
  i: integer;
begin
  Section := '';
  for i := 0 to Length(SectionStrings) - 1 do
  begin
    if Section <> '' then
      Section := Section + '\';
    Section := Section + SectionStrings[i];
  end;
  for i := 0 to Settings.Count - 1 do
    StoreSectionSetting(Section, Settings.Settings[i]);
end;
    // server aliases

function TibcIniPersistentInfo.ServerAliasExists(Alias: string): boolean; begin   Result := FIni.ValueExists(sectServers, Alias); end; procedure TibcIniPersistentInfo.DeleteServerAlias(Alias: string);
var
  SubAliases: TStrings;
  I: integer;
begin
  // delete aliases
  SubAliases := TStringList.Create;
  try
    GetDatabaseAliases(Alias, SubAliases);
    for I := 0 to SubAliases.Count - 1 do
      DeleteDatabaseAlias(Alias, SubAliases[I]);
    FIni.EraseSection(FormatSection([sectServers, Alias, sectDatabases]));

    GetBackupAliases(Alias, SubAliases);
    for I := 0 to SubAliases.Count - 1 do
      DeleteBackupAlias(Alias, SubAliases[I]);
    FIni.EraseSection(FormatSection([sectServers, Alias, sectBackupFiles]));
  finally
    SubAliases.Free;
  end;
  // delete props
  FIni.EraseSection(FormatSection([sectServers, Alias]));
  // delete key
  FIni.DeleteKey(sectServers, Alias);
end;

procedure TibcIniPersistentInfo.RenameServerAlias(SrcAlias, DestAlias: string);
var
  ServerProps: TibcServerProps;
begin
  // check if dest doesn't exist
  GetServerProps(SrcAlias, ServerProps);
  StoreServerProps(DestAlias, ServerProps);
  DeleteServerAlias(SrcAlias);
//  FIni.DeleteKey(sectServers, SrcAlias);
//  FIni.WriteString(sectServers, DestAlias, '');
end;

procedure TibcIniPersistentInfo.GetServerAliases(Aliases: TStrings);
begin
  FIni.ReadSection(sectServers, Aliases);
end;

procedure TibcIniPersistentInfo.GetServerProps(Alias: string; var ServerProps: TibcServerProps);
var
  AliasSection: string;
  iProtocol: integer;
begin
  AliasSection := FormatSection([sectServers, Alias]);
  // check if exists
  if not FIni.ValueExists(sectServers, Alias) then
    raise EIBCPersistent.Create(Format(EPACCESSSERVERALIAS, [Alias]));

  // raise exception if values don't exists
  ServerProps.ServerName := FIni.ReadString(AliasSection, pkeyServerName, '');
  ServerProps.UserName := FIni.ReadString(AliasSection, pkeyUserName, '');
  ServerProps.Description := FIni.ReadString(AliasSection, pkeyDescription, '');
  iProtocol := FIni.ReadInteger(AliasSection, pkeyProtocol, integer(cpLocal));
  ServerProps.Protocol := TIB_Protocol(iProtocol);
  ServerProps.Accessed := FIni.ReadDateTime(AliasSection, pkeyAccessed, 0);
  ServerProps.Created := FIni.ReadDateTime(AliasSection, pkeyCreated, 0);
end;

procedure TibcIniPersistentInfo.StoreServerProps(Alias: string; ServerProps: TibcServerProps);
var
  AliasSection: string;
begin
  FIni.WriteString(sectServers, Alias, '');
  AliasSection := FormatSection([sectServers, Alias]);
  FIni.WriteString(AliasSection, pkeyServerName, ServerProps.ServerName);
  FIni.WriteString(AliasSection, pkeyUserName, ServerProps.UserName);
  FIni.WriteString(AliasSection, pkeyDescription, ServerProps.Description);
  FIni.WriteInteger(AliasSection, pkeyProtocol, integer(ServerProps.Protocol));
  FIni.WriteDateTime(AliasSection, pkeyAccessed, ServerProps.Accessed);
  FIni.WriteDateTime(AliasSection, pkeyCreated, ServerProps.Created);
end;

  // database aliases
function TibcIniPersistentInfo.DatabaseAliasExists(ServerAlias, DatabaseAlias: string): boolean;
begin
  Result := FIni.ValueExists(FormatSection([sectServers, ServerAlias, sectDatabases]), DatabaseAlias);
end;

procedure TibcIniPersistentInfo.DeleteDatabaseAlias(ServerAlias, DatabaseAlias: string);
begin
  FIni.EraseSection(FormatSection([sectServers, ServerAlias, sectDatabases, DatabaseAlias]));
  FIni.DeleteKey(FormatSection([sectServers, ServerAlias, sectDatabases]), DatabaseAlias);
end;

procedure TibcIniPersistentInfo.RenameDatabaseAlias(ServerAlias, SrcAlias, DestAlias: string);
var
  dbProps: TibcDatabaseProps;
begin
  // check if dest doesn't exist
  GetDatabaseProps(ServerAlias, SrcAlias, dbProps);
  StoreDatabaseProps(ServerAlias, DestAlias, dbProps);
  DeleteDatabaseAlias(ServerAlias, SrcAlias);
end;

procedure TibcIniPersistentInfo.GetDatabaseAliases(ServerAlias: string; Aliases: TStrings);
begin
  FIni.ReadSection(FormatSection([sectServers, ServerAlias, sectDatabases]), Aliases);
end;

procedure TibcIniPersistentInfo.GetDatabaseProps(ServerAlias, DatabaseAlias: string; var DatabaseProps: TibcDatabaseProps);
var
  Section: string;
begin
  Section := FormatSection([sectServers, ServerAlias, sectDatabases]);
  if not FIni.ValueExists(Section, DatabaseAlias) then
    raise EIBCPersistent.Create(Format(EPACCESSDATABASEALIAS, [DataBaseAlias, ServerAlias]));

  Section := FormatSection([Section, DatabaseAlias]);
  DatabaseProps.DatabaseFiles := StringToDatabaseFiles(FIni.ReadString(Section, pkeyDatabaseFiles, ''));
  DatabaseProps.Username := FIni.ReadString(Section, pkeyUserName, '');
  DatabaseProps.Role := FIni.ReadString(Section, pkeyRole, '');
  DatabaseProps.CaseSensitiveRole :=  FIni.ReadBool(Section, pkeyCaseSensitiveRole, false);
  DatabaseProps.CharacterSet := FIni.ReadString(Section, pkeyCharacterSet, '');
  DatabaseProps.Accessed := FIni.ReadDateTime(Section, pkeyAccessed, 0);
  DatabaseProps.Created := FIni.ReadDateTime(Section, pkeyCreated, 0);
//lm  DatabaseProps.PatchBlankCharset :=  FIni.ReadBool(Section, pkeyPatchBlankCharset, false);
end;

procedure TibcIniPersistentInfo.StoreDatabaseProps(ServerAlias, DatabaseAlias: string; var DatabaseProps: TibcDatabaseProps);
var
  Section: string;
begin
  // check if server alias exists
  Section := FormatSection([sectServers, ServerAlias, sectDatabases]);
  FIni.WriteString(Section, DatabaseAlias, '');

  Section := FormatSection([Section, DatabaseAlias]);
  FIni.WriteString(Section, pkeyDatabaseFiles, DatabaseFilesToString(DatabaseProps.DatabaseFiles));
  FIni.WriteString(Section, pkeyUserName, DatabaseProps.Username);
  FIni.WriteString(Section, 'Role', DatabaseProps.Role);
  FIni.WriteBool(Section, 'CaseSensitiveRole', DatabaseProps.CaseSensitiveRole);
  FIni.WriteString(Section, pkeyCharacterSet, DatabaseProps.CharacterSet);
  FIni.WriteDateTime(Section, pkeyAccessed, DatabaseProps.Accessed);
  FIni.WriteDateTime(Section, pkeyCreated, DatabaseProps.Created);
//lm  FIni.WriteBool(Section, pkeyPatchBlankCharset, DatabaseProps.PatchBlankCharset);
end;

    // backup aliases
function TibcIniPersistentInfo.BackupAliasExists(ServerAlias, BackupAlias: string): boolean;
begin
  Result := FIni.ValueExists(FormatSection([sectServers, ServerAlias, sectBackupFiles]), BackupAlias);
end;

procedure TibcIniPersistentInfo.DeleteBackupAlias(ServerAlias, BackupAlias: string);
begin
  FIni.EraseSection(FormatSection([sectServers, ServerAlias, sectBackupFiles, BackupAlias]));
  FIni.DeleteKey(FormatSection([sectServers, ServerAlias, sectBackupFiles]), BackupAlias);
end;

procedure TibcIniPersistentInfo.RenameBackupAlias(ServerAlias, SrcAlias, DestAlias: string);
var
  bkProps: TibcBackupProps;
begin
  // check if dest doesn't exist
  GetBackupAliasProps(ServerAlias, SrcAlias, bkProps);
  StoreBackupAliasProps(ServerAlias, DestAlias, bkProps);
  DeleteBackupAlias(ServerAlias, SrcAlias);
end;

procedure TibcIniPersistentInfo.GetBackupAliases(ServerAlias: string; Aliases: TStrings);
begin
  FIni.ReadSection(FormatSection([sectServers, ServerAlias, sectBackupFiles]), Aliases);
end;

procedure TibcIniPersistentInfo.GetBackupAliasProps(ServerAlias, BackupAlias: string; var BackupProps: TibcBackupProps);
var
  Section: string;
begin
  Section := FormatSection([sectServers, ServerAlias, sectBackupFiles]);
  if not FIni.ValueExists(Section, BackupAlias) then
    raise EIBCPersistent.Create(Format(EPACCESSBACKUPALIAS, [BackupAlias, ServerAlias]));

  Section := FormatSection([Section, BackupAlias]);
  BackupProps.ServerAlias := FIni.ReadString(Section, pkeyBackupSourceServer, '');
  BackupProps.DatabaseAlias := FIni.ReadString(Section, pkeyBackupSourceDatabase, '');
  BackupProps.BackupFiles := FIni.ReadString(Section, pkeyBackupFiles, '');
  BackupProps.Accessed := FIni.ReadDateTime(Section, pkeyAccessed, 0);
  BackupProps.Created := FIni.ReadDateTime(Section, pkeyCreated, 0);
end;

procedure TibcIniPersistentInfo.StoreBackupAliasProps(ServerAlias, BackupAlias: string; var BackupProps: TibcBackupProps);
var
  Section: string;
begin
  // check if server alias exists
  Section := FormatSection([sectServers, ServerAlias, sectBackupFiles]);
  FIni.WriteString(Section, BackupAlias, '');

  Section := FormatSection([Section, BackupAlias]);
  FIni.WriteString(Section, 'SourceDBServer', BackupProps.ServerAlias);
  FIni.WriteString(Section, 'SourceDBAlias', BackupProps.DatabaseAlias);
  FIni.WriteString(Section, 'BackupFiles', BackupProps.BackupFiles);
//  if not FIni.ValueExists(Section, pkeyCreated) then
    FIni.ReadDateTime(Section, pkeyCreated, 0);
  FIni.ReadDateTime(Section, pkeyAccessed, BackupProps.Accessed);
end;

  // external Apps/Tools
procedure TibcIniPersistentInfo.DoGetExternalApp(Section: string; var AppPath, WorkDir, Params: string);
begin
  AppPath := Fini.ReadString(Section, 'Path', '');
  WorkDir := Fini.ReadString(Section, 'WorkingDir', '');
  Params := Fini.ReadString(Section, 'Params', '');
end;

procedure TibcIniPersistentInfo.DoStoreExternalApp(Section: string; AppPath, WorkDir, Params: string);
begin
  Fini.WriteString(Section, 'Path', AppPath);
  Fini.WriteString(Section, 'WorkingDir', WorkDir);
  Fini.WriteString(Section, 'Params', Params);
end;

procedure TibcIniPersistentInfo.GetExternalApps(ExternalApps: TStrings);
var
  Apps: TOrderedStringList;
  i: integer;
begin
  Apps := TOrderedStringList.Create;
  try
    FIni.ReadSectionValues(sectTools, Apps);
    Apps.Sort;
    ExternalApps.Clear;
    for i := 0 to Apps.Count - 1 do
      ExternalApps.Add(Apps.Values[Apps.Names[i]]);
  finally
    Apps.Free;
 end;
end;

procedure TibcIniPersistentInfo.GetExternalApp(Pos: integer; var Title, AppPath, WorkDir, Params: string);
var
  Section: string;
  Apps: TStrings;
  Id: string;
begin
  Apps := TStringList.Create;
  try
    Fini.ReadSectionValues(sectTools, Apps);
    Id := Apps.Names[Pos];
    Title := Apps.Values[Id];
    Section := FormatSection([sectTOOLS, Id]);
    DoGetExternalApp(Section, AppPath, WorkDir, Params);
  finally
    Apps.Free;
  end;
end;

procedure TibcIniPersistentInfo.AddExternalApp(Title, AppPath, WorkDir, Params: string);
var
  Section: string;
  Apps: TOrderedStringList;
  NextS: string;
begin
  Apps := TOrderedStringList.Create;
  try
    Fini.ReadSectionValues(sectTools, Apps);
    Apps.Sort;
    NextS := IntToStr(Apps.NextAvail);
    Fini.WriteString(sectTools, NextS, Title);
    Section := FormatSection([sectTOOLS, NextS]);
    DoStoreExternalApp(Section, appPath, WorkDir, Params);
  finally
    Apps.Free;
  end;
end;

procedure TibcIniPersistentInfo.StoreExternalApp(Pos: integer; Title, AppPath, WorkDir, Params: string);
var
  Section, Id: string;
  Apps: TOrderedStringList;
begin
  Apps := TOrderedStringList.Create;
  try
    Fini.ReadSectionValues(sectTools, Apps);
    Apps.Sort;
    Id := Apps.Names[Pos];
    Fini.WriteString(sectTools, Id, Title);
    Section := FormatSection([sectTOOLS, Id]);
    DoStoreExternalApp(Section, appPath, WorkDir, Params);
  finally
    Apps.Free;
  end;
end;

procedure TibcIniPersistentInfo.DeleteExternalApp(Pos: integer);
var
  Id: string;
  Section: string;
  Apps: TOrderedStringList;
  i: integer;
  Title, AppPath, WorkDir, Params: string;
begin
  Apps := TOrderedStringList.Create;
  try
    Fini.ReadSectionValues(sectTools, Apps);
    Apps.Sort;
    for i := Pos + 1 to Apps.Count - 1 do
    begin
      // get a section
      Id := Apps.Names[i];
      Title := Apps.Values[Id];
      Section := FormatSection([sectTOOLS, Id]);
      DoGetExternalApp(Section, AppPath, WorkDir, Params);

      // store a section
      Id := Apps.Names[i - 1];
      Section := FormatSection([sectTOOLS, Id]);
      Fini.WriteString(sectTools, Id, Title);
      DoStoreExternalApp(Section, AppPath, WorkDir, Params);
    end;

    // delete last section
    Id := Apps.Names[Apps.Count - 1];
    Section := FormatSection([sectTOOLS, Id]);
    FIni.EraseSection(Section);
    Fini.DeleteKey(sectTools, Id);
  finally
    Apps.Free;
  end;
end;


end.

