{******************************************************************}
{  This Unit was modified by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.
}
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 *
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 *
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 *
 * Contributor(s):  Krzysztof Golko.
}
// IBOConsole Persistentce Interface
// Version 0.97.1

// Modified by Lorenzo Mengoni

unit IBOCPersistentTypes;
{
This unit is to make IBCPersisnt independent of IBOConsole and resuable
}
interface

uses
  SysUtils, Graphics;
  
type
  EIBOCPersistent = class(Exception);

  TPersistentSetting = record
    Name: string;
    Value: Variant;
  end;

{$IFDEF use_TFontProps}
  TFontProps = record
    FontName: String;
    FontSize: Integer;
    FontColor: TColor;
    FontStyle: TFontStyles;
    FontHeight: Integer;
    CharSet:TFontCharset;
  end;
{$ENDIF}

  // general persistent storage class
  TPersistentSettings = class
  private
    FSettings: array of TPersistentSetting;
    function GetValues(Index: integer): Variant;
    procedure SetValues(Index: integer; Value: Variant);
    function GetNames(Index: integer): string;
    procedure SetNames(Index: integer; Value: string);
    function GetSettings(Index: integer): TPersistentSetting;
    procedure SetSettings(Index: integer; Value: TPersistentSetting);

    function GetValueByName(Name: string): Variant;
    procedure SetValueByName(Name: string; Value: Variant);
  public
{$IFDEF use_TFontProps}
    FontProps:TFontProps;
{$ENDIF}
    constructor Create(NumSettings: integer); overload;
    constructor Create(Names: array of string); overload;
//  This is going to be prefereable way to initiate Tpersistentettings
//    constructor Create(InitialSettings: array of TPersistentSetting); overload;
    function Count: integer;
    function IndexOf(Name: string): integer;

    property Values[Index: integer]: Variant read GetValues write SetValues; default;
    property Names[Index: integer]: string read GetNames write SetNames;
    property Settings[Index: integer]: TPersistentSetting read GetSettings write SetSettings;
    property ValueByName[Name: string]: VAriant read GetValueByName write SetValueByName;
  end;

implementation

constructor TPersistentSettings.Create(NumSettings: integer);
begin
  inherited Create;
  SetLength(FSettings, NumSettings);
end;

constructor TPersistentSettings.Create(Names: array of string);
var
  i: integer;
begin
  SetLength(FSettings, Length(Names));
  for i := 0 to Length(Names) - 1 do
    FSettings[i].Name := Names[i];
end;

{constructor TPersistentSettings.Create(InitialSettings: array of TPersistentSetting);
var
  i: integer;
begin
  SetLength(FSettings, Length(InitialSettings));
  for i := 0 to Length(InitialSettings) - 1 do
  begin
    FSettings[i].Name := InitialSettings[i].Name;
    FSettings[i].Value := InitialSettings[i].Value;
  end;
end; }

function TPersistentSettings.GetValues(Index: integer): Variant;
begin
  Result := FSettings[Index].Value;
end;

procedure TPersistentSettings.SetValues(Index: integer; Value: Variant);
begin
  FSettings[Index].Value := Value;
end;

function TPersistentSettings.GetValueByName(Name: string): Variant;
var
  ind: integer;
begin
  ind := IndexOf(Name);
  // exception if name not found
  Result := FSettings[ind].Value;
end;

procedure TPersistentSettings.SetValueByName(Name: string; Value: Variant);
var
  ind: integer;
begin
  ind := IndexOf(Name);
  // exception if name not found
  FSettings[Ind].Value := Value;
end;

function TPersistentSettings.IndexOf(Name: string): integer;
var
  i: integer;
begin
  Result := -1;
  for i := 0 to Length(FSettings) - 1 do
    if SameText(Name, FSettings[i].Name) then
    begin
      Result := i;
      Break;
    end;
end;

function TPersistentSettings.GetNames(Index: integer): string;
begin
  Result := FSettings[Index].Name;
end;

procedure TPersistentSettings.SetNames(Index: integer; Value: string);
begin
  FSettings[Index].Name := Value;
end;

function TPersistentSettings.GetSettings(Index: integer): TPersistentSetting;
begin
  Result := FSettings[Index];
end;

procedure TPersistentSettings.SetSettings(Index: integer; Value: TPersistentSetting);
begin
  FSettings[Index] := Value;
end;

function TPersistentSettings.Count: integer;
begin
  Result := Length(FSettings);
end;


end.
