{******************************************************************}
{  This Unit was modified to by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.
}
{************************************************************************

9.Dicember.2002, Lorenzo:
- Adjusted out of screen positioning of windows

}

unit IBOCPersistentWinReg;
interface

uses
  Windows, SysUtils, Classes,
  Registry,
  Forms, IB_Session, Types, Graphics,
  IBOServices, Variants, IBOCPersistentTypes, IBOCPersistent;

type
  TIBOCWinRegPersistentInfo = class(tibcPersistentInfo)
  private
    FRegSettingsKey: string;
    FRegServersKey: string;
    FRegToolsKey: string;
    FRootKey: string;
    FRegistry: TRegistry;

    procedure InitRegistry;
    procedure DoGetSettings(var Settings: TPersistentSettings);
    procedure DoStoreSettings(Settings: TPersistentSettings);
    // presumes that a registry key is open
    procedure DoGetExternalApp(Pos: integer; var Title, AppPath, WorkDir, Params: string);
    procedure DoStoreExternalApp(Pos: integer; Title, AppPath, WorkDir, Params: string);
  protected
    procedure GetSetting(var Setting: TPersistentSetting); override;
    procedure StoreSetting(Setting: TPersistentSetting); override;
  public
    constructor Create(Root: string = '');
    destructor Destroy; override;
    procedure Update; override;
     // form methods shall be moved to IBCPersistentUtils
    procedure GetFormSettings(AForm: TForm; var Extra: word; Id: string); override;
    procedure StoreFormSettings(AForm: TForm; Extra: word; Id: string); override;
    // preferences
    procedure GetSettings(var Settings: TPersistentSettings); override;
    procedure StoreSettings(Settings: TPersistentSettings); override;
    procedure GetSectionSettings(SectionStrings: array of string; var Settings: TPersistentSettings; MustExist: boolean = false); override;
    procedure StoreSectionSettings(SectionStrings: array of string; Settings: TPersistentSettings); override;
    // server aliases
    function ServerAliasExists(Alias: string): boolean; override;
    procedure DeleteServerAlias(Alias: string); override;
    procedure RenameServerAlias(SrcAlias, DestAlias: string); override;
    procedure GetServerAliases(Aliases: TStrings); override;
    procedure GetServerProps(Alias: string; var ServerProps: TibcServerProps); override;
    procedure StoreServerProps(Alias: string; ServerProps: TibcServerProps); override;
    // database aliases
    function DatabaseAliasExists(ServerAlias, DatabaseAlias: string): boolean; override;
    procedure DeleteDatabaseAlias(ServerAlias, DatabaseAlias: string); override;
    procedure RenameDatabaseAlias(ServerAlias, SrcAlias, DestAlias: string); override;
    procedure GetDatabaseAliases(ServerAlias: string; Aliases: TStrings); override;
    procedure GetDatabaseProps(ServerAlias, DatabaseAlias: string; var DatabaseProps: TibcDatabaseProps); override;
    procedure StoreDatabaseProps(ServerAlias, DatabaseAlias: string; var DatabaseProps: TibcDatabaseProps); override;
    // backup aliases
    function BackupAliasExists(ServerAlias, BackupAlias: string): boolean; override;
    procedure DeleteBackupAlias(ServerAlias, BackupAlias: string); override;
    procedure RenameBackupAlias(ServerAlias, SrcAlias, DestAlias: string); override;
    procedure GetBackupAliases(ServerAlias: string; Aliases: TStrings); override;
    procedure GetBackupAliasProps(ServerAlias, BackupAlias: string; var BackupProps: TibcBackupProps); override;
    procedure StoreBackupAliasProps(ServerAlias, BackupAlias: string; var BackupProps: TibcBackupProps); override;
    // external Apps/Tools
    procedure GetExternalApps(ExternalApps: TStrings); override;
    procedure GetExternalApp(Pos: integer; var Title, AppPath, WorkDir, Params: string); override;
    procedure AddExternalApp(Title, AppPath, WorkDir, Params: string); override;
    procedure StoreExternalApp(Pos: integer; Title, AppPath, WorkDir, Params: string); override;
    procedure DeleteExternalApp(Pos: integer); override;
    property Registry: TRegistry read FRegistry;
  end;
implementation
{$A+}

// The above is to ensure that TibcFormSettings record is 20 bytes long
type
  TIBOCFormSettings = record
     _Top,
     _Left,
     _Height,
     _Width: integer;
     _State: TWindowState;
     _Extra: word ; // since fields in record are word aligned, word has the same size as boolean
  end;

{ TIBOCWinRegPersistentInfo }

constructor TIBOCWinRegPersistentInfo.Create;
begin
  FRegistry := TRegistry.Create;
  InitRegistry;
end;

destructor TIBOCWinRegPersistentInfo.Destroy;
begin
  FRegistry.Free;
end;

procedure TIBOCWinRegPersistentInfo.Update;
begin
  // update is for FIniPersistent and is not needed here
  // just in case in calls CloseKey
  FRegistry.CloseKey;
end;

procedure TIBOCWinRegPersistentInfo.GetSetting(var Setting: TPersistentSetting);
begin
  if FRegistry.ValueExists(Setting.Name) then
    case (VarType(Setting.Value) and varTypeMask) of
      varSmallint: Setting.Value := FRegistry.ReadInteger(Setting.Name);
      varInteger: Setting.Value := FRegistry.ReadInteger(Setting.Name);
      varBoolean: Setting.Value := FRegistry.ReadBool(Setting.Name);
      varString: Setting.Value := FRegistry.ReadString(Setting.Name);
    end;
end;

procedure TIBOCWinRegPersistentInfo.StoreSetting(Setting: TPersistentSetting);
begin
  case (VarType(Setting.Value) and varTypeMask) of
    varSmallint: FRegistry.WriteInteger(Setting.Name, Setting.Value);
    varInteger: FRegistry.WriteInteger(Setting.Name, Setting.Value);
    varBoolean: FRegistry.WriteBool(Setting.Name, Setting.Value);
    varString: FRegistry.WriteString(Setting.Name, Setting.Value);
  end;
end;

procedure TIBOCWinRegPersistentInfo.InitRegistry;
begin
  FRegistry.RootKey := HKEY_CURRENT_USER;
  FRegistry.OpenKey('Software', true);
  FRegistry.OpenKey('Borland', true);
  FRegistry.OpenKey('InterBase', true);
  FRegistry.OpenKey('IBOConsole', true);
  FRootKey := FRegistry.CurrentPath;
  FRegServersKey := Format('\%s\Servers\',[FRegistry.CurrentPath]);
  FRegSettingsKey := Format('\%s\Settings',[FRegistry.CurrentPath]);
  FRegToolsKey := Format('%s\Tools',[FRegSettingsKey]);
  FRegistry.CloseKey;
end;

procedure TIBOCWinRegPersistentInfo.GetFormSettings(AForm: TForm; var Extra: word; Id: string);
var
  wSettings: TIBOCFormSettings;
  R,Ro:TRect;
  x,y:integer;
begin
  if FRegistry.OpenKey(FRegSettingsKey, false) then
  begin
    Extra := 0;
    try
      if FRegistry.ReadBinaryData(Id, wSettings, SizeOf(TIBOCFormSettings)) >= SizeOf(TIBOCFormSettings) then
      begin
        // only wsNrmal and wsMaximized are considered valid windows state
        if (wSettings._State = wsNormal) or (wSettings._State = wsMaximized) then
        begin
          R := Bounds(wSettings._Left, wSettings._Top, wSettings._Width, wSettings._Height);
          if Types.IntersectRect(Ro, Screen.DesktopRect, R)
             and PtInRect(Screen.DesktopRect,R.BottomRight)
          then
          begin
            AForm.BoundsRect := R;
            AForm.WindowState := wSettings._State;
            Extra := wSettings._Extra;
          end
          else
          begin
            if wSettings._Width > Screen.Width then wSettings._Width:=Screen.Width;
            x := (Screen.Width - wSettings._Width) div 2;
            if x < 0 then x:=0;
            if wSettings._Height > Screen.Height then wSettings._Height:=Screen.Height;
            y := (Screen.Height - wSettings._Height) div 2;
            if y < 0 then y:=0;
            R := Bounds(x, y, wSettings._Width, wSettings._Height);
            AForm.BoundsRect := R;
            AForm.WindowState := wSettings._State;
            Extra := wSettings._Extra;
          end
        end;
      end;
    finally
      FRegistry.CloseKey;
    end;
  end;
end;

procedure TIBOCWinRegPersistentInfo.StoreFormSettings(AForm: TForm; Extra: word; Id: string);
var
 wSettings: TIBOCFormSettings;
 WasMaximized: Boolean;
 // thanks to Glenn B. Andersen for providing the code to do special handling
 //        in case a form is maximaised
begin
 if FRegistry.OpenKey(FRegSettingsKey, true) then
 begin
   try
     WasMaximized := AForm.WindowState = wsMaximized;
     if WasMaximized then
       AForm.WindowState := wsNormal;
     wSettings._Top := AForm.Top;
     wSettings._Left := AForm.Left;
     wSettings._Height := AForm.Height;
     wSettings._Width := AForm.Width;
     if WasMaximized then
       wSettings._State := wsMaximized
     else
       wSettings._State := AForm.WindowState;
     wSettings._Extra := Extra;
     FRegistry.WriteBinaryData(Id, wSettings, SizeOf(TIBOCFormSettings));
   finally
     FRegistry.CloseKey;
   end;
 end;
end;

procedure TIBOCWinRegPersistentInfo.DoGetSettings(var Settings: TPersistentSettings);
var
  i: integer;
  TempSetting: TPersistentSetting;
{$IFDEF use_TFontProps}
  s:String;
{$ENDIF}

begin
  for i := 0 to Settings.Count - 1 do
  begin
    TempSetting := Settings.Settings[i];
    GetSetting(TempSetting);
    Settings.Settings[i] := TempSetting;
  end;
{$IFDEF use_TFontProps}
  try
    Settings.FontProps.FontName := FRegistry.ReadString('gFontName');
    Settings.FontProps.FontSize := FRegistry.ReadInteger('gFontSize');
    Settings.FontProps.FontColor := FRegistry.ReadInteger('gFontColor');
    s:= FRegistry.ReadString('gFontStyle');
    Settings.FontProps.FontStyle := [];
    if pos('B',s)>0 then Settings.FontProps.FontStyle := [fsBold] + Settings.FontProps.FontStyle;
    if pos('I',s)>0 then Settings.FontProps.FontStyle := [fsItalic] + Settings.FontProps.FontStyle;
    if pos('U',s)>0 then Settings.FontProps.FontStyle := [fsUnderline] + Settings.FontProps.FontStyle;
    if pos('S',s)>0 then Settings.FontProps.FontStyle := [fsStrikeOut] + Settings.FontProps.FontStyle;
    Settings.FontProps.FontHeight := FRegistry.ReadInteger('gFontHeight');
    Settings.FontProps.CharSet := FRegistry.ReadInteger('gCharSet');
  except
  end;
{$ENDIF}
end;

procedure TIBOCWinRegPersistentInfo.DoStoreSettings(Settings: TPersistentSettings);
var
  i: integer;
{$IFDEF use_TFontProps}
  s:String;
{$ENDIF}
begin
  for i := 0 to Settings.Count - 1 do
    StoreSetting(Settings.Settings[i]);
{$IFDEF use_TFontProps}
  FRegistry.WriteString('gFontName', Settings.FontProps.FontName);
  FRegistry.WriteInteger('gFontSize', Settings.FontProps.FontSize);
  FRegistry.WriteInteger('gFontColor', Settings.FontProps.FontColor);
  s:='';
  if fsBold in Settings.FontProps.FontStyle then s:=s+'B';
  if fsItalic in Settings.FontProps.FontStyle then s:=s+'I';
  if fsUnderline in Settings.FontProps.FontStyle then s:=s+'U';
  if fsStrikeOut in Settings.FontProps.FontStyle then s:=s+'S';
  FRegistry.WriteString('gFontStyle', s);
  FRegistry.WriteInteger('gFontHeight', Settings.FontProps.FontHeight);
  FRegistry.WriteInteger('gCharSet', Settings.FontProps.CharSet);
{$ENDIF}
end;

procedure TIBOCWinRegPersistentInfo.GetSettings(var Settings: TPersistentSettings);
begin
 if FRegistry.OpenKey(FRegSettingsKey, false) then
 begin
   try
     DoGetSettings(Settings);
   finally
     FRegistry.CloseKey;
   end;
 end
{ the exception has been removed to deal with the case when the settings
 doesn't exist, GetSectionSettings handles that problem more acurately
 by providing MustExist parameter defaulting to false, GetSetting is now considered
 a legacy method }
{  else
   raise EIBCPersistent.Create(Format(EPACCESSSECTION, [FRegSettingsKey])); }
end;

procedure TIBOCWinRegPersistentInfo.StoreSettings(Settings: TPersistentSettings);
begin
 if FRegistry.OpenKey(FRegSettingsKey, true) then
 begin
   try
     DoStoreSettings(Settings);
   finally
     FRegistry.CloseKey;
   end;
 end
 else
   raise EIBCPersistent.Create(Format(EPACCESSSECTION, [FRegSettingsKey]));
end;

procedure TIBOCWinRegPersistentInfo.GetSectionSettings(
 SectionStrings: array of string; var Settings: TPersistentSettings; MustExist: boolean = false);
var
  i: integer;
  SectionKey: string;
begin
 SectionKey := FRootKey;
 for i := 0 to Length(SectionStrings) - 1 do
   SectionKey := SectionKey + '\' + SectionStrings[i];
  if FRegistry.OpenKey(SectionKey, false) then
 begin
   try
     DoGetSettings(Settings);
   finally
     FRegistry.CloseKey;
   end;
 end else
   if MustExist then
     raise EIBCPersistent.Create(Format(EPACCESSSECTION, [SectionKey]));
end;

procedure TIBOCWinRegPersistentInfo.StoreSectionSettings(
  SectionStrings: array of string; Settings: TPersistentSettings);
var
  i: integer;
  SectionKey: string;
begin
  SectionKey := FRootKey;
  for i := 0 to Length(SectionStrings) - 1 do
    SectionKey := SectionKey + '\' + SectionStrings[i];
  if FRegistry.OpenKey(SectionKey, true) then
  begin
    try
      DoStoreSettings(Settings);
    finally
      FRegistry.CloseKey;
    end;
  end
  else
    raise EIBCPersistent.Create(Format(EPACCESSSECTION, [SectionKey]));
end;

function TIBOCWinRegPersistentInfo.ServerAliasExists(Alias: string): boolean;
begin
  Result := FRegistry.KeyExists(Format('%s%s',[FRegServersKey, Alias]));
end;

procedure TIBOCWinRegPersistentInfo.DeleteServerAlias(Alias: string);
begin
  FRegistry.CloseKey;
  // what about subkeys under Win NT ???
  // FRegistry.DeleteKey(Format('%s%s\Databases',[gRegServersKey,Node]));
  FRegistry.DeleteKey(Format('%s%s',[FRegServersKey, Alias]));
end;

procedure TIBOCWinRegPersistentInfo.RenameServerAlias(SrcAlias, DestAlias: string);
begin
  // if an alias already exists an exception should be raised here???
  FRegistry.MoveKey(Format('%s%s',[FRegServersKey, SrcAlias]),
       Format('%s%s',[FRegServersKey, DestAlias]), true);
end;

procedure TIBOCWinRegPersistentInfo.GetServerAliases(Aliases: TStrings);
begin
  if FRegistry.OpenKey(FRegServersKey, false) then
  begin
    FRegistry.GetKeyNames(Aliases);
    FRegistry.CloseKey;
  end
  else
    Aliases.Clear;
end;

procedure TIBOCWinRegPersistentInfo.GetServerProps(Alias: string; var ServerProps: TibcServerProps);
begin
  ServerProps.ServerName := '';
  ServerProps.UserName := '';
  ServerProps.Description := '';
  ServerProps.Protocol := cpLocal;
  ServerProps.Created := 0;
  ServerProps.Accessed := 0;
  if FRegistry.OpenKey(Format('%s%s',[FRegServersKey, Alias]), false) then
  begin
    try
      case FRegistry.ReadInteger('Protocol') of
        0: ServerProps.Protocol := cpTCP_IP;
        1: ServerProps.Protocol := cpNetBEUI;
        2: ServerProps.Protocol := cpNovell;
        3: ServerProps.Protocol := cpLocal;
      end;
      ServerProps.ServerName := FRegistry.ReadString('ServerName');
      ServerProps.UserName := FRegistry.ReadString('UserName');
      ServerProps.Description := FRegistry.ReadString('Description');
      if FRegistry.ValueExists('Created') then
        ServerProps.Created := FRegistry.ReadDateTime ('Created');
      if FRegistry.ValueExists('Last Accessed') then
        ServerProps.Accessed := FRegistry.ReadDateTime ('Last Accessed');
   finally
     FRegistry.CloseKey;
   end;
 end
 else
   raise EIBCPersistent.Create(Format(EPACCESSSERVERALIAS, [Alias]));
end;

procedure TIBOCWinRegPersistentInfo.StoreServerProps(Alias: string; ServerProps: TibcServerProps);
begin
 if FRegistry.OpenKey(Format('%s%s',[FRegServersKey, Alias]), true) then
 begin
   try
     FRegistry.WriteString('ServerName', ServerProps.ServerName);
     case ServerProps.Protocol of
       cpTCP_IP: FRegistry.WriteInteger('Protocol',0);
       cpNETBeui: FRegistry.WriteInteger('Protocol',1);
       cpNovell: FRegistry.WriteInteger('Protocol',2);
       cpLocal: FRegistry.WriteInteger('Protocol',3);
     end;
     FRegistry.WriteString('Username', ServerProps.Username);
     FRegistry.WriteString('Description', ServerProps.Description);
     FRegistry.WriteDateTime('Last Accessed', ServerProps.Accessed);
     if ServerProps.Created <> 0 then
       FRegistry.WriteDateTime('Created', ServerProps.Created);
   finally
     FRegistry.CloseKey;
   end;
 end
 else
   raise EIBCPersistent.Create('Persistent data write error. Cannot access server alias: ' + Alias);
end;

function TIBOCWinRegPersistentInfo.DatabaseAliasExists(ServerAlias, DatabaseAlias: string): boolean;
begin
  Result := FRegistry.KeyExists(Format('%s%s\Databases\%s',[FRegServersKey, ServerAlias, DatabaseAlias]));
end;

procedure TIBOCWinRegPersistentInfo.DeleteDatabaseAlias(ServerAlias, DatabaseAlias: string);
begin
  FRegistry.CloseKey;
  FRegistry.DeleteKey(Format('%s%s\Databases\%s',[FRegServersKey, ServerAlias, DatabaseAlias]));
end;

procedure TIBOCWinRegPersistentInfo.RenameDatabaseAlias(ServerAlias, SrcAlias, DestAlias: string);
begin
  // if an alias already exists an exception should be raised here???
  FRegistry.MoveKey(Format('%s%s\Databases\%s',[FRegServersKey, ServerAlias, SrcAlias]),
        Format('%s%s\Databases\%s',[FRegServersKey, ServerAlias, DestAlias]), true);
  FRegistry.CloseKey;
end;

procedure TIBOCWinRegPersistentInfo.GetDatabaseAliases(ServerAlias: string; Aliases: TStrings);
begin
  Aliases.Clear;
  if FRegistry.OpenKey(Format('%s%s\Databases',[FRegServersKey, ServerAlias]), false) then
  begin
    FRegistry.GetKeyNames(Aliases);
    FRegistry.CloseKey;
  end;
end;

procedure TIBOCWinRegPersistentInfo.GetDatabaseProps(ServerAlias, DatabaseAlias: string; var DatabaseProps: TibcDatabaseProps);
begin
  DatabaseProps.DatabaseFiles := '';
  DatabaseProps.UserName := '';
  DatabaseProps.Role := '';
  DatabaseProps.CharacterSet := '';
  DatabaseProps.CaseSensitiveRole := false;
  DatabaseProps.Accessed := 0;
  DatabaseProps.Created := 0;
  if FRegistry.OpenKey(Format('%s%s\Databases\%s',[FRegServersKey, ServerAlias, DatabaseAlias]), false) then
  begin
    try
      DatabaseProps.DatabaseFiles := FRegistry.ReadString('DatabaseFiles');
      DatabaseProps.UserName := FRegistry.ReadString('Username');
      DatabaseProps.Role := FRegistry.ReadString('Role');
      DatabaseProps.CharacterSet := FRegistry.ReadString('CharacterSet');
      if FRegistry.ValueExists('CaseSensitiveRole') then
        DatabaseProps.CaseSensitiveRole := FRegistry.ReadBool('CaseSensitiveRole');
      if FRegistry.ValueExists('Last Accessed') then
        DatabaseProps.Accessed := FRegistry.ReadDateTime('Last accessed');
      if FRegistry.ValueExists('Created') then
        DatabaseProps.Created := FRegistry.ReadDateTime('Created');
    finally
      FRegistry.CloseKey;
    end
  end
  else
    raise EIBCPersistent.Create(Format(EPACCESSDATABASEALIAS, [DatabaseAlias, ServerAlias]));
end;

procedure TIBOCWinRegPersistentInfo.StoreDatabaseProps(ServerAlias, DatabaseAlias: string; var DatabaseProps: TibcDatabaseProps);
begin
  if FRegistry.OpenKey(Format('%s%s\Databases\%s',[FRegServersKey, ServerAlias, DatabaseAlias]), true) then
  begin
    try
      FRegistry.WriteString('DatabaseFiles', DatabaseProps.DatabaseFiles);
      FRegistry.WriteString('Username', DatabaseProps.Username);
      FRegistry.WriteString('Role', DatabaseProps.Role);
      FRegistry.WriteBool('CaseSensitiveRole', DatabaseProps.CaseSensitiveRole);
      FRegistry.WriteString('CharacterSet', DatabaseProps.CharacterSet);
      FRegistry.WriteDateTime('Last Accessed', DatabaseProps.Accessed);
      if DatabaseProps.Created <> 0 then
        FRegistry.WriteDateTime('Created', DatabaseProps.Created);
    finally
      FRegistry.CloseKey;
    end;
  end
  else
    raise EIBCPersistent.Create('Persistent data write error. Cannot access database alias: ' + DatabaseAlias);
end;

// backup aliases
function TIBOCWinRegPersistentInfo.BackupAliasExists(ServerAlias, BackupAlias: string): boolean;
begin
  Result := FRegistry.KeyExists(Format('%s%s\Backup Files\%s',[FRegServersKey, ServerAlias, BackupAlias]));
end;

procedure TIBOCWinRegPersistentInfo.DeleteBackupAlias(ServerAlias, BackupAlias: string);
begin
  FRegistry.CloseKey;
  FRegistry.DeleteKey(Format('%s%s\Backup Files\%s',[FRegServersKey, ServerAlias, BackupAlias]));
end;

procedure TIBOCWinRegPersistentInfo.GetBackupAliases(ServerAlias: string; Aliases: TStrings);
begin
  Aliases.Clear;
  if FRegistry.OpenKey(Format('%s%s\Backup Files',[FRegServersKey, ServerAlias]), false) then
  begin
    FRegistry.GetKeyNames(Aliases);
    FRegistry.CloseKey;
  end;
end;

procedure TIBOCWinRegPersistentInfo.GetBackupAliasProps(ServerAlias, BackupAlias: string; var BackupProps: TibcBackupProps);
begin
  BackupProps.ServerAlias := '';
  BackupProps.DatabaseAlias := '';
  BackupProps.BackupFiles := '';
  BackupProps.Created := 0;
  BackupProps.Accessed := 0;
  if FRegistry.OpenKey(Format('%s%s\Backup Files\%s',[FRegServersKey, ServerAlias, BackupAlias]), false) then
  begin
    BackupProps.ServerAlias := FRegistry.ReadString('SourceDBServer');
    BackupProps.DatabaseAlias := FRegistry.ReadString('SourceDBAlias');
    BackupProps.BackupFiles := FRegistry.ReadString('BackupFiles');
    if FRegistry.ValueExists('Created') then
      BackupProps.Created := FRegistry.ReadDateTime('Created');
    if FRegistry.ValueExists('Accessed') then
      BackupProps.Accessed := FRegistry.ReadDateTime('Accessed');
  end;
end;

procedure TIBOCWinRegPersistentInfo.StoreBackupAliasProps(ServerAlias, BackupAlias: string; var BackupProps: TibcBackupProps);
begin
  if FRegistry.OpenKey(Format('%s%s\Backup Files\%s',[FRegServersKey, ServerAlias, BackupAlias]), true) then
  begin
    FRegistry.WriteString('SourceDBServer', BackupProps.ServerAlias);
    FRegistry.WriteString('SourceDBAlias', BackupProps.DatabaseAlias);
    FRegistry.WriteString('BackupFiles', BackupProps.BackupFiles);
    if not FRegistry.ValueExists('Created') then
      FRegistry.WriteDateTime('Created', BackupProps.Created);
    FRegistry.WriteDateTime ('Accessed', BackupProps.Accessed);
    FRegistry.CloseKey();
  end;
end;

procedure TIBOCWinRegPersistentInfo.RenameBackupAlias(ServerAlias, SrcAlias, DestAlias: string);
begin
  if SrcAlias = DestAlias then Exit;
  // if an alias already exists an exception should be raised here???
  FRegistry.CloseKey;
  FRegistry.MoveKey(Format('%s%s\Backup Files\%s',[FRegServersKey, ServerAlias, SrcAlias]),
  Format('%s%s\Backup Files\%s',[FRegServersKey, ServerAlias, DestAlias]), true);
end;

// external Apps/Tools
procedure TIBOCWinRegPersistentInfo.GetExternalApps(ExternalApps: TStrings);
var
  iCount, i: integer;
begin
  ExternalApps.Clear;
  if FRegistry.OpenKey(FRegToolsKey, false) and FRegistry.ValueExists('Count') then
  begin
    iCount := FRegistry.ReadInteger('Count');
    for i := 0 to iCount - 1 do
      ExternalApps.Add(FRegistry.ReadString(Format('Title%d', [i])));
    FRegistry.CloseKey;
  end;
end;

procedure TIBOCWinRegPersistentInfo.DoGetExternalApp(Pos: integer; var Title, AppPath, WorkDir, Params: string);
begin
  Title := FRegistry.ReadString(Format('Title%d',[Pos]));
  AppPath := FRegistry.ReadString(Format('Path%d',[Pos]));
  WorkDir := FRegistry.ReadString (Format('WorkingDir%d',[Pos]));
  Params := FRegistry.ReadString (Format('Params%d',[Pos]));
end;

procedure TIBOCWinRegPersistentInfo.DoStoreExternalApp(Pos: integer; Title, AppPath, WorkDir, Params: string);
begin
  FRegistry.WriteString(Format('Title%d', [Pos]), Title);
  FRegistry.WriteString(Format('Path%d', [Pos]), AppPath);
  FRegistry.WriteString(Format('WorkingDir%d', [Pos]), WorkDir);
  FRegistry.WriteString(Format('Params%d', [Pos]), Params);
end;

procedure TIBOCWinRegPersistentInfo.GetExternalApp(Pos: integer; var Title, AppPath, WorkDir, Params: string);
begin
  if FRegistry.OpenKey(FRegToolsKey, false) then
  begin
    DoGetExternalApp(Pos, Title, AppPath, WorkDir, Params);
    FRegistry.CloseKey;
  end;
end;

procedure TIBOCWinRegPersistentInfo.AddExternalApp(Title, AppPath, WorkDir, Params: string);
var
 iPos : integer;
begin
  if FRegistry.OpenKey(FRegToolsKey, true) then
  begin
    if FRegistry.ValueExists('Count') then
      iPos := FRegistry.ReadInteger('Count')
    else
      iPos := 0;
    DoStoreExternalApp(iPos, Title, AppPath, WorkDir, Params);
    Inc(iPos);
    FRegistry.WriteInteger('Count', iPos);
    FRegistry.CloseKey;
  end;
end;

procedure TIBOCWinRegPersistentInfo.StoreExternalApp(Pos: integer; Title, AppPath, WorkDir, Params: string);
begin
  if FRegistry.OpenKey(FRegToolsKey, true) then
  begin
    DoStoreExternalApp(Pos, Title, AppPath, WorkDir, Params);
    FRegistry.CloseKey;
  end;
end;

procedure TIBOCWinRegPersistentInfo.DeleteExternalApp(Pos: integer);
var
  i, iCnt: integer;
begin
  if FRegistry.OpenKey(FRegToolsKey, false) then
  begin
    iCnt := FRegistry.ReadInteger('Count');
    FRegistry.DeleteValue(Format('Title%d', [Pos]));
    FRegistry.DeleteValue(Format('Path%d', [Pos]));
    FRegistry.DeleteValue(Format('WorkingDir%d', [Pos]));
    FRegistry.DeleteValue(Format('Params%d', [Pos]));
    for i := Pos + 1 to iCnt - 1 do
    begin
      FRegistry.RenameValue(Format('Title%d', [i]), Format('Title%d', [i - 1]));
      FRegistry.RenameValue(Format('Path%d', [i]), Format('Path%d', [i - 1]));
      FRegistry.RenameValue(Format('WorkingDir%d', [i]), Format('WorkingDir%d', [i - 1]));
      FRegistry.RenameValue(Format('Params%d', [i]), Format('Params%d', [i - 1]));
    end;
    if iCnt > 0 then
    begin
      FRegistry.WriteInteger('Count', iCnt - 1);
    end
    else
      FRegistry.DeleteKey('Count');
    FRegistry.CloseKey;
  end;
end;


end.
