{************************************************************************
  This Unit was createed by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed under the License is distributed on
    an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
    express or implied. See the ORIGINAL Copyright

16.March.2002, Lorenzo:
- Created
}
unit IBOCPersistentXML;

interface
 
uses
  SysUtils, Classes, Forms, IB_Session, Types, Graphics, XMLParser,
  IBOServices, Variants, IBOCPersistentTypes, IBOCPersistent;

type
  TIBOCPersistentInfoXML = class(tibcPersistentInfo)
  private
    fParser :TXMLParser;
    FRootNode: string;
    FRegSettingsNode: string;
    FRegServersKey: string;
    FRegToolsKey: string;
    procedure InitRegistry;
    procedure DoGetSettings(var Settings: TPersistentSettings);
    procedure DoStoreSettings(Settings: TPersistentSettings);
    // presumes that a registry key is open
    procedure DoGetExternalApp(Pos: integer; var Title, AppPath, WorkDir, Params: string);
    procedure DoStoreExternalApp(Pos: integer; Title, AppPath, WorkDir, Params: string);
  protected
    procedure GetSetting(var Setting: TPersistentSetting); override;
    procedure StoreSetting(Setting: TPersistentSetting); override;
  public
    constructor Create(Root: string = '');
    destructor Destroy; override;
    procedure Update; override;
     // form methods shall be moved to IBCPersistentUtils
    procedure GetFormSettings(AForm: TForm; var Extra: word; Id: string); override;
    procedure StoreFormSettings(AForm: TForm; Extra: word; Id: string); override;
    // preferences
    procedure GetSettings(var Settings: TPersistentSettings); override;
    procedure StoreSettings(Settings: TPersistentSettings); override;
    procedure GetSectionSettings(SectionStrings: array of string; var Settings: TPersistentSettings; MustExist: boolean = false); override;
    procedure StoreSectionSettings(SectionStrings: array of string; Settings: TPersistentSettings); override;
    // server aliases
    function ServerAliasExists(Alias: string): boolean; override;
    procedure DeleteServerAlias(Alias: string); override;
    procedure RenameServerAlias(SrcAlias, DestAlias: string); override;
    procedure GetServerAliases(Aliases: TStrings); override;
    procedure GetServerProps(Alias: string; var ServerProps: TibcServerProps); override;
    procedure StoreServerProps(Alias: string; ServerProps: TibcServerProps); override;
    // database aliases
    function DatabaseAliasExists(ServerAlias, DatabaseAlias: string): boolean; override;
    procedure DeleteDatabaseAlias(ServerAlias, DatabaseAlias: string); override;
    procedure RenameDatabaseAlias(ServerAlias, SrcAlias, DestAlias: string); override;
    procedure GetDatabaseAliases(ServerAlias: string; Aliases: TStrings); override;
    procedure GetDatabaseProps(ServerAlias, DatabaseAlias: string; var DatabaseProps: TibcDatabaseProps); override;
    procedure StoreDatabaseProps(ServerAlias, DatabaseAlias: string; var DatabaseProps: TibcDatabaseProps); override;
    // backup aliases
    function BackupAliasExists(ServerAlias, BackupAlias: string): boolean; override;
    procedure DeleteBackupAlias(ServerAlias, BackupAlias: string); override;
    procedure RenameBackupAlias(ServerAlias, SrcAlias, DestAlias: string); override;
    procedure GetBackupAliases(ServerAlias: string; Aliases: TStrings); override;
    procedure GetBackupAliasProps(ServerAlias, BackupAlias: string; var BackupProps: TibcBackupProps); override;
    procedure StoreBackupAliasProps(ServerAlias, BackupAlias: string; var BackupProps: TibcBackupProps); override;
    // external Apps/Tools
    procedure GetExternalApps(ExternalApps: TStrings); override;
    procedure GetExternalApp(Pos: integer; var Title, AppPath, WorkDir, Params: string); override;
    procedure AddExternalApp(Title, AppPath, WorkDir, Params: string); override;
    procedure StoreExternalApp(Pos: integer; Title, AppPath, WorkDir, Params: string); override;
    procedure DeleteExternalApp(Pos: integer); override;
  end;
implementation
{$A+}

// The above is to ensure that TibcFormSettings record is 20 bytes long
type
  TIBOCFormSettings = record
     _Top,
     _Left,
     _Height,
     _Width: integer;
     _State: TWindowState;
     _Extra: word ; // since fields in record are word aligned, word has the same size as boolean
  end;

procedure ReadFormSettings(const Parser:TXMLParser; const ID:string; var rec:TIBOCFormSettings);
begin
end;

procedure WriteFormSettings(const Parser:TXMLParser; const ID:String; const rec:TIBOCFormSettings);
begin
end;

procedure ReadServerProps(const Parser:TXMLParser; const ID:string; var ServerProps: TibcServerProps);
begin
end;

procedure WriteServerProps(const Parser:TXMLParser; const ID:String; const ServerProps: TibcServerProps);
begin
end;

    { TibcPersistentInfo }

constructor TIBOCPersistentInfoXML.Create;
begin
  InitRegistry;
  fParser :=TXMLParser.create;
  FRootNode := 'iboconsole';
end;

destructor TIBOCPersistentInfoXML.Destroy;
begin
  fParser.Free;
end;

procedure TIBOCPersistentInfoXML.Update;
begin
  fParser.SaveToFile(ChangeFileExt(Application.ExeName,'.xml'));
end;

procedure TIBOCPersistentInfoXML.GetSetting(var Setting: TPersistentSetting);
begin
  if fParser.NodeExists(Setting.Name) then
   case (VarType(Setting.Value) and varTypeMask) of
     varSmallint: Setting.Value := fParser.ReadInteger(Setting.Name);
     varInteger: Setting.Value := fParser.ReadInteger(Setting.Name);
     varBoolean: Setting.Value := fParser.ReadBool(Setting.Name);
     varString: Setting.Value := fParser.ReadString(Setting.Name);
   end;
end;

procedure TIBOCPersistentInfoXML.StoreSetting(Setting: TPersistentSetting);
begin
  case (VarType(Setting.Value) and varTypeMask) of
    varSmallint: fParser.WriteInteger(Setting.Name, Setting.Value);
    varInteger: fParser.WriteInteger(Setting.Name, Setting.Value);
    varBoolean: fParser.WriteBool(Setting.Name, Setting.Value);
    varString: fParser.WriteString(Setting.Name, Setting.Value);
  end;
end;

procedure TIBOCPersistentInfoXML.InitRegistry;
var nf:String;
begin
  nf := ChangeFileExt(Application.ExeName,'.xml');
  if FileExists(nf) then
    fParser.LoadFromFile(nf)
  else
    fParser.Text := '';
//  FRootKey := fParser.CurrentPath;
//  FRegServersKey := Format('\%s\Servers\',[fParser.CurrentPath]);
//  FRegSettingsKey := Format('\%s\Settings',[fParser.CurrentPath]);
//  FRegToolsKey := Format('%s\Tools',[FRegSettingsKey]);
//  fParser.CloseKey;
end;

procedure TIBOCPersistentInfoXML.GetFormSettings(AForm: TForm; var Extra: word; Id: string);
var
  wSettings: TIBOCFormSettings;
  R,Ro:TRect;
  x,y:integer;
begin
  if fParser.FindNode(FRegSettingsNode, false) then
  begin
    Extra := 0;
    try
      ReadFormSettings(fParser, Id, wSettings);
      begin
        // only wsNrmal and wsMaximized are considered valid windows state
        if (wSettings._State = wsNormal) or (wSettings._State = wsMaximized) then
        begin
          R := Bounds(wSettings._Left, wSettings._Top, wSettings._Width, wSettings._Height);
          if Types.IntersectRect(Ro, Screen.DesktopRect, R)
             and PtInRect(Screen.DesktopRect,R.BottomRight)
          then
          begin
            AForm.BoundsRect := R;
            AForm.WindowState := wSettings._State;
            Extra := wSettings._Extra;
          end
          else
          begin
            if wSettings._Width > Screen.Width then wSettings._Width:=Screen.Width;
            x := (Screen.Width - wSettings._Width) div 2;
            if x < 0 then x:=0;
            if wSettings._Height > Screen.Height then wSettings._Height:=Screen.Height;
            y := (Screen.Height - wSettings._Height) div 2;
            if y < 0 then y:=0;
            R := Bounds(x, y, wSettings._Width, wSettings._Height);
            AForm.BoundsRect := R;
            AForm.WindowState := wSettings._State;
            Extra := wSettings._Extra;
          end
        end;
      end;
    finally
    end;
  end;
end;

procedure TIBOCPersistentInfoXML.StoreFormSettings(AForm: TForm; Extra: word; Id: string);
var
 wSettings: TIBOCFormSettings;
 WasMaximized: Boolean;
 // thanks to Glenn B. Andersen for providing the code to do special handling
 //        in case a form is maximaised
begin
 if fParser.FindNode(FRegSettingsNode, false) then
 begin
   try
     WasMaximized := AForm.WindowState = wsMaximized;
     if WasMaximized then
       AForm.WindowState := wsNormal;
     wSettings._Top := AForm.Top;
     wSettings._Left := AForm.Left;
     wSettings._Height := AForm.Height;
     wSettings._Width := AForm.Width;
     if WasMaximized then
       wSettings._State := wsMaximized
     else
       wSettings._State := AForm.WindowState;
     wSettings._Extra := Extra;
     WriteFormSettings(fParser,Id, wSettings);
   finally
   end;
 end;
end;

procedure TIBOCPersistentInfoXML.DoGetSettings(var Settings: TPersistentSettings);
var
 i: integer;
 TempSetting: TPersistentSetting;
 s:String;

begin
  for i := 0 to Settings.Count - 1 do
  begin
    TempSetting := Settings.Settings[i];
    GetSetting(TempSetting);
    Settings.Settings[i] := TempSetting;
  end;
  try
    Settings.FontProps.FontName := fParser.ReadString('gFontName');
    Settings.FontProps.FontSize := fParser.ReadInteger('gFontSize');
    Settings.FontProps.FontColor := fParser.ReadInteger('gFontColor');
    s:= fParser.ReadString('gFontStyle');
    Settings.FontProps.FontStyle := [];
    if pos('B',s)>0 then Settings.FontProps.FontStyle := [fsBold] + Settings.FontProps.FontStyle;
    if pos('I',s)>0 then Settings.FontProps.FontStyle := [fsItalic] + Settings.FontProps.FontStyle;
    if pos('U',s)>0 then Settings.FontProps.FontStyle := [fsUnderline] + Settings.FontProps.FontStyle;
    if pos('S',s)>0 then Settings.FontProps.FontStyle := [fsStrikeOut] + Settings.FontProps.FontStyle;
    Settings.FontProps.FontHeight := fParser.ReadInteger('gFontHeight');
    Settings.FontProps.CharSet := fParser.ReadInteger('gCharSet');
  except
  end;
end;

procedure TIBOCPersistentInfoXML.DoStoreSettings(Settings: TPersistentSettings);
var
  i: integer;
  s:String;
begin
  for i := 0 to Settings.Count - 1 do
    StoreSetting(Settings.Settings[i]);
  fParser.WriteString('gFontName', Settings.FontProps.FontName);
  fParser.WriteInteger('gFontSize', Settings.FontProps.FontSize);
  fParser.WriteInteger('gFontColor', Settings.FontProps.FontColor);
  s:='';
  if fsBold in Settings.FontProps.FontStyle then s:=s+'B';
  if fsItalic in Settings.FontProps.FontStyle then s:=s+'I';
  if fsUnderline in Settings.FontProps.FontStyle then s:=s+'U';
  if fsStrikeOut in Settings.FontProps.FontStyle then s:=s+'S';
  fParser.WriteString('gFontStyle', s);
  fParser.WriteInteger('gFontHeight', Settings.FontProps.FontHeight);
  fParser.WriteInteger('gCharSet', Settings.FontProps.CharSet);
end;

procedure TIBOCPersistentInfoXML.GetSettings(var Settings: TPersistentSettings);
begin
 if fParser.FindNode(FRegSettingsNode, false) then
 begin
   try
     DoGetSettings(Settings);
   finally
   end;
 end
{ the exception has been removed to deal with the case when the settings
 doesn't exist, GetSectionSettings handles that problem more acurately
 by providing MustExist parameter defaulting to false, GetSetting is now considered
 a legacy method }
{  else
   raise EIBCPersistent.Create(Format(EPACCESSSECTION, [FRegSettingsKey])); }
end;

procedure TIBOCPersistentInfoXML.StoreSettings(Settings: TPersistentSettings);
begin
 if fParser.FindNode(FRegSettingsNode, false) then
 begin
   try
     DoStoreSettings(Settings);
   finally
   end;
 end
 else
   raise EIBCPersistent.Create(Format(EPACCESSSECTION, [FRegSettingsNode]));
end;

procedure TIBOCPersistentInfoXML.GetSectionSettings(SectionStrings: array of string;
                                                    var Settings: TPersistentSettings;
                                                    MustExist: boolean = false);
var
  i: integer;
begin
  for i := 0 to Length(SectionStrings) - 1 do
  if fParser.FindNode(FRootNode, True)
     and fParser.FindNode(SectionStrings[i], false)
  then
  begin
    try
      DoGetSettings(Settings);
    finally
    end;
  end
  else
  if MustExist then
    raise EIBCPersistent.Create(Format(EPACCESSSECTION, [SectionStrings[i]]));
end;

procedure TIBOCPersistentInfoXML.StoreSectionSettings(
  SectionStrings: array of string; Settings: TPersistentSettings);
var
  i: integer;
begin
  for i := 0 to Length(SectionStrings) - 1 do
  if fParser.FindNode(FRootNode, True)
     and fParser.FindNode(SectionStrings[i], false)
  then
  begin
    try
      DoStoreSettings(Settings);
    finally
    end;
  end
  else
    raise EIBCPersistent.Create(Format(EPACCESSSECTION, [SectionStrings[i]]));
end;

function TIBOCPersistentInfoXML.ServerAliasExists(Alias: string): boolean;
begin
  Result := fParser.NodeExists([FRegServersKey, Alias]);
end;

procedure TIBOCPersistentInfoXML.DeleteServerAlias(Alias: string);
begin
  // what about subkeys under Win NT ???
  // fParser.DeleteKey(Format('%s%s\Databases',[gRegServersKey,Node]));
  fParser.DeleteNode([FRegServersKey, Alias]);
end;

procedure TIBOCPersistentInfoXML.RenameServerAlias(SrcAlias, DestAlias: string);
begin
  // if an alias already exists an exception should be raised here???
//  fParser.MoveKey(Format('%s%s',[FRegServersKey, SrcAlias]),
//       Format('%s%s',[FRegServersKey, DestAlias]), true);
end;

procedure TIBOCPersistentInfoXML.GetServerAliases(Aliases: TStrings);
begin
  if fParser.FindNode(FRegServersKey, true) then
  begin
    fParser.GetNodeNames([FRegServersKey],Aliases);
  end
  else
    Aliases.Clear;
end;

procedure TIBOCPersistentInfoXML.GetServerProps(Alias: string; var ServerProps: TibcServerProps);
begin
  ServerProps.ServerName := '';
  ServerProps.UserName := '';
  ServerProps.Description := '';
  ServerProps.Protocol := cpLocal;
  ServerProps.Created := 0;
  ServerProps.Accessed := 0;
  if fParser.FindNode([FRegServersKey], false) then
  begin
    ReadServerProps(fParser,Alias,ServerProps);
    try
      case fParser.ReadInteger('Protocol') of
        0: ServerProps.Protocol := cpTCP_IP;
        1: ServerProps.Protocol := cpNetBEUI;
        2: ServerProps.Protocol := cpNovell;
        3: ServerProps.Protocol := cpLocal;
      end;
      ServerProps.ServerName := fParser.ReadString('ServerName');
      ServerProps.UserName := fParser.ReadString('UserName');
      ServerProps.Description := fParser.ReadString('Description');
      ServerProps.Created := fParser.ReadDateTime ('Created');
      ServerProps.Accessed := fParser.ReadDateTime ('Last Accessed');
   finally
   end;
 end
 else
   raise EIBCPersistent.Create(Format(EPACCESSSERVERALIAS, [Alias]));
end;

procedure TIBOCPersistentInfoXML.StoreServerProps(Alias: string; ServerProps: TibcServerProps);
begin
(* if fParser.OpenKey(Format('%s%s',[FRegServersKey, Alias]), true) then
 begin
   try
     fParser.WriteString('ServerName', ServerProps.ServerName);
     case ServerProps.Protocol of
       cpTCP_IP: fParser.WriteInteger('Protocol',0);
       cpNETBeui: fParser.WriteInteger('Protocol',1);
       cpNovell: fParser.WriteInteger('Protocol',2);
       cpLocal: fParser.WriteInteger('Protocol',3);
     end;
     fParser.WriteString('Username', ServerProps.Username);
     fParser.WriteString('Description', ServerProps.Description);
     fParser.WriteDateTime('Last Accessed', ServerProps.Accessed);
     if ServerProps.Created <> 0 then
       fParser.WriteDateTime('Created', ServerProps.Created);
   finally
     fParser.CloseKey;
   end;
 end
 else
   raise EIBCPersistent.Create('Persistent data write error. Cannot access server alias: ' + Alias);
   *)
end;

function TIBOCPersistentInfoXML.DatabaseAliasExists(ServerAlias, DatabaseAlias: string): boolean;
begin
//  Result := fParser.KeyExists(Format('%s%s\Databases\%s',[FRegServersKey, ServerAlias, DatabaseAlias]));
end;

procedure TIBOCPersistentInfoXML.DeleteDatabaseAlias(ServerAlias, DatabaseAlias: string);
begin
//  fParser.DeleteKey(Format('%s%s\Databases\%s',[FRegServersKey, ServerAlias, DatabaseAlias]));
end;

procedure TIBOCPersistentInfoXML.RenameDatabaseAlias(ServerAlias, SrcAlias, DestAlias: string);
begin
  // if an alias already exists an exception should be raised here???
//  fParser.MoveKey(Format('%s%s\Databases\%s',[FRegServersKey, ServerAlias, SrcAlias]),
//        Format('%s%s\Databases\%s',[FRegServersKey, ServerAlias, DestAlias]), true);
//  fParser.CloseKey;
end;

procedure TIBOCPersistentInfoXML.GetDatabaseAliases(ServerAlias: string; Aliases: TStrings);
begin
(*  Aliases.Clear;
  if fParser.OpenKey(Format('%s%s\Databases',[FRegServersKey, ServerAlias]), false) then
  begin
    fParser.GetKeyNames(Aliases);
    fParser.CloseKey;
  end;*)
end;

procedure TIBOCPersistentInfoXML.GetDatabaseProps(ServerAlias, DatabaseAlias: string; var DatabaseProps: TibcDatabaseProps);
begin
(*  DatabaseProps.DatabaseFiles := '';
  DatabaseProps.UserName := '';
  DatabaseProps.Role := '';
  DatabaseProps.CharacterSet := '';
  DatabaseProps.CaseSensitiveRole := false;
  DatabaseProps.Accessed := 0;
  DatabaseProps.Created := 0;
  if fParser.OpenKey(Format('%s%s\Databases\%s',[FRegServersKey, ServerAlias, DatabaseAlias]), false) then
  begin
    try
      DatabaseProps.DatabaseFiles := fParser.ReadString('DatabaseFiles');
      DatabaseProps.UserName := fParser.ReadString('Username');
      DatabaseProps.Role := fParser.ReadString('Role');
      DatabaseProps.CharacterSet := fParser.ReadString('CharacterSet');
      if fParser.ValueExists('CaseSensitiveRole') then
        DatabaseProps.CaseSensitiveRole := fParser.ReadBool('CaseSensitiveRole');
      if fParser.ValueExists('Last Accessed') then
        DatabaseProps.Accessed := fParser.ReadDateTime('Last accessed');
      if fParser.ValueExists('Created') then
        DatabaseProps.Created := fParser.ReadDateTime('Created');
    finally
      fParser.CloseKey;
    end
  end
  else
    raise EIBCPersistent.Create(Format(EPACCESSDATABASEALIAS, [DatabaseAlias, ServerAlias]));
    *)
end;

procedure TIBOCPersistentInfoXML.StoreDatabaseProps(ServerAlias, DatabaseAlias: string; var DatabaseProps: TibcDatabaseProps);
begin
(*  if fParser.OpenKey(Format('%s%s\Databases\%s',[FRegServersKey, ServerAlias, DatabaseAlias]), true) then
  begin
    try
      fParser.WriteString('DatabaseFiles', DatabaseProps.DatabaseFiles);
      fParser.WriteString('Username', DatabaseProps.Username);
      fParser.WriteString('Role', DatabaseProps.Role);
      fParser.WriteBool('CaseSensitiveRole', DatabaseProps.CaseSensitiveRole);
      fParser.WriteString('CharacterSet', DatabaseProps.CharacterSet);
      fParser.WriteDateTime('Last Accessed', DatabaseProps.Accessed);
      if DatabaseProps.Created <> 0 then
        fParser.WriteDateTime('Created', DatabaseProps.Created);
    finally
      fParser.CloseKey;
    end;
  end
  else
    raise EIBCPersistent.Create('Persistent data write error. Cannot access database alias: ' + DatabaseAlias);*)
end;

// backup aliases
function TIBOCPersistentInfoXML.BackupAliasExists(ServerAlias, BackupAlias: string): boolean;
begin
  Result := fParser.KeyExists(Format('%s%s\Backup Files\%s',[FRegServersKey, ServerAlias, BackupAlias]));
end;

procedure TIBOCPersistentInfoXML.DeleteBackupAlias(ServerAlias, BackupAlias: string);
begin
  fParser.CloseKey;
  fParser.DeleteKey(Format('%s%s\Backup Files\%s',[FRegServersKey, ServerAlias, BackupAlias]));
end;

procedure TIBOCPersistentInfoXML.GetBackupAliases(ServerAlias: string; Aliases: TStrings);
begin
  Aliases.Clear;
  if fParser.OpenKey(Format('%s%s\Backup Files',[FRegServersKey, ServerAlias]), false) then
  begin
    fParser.GetKeyNames(Aliases);
    fParser.CloseKey;
  end;
end;

procedure TIBOCPersistentInfoXML.GetBackupAliasProps(ServerAlias, BackupAlias: string; var BackupProps: TibcBackupProps);
begin
  BackupProps.ServerAlias := '';
  BackupProps.DatabaseAlias := '';
  BackupProps.BackupFiles := '';
  BackupProps.Created := 0;
  BackupProps.Accessed := 0;
  if fParser.OpenKey(Format('%s%s\Backup Files\%s',[FRegServersKey, ServerAlias, BackupAlias]), false) then
  begin
    BackupProps.ServerAlias := fParser.ReadString('SourceDBServer');
    BackupProps.DatabaseAlias := fParser.ReadString('SourceDBAlias');
    BackupProps.BackupFiles := fParser.ReadString('BackupFiles');
    if fParser.ValueExists('Created') then
      BackupProps.Created := fParser.ReadDateTime('Created');
    if fParser.ValueExists('Accessed') then
      BackupProps.Accessed := fParser.ReadDateTime('Accessed');
  end;
end;

procedure TIBOCPersistentInfoXML.StoreBackupAliasProps(ServerAlias, BackupAlias: string; var BackupProps: TibcBackupProps);
begin
  if fParser.OpenKey(Format('%s%s\Backup Files\%s',[FRegServersKey, ServerAlias, BackupAlias]), true) then
  begin
    fParser.WriteString('SourceDBServer', BackupProps.ServerAlias);
    fParser.WriteString('SourceDBAlias', BackupProps.DatabaseAlias);
    fParser.WriteString('BackupFiles', BackupProps.BackupFiles);
    if not fParser.ValueExists('Created') then
      fParser.WriteDateTime('Created', BackupProps.Created);
    fParser.WriteDateTime ('Accessed', BackupProps.Accessed);
    fParser.CloseKey();
  end;
end;

procedure TIBOCPersistentInfoXML.RenameBackupAlias(ServerAlias, SrcAlias, DestAlias: string);
begin
  if SrcAlias = DestAlias then Exit;
  // if an alias already exists an exception should be raised here???
  fParser.CloseKey;
  fParser.MoveKey(Format('%s%s\Backup Files\%s',[FRegServersKey, ServerAlias, SrcAlias]),
  Format('%s%s\Backup Files\%s',[FRegServersKey, ServerAlias, DestAlias]), true);
end;

// external Apps/Tools
procedure TIBOCPersistentInfoXML.GetExternalApps(ExternalApps: TStrings);
var
  iCount, i: integer;
begin
  ExternalApps.Clear;
  if fParser.OpenKey(FRegToolsKey, false) and fParser.ValueExists('Count') then
  begin
    iCount := fParser.ReadInteger('Count');
    for i := 0 to iCount - 1 do
      ExternalApps.Add(fParser.ReadString(Format('Title%d', [i])));
    fParser.CloseKey;
  end;
end;

procedure TIBOCPersistentInfoXML.DoGetExternalApp(Pos: integer; var Title, AppPath, WorkDir, Params: string);
begin
  Title := fParser.ReadString(Format('Title%d',[Pos]));
  AppPath := fParser.ReadString(Format('Path%d',[Pos]));
  WorkDir := fParser.ReadString (Format('WorkingDir%d',[Pos]));
  Params := fParser.ReadString (Format('Params%d',[Pos]));
end;

procedure TIBOCPersistentInfoXML.DoStoreExternalApp(Pos: integer; Title, AppPath, WorkDir, Params: string);
begin
  fParser.WriteString(Format('Title%d', [Pos]), Title);
  fParser.WriteString(Format('Path%d', [Pos]), AppPath);
  fParser.WriteString(Format('WorkingDir%d', [Pos]), WorkDir);
  fParser.WriteString(Format('Params%d', [Pos]), Params);
end;

procedure TIBOCPersistentInfoXML.GetExternalApp(Pos: integer; var Title, AppPath, WorkDir, Params: string);
begin
  if fParser.OpenKey(FRegToolsKey, false) then
  begin
    DoGetExternalApp(Pos, Title, AppPath, WorkDir, Params);
    fParser.CloseKey;
  end;
end;

procedure TIBOCPersistentInfoXML.AddExternalApp(Title, AppPath, WorkDir, Params: string);
var
 iPos : integer;
begin
  if fParser.OpenKey(FRegToolsKey, true) then
  begin
    if fParser.ValueExists('Count') then
      iPos := fParser.ReadInteger('Count')
    else
      iPos := 0;
    DoStoreExternalApp(iPos, Title, AppPath, WorkDir, Params);
    Inc(iPos);
    fParser.WriteInteger('Count', iPos);
    fParser.CloseKey;
  end;
end;

procedure TIBOCPersistentInfoXML.StoreExternalApp(Pos: integer; Title, AppPath, WorkDir, Params: string);
begin
  if fParser.OpenKey(FRegToolsKey, true) then
  begin
    DoStoreExternalApp(Pos, Title, AppPath, WorkDir, Params);
    fParser.CloseKey;
  end;
end;

procedure TIBOCPersistentInfoXML.DeleteExternalApp(Pos: integer);
var
  i, iCnt: integer;
begin
  if fParser.OpenKey(FRegToolsKey, false) then
  begin
    iCnt := fParser.ReadInteger('Count');
    fParser.DeleteValue(Format('Title%d', [Pos]));
    fParser.DeleteValue(Format('Path%d', [Pos]));
    fParser.DeleteValue(Format('WorkingDir%d', [Pos]));
    fParser.DeleteValue(Format('Params%d', [Pos]));
    for i := Pos + 1 to iCnt - 1 do
    begin
      fParser.RenameValue(Format('Title%d', [i]), Format('Title%d', [i - 1]));
      fParser.RenameValue(Format('Path%d', [i]), Format('Path%d', [i - 1]));
      fParser.RenameValue(Format('WorkingDir%d', [i]), Format('WorkingDir%d', [i - 1]));
      fParser.RenameValue(Format('Params%d', [i]), Format('Params%d', [i - 1]));
    end;
    if iCnt > 0 then
    begin
      fParser.WriteInteger('Count', iCnt - 1);
    end
    else
      fParser.DeleteKey('Count');
    fParser.CloseKey;
  end;
end;

end.
