{************************************************************************
  This Unit was created  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  Under the GPL License from Free Software Foundation
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

************************************************************************}
unit IBOConsoleForm;

interface

uses
{$IFDEF LINUX}
  Types, QGraphics, QForms, QDialogs, IBCXDummies, QStdActns, QActnList, QMenus,
  QTypes, QComCtrls, QExtCtrls, QStdCtrls, QDBGrids, QGrids, QControls,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Graphics, Controls, Forms, Dialogs, Menus, ComCtrls, ToolWin,
  ExtCtrls, StdCtrls, Grids, DBGrids, ImgList, StdActns, ActnList,
{$ENDIF}
  Classes, dmuNode, frmuWindowList, SynEditHighlighter, SynHighlighterSQL,
  SynEdit, SynMemo;

const WM_CaptionChanged = WM_USER + 1;

type
  TIBOConsoleForm = class;
  TIBOConsoleFormClass = class of TIBOConsoleForm;
  TIBOConsoleDatabaseForm = class;
  TIBOConsoleDatabaseFormClass = class of TIBOConsoleDatabaseForm;
  TIBOConsoleObjectForm = class;
  TIBOConsoleObjectFormClass = class of TIBOConsoleObjectForm;
  TIBOConsoleEditor = class;
  TIBOConsoleEditorClass = class of TIBOConsoleEditor;

  TIBOConsoleForm = class(TForm)
  private
    FStart:boolean;
    FImageIndex: integer;
    FOnStart: TNotifyEvent;
    FFormId: String;
    FUseFormHelpContext: boolean;
    procedure SetImageIndex(const Value: integer);
    procedure SetFormId(const Value: String);
    //procedure SetUseFormHelpContext(const Value: boolean);
  protected
    procedure WndProc(var Message: TMessage); override;
    procedure WMCaptionChanged(var msg: TMessage); message WM_CaptionChanged;
    procedure DoCreate; override;
    procedure DoDestroy; override;
    procedure DoShow; override;
    procedure DoHide; override;
    procedure DoOnStart; virtual;
    function GetExtra:Word; virtual;
    procedure SetExtra(const Value:Word); virtual;
    procedure WMNCLButtonDown( var Message: TWMNCLBUTTONDOWN ); message WM_NCLBUTTONDOWN ;
  public
    constructor Create(aOwner:Tcomponent); override;
    property ImageIndex: integer read FImageIndex write SetImageIndex;
    property Start:boolean Read FStart;
    property OnStart:TNotifyEvent read FOnStart write FOnStart;
    procedure UseFont(const aName: TFontName; const aCharset: TFontCharset); virtual;
    procedure UseSyntaxFont(const aName: TFontName; const aCharset: TFontCharset); virtual;
    property FormId:String read FFormId write SetFormId;
    Property Extra:Word read GetExtra write SetExtra;
    property UseFormHelpContext:boolean read FUseFormHelpContext;
  end;

  TIBOConsoleDatabaseForm = class(TIBOConsoleForm)
  private
    FDataModule: TDmN;
  protected
    procedure SetDataModule(const Value: TDmN); virtual;
  public
    constructor Create(aOwner:Tcomponent); override;
    property DataModule:TDmN read FDataModule write SetDataModule;
  end;

  TIBOConsoleObjectForm = class(TIBOConsoleDatabaseForm)
  private
    FObjectType: integer;
    FObjectName: String;
    FDescriptionModified: boolean;
    FDescription: String;
    FEditDescription: String;
    fsObjectType: String;
    fLockUpdate: boolean;
    procedure SetEditDescription(const Value: String);
    procedure LoadObject;
    procedure SaveDescription;
    procedure LoadDescription;
  protected
    procedure DoLoadObject; virtual; abstract;
    procedure SetObjectName(const Value: String); virtual;
    procedure SetObjectType(const Value: integer); virtual;
    procedure SignalDescriptionModified; virtual;
    procedure SignalObjectNameChanged; virtual;
    property EditDescription:String read FEditDescription write SetEditDescription;
  public
    function InternalCloseQuery : Boolean; virtual;
    constructor Create(aOwner:Tcomponent); override;
    property sObjectType: String read fsObjectType;
    property ObjectType: integer read FObjectType write SetObjectType;
    property ObjectName:String read FObjectName write SetObjectName;
    property DescriptionModified:boolean read FDescriptionModified;
    property Description:String read FDescription;
    procedure SignalObjectRefresh(const aDataModule: TDmN; const aObjType: integer; const aObjName:String=''); virtual;
    property LockUpdate:boolean read fLockUpdate;
  end;

  TIBOConsoleEditor = class(TIBOConsoleObjectForm)
  private
    FObjectModified: boolean;
    FIsNewObject: Boolean;
    FNewName: string;
    FCompileErrors: String;
    Fuppercase_identifier: boolean;
    procedure SetNewName(const Value: string);
    procedure SetCompileErrors(const Value: String);
    procedure Setuppercase_identifier(const Value: boolean);
  protected
    procedure SetIsNewObject(const Value: Boolean); virtual;
    procedure SetObjectModified(const Value: boolean); virtual;
    procedure SetObjectName(const Value: String); override;
    procedure SignalObjectModified; virtual;
    procedure SignalNewObject; virtual;
    function CompileNewObject:boolean; virtual;
    function AlterObject:boolean; virtual;
    property NewName:string read FNewName write SetNewName;
  public
    function InternalCloseQuery : Boolean; override;
    function Compile:boolean;
    constructor Create(aOwner:Tcomponent); override;
    procedure NewObject;
    property IsNewObject:Boolean read FIsNewObject write SetIsNewObject;
    property ObjectModified:boolean read FObjectModified write SetObjectModified;
    property CompileErrors:String read FCompileErrors write SetCompileErrors;
    property uppercase_identifier:boolean read Fuppercase_identifier write Setuppercase_identifier;
  end;

implementation

uses frmuMessage, frmuMain, SysUtils, IBOCGlobal, dmuMain, LMMessageDialog;


{ TIBOConsoleForm }

constructor TIBOConsoleForm.Create(aOwner: Tcomponent);
begin
  inherited;
  FUseFormHelpContext := biHelp in BorderIcons;
  fStart := True;
  FFormId := '';
  UseFont(gAppSettings[FONT_NAME],gAppSettings[FONT_CHARSET]);
  UseSyntaxFont(gAppSettings[SYNTAX_FONT_NAME],gAppSettings[SYNTAX_FONT_CHARSET]);
  if Assigned(gGUILanguageFile) then
    GetAllCaptionAndText('',Self,gGUILanguageFile);
end;

procedure TIBOConsoleForm.WndProc(var Message: TMessage);
begin
  if Message.Msg = WM_SETTEXT then
     PostMessage(Handle,WM_CaptionChanged,0,0);
  inherited WndProc(Message);
end;

procedure TIBOConsoleForm.WMCaptionChanged(var msg: TMessage);
begin
  if Assigned(OpenWindows) then
  try
    OpenWindows.UpdateWindowList(Caption,Self);
  except
  end;
end;

procedure TIBOConsoleForm.WMNCLButtonDown( var Message: TWMNCLButtonDown );
var
  ScreenPt: TPoint;
  ClientPt: TPoint;
begin
  if FUseFormHelpContext then
  begin
    ScreenPt.X := Message.XCursor;
    ScreenPt.Y := Message.YCursor;
    ClientPt := ScreenToClient( ScreenPt );
    if( ClientPt.X > Width-45 )and (ClientPt.X < Width-29) then
    begin
      Application.HelpCommand(HELP_CONTEXT, HelpContext);
      Message.Result := 0;
    end
    else
      inherited;
  end
  else
    inherited;
end;


procedure TIBOConsoleForm.SetImageIndex(const Value: integer);
{$IFDEF MSWINDOWS}
var
  lIcon : TIcon;
{$ENDIF}
begin
  FImageIndex := Value;
{$IFDEF MSWINDOWS}
  if Assigned(dmMain) then
  begin
    lIcon := TIcon.Create;
    try
      dmMain.imgTreeview.GetIcon(ImageIndex, lIcon);
      Icon := lIcon;
    finally
      lIcon.Free;
    end;
  end;
{$ENDIF}
end;

procedure TIBOConsoleForm.DoCreate;
begin
  inherited;
  FUseFormHelpContext := biHelp in BorderIcons;
  if Assigned(OpenWindows) then
  try
    OpenWindows.Add(Caption,Self);
  except
  end;
end;

procedure TIBOConsoleForm.DoDestroy;
begin
  if Assigned(OpenWindows) then
  try
    OpenWindows.Remove(Self);
  except
  end;
  inherited;
end;

procedure TIBOConsoleForm.DoShow;
var e:word;
begin
  inherited;
  if not gApplShutdown then
  if fStart then
  begin
    fStart := False;
    UseFont(gAppSettings[FONT_NAME],gAppSettings[FONT_CHARSET]);
    UseSyntaxFont(gAppSettings[SYNTAX_FONT_NAME],gAppSettings[SYNTAX_FONT_CHARSET]);
    if FormId <>'' then
    begin
      e := Extra;
      gPersistentInfo.GetFormSettings(Self, E, FormId);
      Extra := e;
    end;
    DoOnStart;
    if Assigned(FOnStart) then FOnStart(Self);
{$ifdef initlangfile} WriteAllCaptionAndText('',Self,gTransIniFile); {$endif}
  end;
end;

procedure TIBOConsoleForm.UseFont(const aName: TFontName; const aCharset: TFontCharset);
begin
  SetAllFontsNameCharset(Self,aName,aCharSet);
end;

procedure TIBOConsoleForm.UseSyntaxFont(const aName: TFontName; const aCharset: TFontCharset);
begin
  SetAllSyntaxFontsNameCharset(Self, aName, aCharset);
end;

procedure TIBOConsoleForm.SetFormId(const Value: String);
begin
  FFormId := Value;
end;

procedure TIBOConsoleForm.DoHide;
begin
  if not gApplShutdown then
  begin
    if FormId <>'' then gPersistentInfo.StoreFormSettings(Self, Extra, FormId);
  end;
  inherited;
end;

function TIBOConsoleForm.GetExtra: Word;
begin
  result := 0;
end;

procedure TIBOConsoleForm.SetExtra(const Value: Word);
begin

end;

//procedure TIBOConsoleForm.SetUseFormHelpContext(const Value: boolean);
//begin
//  FUseFormHelpContext := Value;
//end;

procedure TIBOConsoleForm.DoOnStart;
begin
  // do nothing
end;

{ TIBOConsoleDatabaseForm }

constructor TIBOConsoleDatabaseForm.Create(aOwner: Tcomponent);
begin
  inherited;
  FDataModule := Nil;
end;

procedure TIBOConsoleDatabaseForm.SetDataModule(const Value: TDmN);
begin
  FDataModule := Value;
end;

{ TIBOConsoleEditor }


function TIBOConsoleEditor.AlterObject: boolean;
begin
  Result := True;
end;

type TCompProc = function: boolean of object;

function TIBOConsoleEditor.Compile: boolean;
var CompProc:TCompProc;
    notify : TEventObjectNotify;
begin
  if NewName = '' then
  begin
    LMMessageDlg(8,[GetTypeArticle(fObjectType),GetLangType(fObjectType)], mtError, [mbOK], 0);
    Result := False;
    Exit;
  end;

  if AnsiSameText(FNewName,'NEW_'+fsObjectType) then
  begin
    if LMMessageDlg(9,[GetTypeArticle(fObjectType),GetLangType(fObjectType)], mtConfirmation, [mbYes, mbNo], 0) = mrNo then
    begin
      Result := False;
      Exit;
    end;
  end;

  if IsNewObject then
  begin
    CompProc := CompileNewObject;
    notify := frmMain.EventObjectNew;
  end
  else
  begin
    CompProc := AlterObject;
    notify := frmMain.EventObjectRefresh;
  end;
  fLockUpdate := true;
  try
    if CompProc then
    begin
      ObjectName := NewName;
      IsNewObject := False;
      ObjectModified := False;
      notify(DataModule,ObjectType,ObjectName);
      Result := True;
    end
    else
    begin
      Result := False;
      DisplayMsg(ERR_DDL_EXECUTION,CompileErrors);
    end;
    SaveDescription;
  finally
    fLockUpdate := False;
  end;
end;


function TIBOConsoleEditor.CompileNewObject: boolean;
begin
  Result := True;
end;

constructor TIBOConsoleEditor.Create(aOwner: Tcomponent);
begin
  inherited;
  FIsNewObject := False;
  FObjectModified := False;
  fCompileErrors := '';
  Fuppercase_identifier := gAppSettings[force_uppercase_identifier];
end;

function TIBOConsoleEditor.InternalCloseQuery: Boolean;
begin
  Result := inherited InternalCloseQuery;
  if Result and ObjectModified then
  begin
    case LMMessageDlg(10,[GetTypeArticle(fObjectType),GetLangType(fObjectType),ObjectName], mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
      mrYes:      Result := Compile;
      mrCancel :  Result := False;
      mrNo :
        begin
          Result := True;
          ObjectModified := False;
        end;
    end;
  end;
end;


procedure TIBOConsoleEditor.NewObject;
begin
  FIsNewObject := True;
  ObjectName := UpperCase('NEW_'+fsObjectType);
  ObjectModified := False;
  SignalNewObject;
end;

procedure TIBOConsoleEditor.SetCompileErrors(const Value: String);
begin
  FCompileErrors := Value;
end;

procedure TIBOConsoleEditor.Setuppercase_identifier(
  const Value: boolean);
begin
  Fuppercase_identifier := Value;
end;

procedure TIBOConsoleEditor.SetIsNewObject(const Value: Boolean);
begin
  FIsNewObject := Value;
end;

procedure TIBOConsoleEditor.SetNewName(const Value: string);
begin
  if uppercase_identifier then
    FNewName := ConvertNameUppercase(Value)
  else
    FNewName := Value;
end;

procedure TIBOConsoleEditor.SetObjectModified(const Value: boolean);
begin
  FObjectModified := Value;
  SignalObjectModified;
end;

procedure TIBOConsoleEditor.SetObjectName(const Value: String);
begin
  inherited SetObjectName(Value);
  if not IsNewObject then
  begin
    LoadObject;
  end;
  ObjectModified := False;
end;

procedure TIBOConsoleEditor.SignalNewObject;
begin

end;

procedure TIBOConsoleEditor.SignalObjectModified;
begin

end;

{ TIBOConsoleObjectForm }

constructor TIBOConsoleObjectForm.Create(aOwner: Tcomponent);
begin
  inherited;
  FDataModule := nil;
  FObjectName := '';
  fLockUpdate := False;
  //FObjectType := -1;
end;

function TIBOConsoleObjectForm.InternalCloseQuery: Boolean;
begin
  Result := True;
  if DescriptionModified then SaveDescription;
end;

procedure TIBOConsoleObjectForm.LoadDescription;
begin
  if Assigned(Datamodule) then
  begin
    fDescription := DataModule.GetObjectDescription(ObjectType,ObjectName);
    FEditDescription := FDescription;
  end;
  fDescriptionModified := False;
end;

procedure TIBOConsoleObjectForm.LoadObject;
begin
  LoadDescription;
  DoLoadObject;
end;

procedure TIBOConsoleObjectForm.SaveDescription;
begin
  if Assigned(Datamodule) then
  begin
    DataModule.ObjectDescriptionUpdate(ObjectType,ObjectName,FEditDescription);
    FDescription := FEditDescription;
    frmMain.ObjectListRefresh(DataModule, ObjectType, ObjectName);
  end;
  fDescriptionModified := False;
end;

procedure TIBOConsoleObjectForm.SetEditDescription(const Value: String);
begin
  FEditDescription := Value;
  if not FDescriptionModified then
  begin
    FDescriptionModified := True;
    SignalDescriptionModified;
  end;
end;

procedure TIBOConsoleObjectForm.SetObjectName(const Value: String);
begin
  FObjectName := Value;
  SignalObjectNameChanged;
end;

procedure TIBOConsoleObjectForm.SetObjectType(const Value: integer);
begin
  FObjectType := Value;
  if ObjectType in [low(NODE_ARRAY)..high(NODE_ARRAY)] then
  begin
    fsObjectType := NODE_ARRAY[ObjectType];
    ImageIndex := NodeImg(FObjectType);
  end
  else
  begin
    fsObjectType := '??';
    ImageIndex := -1;
  end;
end;

procedure TIBOConsoleObjectForm.SignalDescriptionModified;
begin

end;

procedure TIBOConsoleObjectForm.SignalObjectNameChanged;
begin
  Caption := GetLangType(fsObjectType) + ' - ' + ObjectName;
end;

procedure TIBOConsoleObjectForm.SignalObjectRefresh(const aDataModule: TDmN;
  const aObjType: integer; const aObjName: String);
begin
  if fLockUpdate then exit;
  if (aDataModule = FDataModule)
     and (aObjType = FObjectType)
     and (aObjName = FObjectName)
  then
    DoLoadObject;
end;

end.
