{******************************************************************}
{  This Unit was created to by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed under the License GPL on "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.
}
unit LMMessageDialog;

interface

uses
{$IFDEF LINUX}
  QDialogs, QGraphics, QForms, QDialogs, QStdActns, QActnList, QMenus,
  QTypes, QComCtrls, QExtCtrls, QStdCtrls, QDBGrids, QGrids, QControls,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Graphics, Controls, Forms, Dialogs, Menus, ComCtrls, ToolWin,
  ExtCtrls, StdCtrls, Grids, DBGrids, ImgList, StdActns, ActnList,
{$ENDIF}
  SysUtils, Types, Buttons, Classes;


type
  TLMMessage = class(TForm)
    imgError: TImage;
    imgInformation: TImage;
    imgWarning: TImage;
    BitBtnAbort: TBitBtn;
    BitBtnAll: TBitBtn;
    BitBtnCancel: TBitBtn;
    BitBtnClose: TBitBtn;
    BitBtnHelp: TBitBtn;
    BitBtnIgnore: TBitBtn;
    BitBtnNo: TBitBtn;
    BitBtnOK: TBitBtn;
    BitBtnRetry: TBitBtn;
    BitBtnYes: TBitBtn;
    imgConfirmation: TImage;
    BitBtnYesToAll: TBitBtn;
    BitBtnNoToAll: TBitBtn;
    LabelMsg: TLabel;
    procedure BitBtnHelpClick(Sender: TObject);
  private
    { Private declarations }
    fHelpCtx:integer;
    fButtons: TMsgDlgButtons;
  public
    { Public declarations }
  end;

function LMMessageDlg(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint): Integer;  overload;

function LMMessageDlg(const IdPrompt: integer; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint): Integer; overload;

function LMMessageDlg(const IdPrompt: integer; const Args:array of const;
  DlgType: TMsgDlgType; Buttons: TMsgDlgButtons; HelpCtx: Longint): Integer; overload;

function LMMessageDlg(const Msg: string; const Args:array of const;
  DlgType: TMsgDlgType; Buttons: TMsgDlgButtons; HelpCtx: Longint): Integer; overload;

implementation

uses IBOCGlobal, zluContextHelp;

{$R *.dfm}

const
  bleft=24;
  bWidth=97;
  bHGap=128-24;
  bVGap=208-176;

function LMMessageDlg(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint): Integer;
var x:TLMMessage;
    nb:integer;
    _left:integer;
    _top:integer;
    maxY:integer;
    maxX:integer;
    imgY:integer;
    bCount:integer;
    bWidth:integer;

  procedure sb(b:TBitBtn);
  var bx,by:integer;
  begin
    by := {(nb div 4)*bVGap +} _top ;
    bx := (nb {mod 4})*bHGap + _left;
    b.Left := bx;
    b.Top := by;
    b.Visible := true;
    b.Default := nb=0;
    maxY:=b.Top+b.Height;
    //if (b.Left+b.Width)>MaxX then MaxX := b.Left+b.Width;
    b.Font.Name := gAppSettings[FONT_NAME];
    b.Font.Charset := gAppSettings[FONT_CHARSET];
  end;

begin
  x := TLMMessage.Create(Application);
  x.Font.Name := gAppSettings[FONT_NAME];
  x.Font.Charset := gAppSettings[FONT_CHARSET];
  x.fHelpCtx := HelpCtx;
  x.fButtons := Buttons;
  x.Caption := Application.Title;
  x.LabelMsg.Caption := Msg;
  maxX := x.LabelMsg.Left+x.LabelMsg.Width;
  x.LabelMsg.Font.Name := gAppSettings[FONT_NAME];
  x.LabelMsg.Font.Charset := gAppSettings[FONT_CHARSET];
  case DlgType of
    mtWarning     :x.imgWarning.Visible := true;
    mtError       :x.imgError.Visible := true;
    mtInformation :x.imgInformation.Visible := true;
    mtConfirmation:x.imgConfirmation.Visible := true;
  end;
  case DlgType of
    mtWarning     :x.Caption := Application.Title+' - '+GetLangText('Warning');
    mtError       :x.Caption := Application.Title+' - '+GetLangText('Error');
    mtInformation :x.Caption := Application.Title+' - '+GetLangText('Information');
    mtConfirmation:x.Caption := Application.Title+' - '+GetLangText('Confirmation');
  end;
  bCount:=0;
  if mbYes      in Buttons then begin inc(bCount); end;
  if mbNo       in Buttons then begin inc(bCount); end;
  if mbCancel   in Buttons then begin inc(bCount); end;
  if mbOK       in Buttons then begin inc(bCount); end;
  if mbAbort    in Buttons then begin inc(bCount); end;
  if mbRetry    in Buttons then begin inc(bCount); end;
  if mbIgnore   in Buttons then begin inc(bCount); end;
  if mbYesToAll in Buttons then begin inc(bCount); end;
  if mbNoToAll  in Buttons then begin inc(bCount); end;
  if mbAll      in Buttons then begin inc(bCount); end;
  if mbHelp     in Buttons then begin inc(bCount); end;
  bWidth := bcount*bHGap;
  if maxX < bWidth then
  begin
    maxX := bWidth;
    _left := 4;
  end
  else
  begin
    _left := (maxX+8-bWidth) div 2;
  end;
  _top := x.LabelMsg.top + x.LabelMsg.Height + 4;
  imgY := x.imgConfirmation.Top+x.imgConfirmation.Height+4;
  if _top < imgY then _top := imgY;

  x.BitBtnAbort .Caption   := GetLangText('Abort');
  x.BitBtnAll   .Caption   := GetLangText('All');
  x.BitBtnCancel.Caption   := GetLangText('Cancel');
  x.BitBtnClose .Caption   := GetLangText('Close');
  x.BitBtnHelp  .Caption   := GetLangText('Help');
  x.BitBtnIgnore.Caption   := GetLangText('Ignore');
  x.BitBtnNo    .Caption   := GetLangText('No');
  x.BitBtnOK    .Caption   := GetLangText('OK');
  x.BitBtnRetry .Caption   := GetLangText('Retry');
  x.BitBtnYes   .Caption   := GetLangText('Yes');
  x.BitBtnYesToAll.Caption := GetLangText('Yes To All');
  x.BitBtnNoToAll.Caption  := GetLangText('No To All');

  nb:=-1;
  if mbYes      in Buttons then begin inc(nb); sb(x.BitBtnYes); end;
  if mbNo       in Buttons then begin inc(nb); sb(x.BitBtnNo     ); end;
  if mbCancel   in Buttons then begin inc(nb); sb(x.BitBtnCancel ); end;
  if mbOK       in Buttons then begin inc(nb); sb(x.BitBtnOK     ); end;
  if mbAbort    in Buttons then begin inc(nb); sb(x.BitBtnAbort  ); end;
  if mbRetry    in Buttons then begin inc(nb); sb(x.BitBtnRetry  ); end;
  if mbIgnore   in Buttons then begin inc(nb); sb(x.BitBtnIgnore ); end;
  if mbYesToAll in Buttons then begin inc(nb); sb(x.BitBtnYesToAll); end;
  if mbNoToAll  in Buttons then begin inc(nb); sb(x.BitBtnNoToAll ); end;
  if mbAll      in Buttons then begin inc(nb); sb(x.BitBtnAll    ); end;
  if mbHelp     in Buttons then begin inc(nb); sb(x.BitBtnHelp); end;

  x.ClientHeight := maxY+4;
  x.ClientWidth := maxX+8;
  Result := x.ShowModal;
  x.Release;
end;

function LMMessageDlg(const IdPrompt: integer; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint): Integer;
var p:string;
begin
  p:=GetPrompt(IdPrompt);
  Result := LMMessageDlg(p, DlgType, Buttons, HelpCtx);
end;

function LMMessageDlg(const IdPrompt: integer; const Args:array of const;
  DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint): Integer; overload;
var p:string;
begin
  p:=GetPrompt(IdPrompt);
  Result := LMMessageDlg(format(p, Args), DlgType, Buttons, HelpCtx);
end;

function LMMessageDlg(const Msg: string; const Args:array of const;
  DlgType: TMsgDlgType; Buttons: TMsgDlgButtons; HelpCtx: Longint): Integer; overload;
begin
  Result := LMMessageDlg(format(Msg, Args), DlgType, Buttons, HelpCtx);
end;

procedure TLMMessage.BitBtnHelpClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_CONTEXT,fHelpCtx);
end;

end.
