{
======================================================================
Project    : Delphi components
File       : XLSExport.pas
Purpose    : easy to use wrapper for TXLSFile
Created    : Achim Kalwa
======================================================================
}
unit XLSExport;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  DB,
  XLSFile;

type
  EDataSetMissing = Class(Exception);
  EFileNameMissing = Class(Exception);
  EDatasetNotActive = class(Exception);

  TXLSExport = class(TComponent)
  private
    { Private-Deklarationen }
    FDataset: TDataSet;
    FFileName : TFileName;
  protected
    { Protected-Deklarationen }
    procedure Notification(AComponent: TComponent; Operation: TOperation); Override;
  public
    { Public-Deklarationen }
    procedure ExportDataSet;
  published
    { Published-Deklarationen }
    property DataSet : TDataSet read FDataset write FDataSet;
    property FileName : TFileName read FFileName write FFileName;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Diverse', [TXLSExport]);
end;

{ TXLSExport }

procedure TXLSExport.ExportDataSet;
const
  DefAttr : TSetOfAtribut = [];
var
  XLSFile   : TXLSfile;
  i         : Integer;
  Col, Row  : Integer;
  aField    : TField;
begin
  if not Assigned(FDataSet) then Raise EDataSetMissing.Create('no dataset connected');
  if not FDataset.Active then Raise EDatasetNotActive.Create('datset not active');
  if Length(FFileName)=0 then Raise EFileNameMissing.Create('export filename missing');

  FDataset.DisableControls;
  FDataset.First;
  try
    XLSFile := TXLSFile.Create(Self);
    XLSFile.FileName := FFileName;
    Row := 1;

    { write field names in first row }
    for i := 0 to FDataSet.Fields.Count-1 do begin
      Col := i+1;
      XLSFile.AddStrCell(Col, Row, DefAttr, FDataset.Fields[i].FieldName);
    end;

    { export every row }
    Inc(Row);
    while not FDataset.Eof do begin
      for i := 0 to FDataset.Fields.Count-1 do begin

        aField := FDataset.Fields[i];
        Col := i+1;

        case aField.DataType of
          ftSmallint,
          ftInteger,
          ftWord:
            XLSFile.AddWordCell(Col, Row, DefAttr, aField.AsInteger);

          ftCurrency,
          ftFloat:
            XLSFile.AddDoubleCell(Col, Row, DefAttr, aField.AsFloat);

          ftString,
          ftWideString:
            XLSFile.AddStrCell(Col, Row, DefAttr, aField.AsString);

          ftDate:
            XLSFile.AddStrCell(Col, Row, DefAttr, DateToStr(aField.AsDateTime));

          ftTime:
            XLSFile.AddStrCell(Col, Row, DefAttr, TimeToStr(aField.AsDateTime));

          ftDateTime:
            XLSFile.AddStrCell(Col, Row, DefAttr, DateTimeToStr(aField.AsDateTime));
        end;                            { case }
      end;                              { for i }
      FDataset.Next;
      Inc(Row);
    end;                                { while not Eof }
    XLSFile.write;
  finally
    FreeAndNil(XLSFile);
    FDataset.EnableControls;
  end;                                  { try..finally }
end;                                    { ExportDataSet }

procedure TXLSExport.Notification(AComponent: TComponent; Operation: TOperation);
begin
  if (Operation = opRemove) and (Assigned(FDataset)) and (AComponent = FDataset)
  then begin
    FDataset := nil;
  end;
end;

end.
