{************************************************************************}
{  This Unit was created 
   by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com
}

{************************************************************************
  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.
                                                                         }
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }

{
13.August.2002, Lorenzo:
- updated Code for trigger metadata to get position and description

15.August.2002, Lorenzo:
- Bugfix in extraction domain metadata with collation

8.june.2003, Lorenzo:
- fixed metadata generation of numeric(x,x) field types

15.December.2003, Hideo Tanaka:
- added use of GetTriggerTypes

}
unit dmuNode;

interface

uses
{$IFDEF LINUX}
  Types, QGraphics, QForms, QDialogs, QTypes,  QStdCtrls, QDBGrids, QControls,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Graphics, Controls, Forms, Dialogs, StdCtrls, ImgList,
  CommCtrl, ComCtrls, ActnList, Grids, DBGrids,
{$ENDIF}
  Db, zluibcClasses,  IBOCPersistent, DBTables, SynEditHighlighter, SynHighlighterSQL,
  SysUtils, Classes, IB_Components, IBOExtract, IB_Session, IBODataset, Types,
  IB_Process, IBOServices, IB_Script;

const

  iSMALLINT         = 0;
  iINTEGER          = 1;
  iFLOAT            = 2;
  iDOUBLE_PRECISION = 3;
  iDECIMAL          = 4;
  iNUMERIC          = 5;
  iDATE             = 6;
  iTIME             = 7;
  iTIMESTAMP        = 8;
  iCHAR             = 9;
  iVARCHAR          = 10;
  iBLOB             = 11;

   sDataType :array [0..11] of string = (
  'SMALLINT',
  'INTEGER',
  'FLOAT',
  'DOUBLE PRECISION',
  'DECIMAL',
  'NUMERIC',
  'DATE',
  'TIME',
  'TIMESTAMP',
  'CHAR',
  'VARCHAR',
  'BLOB');


type
  TArrayDim = record
    LBound:integer;
    UBound:integer;
  end;

  TArrayDef = array of TArrayDim;

  TColumnDef = record
    //TableName   : String;
    ColumnName  : String;
    DataType    : String;
    Precision   : integer;
    Scale       : integer;
    BlobSubType : String;
    BlobCharSet : String;
    BlobCollate : String;
    BlobSegSize : integer;
    CharLength  : integer;
    CharCharSet : String;
    CharCollate : String;
    ArrayDim    : TArrayDef;
    Domain      : String;
    Default     : String;
    Computed    : String;
    NotNull     : boolean;
    Pk          : boolean;
    UseDomain   : boolean;
    UseComputed : boolean;
  end;

  TIndexDef = record
    Name        : String;
    Table       : String;
    IndexType   : integer;
    Unique      : integer;
    Inactive    : integer;
    Fields      : TStringDynArray;
    Description : String;
  end;

  TConstraintDef = record
     Name:String;
     cType:String;
     Deferrable:String;
     InitiallyDeferred:String;
     IndexName:String;
  end;

  TRefConstraintDef = record
     Name:String;                   //   ' A.RDB$CONSTRAINT_NAME,'+
     Deferrable:String;             //   ' A.RDB$DEFERRABLE,'+
     InitiallyDeferred:String;      //   ' A.RDB$INITIALLY_DEFERRED, ' +
     Match_Option:String;           //   ' B.RDB$MATCH_OPTION,'+
     Update_Rule:String;            //   ' B.RDB$UPDATE_RULE,'+
     DeleteRule:String;             //   ' B.RDB$DELETE_RULE,'+
     IndexName:String;              //   ' A.RDB$INDEX_NAME,'+
     RelationName:String;           //   ' C.RDB$RELATION_NAME ' +
  end;

  TDomainDef=record
    DomainName  : string;
    DataType    : String;
    Precision   : integer;
    Scale       : integer;
    BlobSubType : String;
    BlobCharSet : String;
    BlobCollate : String;
    BlobSegSize : String;
    CharLength  : integer;
    CharCharSet : String;
    CharCollate : String;
    ArrayDim    : TArrayDef;
    Domain      : String;
    Default     : String;
    Check       : String;
    NotNull     : boolean;
  end;


type
  TibcDatabaseNode = class;

  TDmN = class(TDataModule)
    lQry: TIB_Cursor;
    IBOExtract: TIBOExtract;
    IBOQuery1: TIBOQuery;
    lQry2: TIB_Cursor;
    Connection: TIB_Connection;
    Transaction: TIB_Transaction;
    TableData: TIBOQuery;
    DDLTransaction: TIBOTransaction;
    IB_Session1: TIB_Session;
    Query: TIB_Cursor;
    QryDataSource: TDataSource;
    DataSource: TDataSource;
    DataSet: TIBOQuery;
    IB_DSQL: TIB_DSQL;
    IB_Script1: TIB_Script;
    procedure DataModuleCreate(Sender: TObject);

  private
    FDatabaseNode: TibcDatabaseNode;
    FServerNode: TibcServerNode;
    FAutoDDL: boolean;
    function ExecuteQuery(const SSQL: String): Integer; overload;
    function ExecuteQuery(const SSQL: String; const params:array of variant): Integer; overload;
    function ExecuteQuery2(const SSQL: String): Integer;
    procedure SetDatabaseNode(const Value: TibcDatabaseNode);
    procedure SetServerNode(const Value: TibcServerNode);
    function sGetFieldType(FieldType,FieldSubType,FieldScale,FieldPrec: integer): String;
    procedure GetArrayDef(FieldName: String; var ArrayDef: TArrayDef);
    procedure SetAutoDDL(const Value: boolean);
    function PrepareQuery(const SSQL: String): Integer;
    { Private declarations }
  public
    { Public declarations }
    function IsSystemTable(const ObjName): boolean;
    procedure GetUsersList(List: TStrings);
    function IndexNameExist(const IndexName: string): boolean;
    function TriggerNameExist(const TriggerName: string): boolean;
    function ConstraintNameExist(const ConstraintName: string): boolean;
    function ExecuteDDLScript(DDL: String; var error: string): boolean;
    procedure GetPKList(const RelationName: String;  var PK:TStringDynArray);
    function GetCheckConstSource(const TableName, CheckName: string): String;
    function GetConstraintData(var Data: TConstraintDef; const TableName, ConstraintName: string): integer;
    function GetConstList(var ObjectList: TStringList;
      const TableName: string): integer;
    function GetReferentialConstData(var Data: TRefConstraintDef;
      const TableName, ConstraintName: string): integer;
    function GetIndexData(var Data: TIndexDef; const IndexName: string): integer;
    function GetColumnData(var Data: TColumnDef; const TableName, FieldName: string): integer;
    function GetColumnList(var Fields: TStringDynArray; const TableName: string): integer; overload;
    function GetColumnList(Fields: TStrings; const TableName: string): integer; overload;
    function ExecuteSql(const SSQL: String; var Error: string): boolean;
    procedure CommitDDL;
    procedure RollbackDDL;
    function ExecuteDDL(DDL: String; var error: string): boolean; overload;
    function ExecuteDDL(DDL: TStrings; var error: string): boolean; overload;
    procedure GetDependencies(tvDependencies, tvDependents: TTreeView; const ObjName: String; const ObjType: Integer);
    procedure GetPermissions(lvPermissions: TListView; const ObjName: String; const ObjType: Integer);
    function GetObjectDescription(NodeType: integer; ObjectName: String): String;
    function DropObject(NodeType: integer; NodeName: String; var error:String; DoPrompt:boolean = true): boolean;
    function GetBlobFilterList(var ObjectList: TStringList; const SystemData: boolean): integer;
    procedure GetCharSetNames(S: TStrings);
    function GetCheckConstList(var ObjectList: TStringList; const TableName: string): integer;
    procedure GetCollationNames(S: TStrings);
    function GetColumnList(var ObjectList: TStringList; const TableName: string): integer; overload;
    function GetDBCharSetName(CharSetID: Integer): String;
    function GetDBCollationName(CollationID, CharSetID: Integer): String;
    function GetDBFiles(var ObjectList: TStringList): integer;
    function GetDomainData(var ObjectList: TStringList; const ShowSystem: boolean): integer;
    procedure GetDomainData3(const DomainName: String; var Data: TDomainDef);
    function GetDomainList(var ObjectList: TStringList; const SystemData: boolean): integer;
    function GetExceptionData(var ObjectList: TStringList; const ExceptionName: String): integer;
    function GetExceptionList(var ObjectList: TStringList; const SystemData: boolean): integer;
    function GetFilterData(var ObjectList: TStringList; const FuncName: String): integer;
    function GetFunctiondata(var ObjectList: TStringList; out ModuleName, EntryPoint, Returnval: String; const FuncName: String): integer;
    function GetFunctionList(var ObjectList: TStringList; const SystemData: boolean): integer;
    function GetGeneratorData(var ObjectList: TStringList; const Showsystem: boolean): integer;
    function GetGeneratorList(var ObjectList: TStringList; const SystemData: boolean): integer;
    function GetIndexList(var ObjectList: TStringList; const TableName: string): integer;
    function GetNextGenValue(const GenName: String): String;
    function GetOwnerInfo(NodeName:String; var OwnerName, Description: string; NodeType:integer): integer;
    function GetProcedureInfo(var ObjectList, Source: TStringList; const ProcName: String): integer;
    function GetProcedureList(var ObjectList: TStringList; const SystemData: boolean): integer;
    function GetProcedureSource(var ObjectList: TStringList; const ProcName: String): integer; overload;
    function GetProcedureSource(const ProcName: String): String; overload;
    function GetReferentialConstList(var ObjectList: TStringList; const TableName: string): integer;
    function GetRoleData(var ObjectList: TStringList; const RoleName: String): integer;
    function GetRoleList(var ObjectList: TStringList): integer;
    function GetTableList(var ObjectList: TStringList; const SystemData: boolean): integer;
    function GetTriggerList(var ObjectList: TStringList; const TableName: string): integer;
    function GetUniqueConstList(var ObjectList: TStringList; const TableName: string): integer;
    function GetViewData(var ObjectList: TStringList; const ViewName: String): integer;
    function GetViewList(var ObjectList: TStringList; const SystemData: boolean): integer;
    procedure ObjectDescriptionUpdate(NodeType: integer; ObjectName, Description: String);
    procedure StartTransaction(ObjectList: TStringList; const ObjectStr: String);
    property ServerNode: TibcServerNode read FServerNode write SetServerNode;
    property DatabaseNode: TibcDatabaseNode read FDatabaseNode write SetDatabaseNode;
    function GetTriggerData(const TriggerName : String; var TriggerType, Active, Table, FireOrder, Source, Description:String): integer;
    property AutoDDL:boolean read FAutoDDL write SetAutoDDL;
  end;

  TibcDatabaseNode = class(TibcTreeNode)
  private
    FDatabaseFiles: TStringList;
    FUserName: string;
    FPassword: string;
    FRole: string;
    FCaseSensitiveRole: boolean;
    FDefaultBackupAlias: string;
    FSQLBuffer: TStringList;
    FSQLBufferPos: integer;

{$IFDEF LINUX}
    FTablesID: QListViewItemH;
    FDomainsID: QListViewItemH;
    FViewsID: QListViewItemH;
    FProceduresID: QListViewItemH;
    FFunctionsID: QListViewItemH;
    FGeneratorsID: QListViewItemH;
    FExceptionsID: QListViewItemH;
    FFiltersID: QListViewItemH;
    FRolesID: QListViewItemH;
    FTriggersID: QListViewItemH;
{$ENDIF}
{$IFDEF MSWINDOWS}
    FTablesID: HTreeItem;
    FDomainsID: HTreeItem;
    FViewsID: HTreeItem;
    FProceduresID: HTreeItem;
    FFunctionsID: HTreeItem;
    FGeneratorsID: HTreeItem;
    FExceptionsID: HTreeItem;
    FFiltersID: HTreeItem;
    FRolesID: HTreeItem;
    FTriggersID: HTreeItem;
{$ENDIF}

    FCharacterSet: string;
    fDmN:TDmN;
//lm    fPatchBlankCharset:boolean;

  public
{$IFDEF LINUX}
    constructor Create(AOwner: TComponent; const NodeID: QListViewItemH;
      const NodeName: string; const NodeType: word;
      const DatabaseProps: TibcDatabaseProps; const Password: string; NewDatabase: TIB_Connection);
{$ENDIF}
{$IFDEF MSWINDOWS}
{    constructor Create(AOwner: TComponent; const NodeID: HTreeItem;
      const NodeName: string; const NodeType: word;
      const DatabaseFiles: TStrings; var NewDatabase: TIB_Connection); }
  constructor Create(AOwner: TComponent; const NodeID: HTreeItem; const NodeName: string;
  const NodeType: word; const DatabaseProps: TibcDatabaseProps; const Password: string);
{$ENDIF}
    destructor Destroy(); override;
    property DatabaseFiles: TStringList read FDatabaseFiles write FDatabaseFiles;
    property UserName: string read FUserName write FUserName;
    property Password: string read FPassword write FPassword;
    property Role: string read FRole write FRole;
    property CaseSensitiveRole: boolean read FCaseSensitiveRole write FCaseSensitiveRole;
    property DefaultBackupAlias: string read FDefaultBackupAlias write FDefaultBackupAlias;
    property SQLBuffer: TStringList read FSQLBuffer write FSQLBuffer;
    property SQLBufferPos: integer read FSQLBufferPos write FSQLBufferPos;

{$IFDEF LINUX}
    property TablesID: QListViewItemH read FTablesID write FTablesID;
    property DomainsID: QListViewItemH read FDomainsID write FDomainsID;
    property ViewsID: QListViewItemH read FViewsID write FViewsID;
    property ProceduresID: QListViewItemH read FProceduresID write FProceduresID;
    property FunctionsID: QListViewItemH read FFunctionsID write FFunctionsID;
    property GeneratorsID: QListViewItemH read FGeneratorsID write FGeneratorsID;
    property ExceptionsID: QListViewItemH read FExceptionsID write FExceptionsID;
    property FiltersID: QListViewItemH read FFiltersID write FFiltersID;
    property RolesID: QListViewItemH read FRolesID write FRolesID;
    property TriggersID: QListViewItemH read FTriggersID write FTriggersID;
{$ENDIF}
{$IFDEF MSWINDOWS}
    property TablesID: HTreeItem read FTablesID write FTablesID;
    property DomainsID: HTreeItem read FDomainsID write FDomainsID;
    property ViewsID: HTreeItem read FViewsID write FViewsID;
    property ProceduresID: HTreeItem read FProceduresID write FProceduresID;
    property FunctionsID: HTreeItem read FFunctionsID write FFunctionsID;
    property GeneratorsID: HTreeItem read FGeneratorsID write FGeneratorsID;
    property ExceptionsID: HTreeItem read FExceptionsID write FExceptionsID;
    property FiltersID: HTreeItem read FFiltersID write FFiltersID;
    property RolesID: HTreeItem read FRolesID write FRolesID;
    property TriggersID: HTreeItem read FTriggersID write FTriggersID;
{$ENDIF}
    property CharacterSet: string read FCharacterSet write FCharacterSet;
    property DataModule:TDmN read fDmN;
//lm    property PatchBlankCharset:boolean read fPatchBlankCharset write fPatchBlankCharset;
  end;

var
  DmN: TDmN;

procedure ResetColumnDef(Var Data:TColumnDef);
procedure AssignColumnDef(Var Dest:TColumnDef; Src:TColumnDef);
function GetSqlFieldDef(Var fData:TColumnDef; SQLDialect:integer;  inc_NoNull:boolean; inc_Default:boolean):String;
function GetDataType(Var fData:TColumnDef; SQLDialect:integer = 3;  inc_NoNull:boolean = false; inc_Default:boolean=False): String;
procedure AssignIndexDef(Var Dest:TIndexDef; Src:TIndexDef);
function CompareColumnDef(const Dest:TColumnDef; Src:TColumnDef):boolean;
function CompareColumnDataType(const Dest:TColumnDef; Src:TColumnDef):boolean;

implementation

{$R *.dfm}

uses
  IBOCGlobal, frmuMessage, zluUtility, IBOCPersistentTypes,
  IBOAdminUtils, IB_Schema, IB_Header, frmuMain, frmuWindowList,
  LMMessageDialog;


procedure ResetColumnDef(Var Data:TColumnDef);
begin
  Data.ColumnName   := '';
  Data.DataType     := '';
  Data.Precision    := 0;
  Data.Scale        := 0;
  Data.BlobSubType  := '';
  Data.BlobCharSet  := '';
  Data.BlobCollate  := '';
  Data.BlobSegSize  := 0;
  Data.CharLength   := 0;
  Data.CharCharSet  := '';
  Data.CharCollate  := '';
  Data.Domain       := '';
  Data.Default      := '';
  Data.Computed     := '';
  Data.NotNull      := False;
  Data.Pk           := False;
  Data.UseDomain    := False;
  Data.UseComputed  := False;
  SetLength(Data.ArrayDim,0);
end;

procedure AssignColumnDef(Var Dest:TColumnDef; Src:TColumnDef);
var n:integer;
begin
  Dest.ColumnName   :=  Src.ColumnName ;
  Dest.DataType     :=  Src.DataType   ;
  Dest.Precision    :=  Src.Precision  ;
  Dest.Scale        :=  Src.Scale      ;
  Dest.BlobSubType  :=  Src.BlobSubType;
  Dest.BlobCharSet  :=  Src.BlobCharSet;
  Dest.BlobCollate  :=  Src.BlobCollate;
  Dest.BlobSegSize  :=  Src.BlobSegSize;
  Dest.CharLength   :=  Src.CharLength ;
  Dest.CharCharSet  :=  Src.CharCharSet;
  Dest.CharCollate  :=  Src.CharCollate;
  Dest.Domain       :=  Src.Domain     ;
  Dest.Default      :=  Src.Default;
  Dest.Computed     :=  Src.Computed   ;
  Dest.NotNull      :=  Src.NotNull    ;
  Dest.Pk           :=  Src.Pk         ;
  Dest.UseDomain    :=  Src.UseDomain  ;
  Dest.UseComputed  :=  Src.UseComputed;
  SetLength(Dest.ArrayDim,Length(Src.ArrayDim));
  for n:= Low(Src.ArrayDim) to high(Src.ArrayDim) do
    Dest.ArrayDim[n] := Src.ArrayDim[n];
end;

procedure AssignIndexDef(Var Dest:TIndexDef; Src:TIndexDef);
var n:integer;
begin
  Dest.Name        :=  Src.Name ;
  Dest.Table       :=  Src.Table   ;
  Dest.IndexType   :=  Src.IndexType  ;
  Dest.Unique      :=  Src.Unique      ;
  Dest.Inactive    :=  Src.Inactive;
  Dest.Description :=  Src.Description;
  SetLength(Dest.Fields,Length(Src.Fields));
  for n:= Low(Src.Fields) to high(Src.Fields) do
    Dest.Fields[n] := Src.Fields[n];
end;

function CompareColumnDef(const Dest:TColumnDef; Src:TColumnDef):boolean;
var n:integer;
begin
  Result :=
  (Dest.ColumnName   <>  Src.ColumnName ) or
  (Dest.DataType     <>  Src.DataType   ) or
  (Dest.Precision    <>  Src.Precision  ) or
  (Dest.Scale        <>  Src.Scale      ) or
  (Dest.BlobSubType  <>  Src.BlobSubType) or
  (Dest.BlobCharSet  <>  Src.BlobCharSet) or
  (Dest.BlobCollate  <>  Src.BlobCollate) or
  (Dest.BlobSegSize  <>  Src.BlobSegSize) or
  (Dest.CharLength   <>  Src.CharLength ) or
  (Dest.CharCharSet  <>  Src.CharCharSet) or
  (Dest.CharCollate  <>  Src.CharCollate) or
  (Dest.Domain       <>  Src.Domain     ) or
  (Dest.Default      <>  Src.Default    ) or
  (Dest.Computed     <>  Src.Computed   ) or
  (Dest.NotNull      <>  Src.NotNull    ) or
  (Dest.Pk           <>  Src.Pk         ) or
  (Dest.UseDomain    <>  Src.UseDomain  ) or
  (Dest.UseComputed  <>  Src.UseComputed) or
  (Length(Dest.ArrayDim) <> Length(Src.ArrayDim));
  if not result then
    for n:= Low(Src.ArrayDim) to high(Src.ArrayDim) do
    begin
      if (Dest.ArrayDim[n].LBound <> Src.ArrayDim[n].LBound) or
         (Dest.ArrayDim[n].UBound <> Src.ArrayDim[n].UBound)
      then
      begin
        Result := True;
        exit;
      end;
    end;
end;

function CompareColumnDataType(const Dest:TColumnDef; Src:TColumnDef):boolean;
var n:integer;
begin
  Result :=
  (Dest.DataType     <>  Src.DataType   ) or
  (Dest.Precision    <>  Src.Precision  ) or
  (Dest.Scale        <>  Src.Scale      ) or
  (Dest.BlobSubType  <>  Src.BlobSubType) or
  (Dest.BlobCharSet  <>  Src.BlobCharSet) or
  (Dest.BlobCollate  <>  Src.BlobCollate) or
  (Dest.BlobSegSize  <>  Src.BlobSegSize) or
  (Dest.CharLength   <>  Src.CharLength ) or
  (Dest.CharCharSet  <>  Src.CharCharSet) or
  (Dest.CharCollate  <>  Src.CharCollate) or
  (Length(Dest.ArrayDim) <> Length(Src.ArrayDim));
  if not result then
    for n:= Low(Src.ArrayDim) to high(Src.ArrayDim) do
    begin
      if (Dest.ArrayDim[n].LBound <> Src.ArrayDim[n].LBound) or
         (Dest.ArrayDim[n].UBound <> Src.ArrayDim[n].UBound)
      then
      begin
        Result := True;
        exit;
      end;
    end;
end;



function GetDataType(Var fData:TColumnDef; SQLDialect:integer = 3;  inc_NoNull:boolean = false; inc_Default:boolean=False): String;

  procedure AddDimArray;
  var
    Idx : Integer;
  begin
    Result := Result + ' [';
    for Idx := Low(fData.ArrayDim) to High(fData.ArrayDim) do
    begin
      Result := Result + IntToStr(fData.ArrayDim[Idx].LBound) + ':' +
                         IntToStr(fData.ArrayDim[Idx].UBound);
      if idx < High(fData.ArrayDim) then
        Result := Result + ', ';
    end;
    Result := Result + ']';
  end;

begin
  if fData.UseDomain then
     Result := QuoteIdentifier(SQLDialect,fData.Domain)
  else
  if fData.UseComputed then
     Result := 'computed by ' + Trim(fData.Computed)
  else
  begin
    Result := fData.DataType;
    if (fData.DataType = 'DECIMAL') or
       (fData.DataType = 'NUMERIC')
    then
    begin
      Result := Result + '(' + IntToStr(fData.Precision) + ', ' + IntToStr(fData.Scale) + ')';
      if Length(fData.ArrayDim) > 0 then
        AddDimArray;
    end
    else
    if (fData.DataType = 'CHAR') or
       (fData.DataType = 'VARCHAR')
    then
    begin
      Result := Result + '(' + IntToStr(fData.CharLength) + ')';
      if Length(fData.ArrayDim) > 0 then
        AddDimArray;
      if not ((fData.CharCharSet = '') or (fData.CharCharSet = 'NONE')) then
        Result := Result + ' character set ' + fData.CharCharSet;
      if (fData.CharCollate <> '') then
        Result := Result + ' collate ' + QuoteIdentifier(SQLDialect,fData.CharCollate);
    end
    else
    if (fData.DataType = 'BLOB')
    then
    begin
      Result := Result + ' sub_type ' + fData.BlobSubType;
      Result := Result + ' segment size ' + IntToStr(fData.BlobSegSize);
      if not ((fData.BlobCharSet = '') or (fData.BlobCharSet = 'NONE')) then
        Result := Result + ' character set ' + fData.BlobCharSet;
      if (fData.BlobCollate <> '') then
        Result := Result + ' collate ' + QuoteIdentifier(SQLDialect,fData.BlobCollate);
    end
    else
    begin
      if Length(fData.ArrayDim) > 0 then
        AddDimArray;
    end;
  end;

  if not fData.UseComputed then
  begin
    if inc_Default and (fData.Default <> '') then
    begin
      Result := Result + ' default ' + fData.Default;
    end;
    if fData.NotNull and inc_NoNull then
    begin
      Result := Result + ' not null';
    end;
  end;

end;

function GetSqlFieldDef(Var fData:TColumnDef; SQLDialect:integer;  inc_NoNull:boolean; inc_Default:boolean):String;
begin
  Result := QuoteIdentifier(SQLDialect,fData.ColumnName) + ' ' + GetDataType(fData,SQLDialect,inc_NoNull,inc_Default);
end;


function TDmN.GetBlobFilterList(var ObjectList: TStringList;  const SystemData: boolean): integer;
const
  SBlobFilter =
    'SELECT RDB$FUNCTION_NAME,RDB$MODULE_NAME,RDB$ENTRYPOINT, ' +
    '  RDB$INPUT_SUB_TYPE,RDB$OUTPUT_SUB_TYPE, RDB$DESCRIPTION FROM RDB$FILTERS ' +
    'ORDER BY RDB$FUNCTION_NAME ';

  SBlobWithSystem =
    'SELECT RDB$FUNCTION_NAME,RDB$MODULE_NAME,RDB$ENTRYPOINT, ' +
    '  RDB$INPUT_SUB_TYPE,RDB$OUTPUT_SUB_TYPE, RDB$DESCRIPTION FROM RDB$FILTERS ' +
    'WHERE RDB$SYSTEM_FLAG <> 1 OR RDB$SYSTEM_FLAG is NULL ' +
    'ORDER BY RDB$FUNCTION_NAME ';

begin
  StartTransaction(ObjectList, Format(GetLangText('Name')+'%s'+
                                      GetLangText('Module')+'%s'+
                                      GetLangText('Entry')+'%s'+
                                      GetLangText('Input')+'%s'+
                                      GetLangText('Output')+'%s'+
                                      GetLangText('Description'),[DEL,DEL,DEL,DEL,DEL]));
  try
    if not SystemData then
      Result := ExecuteQuery(SBlobWithSystem)
    else
      Result := ExecuteQuery(SBlobFilter);
    with lQry do
    begin
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s%s%s%s%s%s%s%s%s',[Fields[0].AsString,
          DEL,Fields[1].AsString,DEL,Fields[2].AsString,DEL,Fields[3].AsString,
          DEL,Fields[4].AsString, DEL, Fields[5].AsString]));
        Next;
      end;
      Close;
    end;
  finally
  end;
end;

function TDmN.GetCheckConstList(var ObjectList: TStringList;  const TableName: string): integer;
const
  SCheckList =
    'SELECT '+
    ' A.RDB$CONSTRAINT_NAME,'+
    ' A.RDB$DEFERRABLE,'+
    ' A.RDB$INITIALLY_DEFERRED,'+
    ' C.RDB$TRIGGER_SOURCE ' +
    'FROM '+
    ' RDB$RELATION_CONSTRAINTS A,'+
    ' RDB$CHECK_CONSTRAINTS B,'+
    ' RDB$TRIGGERS C ' +
    'WHERE '+
    ' A.RDB$CONSTRAINT_NAME = B.RDB$CONSTRAINT_NAME ' +
    ' AND B.RDB$TRIGGER_NAME = C.RDB$TRIGGER_NAME ' +
    ' AND A.RDB$RELATION_NAME = ''%s'' ' +
    ' AND A.RDB$CONSTRAINT_TYPE = ''CHECK'' ' +
    'ORDER BY A.RDB$CONSTRAINT_NAME';
var
  lLastConstraint: string;
begin
  StartTransaction(ObjectList,
    Format(GetLangText('Name')+'%s'+GetLangText('Can Defer')+'%s'+GetLangText('Initially Deferred'), [DEL,DEL]));
  try
    Result := ExecuteQuery(Format(SCheckList, [TableName]));
    with lQry do
    begin
      lLastConstraint := '';
      while not EOF do
      begin
        if lLastConstraint <> Fields[0].AsString then
        begin
          ObjectList.Add(Format('%s%s%s%s%s%s%s',[Trim(Fields[0].AsString),DEL,Trim(Fields[1].AsString),DEL,
            Trim(Fields[2].AsString),DEL,Trim(Fields[3].AsString)]));
        end;
        lLastConstraint := Fields[0].AsString;
        Next;
      end;
    end;
  finally
    lQry.Close;
  end;
end;


function TDmN.ConstraintNameExist(const ConstraintName: string): boolean;
const
  SCheckList =
    'SELECT '+
    ' A.RDB$CONSTRAINT_NAME '+
    ' FROM '+
    ' RDB$RELATION_CONSTRAINTS A '+
    ' WHERE '+
    ' UPPER(A.RDB$CONSTRAINT_NAME) = %s;' ;
begin
  result := false;
  try
    try
      ExecuteQuery(Format(SCheckList, [AnsiQuotedStr(ConstraintName,#39)]));
      result := not lQry.EOF;
    except
    end;
  finally
    lQry.Close;
  end;
end;


function TDmN.GetCheckConstSource(const TableName,CheckName: string): String;
const
  SCheckList =
    'SELECT '+
    ' C.RDB$TRIGGER_SOURCE ' +
    'FROM '+
    ' RDB$RELATION_CONSTRAINTS A,'+
    ' RDB$CHECK_CONSTRAINTS B,'+
    ' RDB$TRIGGERS C ' +
    'WHERE '+
    ' A.RDB$CONSTRAINT_NAME = B.RDB$CONSTRAINT_NAME ' +
    ' AND B.RDB$TRIGGER_NAME = C.RDB$TRIGGER_NAME ' +
    ' AND A.RDB$RELATION_NAME = ''%s'' ' +
    ' AND A.RDB$CONSTRAINT_TYPE = ''CHECK'' ' +
    ' AND B.RDB$CONSTRAINT_NAME = ''%s''' +
    'ORDER BY A.RDB$CONSTRAINT_NAME';
begin
  Result :='';
  try
    ExecuteQuery(Format(SCheckList, [TableName,CheckName]));
    with lQry do
    begin
      while not EOF do
      begin
        Result := Trim(Fields[0].AsString);
        Next;
      end;
    end;
  finally
    lQry.Close;
  end;
end;


function TDmN.GetColumnList(var ObjectList: TStringList; 
                               const TableName: string): integer;
const
  ColumnSQL =
    'SELECT '+
    ' A.RDB$FIELD_NAME, '+
    ' A.RDB$FIELD_SOURCE,'+
    ' B.RDB$FIELD_TYPE, '+
    ' B.RDB$SEGMENT_LENGTH, ' +
    ' B.RDB$FIELD_SUB_TYPE,'+
    ' B.RDB$FIELD_LENGTH,'+
    ' B.RDB$FIELD_SCALE, ' +
    ' B.RDB$DEFAULT_SOURCE DEF_DOM,'+
    ' A.RDB$DEFAULT_SOURCE DEF_NATIVE, ' +
    ' A.RDB$NULL_FLAG NULLS1, '+
    ' B.RDB$NULL_FLAG NULLS2, '+
    ' B.RDB$SYSTEM_FLAG, ' +
    ' B.RDB$DIMENSIONS, '+
    ' B.RDB$CHARACTER_LENGTH, ' +
    ' %s, ' +
    ' B.RDB$CHARACTER_SET_ID,'+
    ' B.RDB$COLLATION_ID ' +
    ' FROM '+
    ' RDB$RELATION_FIELDS A, '+
    ' RDB$FIELDS B ' +
    ' WHERE ' +
    ' A.RDB$RELATION_NAME = ''%s'' AND ' +
    ' A.RDB$FIELD_SOURCE = B.RDB$FIELD_NAME ' +
    ' ORDER BY A.RDB$RELATION_NAME, RDB$FIELD_POSITION ';

var
  Charset, Collation,
  lFieldType,lAllowNulls,lSQLStr,lDefault : string;
  len: integer;
begin
  StartTransaction(ObjectList,
    Format(GetLangText('Name')+'%s'+GetLangText('Type')+'%s'+GetLangText('Character Set')+'%s'+GetLangText('Collation')+'%s'+GetLangText('Default Value')+'%s'+GetLangText('Allow Nulls'),
    [DEL,DEL,DEL,DEL,DEL]));

  IBOExtract.Database := Connection;
  IBOExtract.Transaction := Connection.DefaultTransaction;
  if IBOExtract.Database.Characteristics.dbODS_Version  >= ODS_VERSION10 then
    lSQLStr := Format(ColumnSQL, ['B.RDB$FIELD_PRECISION', TableName])
  else
    lSQLStr := Format(ColumnSQL, ['0 RDB$FIELD_PRECISION', TableName]);
  try
    Result := ExecuteQuery(lSQLStr);
    with lQry do
      while not EOF do
      begin
        lFieldType := '';
        if FieldByName('RDB$DIMENSIONS').AsInteger > 0 then
           lFieldType := IBOExtract.GetArrayField(Trim(FieldByName('RDB$FIELD_SOURCE').AsString));

        if Pos('RDB$', TableName) = 1 then
          len := FieldByName('RDB$FIELD_LENGTH').AsInteger
        else
          len := FieldByName('RDB$CHARACTER_LENGTH').AsInteger;

        lFieldType := Format('%s %s',[lFieldType , IBOExtract.GetFieldType (FieldByName('RDB$FIELD_TYPE').AsInteger,
                        FieldByName('RDB$FIELD_SUB_TYPE').AsInteger,
                        FieldByName('RDB$FIELD_SCALE').AsInteger,
                        Len,
                        FieldByName('RDB$FIELD_PRECISION').AsInteger,
                        FieldByName('RDB$SEGMENT_LENGTH').AsInteger)]);
        Charset := '';
        if not (FieldByName('RDB$CHARACTER_SET_ID').IsNull) and
           (FieldByName('RDB$CHARACTER_SET_ID').AsInteger <> 0) then
          Charset := IBOExtract.GetCharacterSets(FieldByName('RDB$CHARACTER_SET_ID').AsInteger, 0, false);

        Collation := '';
        if (not FieldByName('RDB$COLLATION_ID').IsNull) and
           (FieldByName('RDB$COLLATION_ID').AsInteger <> 0) then
          Collation := IBOExtract.GetCharacterSets(FieldByName('RDB$CHARACTER_SET_ID').AsInteger,
                                                  FieldByName('RDB$COLLATION_ID').AsInteger, true);


        lAllowNulls := GetLangText('Yes');
        if FieldByName('NULLS1').AsInteger = 1 then lAllowNulls := GetLangText('No');
        if FieldByName('NULLS2').AsInteger = 1 then lAllowNulls := GetLangText('No');

        lDefault := '';
        if not FieldByName('DEF_DOM').IsNull then
          lDefault := Trim(FieldByName('DEF_DOM').AsString);

        if not FieldByName('DEF_NATIVE').IsNull then
          lDefault := Trim(FieldByName('DEF_NATIVE').AsString);

        if (Pos('RDB$', FieldByName('RDB$FIELD_SOURCE').AsString) = 0) or
           (FieldByName('RDB$SYSTEM_FLAG').AsInteger = 1) then
          lFieldType := Format('(%s) %s', [Trim(FieldByName('RDB$FIELD_SOURCE').AsString), lFieldType]);

        ObjectList.Add(Format('%s%s%s%s%s%s%s%s%s%s%s',[FieldByName('RDB$FIELD_NAME').AsString,DEL,
                                                lFieldType,DEL,
                                                Charset,DEL,
                                                Collation,DEL,
                                                lDefault, DEL,
                                                lAllowNulls]));
        Next;
      end;
  finally
    lQry.Close;
  end;
end;

function GetColumnTypesIndex(FieldType:integer):integer;
var
  i : Integer;
begin
  Result := -1;
  for i:= 0 to High(ColumnTypes) do
    if FieldType = ColumnTypes[i].SQLType then
    begin
      result := i;
      exit;
    end;
end;

function TDmN.sGetFieldType(FieldType,FieldSubType,FieldScale,FieldPrec:integer): String;
var
  i : Integer;
  PrecisionKnown : Boolean;
begin
  Result := '';
  i := GetColumnTypesIndex(FieldType);
  if i = -1 then exit;
  PrecisionKnown := FALSE;
  if Connection.Characteristics.dbODS_Version >= ODS_VERSION10 then
  begin
    if FieldType in [blr_short, blr_long, blr_int64] then
    begin
      { We are ODS >= 10 and could be any Dialect }
      if (Connection.SQLDialect >= 3) and
         (FieldPrec <> 0) and
         (FieldSubType > 0) and
         (FieldSubType <= MAX_INTSUBTYPES) then
      begin
        Result := IntegralSubtypes [FieldSubType];
        PrecisionKnown := true;
      end;
    end;
  end;
  if PrecisionKnown = false then
  begin
    { Take a stab at numerics and decimals }
    if (FieldType = blr_short) and
        (FieldScale < 0) then
      Result := 'NUMERIC'
    else
      if (FieldType = blr_long) and
          (FieldScale < 0) then
        Result := 'NUMERIC'
      else
        if (FieldType = blr_double) and
            (FieldScale  < 0) then
          Result := 'NUMERIC'
        else
          Result := ColumnTypes[i].TypeName;
  end;
end;

const
  ArraySQL =
    'SELECT * FROM RDB$FIELD_DIMENSIONS FDIM ' +
    'WHERE ' +
    '  FDIM.RDB$FIELD_NAME = :FIELDNAME ' +
    'ORDER BY FDIM.RDB$DIMENSION';

procedure TDmN.GetArrayDef(FieldName: String; var ArrayDef:TArrayDef);
var
  qryArray : TIB_Cursor;
  n:integer;
begin
  n:=0;
  qryArray := TIB_Cursor.Create(Connection);
  try
    qryArray.IB_Connection := Connection;
    qryArray.IB_Transaction := Transaction;
    qryArray.SQL.Text :=(ArraySQL);
    qryArray.Prepare;
    qryArray.Params.ByName('FIELDNAME').AsString := FieldName;
    qryArray.Open;
    while not qryArray.Eof do
    begin
      if n>high(ArrayDef) then
        SetLength(ArrayDef,n+1);
      ArrayDef[n].LBound := qryArray.FieldByName('RDB$LOWER_BOUND').AsInteger;
      ArrayDef[n].UBound := qryArray.FieldByName('RDB$UPPER_BOUND').AsInteger;
      qryArray.Next;
      inc(n);
    end;
  finally
    qryArray.Free;
  end;
  SetLength(ArrayDef,n);
end;


function TDmN.GetColumnData(var Data: TColumnDef;
                               const TableName, FieldName: string): integer;
const
  ColumnSQL =
    ' SELECT                                    '+
    ' A.RDB$FIELD_NAME,                         '+
    ' A.RDB$FIELD_SOURCE,                       '+
    ' B.RDB$FIELD_TYPE,                         '+
    ' B.RDB$SEGMENT_LENGTH,                     '+
    ' B.RDB$FIELD_SUB_TYPE,                     '+
    ' B.RDB$FIELD_LENGTH,                       '+
    ' B.RDB$FIELD_SCALE,                        '+
    ' B.RDB$DEFAULT_SOURCE DEF_DOM,             '+
    ' A.RDB$DEFAULT_SOURCE DEF_NATIVE,          '+
    ' A.RDB$NULL_FLAG NULLS1,                   '+
    ' B.RDB$NULL_FLAG NULLS2,                   '+
    ' B.RDB$SYSTEM_FLAG,                        '+
    ' B.RDB$DIMENSIONS,                         '+
    ' B.RDB$CHARACTER_LENGTH,                   '+
    ' %s,                                       '+
    ' B.RDB$CHARACTER_SET_ID,                   '+
    ' B.RDB$COLLATION_ID,                       '+
    ' B.RDB$COMPUTED_SOURCE,                    '+
    ' B.RDB$COMPUTED_BLR,                       '+
    ' B.RDB$VALIDATION_SOURCE                   '+
    ' FROM                                      '+
    ' RDB$RELATION_FIELDS A,                    '+
    ' RDB$FIELDS B                              '+
    ' WHERE                                     '+
    ' A.RDB$RELATION_NAME = :RELATION_NAME AND  '+
    ' A.RDB$FIELD_SOURCE = B.RDB$FIELD_NAME     ';
    //' AND A.RDB$FIELD_NAME = :FIELD_NAME        ';
    //'ORDER BY A.RDB$RELATION_NAME, RDB$FIELD_POSITION ';

var
  Charset,
  Collation,
  lSQLStr,
  lDefault : string;
  len: integer;
begin
  if Connection.Characteristics.dbODS_Version >= ODS_VERSION10 then
    lSQLStr := Format(ColumnSQL, ['B.RDB$FIELD_PRECISION'])
  else
    lSQLStr := Format(ColumnSQL, ['0 RDB$FIELD_PRECISION']);

  try
    ResetColumnDef(Data);
    PrepareQuery(lSQLStr);
    //lQry.ParamByName('FIELD_NAME').AsString := FieldName;
    lQry.ParamByName('RELATION_NAME').AsString := TableName;
    lQry.Execute;
    if lQry.Eof then
      result := EMPTY
    else
      result := SUCCESS;
    while (not lQry.EOF) and (Trim(lQry.FieldByName('RDB$FIELD_NAME').AsString)<>FieldName) do
    begin
      lQry.Next;
    end;

    if not lQry.EOF then
    begin
      Data.ColumnName := FieldName;
      if not lQry.FieldByName('RDB$COMPUTED_BLR').IsNull then
      begin
        Data.UseComputed := True;
        if not lQry.FieldByName('RDB$COMPUTED_SOURCE').IsNull then
          Data.Computed := Trim(lQry.FieldByName('RDB$COMPUTED_SOURCE').AsString);
      end
      else
      begin
        if lQry.FieldByName('RDB$DIMENSIONS').AsInteger > 0 then
           GetArrayDef(Trim(lQry.FieldByName('RDB$FIELD_SOURCE').AsString),Data.ArrayDim);

        Data.DataType := sGetFieldType (lQry.FieldByName('RDB$FIELD_TYPE').AsInteger,
                        lQry.FieldByName('RDB$FIELD_SUB_TYPE').AsInteger,
                        lQry.FieldByName('RDB$FIELD_SCALE').AsInteger,
                        lQry.FieldByName('RDB$FIELD_PRECISION').AsInteger);

        if Pos('RDB$', TableName) = 1 then
          len := lQry.FieldByName('RDB$FIELD_LENGTH').AsInteger
        else
          len := lQry.FieldByName('RDB$CHARACTER_LENGTH').AsInteger;

        Data.CharLength := len;


        if not (lQry.FieldByName('RDB$CHARACTER_SET_ID').IsNull) and
           (lQry.FieldByName('RDB$CHARACTER_SET_ID').AsInteger <> 0) then
        begin
          Charset := IBOExtract.GetCharacterSets(lQry.FieldByName('RDB$CHARACTER_SET_ID').AsInteger, 0, false);
          if Data.DataType = 'BLOB' then
            Data.BlobCharSet := Charset
          else
            Data.CharCharSet := Charset;
        end;

        Collation := '';
        if (not lQry.FieldByName('RDB$COLLATION_ID').IsNull) and
           (lQry.FieldByName('RDB$COLLATION_ID').AsInteger <> 0) then
        begin
          Collation := IBOExtract.GetCharacterSets(lQry.FieldByName('RDB$CHARACTER_SET_ID').AsInteger,
                                                  lQry.FieldByName('RDB$COLLATION_ID').AsInteger, true);
          if Data.DataType = 'BLOB' then
            Data.BlobCollate := Charset
          else
            Data.CharCollate := Charset;
        end;

        if lQry.FieldByName('NULLS1').AsInteger = 1 then Data.NotNull := True;
        if lQry.FieldByName('NULLS2').AsInteger = 1 then Data.NotNull := True;

        lDefault := '';
        if not lQry.FieldByName('DEF_DOM').IsNull then
          lDefault := Trim(lQry.FieldByName('DEF_DOM').AsString);

        if not lQry.FieldByName('DEF_NATIVE').IsNull then
          lDefault := Trim(lQry.FieldByName('DEF_NATIVE').AsString);

        if lDefault <> '' then
          Data.Default := lDefault;

        if (Pos('RDB$', lQry.FieldByName('RDB$FIELD_SOURCE').AsString) = 0) or
           (lQry.FieldByName('RDB$SYSTEM_FLAG').AsInteger = 1) then
        begin
          Data.Domain := Trim(lQry.FieldByName('RDB$FIELD_SOURCE').AsString);
          Data.UseDomain := True;
        end;
      end;
    end
    else
      SetLength(Data.ArrayDim,0);
  finally
    lQry.Close;
  end;
end;


function TDmN.GetDomainList(var ObjectList: TStringList;
   const SystemData: boolean): integer;
const
  SDomainList =
    'SELECT RDB$FIELD_NAME,RDB$DESCRIPTION FROM RDB$FIELDS ' +
    'WHERE RDB$FIELD_NAME NOT STARTING WITH ''RDB$'' ' +
    '  AND RDB$SYSTEM_FLAG = 0 OR RDB$SYSTEM_FLAG is NULL ' +
    'ORDER BY RDB$FIELD_NAME';

  SDomainWithSystem =
    'SELECT RDB$FIELD_NAME,RDB$DESCRIPTION FROM RDB$FIELDS ' +
    'WHERE (RDB$FIELD_NAME NOT STARTING WITH ''RDB$'' ' +
    '  AND RDB$SYSTEM_FLAG = 0 OR RDB$SYSTEM_FLAG is NULL) ' +
    '  OR rdb$system_flag = 1 ' +
    'ORDER BY RDB$FIELD_NAME';

begin
  StartTransaction(ObjectList, Format(GetLangText('Name')+'%s'+GetLangText('Description'), [DEL]));
  try
    if not SystemData then
      Result := ExecuteQuery(SDomainList)
    else
      Result := ExecuteQuery(SDomainWithSystem);
    with lQry do
      while not EOF do
      begin
        ObjectList.Append (Fields[0].AsString+DEL+Fields[1].AsString);
        result := Success;
        Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetExceptionList(var ObjectList: TStringList;
   const SystemData: boolean): integer;
const
  SExceptionList =
    'SELECT RDB$EXCEPTION_NAME,RDB$MESSAGE,RDB$DESCRIPTION FROM RDB$EXCEPTIONS ' +
    'ORDER BY RDB$EXCEPTION_NUMBER';

  SExceptionListSystem =
    'SELECT RDB$EXCEPTION_NAME,RDB$MESSAGE,RDB$DESCRIPTION FROM RDB$EXCEPTIONS ' +
    'WHERE RDB$SYSTEM_FLAG <> 1 OR RDB$SYSTEM_FLAG is NULL ' +
    'ORDER BY RDB$EXCEPTION_NUMBER';

begin
  StartTransaction(ObjectList, Format(GetLangText('Name')+'%s'+GetLangText('Message')+'%s'+GetLangText('Description'),[DEL,DEL]));
  try
    if not SystemData then
      Result := ExecuteQuery(SExceptionList)
    else
      Result := ExecuteQuery(SExceptionListSystem);
    with lQry do
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s%s%s',[Fields[0].AsString,DEL,Fields[1].AsString,
          DEL, Fields[2].AsString]));
        Next;
      end
  finally
    lQry.Close;
    //Connection.DefaultTransaction.Commit;
  end;
end;

function TDmN.GetFunctionList(var ObjectList: TStringList;  const SystemData: boolean): integer;
const
  SFunctionList =
    'SELECT RDB$FUNCTION_NAME,RDB$MODULE_NAME,RDB$ENTRYPOINT,RDB$DESCRIPTION ' +
    'FROM RDB$FUNCTIONS ' +
    'ORDER BY RDB$FUNCTION_NAME';
begin
  StartTransaction(ObjectList, Format(GetLangText('Name')+'%s'+GetLangText('Module')+'%s'+GetLangText('Entry')+'%s'+GetLangText('Description'),[DEL,DEL,DEL]));
  try
    Result := ExecuteQuery(SFunctionList);
    with lQry do
    begin
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s%s%s%s%s',[Fields[0].AsString,DEL,
          Fields[1].AsString,DEL,Fields[2].AsString, DEL, Fields[3].AsString]));
        Next;
      end
    end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetGeneratorList(var ObjectList: TStringList;  const SystemData: boolean): integer;
const
  SGeneratorListSystem   =
    'SELECT RDB$GENERATOR_NAME FROM RDB$GENERATORS ' +
    'ORDER BY RDB$GENERATOR_NAME';

  SGeneratorList =
    'SELECT RDB$GENERATOR_NAME FROM RDB$GENERATORS ' +
    'WHERE RDB$SYSTEM_FLAG <> 1 OR RDB$SYSTEM_FLAG is NULL ' +
    'ORDER BY RDB$GENERATOR_NAME';

  SGenValue = 'SELECT GEN_ID(%s, 0) FROM RDB$DATABASE';
var
  lqryGetGenNextVal: TIB_Cursor;
  lCurrGenVal: string;
begin
  StartTransaction(ObjectList, Format(GetLangText('Name')+'%s'+GetLangText('Current Value'),[DEL]));
  lqryGetGenNextVal := TIB_Cursor.Create(nil);
  lqryGetGenNextVal.IB_Connection := Connection;
  lqryGetGenNextVal.IB_Transaction := Connection.DefaultTransaction;
  try
    if not SystemData then
      Result := ExecuteQuery(SGeneratorList)
    else
      Result := ExecuteQuery(SGeneratorListSystem);
    while not lQry.EOF do
    begin
      if lQry.Fields[0].AsString <> '' then
      begin
        lCurrGenVal := '0';
        lqryGetGenNextVal.Close;
        lqryGetGenNextVal.SQL.Clear;
        lqryGetGenNextVal.SQL.Text :=(Format(SGenValue,
          [QuoteIdentifier(Connection.SQLDialect, lQry.Fields[0].AsString)]));
        lqryGetGenNextVal.Prepare;
        lqryGetGenNextVal.Open;
        if not lqryGetGenNextVal.EOF then
          lCurrGenVal := lqryGetGenNextVal.Fields[0].AsString;
        ObjectList.Add(Format('%s%s%s',[lQry.Fields[0].AsString,DEL,lCurrGenVal]));
      end;
      lQry.Next;
    end;
  finally
    lqryGetGenNextVal.Close;
    lqryGetGenNextVal.Free;
    ////lQry.IB_Transaction.Commit;
    lQry.Close;
  end;
end;

function TDmN.GetIndexList(var ObjectList: TStringList;  const TableName: string): integer;
const
  SIndexList =
    'SELECT RDB$INDEX_NAME'+
    ' ,RDB$UNIQUE_FLAG'+
    ' ,RDB$INDEX_TYPE'+
    ' ,RDB$INDEX_INACTIVE ' +
    ' ,RDB$DESCRIPTION '+
    ' FROM RDB$INDICES WHERE RDB$RELATION_NAME = ''%s'' ' +
    ' ORDER BY RDB$INDEX_NAME';

  SIndexFieldList =
    ' SELECT RDB$FIELD_NAME FROM RDB$INDEX_SEGMENTS '+
    ' WHERE RDB$INDEX_NAME = ''%s'''+
    ' ORDER BY RDB$FIELD_POSITION;';

var
  lUnique,lDescending,lActive, lFList, lDescription: string;
begin
  StartTransaction(ObjectList,
    Format(GetLangText('Name')+'%s'+GetLangText('Unique')+'%s'+GetLangText('Descending')+'%s'+GetLangText('Active')+'%s'+GetLangText('Fields'),[DEL,DEL,DEL,DEL]));
  try
    Result := ExecuteQuery(Format(SIndexList, [TableName]));
    while not lQry.EOF do
    begin
      if lQry.FieldbyName('RDB$UNIQUE_FLAG').AsInteger = 1 then
        lUnique := GetLangText('Yes')
      else
        lUnique := GetLangText('No');
      if lQry.FieldbyName('RDB$INDEX_TYPE').AsInteger = 1 then
        lDescending := GetLangText('Yes')
      else
        lDescending := GetLangText('No');
      if lQry.FieldByName('RDB$INDEX_INACTIVE').AsInteger = 1 then
        lActive := GetLangText('No')
      else
        lActive := GetLangText('Yes');

      if lQry.FieldByName('RDB$DESCRIPTION').IsNull then
        lDescription := NULL_FLAG
      else
        lDescription := lQry.FieldByName('RDB$DESCRIPTION').AsString;

      ExecuteQuery2(Format(SIndexFieldList, [lQry.Fields[0].asString]));
      lFList := '';
      while not lQry2.EOF do
      begin
        lFList := lFList + lQry2.Fields[0].asString + ', ';
        lQry2.Next;
      end;
      if lFList <> '' then
        Delete(lFList,Length(lFList)-1,2);
                          // 1   2   3   4   5   6
      ObjectList.Add(Format('%s%s%s%s%s%s%s%s%s%s%s',[lQry.Fields[0].AsString,DEL,
                                                      lUnique,DEL,
                                                      lDescending,DEL,
                                                      lActive,DEL,
                                                      lFList,DEL,
                                                      lDescription]));
      lQry.Next;
    end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.IndexNameExist(const IndexName: string): boolean;
const
  SIndexList =
    'SELECT RDB$INDEX_NAME ' +
     'FROM RDB$INDICES WHERE UPPER(RDB$INDEX_NAME) = %s ';

begin
  Result := False;
  try
    try
      ExecuteQuery(Format(SIndexList, [ansiQuotedStr(IndexName,#39)]));
      result := not lQry.EOF;
    except
    end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;


function TDmN.GetProcedureList(var ObjectList: TStringList;
   const SystemData: boolean): integer;
const
  SProcedureList =
    'SELECT RDB$PROCEDURE_NAME,RDB$OWNER_NAME, RDB$DESCRIPTION FROM RDB$PROCEDURES ' +
    'ORDER BY RDB$PROCEDURE_NAME';

  SProcedureListSystem =
    'SELECT RDB$PROCEDURE_NAME,RDB$OWNER_NAME, RDB$DESCRIPTION FROM RDB$PROCEDURES ' +
    'WHERE RDB$SYSTEM_FLAG <> 1 OR RDB$SYSTEM_FLAG is NULL ' +
    'ORDER BY RDB$PROCEDURE_NAME';

begin
  StartTransaction(ObjectList,
    Format(GetLangText('Name')+'%s'+GetLangText('Owner')+'%s'+GetLangText('Description'),[DEL,DEL]));
  try
    if SystemData then
      Result := ExecuteQuery(SProcedureList)
    else
      Result := ExecuteQuery(SProcedureListSystem);
    while not lQry.EOF do
    begin
      ObjectList.Add(Format('%s%s%s%s%s',
        [lQry.Fields[0].AsString,DEL,lQry.Fields[1].AsString, DEL, lQry.Fields[2].AsString]));
      lQry.Next;
    end
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetReferentialConstList(var ObjectList: TStringList;  const TableName: string): integer;
const
  SReferentialList =
    'SELECT '+
    ' A.RDB$CONSTRAINT_NAME,'+
    ' A.RDB$DEFERRABLE,'+
    ' A.RDB$INITIALLY_DEFERRED, ' +
    ' B.RDB$MATCH_OPTION,'+
    ' B.RDB$UPDATE_RULE,'+
    ' B.RDB$DELETE_RULE,'+
    ' A.RDB$INDEX_NAME,'+
    ' C.RDB$RELATION_NAME ' +
    'FROM '+
    ' RDB$RELATION_CONSTRAINTS A, '+
    ' RDB$REF_CONSTRAINTS B, '+
    ' RDB$RELATION_CONSTRAINTS C ' +
    'WHERE '+
    ' A.RDB$CONSTRAINT_NAME = B.RDB$CONSTRAINT_NAME ' +
    ' AND C.RDB$CONSTRAINT_NAME = B.RDB$CONST_NAME_UQ ' +
    ' AND A.RDB$RELATION_NAME = ''%s'' ' +
    ' AND (A.RDB$CONSTRAINT_TYPE = ''PRIMARY KEY'' OR A.RDB$CONSTRAINT_TYPE = ''FOREIGN KEY'') ' +
    'ORDER BY A.RDB$CONSTRAINT_NAME';

begin
  StartTransaction(ObjectList,
    Format(GetLangText('Name')+'%s'+GetLangText('Can Defer')+'%s'+GetLangText('Initially Deferred')+'%s'+GetLangText('Match Option')+'%s'+GetLangText('Update Rule')+'%s'+GetLangText('Delete Rule')+'%s'+GetLangText('Index')+'%s'+GetLangText('Reference Table'),
                  [DEL,DEL,DEL,DEL,DEL,DEL,DEL]));
  try
    Result := ExecuteQuery(Format(SReferentialList, [TableName]));
    with lQry do
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s',[
          Fields[0].AsString,DEL,
          Fields[1].AsString,DEL,
          Fields[2].AsString,DEL,
          Fields[3].AsString,DEL,
          Fields[4].AsString,DEL,
          Fields[5].AsString,DEL,
          Fields[6].AsString,DEL,
          Fields[7].AsString]));
        Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetReferentialConstData(var Data: TRefConstraintDef;  const TableName,ConstraintName: string): integer;
const
  SReferentialList =
    'SELECT '+
    ' A.RDB$CONSTRAINT_NAME,'+
    ' A.RDB$DEFERRABLE,'+
    ' A.RDB$INITIALLY_DEFERRED, ' +
    ' B.RDB$MATCH_OPTION,'+
    ' B.RDB$UPDATE_RULE,'+
    ' B.RDB$DELETE_RULE,'+
    ' A.RDB$INDEX_NAME,'+
    ' C.RDB$RELATION_NAME ' +
    'FROM '+
    ' RDB$RELATION_CONSTRAINTS A, '+
    ' RDB$REF_CONSTRAINTS B, '+
    ' RDB$RELATION_CONSTRAINTS C ' +
    'WHERE '+
    ' A.RDB$CONSTRAINT_NAME = B.RDB$CONSTRAINT_NAME ' +
    ' AND C.RDB$CONSTRAINT_NAME = B.RDB$CONST_NAME_UQ ' +
    ' AND A.RDB$RELATION_NAME = ''%s'' ' +
    ' AND A.RDB$CONSTRAINT_NAME = ''%s'' ' +
    'ORDER BY A.RDB$CONSTRAINT_NAME';

begin
  try
    Result := ExecuteQuery(Format(SReferentialList, [TableName,ConstraintName]));
    if not lQry.EOF then
    begin
      Data.Name :=              lQry.Fields[0].AsString;
      Data.Deferrable :=        lQry.Fields[1].AsString;
      Data.InitiallyDeferred := lQry.Fields[2].AsString;
      Data.Match_Option :=      lQry.Fields[3].AsString;
      Data.Update_Rule :=       lQry.Fields[4].AsString;
      Data.DeleteRule :=        lQry.Fields[5].AsString;
      Data.IndexName :=         lQry.Fields[6].AsString;
      Data.RelationName :=      lQry.Fields[7].AsString;
    end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;


function TDmN.GetRoleList(var ObjectList: TStringList): integer;
const
  SRoleList =
    'SELECT RDB$ROLE_NAME,RDB$OWNER_NAME ' +
    'FROM RDB$ROLES ' +
    'ORDER BY RDB$OWNER_NAME, RDB$ROLE_NAME';

begin
  StartTransaction(ObjectList,
    Format(GetLangText('Name')+'%s'+GetLangText('Owner'),[DEL]));
  try
    Result := ExecuteQuery(SRoleList);
    with lQry do
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s',[Fields[0].AsString,DEL,Fields[1].AsString]));
        Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetOwnerInfo(NodeName:string; var OwnerName, Description: string; NodeType:integer): integer;
const
  SProcedureOwner =
    'SELECT RDB$OWNER_NAME, RDB$DESCRIPTION FROM RDB$PROCEDURES ' +
    'WHERE RDB$PROCEDURE_NAME = ''%s''';

  STableOwner =
    'SELECT RDB$OWNER_NAME, RDB$DESCRIPTION FROM RDB$RELATIONS ' +
    'WHERE RDB$RELATION_NAME = ''%s''';

  SRoleOwner =
    'SELECT RDB$OWNER_NAME FROM RDB$ROLES ' +
    'WHERE RDB$ROLE_NAME = ''%s''';

begin
  
  try
    case NodeType of
      NODE_PROCEDURE:
        Result := ExecuteQuery(Format(SProcedureOwner, [NodeName]));
      NODE_TABLE, NODE_VIEW:
        Result := ExecuteQuery(Format(STableOwner, [NodeName]));
      NODE_ROLE:
        Result := ExecuteQuery(Format(SRoleOwner, [NodeName]));
      else
      begin
        result := Failure;
        exit;
      end;
    end;
    with lQry do
    begin
      OwnerName := Fields[0].AsString;
      if NodeType <> NODE_ROLE then
        Description := Fields[1].AsString;
    end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.IsSystemTable(const ObjName): boolean;
const
  STableList =
    'SELECT RDB$SYSTEM_FLAG FROM RDB$RELATIONS ' +
    'WHERE RDB$RELATION_NAME = :ObjName';

begin
  result := False;
  StartTransaction(nil,'');
  try
    ExecuteQuery(STableList);
    with lQry do
      if not EOF then
      begin
        result := Fields[0].AsInteger = 1;
      end;
  finally
    lQry.Close;
  end;
end;


function TDmN.GetTableList(var ObjectList: TStringList;
   const SystemData: boolean): integer;
const
  STableListSystem =
    'SELECT RDB$RELATION_NAME,RDB$OWNER_NAME,RDB$DESCRIPTION FROM RDB$RELATIONS ' +
    'WHERE RDB$RELATION_NAME NOT IN ( ' +
    'SELECT RDB$VIEW_NAME FROM RDB$VIEW_RELATIONS) ' +
    'ORDER BY RDB$RELATION_NAME';

  STableList =
    'SELECT RDB$RELATION_NAME,RDB$OWNER_NAME,RDB$DESCRIPTION FROM RDB$RELATIONS ' +
    'WHERE RDB$RELATION_NAME NOT IN ( ' +
    'SELECT RDB$VIEW_NAME FROM RDB$VIEW_RELATIONS) ' +
    '  AND RDB$SYSTEM_FLAG <> 1 OR RDB$SYSTEM_FLAG is NULL ' +
    'ORDER BY RDB$RELATION_NAME';

begin
  StartTransaction(ObjectList,
    Format(GetLangText('Name')+'%s'+GetLangText('Owner')+'%s'+GetLangText('Description'),[DEL,DEL]));
  try
    if not SystemData then
      Result := ExecuteQuery(STableList)
    else
      Result := ExecuteQuery(STableListSystem);
    with lQry do
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s%s%s',[Fields[0].AsString,DEL,
          Fields[1].AsString,DEL,Fields[2].AsString]));
        Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetTriggerList(var ObjectList: TStringList;
   const TableName: string): integer;
const
  STriggerList =
    'SELECT RDB$TRIGGER_NAME '+
    '      ,RDB$TRIGGER_TYPE '+
    '      ,RDB$TRIGGER_INACTIVE '+
    '      ,RDB$TRIGGER_SOURCE '+
    '      ,RDB$TRIGGER_SEQUENCE '+
    '      ,RDB$DESCRIPTION '+
    ' FROM RDB$TRIGGERS ' +
    ' WHERE RDB$RELATION_NAME = ''%s'' ' +
    '  AND RDB$TRIGGER_NAME NOT IN ' +
    '  (SELECT RDB$TRIGGER_NAME FROM RDB$CHECK_CONSTRAINTS) ' +
    'ORDER BY RDB$TRIGGER_NAME';

var
  lName, lTriggerType, lActive, lSource, lSequence, lDescription: string;
begin
  StartTransaction(ObjectList,
    Format(GetLangText('Name')+'%s'+GetLangText('Type')+'%s'+GetLangText('Active')+'%s'+GetLangText('Position'),[DEL,DEL,DEL]));
  try
    Result := ExecuteQuery(Format(STriggerList, [TableName]));
    with lQry do
      while not EOF do
      begin
        lName := Fields[0].AsString;

//  Update  by hideo tanaka 2003/12/15  start
{
        case FieldbyName('RDB$TRIGGER_TYPE').AsInteger of
          1: lTriggerType := 'before insert';
          2: lTriggerType := 'after insert';
          3: lTriggerType := 'before update';
          4: lTriggerType := 'after update';
          5: lTriggerType := 'before delete';
          6: lTriggerType := 'after delete';
        else
          lTriggerType := '';
        end;
}
        lTriggerType := GetTriggerTypes(FieldbyName('RDB$TRIGGER_TYPE').AsInteger);
//  Update  by hideo tanaka 2003/12/15  end

        if FieldByName('RDB$TRIGGER_INACTIVE').AsInteger = 1 then
          lActive := 'InActive'
        else
          lActive := 'Active';

        if FieldByName('RDB$TRIGGER_SEQUENCE').IsNull then
          lSequence := 'Not Available'
        else
          lSequence := FieldByName('RDB$TRIGGER_SEQUENCE').AsString;

        if FieldByName('RDB$TRIGGER_SOURCE').IsNull then
          lSource := 'Not Available'
        else
          lSource := FieldByName('RDB$TRIGGER_SOURCE').AsString;

        if FieldByName('RDB$DESCRIPTION').IsNull then
          lDescription := NULL_FLAG
        else
          lDescription := FieldByName('RDB$DESCRIPTION').AsString;
                             //1   2   3   4   5   6
        ObjectList.Add(Format('%s%s%s%s%s%s%s%s%s%s%s',[lName,DEL,
                                                        lTriggerType, DEL,
                                                        lActive, DEL,
                                                        lSequence, DEL,
                                                        lSource,DEL,
                                                        lDescription]));
        Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.TriggerNameExist(const TriggerName: string): boolean;
const
  STriggerList =
    ' SELECT RDB$TRIGGER_NAME '+
    ' RDB$TRIGGER_SOURCE FROM RDB$TRIGGERS ' +
    ' WHERE UPPER(RDB$TRIGGER_NAME) = %s ';

begin
  Result := False;
  try
    try
      ExecuteQuery(Format(STriggerList, [AnsiquotedStr(TriggerName,#39)]));
      Result :=  not lQry.EOF;
    except
    end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;


function TDmN.GetUniqueConstList(var ObjectList: TStringList;  const TableName: string): integer;
const
   SConstList =
     'SELECT RDB$CONSTRAINT_NAME,RDB$DEFERRABLE,RDB$INITIALLY_DEFERRED,RDB$INDEX_NAME FROM RDB$RELATION_CONSTRAINTS ' +
     'WHERE RDB$RELATION_NAME = ''%s'' ' +
     'AND RDB$CONSTRAINT_TYPE = ''UNIQUE'' ' +
     'ORDER BY RDB$CONSTRAINT_NAME';

begin
  StartTransaction(ObjectList,
    Format(GetLangText('Name')+'%s'+GetLangText('Can Defer')+'%s'+GetLangText('Initially Deferred')+'%s'+GetLangText('Index'),[DEL,DEL,DEL]));
  try
    Result := ExecuteQuery(Format(SConstList, [TableName]));
    with lQry do
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s%s%s%s%s',[
          Fields[0].AsString,DEL,
          Fields[1].AsString,DEL,
          Fields[2].AsString,DEL,
          Fields[3].AsString]));
        Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetConstList(var ObjectList: TStringList;  const TableName: string): integer;
const
(*
  "RDB$CONSTRAINT_NAME"	CHAR(31) CHARACTER SET UNICODE_FSS,
  "RDB$CONSTRAINT_TYPE"	CHAR(11),
  "RDB$RELATION_NAME"	CHAR(31) CHARACTER SET UNICODE_FSS,
  "RDB$DEFERRABLE"	CHAR(3),
  "RDB$INITIALLY_DEFERRED"	CHAR(3),
  "RDB$INDEX_NAME"	CHAR(31) CHARACTER SET UNICODE_FSS
*)

   SConstList =
     'SELECT RDB$CONSTRAINT_NAME,'+
     ' RDB$CONSTRAINT_TYPE, '+
     ' RDB$DEFERRABLE,'+
     ' RDB$INITIALLY_DEFERRED,'+
     ' RDB$INDEX_NAME '+
     ' FROM RDB$RELATION_CONSTRAINTS ' +
     'WHERE RDB$RELATION_NAME = ''%s'' ' +
     ' AND RDB$CONSTRAINT_TYPE in (''UNIQUE'', ''PRIMARY KEY'' )' +
     'ORDER BY RDB$CONSTRAINT_NAME';

begin
  StartTransaction(ObjectList,
    //        0    1      2           3                  4
    Format(GetLangText('Name')+'%s'+GetLangText('Type')+'%s'+GetLangText('Can Defer')+'%s'+GetLangText('Initially Deferred')+'%s'+GetLangText('Index'),[DEL,DEL,DEL,DEL]));
  try
    Result := ExecuteQuery(Format(SConstList, [TableName]));
    with lQry do
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s%s%s%s%s%s%s',[
          Fields[0].AsString,DEL,
          Fields[1].AsString,DEL,
          Fields[2].AsString,DEL,
          Fields[3].AsString,DEL,
          Fields[4].AsString]));
        Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetConstraintData(var Data: TConstraintDef;   const TableName,ConstraintName: string): integer;
const
(*
  "RDB$CONSTRAINT_NAME"	CHAR(31) CHARACTER SET UNICODE_FSS,
  "RDB$CONSTRAINT_TYPE"	CHAR(11),
  "RDB$RELATION_NAME"	CHAR(31) CHARACTER SET UNICODE_FSS,
  "RDB$DEFERRABLE"	CHAR(3),
  "RDB$INITIALLY_DEFERRED"	CHAR(3),
  "RDB$INDEX_NAME"	CHAR(31) CHARACTER SET UNICODE_FSS
*)

   SConstList =
     'SELECT RDB$CONSTRAINT_NAME,'+
     ' RDB$CONSTRAINT_TYPE, '+
     ' RDB$DEFERRABLE,'+
     ' RDB$INITIALLY_DEFERRED,'+
     ' RDB$INDEX_NAME '+
     ' FROM RDB$RELATION_CONSTRAINTS ' +
     'WHERE RDB$RELATION_NAME = ''%s'' and ' +
     ' RDB$CONSTRAINT_NAME = ''%s'' ' +
     'ORDER BY RDB$CONSTRAINT_NAME';

begin
  try
    Result := ExecuteQuery(Format(SConstList, [TableName,ConstraintName]));
    with lQry do
      if not EOF then
      begin
        Data.Name :=              Fields[0].AsString;
        Data.cType :=             Fields[1].AsString;
        Data.Deferrable :=        Fields[2].AsString;
        Data.InitiallyDeferred := Fields[3].AsString;
        Data.IndexName :=         Fields[4].AsString;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;



function TDmN.GetViewList(var ObjectList: TStringList;  const SystemData: boolean): integer;
const
  SViewList =
    'SELECT DISTINCT A.RDB$RELATION_NAME,A.RDB$OWNER_NAME,A.RDB$DESCRIPTION ' +
    'FROM RDB$RELATIONS A, RDB$VIEW_RELATIONS B ' +
    'WHERE A.RDB$RELATION_NAME = B.RDB$VIEW_NAME ' +
    'ORDER BY B.RDB$VIEW_NAME ';
begin
  StartTransaction(ObjectList,
    Format(GetLangText('Name')+'%s'+GetLangText('Owner')+'%s'+GetLangText('Description'),[DEL,DEL]));
  try
    Result := ExecuteQuery(SViewList);
    with lQry do
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s%s%s',[Fields[0].AsString,DEL,
          Fields[1].AsString,DEL,Fields[2].AsString]));
        Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetDBFiles(var ObjectList: TStringList): integer;
const
  SGDBFiles =
    'SELECT RDB$FILE_NAME, RDB$FILE_LENGTH FROM RDB$FILES ' +
    'WHERE RDB$SHADOW_NUMBER < 1 OR RDB$SHADOW_NUMBER IS NULL ' +
    'ORDER BY RDB$FILE_SEQUENCE ASC';
var
  DidConnect: boolean;
  db1:String;
begin
  DidConnect := False;
  Result := FAILURE;
  try
    DidConnect := not DatabaseNode.DataModule.Connection.Connected;
    if DidConnect then
    begin
      if DatabaseNode.DatabaseFiles.Count > 0 then
        db1 := DatabaseNode.DatabaseFiles.Strings[0]
      else
        raise Exception.Create('Unable to connect, database filename is missing.');
      DatabaseNode.DataModule.Connection.Server := ServerNode.ServerName;
      DatabaseNode.DataModule.Connection.Database := db1;
      DatabaseNode.DataModule.Connection.Protocol := ServerNode.Server.Protocol;
      DatabaseNode.DataModule.Connection.Username := ServerNode.UserName;
      DatabaseNode.DataModule.Connection.Password := ServerNode.Password;
      DatabaseNode.DataModule.Connection.Connected := true;
      Application.ProcessMessages;
    end;

    Result := ExecuteQuery(SGDBFiles);
    with lQry do
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s',[Fields[0].AsString,DEL,Fields[1].AsString]));
        Next;
      end;
  finally
    lQry.Close;
    if DidConnect then
      lQry.IB_Connection.Connected := false;
  end;
end;

function TDmN.GetProcedureInfo(var ObjectList: TStringList;
                                  var Source: TStringList;
   const ProcName: String): integer;
const
  SProcedureInfo =
    'select p.rdb$procedure_source, p.rdb$description, pp.rdb$parameter_name, ' +
    '  pp.rdb$parameter_type, f.rdb$field_type, f.rdb$field_sub_type, ' +
    '  f.rdb$FIELD_length, f.rdb$field_scale, f.rdb$character_length, ' +
    '  %s ' +
    'from rdb$procedures p, ' +
    '  rdb$procedure_parameters pp,rdb$fields f where ' +
    '  p.rdb$procedure_name = ''%s'' and ' +
    '  pp.rdb$procedure_name = p.rdb$procedure_name and ' +
    '  f.rdb$field_name = pp.rdb$field_source ';

var
  lParamType,
  lFieldType: String;
begin
  StartTransaction(ObjectList,
    Format(GetLangText('Parameter')+'%s'+GetLangText('Type')+'%s'+GetLangText('Input/Output'),[DEL,DEL]));
  try
    IBOExtract.Database := Connection;
    IBOExtract.Transaction := Connection.DefaultTransaction;
    if IBOExtract.Database.Characteristics.dbODS_Version  >= ODS_VERSION10 then
      Result := ExecuteQuery(Format(SProcedureInfo, ['f.rdb$field_precision', ProcName]))
    else
      Result := ExecuteQuery(Format(SProcedureInfo, ['0 rdb$field_precision', ProcName]));
    with lQry do
    begin
      if not EOF then
        Source.Text := FieldByName('RDB$PROCEDURE_SOURCE').AsString;
      while not EOF do
      begin
        lFieldType := IBOExtract.GetFieldType (FieldByName('RDB$FIELD_TYPE').AsInteger,
                        FieldByName('RDB$FIELD_SUB_TYPE').AsInteger,
                        FieldByName('RDB$FIELD_SCALE').AsInteger,
                        FieldByName('RDB$CHARACTER_LENGTH').AsInteger,
                        FieldByName('RDB$FIELD_PRECISION').AsInteger,
                        0);
        lParamType := 'Output';
        if FieldByName('RDB$PARAMETER_TYPE').AsInteger = 0 then
          lParamType := 'Input';
        ObjectList.Add(Format('%s%s%s%s%s',[FieldByName('RDB$PARAMETER_NAME').AsString,
                                                DEL,
                                                lFieldType,
                                                DEL,
                                                lParamType]));
        Next;
      end;
    end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetFunctiondata(var ObjectList: TStringList;
                                 out ModuleName,
                                     EntryPoint,
                                     Returnval: String;
                                 
                                 const FuncName: String): integer;
const
  FunctionSQL =
    'SELECT RDB$RETURN_ARGUMENT, RDB$ARGUMENT_POSITION, RDB$MECHANISM, ' +
    '       RDB$MODULE_NAME, RDB$ENTRYPOINT, RDB$FIELD_TYPE, ' +
    '       RDB$FIELD_SCALE, RDB$FIELD_LENGTH, RDB$FIELD_SUB_TYPE, ' +
    '       %s ' +
    'FROM RDB$FUNCTIONS F, RDB$FUNCTION_ARGUMENTS FA ' +
    'WHERE FA.RDB$FUNCTION_NAME = ''%s'' AND ' +
    ' FA.RDB$FUNCTION_NAME = F.RDB$FUNCTION_NAME';

var
  lParamType,
  lFieldType: String;
begin
  StartTransaction(ObjectList,
    Format(GetLangText('Parameter')+'%s'+GetLangText('Type'),[DEL]));
  try
    IBOExtract.Database := Connection;
    IBOExtract.Transaction := Connection.DefaultTransaction;
    if IBOExtract.Database.Characteristics.dbODS_Version  >= ODS_VERSION10 then
      Result := ExecuteQuery(Format(FunctionSQL, ['RDB$FIELD_PRECISION', FuncName]))
    else
      Result := ExecuteQuery(Format(FunctionSQL, ['0 RDB$FIELD_PRECISION', FuncName]));
    with lQry do
      while not EOF do
      begin
        ModuleName := Trim(FieldByName('RDB$MODULE_NAME').AsString);
        EntryPoint := Trim(FieldByName('RDB$ENTRYPOINT').AsString);

        lFieldType := IBOExtract.GetFieldType (FieldByName('RDB$FIELD_TYPE').AsInteger,
                        FieldByName('RDB$FIELD_SUB_TYPE').AsInteger,
                        FieldByName('RDB$FIELD_SCALE').AsInteger,
                        FieldByName('RDB$FIELD_LENGTH').AsInteger,
                        FieldByName('RDB$FIELD_PRECISION').AsInteger,
                        0);

        lParamType := '';
        if Abs(FieldByName('RDB$MECHANISM').AsInteger) = 1 then
          lParamType := 'BY REFERENCE';

        if FieldByName('RDB$MECHANISM').AsInteger < 0 then
          lParamType := Format ('%s FREE IT', [lParamType]);

        if (FieldByName('RDB$RETURN_ARGUMENT').AsInteger <>
            FieldByName('RDB$ARGUMENT_POSITION').AsInteger) then
            ObjectList.Add(Format('Parameter %d%s%s',
                           [FieldByName('RDB$ARGUMENT_POSITION').AsInteger,
                           DEL,
                           lFieldType]))
        else
          ReturnVal := Format ('RETURNS %s %s', [lFieldType, lParamType]);
        Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetFilterData(var ObjectList: TStringList;
   const FuncName: String): integer;
const
  SFilterData = 'select * from rdb$filters order by rdb$function_name';
begin
  
  try
    if FuncName <> '' then
      Result := ExecuteQuery(SFilterData+ ' where rdb$function_name = '+AnsiQuotedStr(FuncName,''''))
    else
      Result := ExecuteQuery(SFilterData);
    with lQry do
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s%s%s%s%s%s',[FieldByName('RDB$MODULE_NAME').AsString,
                       DEL,
                       FieldByName('RDB$ENTRYPOINT').AsString,
                       DEL,
                       FieldByName('RDB$INPUT_SUB_TYPE').AsString,
                       DEL,
                       FieldByName('RDB$OUTPUT_SUB_TYPE').AsString,
                       DEL,
                       FieldByName('RDB$DESCRIPTION').AsString]));
        Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetRoleData(var ObjectList: TStringList;
   const RoleName: String): integer;
const
  SRoleData =
    'select rdb$role_name, rdb$owner_name, rdb$user from rdb$roles, rdb$user_privileges ' +
    'where rdb$relation_name=rdb$role_name ' +
    'order by rdb$role_name ';
begin
  StartTransaction(ObjectList,
    Format(GetLangText('Owner')+'%s'+GetLangText('Member'),[DEL]));
  try
    Result := ExecuteQuery(SRoleData);
    with lQry do
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s',[FieldByName('RDB$OWNER_NAME').AsString,
                       DEL,
                       FieldByName('RDB$USER').AsString]));
        Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetExceptionData(var ObjectList: TStringList;
   const ExceptionName: String): integer;
const
  SExceptionData =  'select * from rdb$exceptions order by rdb$exception_number';
begin
  
  try
    Result := ExecuteQuery(SExceptionData);
    with lQry do
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s%s%s',[FieldByName('RDB$EXCEPTION_NUMBER').AsString,
                       DEL,
                       FieldByName('RDB$MESSAGE').AsString,
                       DEL,
                       FieldByName('RDB$DESCRIPTION').AsString]));
        Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetGeneratorData(var ObjectList: TStringList;
   const Showsystem: boolean): integer;
const
  SGeneratorDataSystem =
    'select * from rdb$Generators ' +
    'order by rdb$generator_name';

  SGeneratorData =
    'select * from rdb$Generators ' +
    'Where RDB$SYSTEM_FLAG <> 1 OR RDB$SYSTEM_FLAG is NULL ' +
    'order by rdb$generator_name';

begin
  
  try
    if ShowSystem then
      Result := ExecuteQuery(SGeneratorDataSystem)
    else
      Result := ExecuteQuery(SGeneratorData);
    with lQry do
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s',[FieldByName('RDB$GENERATOR_ID').AsString,
                       DEL,
                       GetNextGenValue(FieldByName('RDB$GENERATOR_NAME').AsString)]));
        Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetNextGenValue(const GenName: String): String;
var
  v:int64;
begin
  GetGeneratorValue(Connection,Connection.DefaultTransaction,GenName,0,v);
  Result := IntToStr(V);
end;

function TDmN.GetViewData(var ObjectList: TStringList;
   const ViewName: String): integer;
const
  SViewData =
    'select rdb$description, rdb$view_source from rdb$relations ' +
    'where rdb$relation_name in (select rdb$view_name ' +
    '  from rdb$view_relations) ' +
    'ORDER BY RDB$RELATION_ID';
begin
  
  try
    Result := ExecuteQuery(SViewData);
    with lQry do
      while not EOF do
      begin
        ObjectList.Add(Format('%s%s%s',[FieldByName('RDB$DESCRIPTION').AsString,
                       DEL,
                       FieldByName('RDB$VIEW_SOURCE').AsString]));
        Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetDomainData(var ObjectList: TStringList;
   const ShowSystem: boolean): integer;
const
  SDomainData =
    'SELECT * FROM RDB$FIELDS ' +
    'WHERE RDB$FIELD_NAME NOT STARTING WITH ''RDB$'' ' +
    '  AND RDB$SYSTEM_FLAG <> 1 OR RDB$SYSTEM_FLAG is NULL ' +
    'ORDER BY RDB$FIELD_NAME';

  SDomainDataSystem =
    'SELECT * FROM RDB$FIELDS ' +
    //'WHERE (RDB$FIELD_NAME NOT STARTING WITH ''RDB$'' ' +
    //'   AND RDB$SYSTEM_FLAG <> 1 OR RDB$SYSTEM_FLAG is NULL) ' +
    //'   or rdb$system_flag = 1 ' +
    'ORDER BY RDB$FIELD_NAME';
var
  NullFlg,
  Charset,
  Collation,
  Default,
  FieldType: String;
  len: integer;

begin
  
  try
    Screen.Cursor := crHourGlass;
    IBOExtract.Database := Connection;
    IBOExtract.Transaction := Connection.DefaultTransaction;
    if not ShowSystem then
      Result := ExecuteQuery(SDomainData)
    else
      Result := ExecuteQuery(SDomainDataSystem);
    with lQry do
      while not EOF do
      begin
        FieldType := '';
        if FieldByName('RDB$DIMENSIONS').AsInteger > 0 then
          FieldType := IBOExtract.GetArrayField(Trim(FieldByName('RDB$FIELD_NAME').AsString));

        if Showsystem then
          len := FieldByName('RDB$FIELD_LENGTH').AsInteger
        else
          len := FieldByName('RDB$CHARACTER_LENGTH').AsInteger;

        if IBOExtract.Database.Characteristics.dbODS_Version  >= ODS_VERSION10 then
          FieldType := Format('%s %s',[FieldType , IBOExtract.GetFieldType (FieldByName('RDB$FIELD_TYPE').AsInteger,
                                                    FieldByName('RDB$FIELD_SUB_TYPE').AsInteger,
                                                    FieldByName('RDB$FIELD_SCALE').AsInteger,
                                                    Len,
                                                    FieldByName('RDB$FIELD_PRECISION').AsInteger,
                                                    FieldByName('RDB$SEGMENT_LENGTH').AsInteger)])
        else
          FieldType := Format('%s %s',[FieldType , IBOExtract.GetFieldType (FieldByName('RDB$FIELD_TYPE').AsInteger,
                                                    FieldByName('RDB$FIELD_SUB_TYPE').AsInteger,
                                                    FieldByName('RDB$FIELD_SCALE').AsInteger,
                                                    Len,
                                                    0,
                                                    FieldByName('RDB$SEGMENT_LENGTH').AsInteger)]);

        Charset := '';
        if not (FieldByName('RDB$CHARACTER_SET_ID').IsNull) and
           (FieldByName('RDB$CHARACTER_SET_ID').AsInteger <> 0) then
        begin
          Charset := IBOExtract.GetCharacterSets(FieldByName('RDB$CHARACTER_SET_ID').AsInteger, 0, false);
          CharSet := Copy(CharSet, Pos(' CHARACTER SET ', CharSet) + 15, Length(CharSet));
        end;

        Collation := '';
        if (not FieldByName('RDB$COLLATION_ID').IsNull) and
           (FieldByName('RDB$COLLATION_ID').AsInteger <> 0) then
        begin
          Collation := IBOExtract.GetCharacterSets(FieldByName('RDB$CHARACTER_SET_ID').AsInteger,
                         FieldByName('RDB$COLLATION_ID').AsInteger, false);
          Collation := Copy(Collation, Pos(' COLLATE ', Collation) + 9, Length(Collation));
        end;

        Default := '';
        Default := FieldByName('RDB$DEFAULT_SOURCE').AsString;
        Default := Copy(Default, Pos('DEFAULT ', Default) + 8, Length(Default));


        NullFlg := GetLangText('No');
        if FieldByName('RDB$NULL_FLAG').IsNull then
          NullFlg := GetLangText('Yes');

        ObjectList.Add (Format('%s%s%s%s%s%s%s%s%s%s%s%s', [FieldType, DEL,
                                                    Charset, DEL,
                                                    Collation, DEL,
                                                    Default,
                                                    DEL,
                                                    NullFlg,
                                                    DEL,
                                                    FieldByName('RDB$VALIDATION_SOURCE').AsString,
                                                    DEL,
                                                    FieldByName('RDB$DESCRIPTION').AsString]));
        Next;
      end;
  finally
    Screen.Cursor := crDefault;
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetProcedureSource(var ObjectList: TStringList;
   const ProcName: String): integer;
const
  SProcedureSource =
    'select rdb$procedure_source ' +
    'from rdb$procedures ' +
    'where rdb$procedure_name = ''%s'' ';

begin

  try
    Result := ExecuteQuery(Format(SProcedureSource, [ProcName]));
    with lQry do
        if not EOF then
          ObjectList.Add (FieldByName('RDB$PROCEDURE_SOURCE').AsString);
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetProcedureSource(const ProcName: String): String;
const
  SProcedureSource =
    'select rdb$procedure_source ' +
    'from rdb$procedures ' +
    'where rdb$procedure_name = :ProcName ';

begin
  Result := '';
  try
    ExecuteQuery(SProcedureSource,[ProcName]);
    with lQry do
        if not EOF then
          Result := FieldByName('RDB$PROCEDURE_SOURCE').AsString;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;



function TDmN.GetDBCharSetName(CharSetID: Integer): String;
var
  CharSet : String;

begin
  try
    lQry.SQL.Text:=('select rdb$character_set_name from rdb$character_sets where rdb$character_set_id = ' + IntToStr(CharSetID));
    lQry.Open;
    if not (lQry.EOF and lQry.BOF) then
      CharSet := lQry.FieldByName('rdb$character_set_name').AsString
    else
      CharSet := '';
    lQry.Close;

    if (CharSet <> '') and (AnsiUpperCase(CharSet) <> 'NONE') then
    begin
      Result := CharSet;
    end
    else
      Result := '';

  finally
  end;
end;

procedure TDmN.GetCollationNames(S: TStrings);
begin
  S.Clear;
  S.Add('');
  try
    lQry.SQL.Text:=('SELECT DISTINCT RDB$COLLATION_NAME FROM RDB$COLLATIONS ORDER BY RDB$COLLATION_NAME ASC');
    lQry.Open;
    while not lQry.EOF do
    begin
      S.Add(lQry.FieldByName('RDB$COLLATION_NAME').AsString);
      lQry.Next;
    end;
    lQry.Close;
  finally
  end;
end;

function TDmN.GetDBCollationName(CollationID : Integer; CharSetID: Integer): String;
var
  CharSet : String;
begin
  try
    lQry.SQL.Text:='select rdb$collation_name from rdb$collations where rdb$character_set_id = ' + IntToStr(CharSetID) + ' and rdb$collation_id = ' + IntToStr(CollationID);
    lQry.Open;
    if not (lQry.EOF and lQry.BOF) then
      CharSet := lQry.FieldByName('rdb$collation_name').AsString
    else
      CharSet := '';
    lQry.Close;

    if (CharSet <> '') and (AnsiUpperCase(CharSet) <> 'NONE') then
    begin
      Result := CharSet;
    end
    else
      Result := '';
  finally
  end;
end;


procedure TDmN.StartTransaction(ObjectList: TStringList;
    const ObjectStr : String);
begin
//  if Transaction.InTransaction then
//    Transaction.Commit;
  if Assigned(ObjectList) then
    ObjectList.Add(ObjectStr);
end;

function TDmN.ExecuteQuery(const SSQL: String): Integer;
begin
  lQry.SQL.Clear;
  lQry.SQL.Text :=(SSQL);
  try
    lQry.Prepare;
    lQry.Open;
    if lQry.Eof then
      result := EMPTY
    else
      result := SUCCESS;
  except
    on E:EIB_Error do
    begin
      DisplayMsg(ERR_GET_EXCEPTIONS, E.Message);
      result := FAILURE;
    end;
  end;
end;

function TDmN.ExecuteQuery(const SSQL: String; const params: array of variant): Integer;
var n:integer;
begin
  lQry.SQL.Clear;
  lQry.SQL.Text :=(SSQL);
  try
    lQry.Prepare;
    for n:=Low(params) to High(params) do
      lQry.params[n].Value := params[n];
    lQry.Open;
    if lQry.Eof then
      result := EMPTY
    else
      result := SUCCESS;
  except
    on E:EIB_Error do
    begin
      DisplayMsg(ERR_GET_EXCEPTIONS, E.Message);
      result := FAILURE;
    end;
  end;
end;

function TDmN.PrepareQuery(const SSQL: String): Integer;
begin
  lQry.SQL.Clear;
  lQry.SQL.Text :=(SSQL);
  try
    lQry.Prepare;
    result := SUCCESS;
  except
    on E:EIB_Error do
    begin
      DisplayMsg(ERR_GET_EXCEPTIONS, E.Message);
      result := FAILURE;
    end;
  end;
end;

function TDmN.ExecuteQuery2(const SSQL: String): Integer;
begin
  lQry2.SQL.Clear;
  lQry2.SQL.Text :=(SSQL);
  try
    lQry2.Prepare;
    lQry2.Open;
    if lQry2.Eof then
      result := EMPTY
    else
      result := SUCCESS;
  except
    on E:EIB_Error do
    begin
      DisplayMsg(ERR_GET_EXCEPTIONS, E.Message);
      result := FAILURE;
    end;
  end;
end;

procedure TDmN.GetCharSetNames(S: TStrings);

begin
  S.Clear;
  S.Add('');
  try
    lQry.SQL.Text:='select distinct rdb$character_set_name from rdb$character_sets order by rdb$character_set_name asc';
    lQry.Open;
    while not lQry.EOF do
    begin
      S.Add(lQry.FieldByName('rdb$character_set_name').AsString);
      lQry.Next;
    end;
    lQry.Close;
  finally
  end;
end;


procedure TDmN.ObjectDescriptionUpdate(NodeType:integer; ObjectName, Description:String);
var
  Q:TIBOQuery;
  Table, fld, qry: String;
begin
  case NodeType of
    NODE_DOMAINS: 
    begin 
      table := 'RDB$FIELDS';
      fld := 'RDB$FIELD_NAME';
    end;
    NODE_TABLES,
    NODE_VIEWS:
    begin 
      table := 'RDB$RELATIONS'; 
      fld := 'RDB$RELATION_NAME'; 
    end;
    NODE_PROCEDURES:
    begin 
      table := 'RDB$PROCEDURES'; 
      fld := 'RDB$PROCEDURE_NAME'; 
    end;
    NODE_FUNCTIONS: 
    begin 
      table := 'RDB$FUNCTIONS'; 
      fld := 'RDB$FUNCTION_NAME'; 
    end;
    NODE_EXCEPTIONS: 
    begin 
      table := 'RDB$EXCEPTIONS'; 
      fld := 'RDB$EXCEPTION_NAME'; 
    end;
    NODE_BLOB_FILTERS: 
    begin
      table := 'RDB$FILTERS';
      fld := 'RDB$FUNCTION_NAME';
    end;
    NODE_TRIGGERS,
    NODE_TRIGGER:
    begin
      table := 'RDB$TRIGGERS';
      fld := 'RDB$TRIGGER_NAME';
    end;
    NODE_INDEXES,
    NODE_INDEX:
    begin
      table := 'RDB$INDICES';
      fld := 'RDB$INDEX_NAME';
    end
    else
    begin
      table := '';
      fld := '';
    end;
  end;
  if Table <> '' then
  begin
    Q := TIBOQuery.Create(nil);
    try
      begin
        Q.IB_Connection := Connection;
        Q.IB_Transaction := DDLTransaction;
        qry := Format('UPDATE %s SET RDB$DESCRIPTION = :description',[table]);
        qry := Format('%s WHERE %s = ''%s''', [qry, fld, ObjectName]);
        Q.SQL.Text := qry;
        Q.Prepare;
        Q.Params[0].AsString := Description;
        Q.ExecSQL;
        Q.Close;
      end;
    finally
      Q.Free;
    end;
  end;
end;

function TDmN.GetObjectDescription(NodeType:integer; ObjectName:String):String;
var
  q:TIBOQuery;
  Table, fld, qry: String;
begin
  Result := '';
  case NodeType of
    NODE_DOMAINS: 
    begin 
      table := 'RDB$FIELDS';
      fld := 'RDB$FIELD_NAME';
    end;
    NODE_TABLES,
    NODE_VIEWS:
    begin 
      table := 'RDB$RELATIONS'; 
      fld := 'RDB$RELATION_NAME'; 
    end;
    NODE_PROCEDURES:
    begin 
      table := 'RDB$PROCEDURES';
      fld := 'RDB$PROCEDURE_NAME'; 
    end;
    NODE_FUNCTIONS: 
    begin 
      table := 'RDB$FUNCTIONS'; 
      fld := 'RDB$FUNCTION_NAME'; 
    end;
    NODE_EXCEPTIONS: 
    begin 
      table := 'RDB$EXCEPTIONS'; 
      fld := 'RDB$EXCEPTION_NAME'; 
    end;
    NODE_BLOB_FILTERS:
    begin 
      table := 'RDB$FILTERS'; 
      fld := 'RDB$FUNCTION_NAME'; 
    end;
    NODE_TRIGGERS,
    NODE_TRIGGER:
    begin
      table := 'RDB$TRIGGERS';
      fld := 'RDB$TRIGGER_NAME';
    end;
    NODE_INDEXES,
    NODE_INDEX:
    begin
      table := 'RDB$INDICES';
      fld := 'RDB$INDEX_NAME';
    end;
    else
    begin
      table := '';
      fld := '';
    end;
  end;
  if Table <> '' then
  begin
    Q := TIBOQuery.Create(nil);
    try
        Q.IB_Connection := Connection;
        Q.IB_Transaction := DDLTransaction;
        qry := Format('Select RDB$DESCRIPTION  From %s WHERE %s = :obj_name', [table, fld]);
        Q.SQL.Text := qry;
        Q.Prepare;
        Q.Params[0].AsString := ObjectName;
        Q.Open;
        Result := Q.Fields[0].AsString;
        Q.Close;
    finally
      Q.Free;
    end;
  end;
end;

procedure TDmN.SetDatabaseNode(const Value: TibcDatabaseNode);
begin
  FDatabaseNode := Value;
end;

procedure TDmN.SetServerNode(const Value: TibcServerNode);
begin
  FServerNode := Value;
end;

function TDmN.DropObject(NodeType:integer; NodeName:String; var error:String; DoPrompt:boolean = true):boolean;
var
  Qry, Obj: String;
begin
    Result := False;
    error := '';
    Qry := 'DROP %s %s';
    case NodeType of
      NODE_DOMAINS: Obj := 'DOMAIN';
      NODE_TABLES: Obj := 'TABLE';
      NODE_VIEWS: Obj := 'VIEW';
      NODE_PROCEDURES: Obj := 'PROCEDURE';
      NODE_FUNCTIONS: Obj := 'EXTERNAL FUNCTION';
      NODE_EXCEPTIONS: Obj := 'EXCEPTION';
      NODE_BLOB_FILTERS: Obj := 'FILTER';
      NODE_ROLES: Obj := 'ROLE';
      NODE_GENERATORS:
      if Pos('FIREBIRD',UpperCase(Connection.Characteristics.dbVersion))=0 then
      begin
        Qry := 'DELETE FROM RDB$GENERATORS WHERE RDB$GENERATOR_NAME = %s';
        Obj := 'GENERATOR';
      end
      else
      begin
        Obj := 'GENERATOR';
      end
    end;

    if (not DoPrompt) or (LMMessageDlg(124,[NodeName], mtConfirmation, [mbYes, mbNo], 0) = mrYes)
    then
    begin
      OpenWindows.CloseObjectWindow(self, NodeType, NodeName);
      try
        with lQry do
        begin
          IB_Connection := Connection;
          IB_Transaction := DDLTransaction;
          if Obj = 'GENERATOR' then
          begin
            if Pos('FIREBIRD',UpperCase(Connection.Characteristics.dbVersion))=0 then
               Qry := Format(Qry, [AnsiQuotedStr(NodeName,'''')])
            else
               Qry := Format(Qry, [Obj, QuoteIdentifier(Connection.SQLDialect,NodeName)]);
          end
          else
            Qry := Format(Qry, [Obj, QuoteIdentifier(Connection.SQLDialect,NodeName)]);
          Sql.Text:= Qry;
          Prepare;
          Execute;
          Close;
          Result := True;
        end;
      except
        on e:Exception do
        begin
          DisplayMsg(ERR_DDL_EXECUTION,e.Message);
          RollbackDDL;
          Result := False;
        end;
      end;
    end;
    if Result then
    begin
      frmMain.EventObjectDrop(Self, NodeType, NodeName);
    end;
end;


procedure TDmN.GetDependencies(tvDependencies,tvDependents:TTreeView; const ObjName: String; const ObjType: Integer);
var
  RelName, SqlStr: String;
  Query: TIB_Cursor;
  ParentNode, ChildNode: TTreeNode;

begin
  ParentNode := nil;

  case ObjType of
    NODE_DOMAINS:
    begin
      SQLStr := 'select rdb$relation_name, rdb$field_name';
      SQLStr := Format('%s from rdb$relation_fields where', [SqlStr]);
      SQLStr := Format('%s rdb$field_source = ''%s''', [SQLStr, ObjName]);
      SQLStr := Format('%s order by rdb$relation_name, rdb$field_name', [SQLStr]);
    end;
    else
    begin
      SQLStr := 'select rdb$dependent_name, rdb$depended_on_name,';
      SQLStr := Format('%s rdb$field_name, rdb$dependent_type, rdb$depended_on_type', [SQLStr]);
      SQLStr := Format('%s from rdb$dependencies where', [SQLStr]);
      SQLStr := Format('%s rdb$dependent_name= ''%s''', [SQLStr, ObjName]);
      SQLStr := Format('%s order by rdb$field_name, rdb$depended_on_name', [SQLStr]);
    end;
  end;

  Query := TIB_Cursor.Create(Nil);
  try
    try
      Query.IB_Transaction := DDLTransaction;
      Query.IB_Connection := Connection;

        Query.SQL.Text := (SQLStr);
        Query.Prepare;
        Query.Execute;
        tvDependencies.Items.BeginUpdate;
        tvDependencies.Items.Clear;
        if ObjType = NODE_DOMAINS then
        begin
          while not Query.EOF do
          begin
            if AnsiCompareText(RelName, TrimRight(Query.Fields[0].AsString)) <> 0 then
            begin
              RelName := TrimRight(Query.Fields[0].AsString);
              ParentNode := tvDependencies.Items.Add(nil, RelName);
              ParentNode.ImageIndex := NODE_TABLES_IMG;
              ParentNode.SelectedIndex := NODE_TABLES_IMG;
            end;
            ChildNode := tvDependencies.Items.AddChild(ParentNode, TrimRight(Query.Fields[1].AsString));
            ChildNode.ImageIndex := NODE_COLUMNS_IMG;
            ChildNode.SelectedIndex := NODE_COLUMNS_IMG;
            ParentNode.Expand(true);
            Query.Next;
          end;
        end
        else
        begin
          while not Query.EOF do
          begin
            if AnsiCompareText(RelName, TrimRight(Query.Fields[1].AsString)) <> 0 then
            begin
              RelName := TrimRight(Query.Fields[1].AsString);
              if (not (Query.Fields[2].IsNull)) or
                 (Query.Fields[4].AsInteger <> 0) then
              begin
                ParentNode := tvDependencies.Items.Add (nil, RelName);
                ParentNode.ImageIndex := GetImageIndex (Query.Fields[4].AsInteger);
                ParentNode.SelectedIndex := ParentNode.ImageIndex;
              end;
            end;
            if not (Query.Fields[2].IsNull) then
            begin
              ChildNode := tvDependencies.Items.AddChild(ParentNode, TrimRight(Query.Fields[2].AsString));
              ChildNode.ImageIndex := NODE_COLUMNS_IMG;
              ChildNode.SelectedIndex := NODE_COLUMNS_IMG;
              ParentNode.Expand (true);
            end;
            Query.Next;
          end;
          Query.Close;

          { Now get the objects which depend on the current object }
          SQLStr := 'select rdb$dependent_name, rdb$depended_on_name,';
          SQLStr := Format('%s rdb$field_name, rdb$dependent_type, rdb$depended_on_type', [SQLStr]);
          SQLStr := Format('%s from rdb$dependencies where', [SQLStr]);
          SQLStr := Format('%s rdb$depended_on_name= ''%s''', [SQLStr, ObjName]);
          SQLStr := Format('%s order by rdb$dependent_name, rdb$field_name, rdb$depended_on_name', [SQLStr]);
          Query.Sql.Clear;
          Query.SQL.Text :=(SQLStr);
          Query.Prepare;
          Query.Execute;

          tvDependents.Items.BeginUpdate;
          tvDependents.Items.Clear;
          RelName := '';
          while not Query.EOF do
          begin
            if AnsiCompareText (RelName, TrimRight(Query.Fields[0].AsString)) <> 0 then
            begin
              RelName := TrimRight(Query.Fields[0].AsString);
              if (not (Query.Fields[2].IsNull)) or
                 (Query.Fields[3].AsInteger <> 0) then
              begin
                ParentNode := tvDependents.Items.Add (nil, RelName);
                ParentNode.ImageIndex := GetImageIndex (Query.Fields[3].AsInteger);
                ParentNode.SelectedIndex := ParentNode.ImageIndex;
              end;
            end;
            if not (Query.Fields[2].IsNull) then
            begin
              ChildNode := tvDependents.Items.AddChild(ParentNode, TrimRight(Query.Fields[2].AsString));
              ChildNode.ImageIndex := NODE_COLUMNS_IMG;
              ChildNode.SelectedIndex := NODE_COLUMNS_IMG;
              ParentNode.Expand (true);
            end;
            Query.Next;
          end;
        end;
        tvDependents.Items.Endupdate;
        tvDependencies.Items.EndUpdate;
        Query.Close;
    except on E: Exception do
      begin
        LMMessageDlg(E.Message,mtError,[mbok],0);
        if Assigned (Query) then
          Query.Free;
      end;
    end;
  finally
    Query.Free;
  end;
end;


const

  USR_COL = 0;
  SEL_COL = 1;
  DEL_COL = 2;
  INS_COL = 3;
  UPD_COL = 4;
  REF_COL = 5;
  EXE_COL = 6;
  MEM_COL = 7;
  GRANT_COL = 8;

procedure TDmN.GetPermissions(lvPermissions:TListView; const ObjName: String; const ObjType: Integer);
var
  GrantPermission: boolean;
  RelName, SqlStr: String;
  Perm: String;
  Query: TIB_Cursor;
  lCnt, img: integer;
  ListItem: TListItem;

begin
  Query := nil;
  ListItem := nil;
  SQLStr := 'select rdb$user, rdb$privilege, rdb$grant_option,';
  SQLStr := Format('%s rdb$relation_name, rdb$user_type from rdb$user_privileges where', [SqlStr]);
  SQLStr := Format('%s rdb$relation_name = ''%s''', [SQLStr, ObjName]);
  SQLStr := Format('%s order by rdb$user, rdb$privilege', [SQLStr]);

  try
    Query := TIB_Cursor.Create (nil);
    Query.IB_Transaction := DDLTransaction;
    Query.IB_Connection := Connection;
    lvPermissions.Items.Clear;
    with Query do
    begin
      SQL.Text := (SQLStr);
      Prepare;
      Execute;
      while not EOF do
      begin
        if AnsiCompareText (RelName, TrimRight(Fields[0].AsString)) <> 0 then
        begin
          RelName := TrimRight(Fields[0].AsString);
          ListItem := lvPermissions.Items.Add;
          ListItem.Caption := RelName;

          case Fields[4].AsInteger of
            obj_relation: img := NODE_TABLES_IMG;
            obj_view: img := NODE_VIEWS_IMG;
            obj_trigger: img := NODE_TRIGGERS_IMG;
            obj_computed: img := NODE_UNK_IMG;
            obj_validation: img := NODE_UNK_IMG;
            obj_procedure: img := NODE_PROCEDURES_IMG;
            obj_expression_index: img := NODE_UNK;
            obj_exception: img := NODE_EXCEPTIONS_IMG;
            obj_user: img := 25;
            obj_field: img := NODE_COLUMNS_IMG;
            obj_index: img := NODE_INDEXES_IMG;
            obj_user_group: img := 71; //NODE_USERS_IMG;
            obj_sql_role: img := NODE_ROLES_IMG
            else
              img := NODE_UNK_IMG;
          end;
          ListItem.ImageIndex := img;

          for lCnt := 1 to lvPermissions.Columns.Count-1 do
            ListItem.SubItems.Add('');
        end;

        GrantPermission := false;
        if Fields[2].AsInteger = 1 then
          GrantPermission := true;

        Perm := Fields[1].AsString;
        if Perm<>'' then 
        case Perm[1] of
          'D', 'd':
          begin
            ListItem.SubItems[DEL_COL-1] := GetLangText('Y');
            if GrantPermission then
              ListItem.SubItemImages[DEL_COL-1] := IMG_GRANT_OPT
            else
              ListItem.SubItemImages[DEL_COL-1] := IMG_WHITE;
          end;
          'I', 'i':
          begin
            ListItem.SubItems[INS_COL-1] := GetLangText('Y');
            if GrantPermission then
              ListItem.SubItemImages[INS_COL-1] := IMG_GRANT_OPT
            else
              ListItem.SubItemImages[INS_COL-1] := IMG_WHITE;
          end;
          'M', 'm':
          begin
            ListItem.SubItems[MEM_COL-1] := GetLangText('Y');
            if GrantPermission then
              ListItem.SubItemImages[MEM_COL-1] := IMG_GRANT_OPT
            else
              ListItem.SubItemImages[MEM_COL-1] := IMG_WHITE;
          end;
          'R', 'r':
          begin
            ListItem.SubItems[REF_COL-1] := GetLangText('Y');
            if GrantPermission then
              ListItem.SubItemImages[REF_COL-1] := IMG_GRANT_OPT
            else
              ListItem.SubItemImages[REF_COL-1] := IMG_WHITE;
          end;
          'S', 's':
          begin
            ListItem.SubItems[SEL_COL-1] := GetLangText('Y');
            if GrantPermission then
              ListItem.SubItemImages[SEL_COL-1] := IMG_GRANT_OPT
            else
              ListItem.SubItemImages[SEL_COL-1] := IMG_WHITE;
          end;
          'U', 'u':
          begin
            ListItem.SubItems[UPD_COL-1] := GetLangText('Y');
            if GrantPermission then
              ListItem.SubItemImages[UPD_COL-1] := IMG_GRANT_OPT
            else
              ListItem.SubItemImages[UPD_COL-1] := IMG_WHITE;
          end;
          'X', 'x':
          begin
            ListItem.SubItems[EXE_COL-1] := GetLangText('Y');
            if GrantPermission then
              ListItem.SubItemImages[EXE_COL-1] := IMG_GRANT_OPT
            else
              ListItem.SubItemImages[EXE_COL-1] := IMG_WHITE;
          end;
        end;
        Next;
      end;
      Close;
      Free;
    end;
  except on E: Exception do
    begin
      lvPermissions.Items.EndUpdate;
      LMMessageDlg(E.Message,mtError,[mbok],0);
      if Assigned (Query) then
        Query.Free;
    end;
  end;
end;

function TDmN.ExecuteDDL(DDL:String; var error:string):boolean;
begin
  result := false;
  error:='';
  try
    IB_DSQL.IB_Connection := Connection;
    IB_DSQL.IB_Transaction := DDLTransaction;
    IB_DSQL.SQL.Text := DDL;
    IB_DSQL.Execute;
    result := true;
  except
    on e:Exception do
    begin
      error := E.Message;
    end;
  end;
end;

function TDmN.ExecuteDDLScript(DDL:String; var error:string):boolean;
begin
  result := false;
  error:='';
  try
    IB_Script1.IB_Connection := Connection;
    IB_Script1.IB_Transaction := DDLTransaction;
    IB_Script1.SQL.Text := DDL;
    IB_Script1.Execute;
    result := true;
  except
    on e:Exception do
    begin
      error := E.Message;
    end;
  end;
end;


function TDmN.ExecuteDDL(DDL:TStrings; var error:string):boolean;
var n:integer;
begin
  result := false;
  error:='';
  if not Assigned(DDL) then exit;
  IB_DSQL.IB_Connection := Connection;
  IB_DSQL.IB_Transaction := DDLTransaction;
  for n:=0 to ddl.Count-1 do
  try
    IB_DSQL.SQL.Text := DDL[n];
    IB_DSQL.Execute;
  except
    on e:Exception do
    begin
      error := E.Message;
      exit;
    end;
  end;
  result := true;
end;


function TDmN.ExecuteSql(const SSQL: String; var Error:string): boolean;
begin
  result := false;
  error:='';
  try
    Query.IB_Connection := Connection;
    Query.IB_Transaction := Transaction;
    Query.SQL.Text := SSQL;
    Query.Prepare;
    Query.Execute;
    result := true;
  except
    on e:Exception do
    begin
      error := E.Message;
    end;
  end;
end;




{$IFDEF LINUX}
constructor TibcDatabaseNode.Create(AOwner: TComponent; const NodeID: QListViewItemH;
  const NodeName: string; const NodeType: word; const DatabaseProps: TibcDatabaseProps;
  const Password: string; NewDatabase: TIB_Connection);
{$ENDIF}
{$IFDEF MSWINDOWS}

constructor TibcDatabaseNode.Create(AOwner: TComponent; const NodeID: HTreeItem; const NodeName: string;
  const NodeType: word; const DatabaseProps: TibcDatabaseProps; const Password: string);

{$ENDIF}

var
  tmp: string;
begin
  inherited Create(AOwner, NodeID, NodeName, NodeType);

  FDatabaseFiles := TStringList.Create;
  fDmN := TDmn.Create(Self);
  fDmN.DatabaseNode := Self;

  with fDmN.DataSet do
  begin
    IB_Connection := fDmN.Connection;
    IB_Transaction := fDmN.Transaction;
    ObjectView := true;
    SparseArrays := true;
  end;

  fDmN.DataSource.DataSet := fDmN.DataSet;
  fDmN.DataSource.AutoEdit := false;

  fDmN.Query.AutoFetchFirst := true;
  FSQLBuffer := TStringList.Create;
  FSQLBufferPos := 0;
  FDatabaseFiles.Text := DatabaseProps.DatabaseFiles;
  fDmN.Connection.LoginPrompt := false;
  if FDatabaseFiles.Count > 0  then
  begin
    { Force the database to have a full path }
    if ExtractFilePath(FDatabaseFiles.Strings[0]) = '' then
    begin
      tmp := ExtractFilePath(Application.ExeName);
      fDmN.Connection.DatabaseName := tmp + FDatabaseFiles.Strings[0];
    end
    else
      fDmN.Connection.DatabaseName := FDatabaseFiles.Strings[0];
  end;

  FUserName := DatabaseProps.UserName;
  FCharacterSet := DatabaseProps.CharacterSet;
  fDmN.Connection.CharSet := FCharacterSet;
  FRole := DatabaseProps.Role;
  FCaseSensitiveRole := DatabaseProps.CaseSensitiveRole;
  LastAccessed := DatabaseProps.Accessed;
  Created := DatabaseProps.Created;
//lm  fPatchBlankCharset:= DatabaseProps.PatchBlankCharset;
end;

destructor TibcDatabaseNode.Destroy();
begin
  FSQLBuffer.Free;
  FDatabaseFiles.Free;
  fDmn.Free;
  inherited Destroy();
end;

procedure TDmN.CommitDDL;
begin
  DDLTransaction.CommitRetaining;
end;

procedure TDmN.RollbackDDL;
begin
  DDLTransaction.RollbackRetaining;
end;

function TDmN.GetIndexData(var Data: TIndexDef;  const IndexName: string): integer;
const
  SIndexList =
    ' SELECT RDB$INDEX_NAME '+
    ' ,RDB$UNIQUE_FLAG '+
    ' ,RDB$INDEX_TYPE '+
    ' ,RDB$INDEX_INACTIVE ' +
    ' ,RDB$RELATION_NAME '+
    ' ,RDB$DESCRIPTION '+
    ' FROM RDB$INDICES '+
    ' WHERE RDB$INDEX_NAME = %s ';

//Lorenzo
  SIndexFieldList3 =
    ' SELECT "RDB$FIELD_NAME" FROM "RDB$INDEX_SEGMENTS" '+
    ' WHERE "RDB$INDEX_NAME" = %s '+
    ' ORDER BY "RDB$FIELD_POSITION";';

  SIndexFieldList1 =
    ' SELECT RDB$FIELD_NAME FROM RDB$INDEX_SEGMENTS '+
    ' WHERE RDB$INDEX_NAME = %s '+
    ' ORDER BY RDB$FIELD_POSITION ';

var
  n:integer;
  SqlIndexFieldList:String;
begin
  try
    Result := ExecuteQuery(Format(SIndexList, [AnsiQuotedStr(IndexName,'''')]));
    while not lQry.EOF do
    begin
      Data.Name := lQry.FieldbyName('RDB$INDEX_NAME').asString;
      Data.Table := lQry.FieldbyName('RDB$RELATION_NAME').asString;
      Data.Unique := lQry.FieldbyName('RDB$UNIQUE_FLAG').AsInteger;
      Data.IndexType :=  lQry.FieldbyName('RDB$INDEX_TYPE').AsInteger;
      Data.Inactive := lQry.FieldByName('RDB$INDEX_INACTIVE').AsInteger;
      Data.Description := lQry.FieldByName('RDB$DESCRIPTION').AsString;

      n:=0;
      if Connection.SQLDialect = 1 then
        SqlIndexFieldList := Format(SIndexFieldList1, [AnsiQuotedStr(IndexName,'''')])
      else
        SqlIndexFieldList := Format(SIndexFieldList3, [AnsiQuotedStr(IndexName,'''')]);
        
      if ExecuteQuery2(SqlIndexFieldList) <> FAILURE then
      begin
        while not lQry2.EOF do
        begin
          if n>high(Data.fields) then
            SetLength(Data.fields,n+1);
          Data.fields[n] := lQry2.Fields[0].asString;
          inc(n);
          lQry2.Next;
        end;
      end;
      SetLength(Data.fields,n);
      lQry.Next;
    end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

function TDmN.GetColumnList(var Fields : TStringDynArray; const TableName: string): integer;
const
  ColumnSQL =
    'SELECT '+
    ' A.RDB$FIELD_NAME '+
    'FROM '+
    'RDB$RELATION_FIELDS A '+
    'WHERE ' +
    ' A.RDB$RELATION_NAME = %s ' +
    'ORDER BY RDB$FIELD_POSITION ';

var
  n: integer;
begin
  try
    Result := ExecuteQuery(Format(ColumnSQL, [AnsiQuotedStr(TableName,'''')]));
    n:=0;
    while not lQry.EOF do
    begin
      if n>High(Fields) then
        SetLength(Fields,n+1);
      Fields[n] := lQry.FieldByName('RDB$FIELD_NAME').AsString;
      inc(n);
      lQry.Next;
    end;
  finally
    lQry.Close;
    //Connection.DefaultTransaction.Commit;
  end;
end;


function TDmN.GetColumnList(Fields: TStrings; const TableName: string): integer;
const
  ColumnSQL =
    'SELECT '+
    ' A.RDB$FIELD_NAME '+
    'FROM '+
    'RDB$RELATION_FIELDS A '+
    'WHERE ' +
    ' A.RDB$RELATION_NAME = %s ' +
    'ORDER BY RDB$FIELD_POSITION ';

begin
  result := FAILURE;
  if not Assigned(Fields) then exit;
  Fields.Clear;
  try
    Result := ExecuteQuery(Format(ColumnSQL, [AnsiQuotedStr(TableName,'''')]));
    while not lQry.EOF do
    begin
      Fields.Add(Trim(lQry.FieldByName('RDB$FIELD_NAME').AsString));
      lQry.Next;
    end;
    result := SUCCESS;
  finally
    lQry.Close;
    //Connection.DefaultTransaction.Commit;
  end;

end;

const
      SqlPK =
      'select i.rdb$field_name '+
      'from    rdb$relation_constraints r, rdb$index_segments i '+
      'where   r.rdb$relation_name=?TableName and '+
      '        r.rdb$constraint_type=''PRIMARY KEY'' and '+
      '        r.rdb$index_name=i.rdb$index_name '+
      'order by i.rdb$field_position; ';


procedure TDmN.GetPKList(const RelationName:String; var PK:TStringDynArray);
var c:TIB_Cursor;
    n:integer;
begin
  if (RelationName<>'') then
  begin
    c:=TIB_Cursor.Create(nil);
    try
      c.IB_Connection := Connection;
      c.IB_Transaction := DDLTransaction;
      c.SQL.Text := SqlPK;
      c.Prepare;
      c.Params[0].AsString := RelationName;
      c.Open;
      n:=0;
      while not c.eof do
      begin
        if n>high(PK) then setlength(PK,n+1);
        PK[n]:=c.Fields[0].AsString;
        inc(n);
        c.Next;
      end;
      setlength(PK,n);
      c.close;
    finally
      c.free;
    end;
  end;
end;

procedure TDmN.GetUsersList(List:TStrings);
begin
  List.Assign(ServerNode.UserList);
end;


function TDmN.GetTriggerData(const TriggerName: String; var TriggerType,
  Active, Table, FireOrder, Source, Description: String): integer;
const

  STriggerList =
    'SELECT '+
    '  RDB$TRIGGER_NAME  '+
    ' ,RDB$TRIGGER_TYPE  '+
    ' ,RDB$TRIGGER_INACTIVE ' +
    ' ,RDB$TRIGGER_SOURCE '+
    ' ,RDB$TRIGGER_SEQUENCE '+
    ' ,RDB$RELATION_NAME '+
    ' ,RDB$DESCRIPTION '+
    ' FROM RDB$TRIGGERS ' +
    ' WHERE RDB$TRIGGER_NAME = %s ';

begin
  try
    Result := ExecuteQuery(Format(STriggerList, [QuotedStr(TriggerName)]));
      while not lQry.EOF do
      begin
        TriggerType := lQry.FieldbyName('RDB$TRIGGER_TYPE').AsString;
        Active := lQry.FieldByName('RDB$TRIGGER_INACTIVE').AsString;
        Source := lQry.FieldByName('RDB$TRIGGER_SOURCE').AsString;
        FireOrder := lQry.FieldByName('RDB$TRIGGER_SEQUENCE').AsString;
        Table  := lQry.FieldByName('RDB$RELATION_NAME').AsString;
        Description := lQry.FieldByName('RDB$DESCRIPTION').AsString;
        lQry.Next;
      end;
  finally
    lQry.Close;
    ////lQry.IB_Transaction.Commit;
  end;
end;

procedure TDmN.GetDomainData3(const DomainName:String; var Data:TDomainDef);
var
  Idx : Integer;
  Dimensions : Integer;
  FSQLDialect:integer;
begin
  FSQLDialect := Connection.SQLDialect;
  lqry2.SQL.Text :=('select * from rdb$fields where rdb$field_name = ' + AnsiQuotedStr(DomainName, '''') + ';');
  lqry2.Open;
  Case lqry2.FieldByName('rdb$field_type').AsInteger of
    blr_short :
      begin
        if FSQLDialect = 3 then
        begin
          if lqry2.FieldByName('rdb$field_sub_type').IsNull then
          begin
            Data.DataType := 'SMALLINT';
          end
          else
          begin
            case lqry2.FieldByName('rdb$field_sub_type').AsInteger of
              0 :
                begin
                  Data.DataType := 'SMALLINT';
                end;
              1 :
                begin
                  Data.DataType := 'NUMERIC';
                  if FSQLDialect = 3 then
                    Data.Precision := lqry2.FieldByName('rdb$field_precision').AsInteger
                  else
                    Data.Precision := 4;
                  Data.Scale := Abs(lqry2.FieldByName('rdb$field_scale').AsInteger);
                end;
              2 :
                begin
                  Data.DataType := 'DECIMAL';
                  if FSQLDialect = 3 then
                    Data.Precision := lqry2.FieldByName('rdb$field_precision').AsInteger
                  else
                    Data.Precision := 4;
                  Data.Scale := Abs(lqry2.FieldByName('rdb$field_scale').AsInteger);
                end;
            end;
          end;
        end
        else
        begin
          if lqry2.FieldByName('rdb$field_scale').AsInteger <> 0 then
          begin
            Data.DataType := 'DECIMAL';
            Data.Precision := 4;
            Data.Scale := Abs(lqry2.FieldByName('rdb$field_scale').AsInteger);
          end
          else
            Data.DataType := 'SMALLINT';
        end;
      end;

    blr_long :
      begin
        if FSQLDialect = 3 then
        begin
          if lqry2.FieldByName('rdb$field_sub_type').IsNull then
          begin
            Data.DataType := 'INTEGER';
          end
          else
          begin
            case lqry2.FieldByName('rdb$field_sub_type').AsInteger of
              0 :
                begin
                  Data.DataType := 'INTEGER';
                end;
              1 :
                begin
                  Data.DataType := 'NUMERIC';
                  if FSQLDialect = 3 then
                    Data.Precision := lqry2.FieldByName('rdb$field_precision').AsInteger
                  else
                    Data.Precision := 9;
                  Data.Scale := Abs(lqry2.FieldByName('rdb$field_scale').AsInteger);
                end;
              2 :
                begin
                  Data.DataType := 'DECIMAL';
                  if FSQLDialect = 3 then
                    Data.Precision := lqry2.FieldByName('rdb$field_precision').AsInteger
                  else
                    Data.Precision := 9;
                  Data.Scale := Abs(lqry2.FieldByName('rdb$field_scale').AsInteger);
                end;
            end;
          end;
        end
        else
        begin
          if lqry2.FieldByName('rdb$field_scale').AsInteger <> 0 then
          begin
            Data.DataType := 'DECIMAL';
            Data.Precision := 9;
            Data.Scale := Abs(lqry2.FieldByName('rdb$field_scale').AsInteger);
          end
          else
            Data.DataType := 'INTEGER';
        end;
      end;

    blr_int64 :
      begin
        if FSQLDialect = 3 then
        begin
          if lqry2.FieldByName('rdb$field_sub_type').IsNull then
          begin
            Data.DataType := 'DECIMAL';
            Data.Precision := 18;
            Data.Scale := 0;
          end
          else
          begin
            case lqry2.FieldByName('rdb$field_sub_type').AsInteger of
              0 :
                begin
                  Data.DataType := 'DECIMAL';
                  Data.Precision := 18;
                  Data.Scale := 0;
                end;
              1 :
                begin
                  Data.DataType := 'NUMERIC';
                  if FSQLDialect = 3 then
                    Data.Precision := lqry2.FieldByName('rdb$field_precision').AsInteger
                  else
                    Data.Precision := 18;
                  Data.Scale := Abs(lqry2.FieldByName('rdb$field_scale').AsInteger);
                end;
              2 :
                begin
                  Data.DataType := 'DECIMAL';
                  if FSQLDialect = 3 then
                    Data.Precision := lqry2.FieldByName('rdb$field_precision').AsInteger
                  else
                    Data.Precision := 18;
                  Data.Scale := Abs(lqry2.FieldByName('rdb$field_scale').AsInteger);
                end;
            end;
          end;
        end;
      end;

    blr_float :
      begin
        Data.DataType := 'FLOAT';
      end;


    blr_text :
      begin
        Data.DataType := 'CHAR';
        Data.CharLength := lqry2.FieldByName('rdb$field_length').AsInteger;
        Data.CharCharSet := GetDBCharSetName(lqry2.FieldByName('rdb$character_set_id').AsInteger);
        Data.CharCollate := GetDBCollationName(lqry2.FieldByName('rdb$collation_id').AsInteger, lqry2.FieldByName('rdb$character_set_id').AsInteger);
      end;

    blr_double :
      begin
        if lqry2.FieldByName('rdb$field_scale').AsInteger <> 0 then
        begin
          Data.DataType := 'DECIMAL';
          Data.Precision := 15;
          Data.Scale := Abs(lqry2.FieldByName('rdb$field_scale').AsInteger);
        end
        else
          Data.DataType := 'DOUBLE PRECISION';
      end;

    blr_timestamp :
      begin
        if FSQLDialect = 3 then
          Data.DataType := 'TIMESTAMP'
        else
          Data.DataType := 'DATE';
      end;

    blr_sql_time :
      begin
        Data.DataType := 'TIME';
      end;

    blr_sql_date :
      begin
        Data.DataType := 'DATE';
      end;

    blr_varying :
      begin
        Data.DataType := 'VARCHAR';
        Data.CharLength := lqry2.FieldByName('rdb$field_length').AsInteger;
        Data.CharCharSet := GetDBCharSetName(lqry2.FieldByName('rdb$character_set_id').AsInteger);
        Data.CharCollate := GetDBCollationName(lqry2.FieldByName('rdb$collation_id').AsInteger, lqry2.FieldByName('rdb$character_set_id').AsInteger);
      end;


    blr_blob :
      begin
        Data.DataType := 'BLOB';
        Data.BlobSubType := lqry2.FieldByName('rdb$field_sub_type').AsString;
        Data.BlobSegSize := lqry2.FieldByName('rdb$segment_length').AsString;
        Data.BlobCharSet := GetDBCharSetName(lqry2.FieldByName('rdb$character_set_id').AsInteger);
        Data.BlobCollate := GetDBCollationName(lqry2.FieldByName('rdb$collation_id').AsInteger, lqry2.FieldByName('rdb$character_set_id').AsInteger);
      end;
  else
    Data.DataType := '';
  end;

  //-----------defaults-------------

  Data.Default := lqry2.FieldByName('rdb$default_source').AsString;

  //----------check constraint------------

  Data.Check := lqry2.FieldByName('rdb$validation_source').AsString;

  if lqry2.FieldByName('rdb$null_flag').IsNull then
    Data.NotNull := False
  else
    Data.NotNull := True;

  if lqry2.FieldByName('rdb$dimensions').AsInteger > 0 then
  begin
    Dimensions := lqry2.FieldByName('rdb$dimensions').AsInteger;
    SetLength(Data.ArrayDim,Dimensions);
    for Idx := 0 to Dimensions-1 do
    begin
      lqry2.Close;
      lqry2.SQL.Text :=('select rdb$lower_bound, rdb$upper_bound from rdb$field_dimensions where ' +
                    'rdb$dimension = ' + IntToStr(Idx)  + 'and rdb$field_name = ' + AnsiQuotedStr(DomainName, '''') + ';');
      lqry2.Open;
      if not (lqry2.EOF and lqry2.BOF) then
      begin
        with Data.ArrayDim[idx] do
        begin
          LBound := lqry2.FieldByName('rdb$lower_bound').AsInteger;
          UBound := lqry2.FieldByName('rdb$upper_bound').AsInteger;
        end;
      end;
    end;
  end;

  lqry2.Close;
  //lqry2.IB_Transaction.Commit;
end;

procedure TDmN.DataModuleCreate(Sender: TObject);
begin
  FAutoDDL := gAppSettings[AUTO_COMMIT_DDL];
  if FAutoDDL then
    DDLTransaction.AutoCommit := True;
  Connection.DefaultTransaction := Transaction;
end;

procedure TDmN.SetAutoDDL(const Value: boolean);
begin
  FAutoDDL := Value;
  DDLTransaction.AutoCommit := FAutoDDL;
end;

end.
