{******************************************************************}
{  This Unit was created to by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.
}
{******************************************************************}
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{******************************************************************}

unit frameEditorField;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, DB, dmuNode, ActnList, SynEdit, IBOConsoleForm,
  Spin;

type
  TPanelType = (ptNone,ptChar,ptNum,ptBlob);

  TFrameEditorField = class(TFrame)
    edColumn: TEdit;
    Label1: TLabel;
    pgTypes: TPageControl;
    tsSubType: TTabSheet;
    Label3: TLabel;
    cmbDataType: TComboBox;
    tsSubDomain: TTabSheet;
    Label2: TLabel;
    cmbDomain: TComboBox;
    tsComputed: TTabSheet;
    Label6: TLabel;
    edComputed: TSynEdit;
    chkNotNull: TCheckBox;
    pnlArray: TPanel;
    Label11: TLabel;
    lvArray: TListView;
    btnAddDimension: TButton;
    btnDeleteDimension: TButton;
    btnEditDimension: TButton;
    chkPrimaryKey: TCheckBox;
    Label4: TLabel;
    Label9: TLabel;
    Label8: TLabel;
    edLength: TSpinEdit;
    cmbCharCharSet: TComboBox;
    cmbCharCollate: TComboBox;
    Label14: TLabel;
    Label15: TLabel;
    edPrecision: TSpinEdit;
    edScale: TSpinEdit;
    Label18: TLabel;
    Label10: TLabel;
    Label12: TLabel;
    Label16: TLabel;
    edBlobSubType: TEdit;
    cmbBlobCharSet: TComboBox;
    edBlobSegSize: TSpinEdit;
    cmbBlobCollate: TComboBox;
    Panel2: TPanel;
    pDataType: TPanel;
    edDefault: TSynEdit;
    pNumeric: TPanel;
    pChar: TPanel;
    pBlob: TPanel;
    tsDefault: TTabSheet;
    procedure cmbDataTypeChange(Sender: TObject);
    procedure btnAddDimensionClick(Sender: TObject);
    procedure btnDeleteDimensionClick(Sender: TObject);
    procedure btnEditDimensionClick(Sender: TObject);
    procedure cmbDomainChange(Sender: TObject);
    procedure chkNotNullClick(Sender: TObject);
    procedure lvArrayChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure edComputedChange(Sender: TObject);
    procedure edColumnChange(Sender: TObject);
    procedure chkPrimaryKeyClick(Sender: TObject);
    procedure edDefaultChange(Sender: TObject);
    procedure edTableKeyPress(Sender: TObject; var Key: Char);
    procedure cmbDataTypeCloseUp(Sender: TObject);
    procedure cmbDomainCloseUp(Sender: TObject);
  private
    fData:TColumnDef;
    FDataModule: TDmN;
    FObjectModified: boolean;
    FnPanel: TPanelType;
    FOnObjectModified: TNotifyEvent;
    { Private declarations }
    procedure GetDomainList;
    procedure ResetDataDisplay;
    procedure SetData(const Value: TColumnDef);
    procedure SetnPanel(const Value: TPanelType);
    procedure SetOnObjectModified(const Value: TNotifyEvent);
  protected
    procedure SetDataModule(const Value: TDmN);
    procedure SetObjectModified(const Value: boolean);
    property nPanel:TPanelType read FnPanel write SetnPanel;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure ResetData;
    procedure DisplayData;
    property Data:TColumnDef read FData write SetData;
    property DataModule: TDmN read FDataModule write SetDataModule;
    property ObjectModified:boolean read FObjectModified write SetObjectModified;
    property OnObjectModified:TNotifyEvent read FOnObjectModified write SetOnObjectModified;
    function Validate:boolean;
  end;

implementation

uses
  frmuArrayDialog, dmuMain, frmuMain, zluUtility, IBOCGlobal, IBOAdminUtils,
  frmuMessage, LMMessageDialog;

{$R *.DFM}

const
  PG_DOMAIN = 0;
  PG_RAW_TYPE = 1;
  PG_COMPUTED = 2;



procedure TFrameEditorField.GetDomainList;
var
  I : Integer;
  l:TstringList;
  lsCurrLine:String;
begin
  cmbDomain.Items.Clear;
  l:=TstringList.Create;
  if Assigned(DataModule) then
  try
    DataModule.GetDomainList(l,False);

    for i := 1  to l.Count - 1 do
    begin
      lsCurrLine := l.Strings[i];
      cmbDomain.Items.Add(GetNextField(lsCurrLine, DEL));
    end;
  finally
    l.Free;
  end;
end;

procedure TFrameEditorField.cmbDataTypeChange(Sender: TObject);
begin
  if Sender = cmbDataType then
  begin
    fData.DataType := cmbDataType.Text;
    case cmbDataType.ItemIndex of
      iDECIMAL, iNUMERIC : nPanel := ptNum;
      iCHAR, iVARCHAR    : nPanel := ptChar;
      iBLOB              : nPanel := ptBlob;
      else   nPanel := ptNone;
    end;
  end
  else
  if Sender = edBlobSubType then
  begin
    fData.BlobSubType := edBlobSubType.Text;
  end
  else
  if Sender = cmbBlobCharSet then
  begin
    fData.BlobCharSet := cmbBlobCharSet.Text;
    if fData.BlobCharSet = 'NONE' then fData.BlobCharSet := '';
  end
  else
  if Sender = cmbBlobCollate then
  begin
    fData.BlobCollate := cmbBlobCollate.Text;
  end
  else
  if Sender = edBlobSegSize then
  begin
    fData.BlobSegSize := edBlobSegSize.Value;
  end
  else
  if Sender = edLength then
  begin
    fData.CharLength := edLength.Value;
  end
  else
  if Sender = cmbCharCharSet then
  begin
    fData.CharCharSet := cmbCharCharSet.Text;
    if fData.CharCharSet = 'NONE' then fData.CharCharSet := '';
  end
  else
  if Sender = cmbCharCollate then
  begin
    fData.CharCollate := cmbCharCollate.Text;
  end
  else
  if Sender = edPrecision then
  begin
    fData.Precision := edPrecision.Value;
  end
  else
  if Sender = edScale then
  begin
    fData.Scale := edScale.Value;
  end;
  ObjectModified:= True;
end;

procedure TFrameEditorField.btnAddDimensionClick(Sender: TObject);
var
  frmArrayDialog: TfrmArrayDialog;

begin
  frmArrayDialog := TfrmArrayDialog.Create(Self);
  try
    if frmArrayDialog.ShowModal = mrOK then
    begin
      with lvArray.Items.Add do
      begin
        Caption := frmArrayDialog.edLBound.Text;
        SubItems.Add(frmArrayDialog.edUBound.Text);
      end;
    end;
  finally
    frmArrayDialog.Free;
  end;
end;

procedure TFrameEditorField.btnDeleteDimensionClick(Sender: TObject);
begin
  if lvArray.Selected <> nil then
  begin
    if LMMessageDlg(51, mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      lvArray.Selected.Delete;
  end;
end;

procedure TFrameEditorField.btnEditDimensionClick(Sender: TObject);
var
  frmArrayDialog: TfrmArrayDialog;
begin
  if lvArray.Selected <> nil then
  begin
    frmArrayDialog := TfrmArrayDialog.Create(Self);
    try
      frmArrayDialog.edLBound.Value := StrToInt(lvArray.Selected.Caption);
      frmArrayDialog.edUBound.Value := StrToInt(lvArray.Selected.SubItems[0]);
      if frmArrayDialog.ShowModal = mrOK then
      begin
        with lvArray.Items.Add do
        begin
          lvArray.Selected.Caption := frmArrayDialog.edLBound.Text;
          lvArray.Selected.SubItems[0] := frmArrayDialog.edUBound.Text;
        end;
      end;
    finally
      frmArrayDialog.Free;
    end;
  end;
end;

procedure TFrameEditorField.SetDataModule(const Value: TDmN);
begin
  fDataModule := Value;
  GetDomainList;
end;

procedure TFrameEditorField.DisplayData;
begin
  ResetDataDisplay;
  edColumn.Text := Data.ColumnName;
  edDefault.Text := Data.Default;
  if Data.UseComputed then
  begin
    edComputed.Lines.Text := Data.Computed;
    cmbDataType.ItemIndex :=-1;
    pgTypes.ActivePage := tsComputed;
  end
  else
  if Data.UseDomain then
  begin
    cmbDomain.ItemIndex := cmbDomain.Items.IndexOf(Data.Domain);
    cmbDataType.ItemIndex :=-1;
    pgTypes.ActivePage := tsSubDomain;
  end
  else
  begin
    pgTypes.ActivePage := tsSubType;
    chkNotNull.Checked := Data.NotNull;
    cmbDataType.ItemIndex := cmbDataType.Items.IndexOf(Data.DataType);
    case cmbDataType.ItemIndex of
      iDECIMAL,
      iNUMERIC :
      begin
        nPanel := ptNum;
        edPrecision.Value := Data.Precision;
        edScale.Value := Data.Scale;
      end;
      iCHAR,iVARCHAR :
      begin
        nPanel := ptChar;
        edLength.Value := Data.CharLength;
        cmbCharCharSet.ItemIndex := cmbCharCharSet.Items.IndexOf(Data.CharCharSet);
        cmbCharCollate.ItemIndex := cmbCharCollate.Items.IndexOf(Data.CharCollate);
      end;
      iBLOB    :
      begin
        nPanel := ptBlob;
        edBlobSubType.Text := Data.BlobSubType;
        edBlobSegSize.Value := Data.BlobSegSize;
        cmbBlobCharSet.ItemIndex := cmbBlobCharSet.Items.IndexOf(Data.BlobCharSet);
        cmbBlobCollate.ItemIndex := cmbBlobCollate.Items.IndexOf(Data.BlobCollate);
      end;
    else
      nPanel := ptNone;
    end;

  end;
  ObjectModified := False;
end;



procedure TFrameEditorField.SetObjectModified(const Value: boolean);
begin
  fObjectModified := Value;
  if Assigned(FOnObjectModified) then FOnObjectModified(self);
end;

procedure TFrameEditorField.cmbDomainChange(Sender: TObject);
begin
  fData.Domain := Trim(cmbDomain.Text);
  fData.UseDomain := fData.Domain <> '';
  ObjectModified := True;
end;

procedure TFrameEditorField.chkNotNullClick(Sender: TObject);
begin
  fData.NotNull := chkNotNull.Checked;
  ObjectModified:= True;
end;

procedure TFrameEditorField.lvArrayChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
var Idx:integer;
begin
  ObjectModified := True;
  for Idx := 0 to lvArray.Items.Count - 1 do
  begin
    if Idx>High(fData.ArrayDim) then
      SetLength(fData.ArrayDim,Idx+1);
    fData.ArrayDim[idx].LBound := StrTointDef(lvArray.Items.Item[Idx].Caption,0);
    fData.ArrayDim[idx].UBound := StrTointDef(lvArray.Items.Item[Idx].SubItems[0],0);
  end;
end;

procedure TFrameEditorField.edComputedChange(Sender: TObject);
begin
  fData.Computed :=  Trim(edComputed.Text);
  fData.UseComputed := fData.Computed <> '';
  ObjectModified := True;
end;

procedure TFrameEditorField.edColumnChange(Sender: TObject);
begin
  fData.ColumnName := edColumn.Text;
  ObjectModified := True;
end;

procedure TFrameEditorField.ResetDataDisplay;
begin
  cmbDataType.Text := '';
  edBlobSubType.Clear;
  cmbBlobCharSet.Text := '';
  cmbBlobCollate.Text := '';
  edBlobSegSize.Clear;
  lvArray.Clear;
  edPrecision.Clear;
  edScale.Clear;
  edLength.Clear;
  cmbCharCharSet.Text := '';
  cmbCharCollate.Text := '';
  chkNotNull.Checked := False;
  edComputed.ClearAll;
  edDefault.ClearAll;
  cmbDomain.Text := '';
  chkPrimaryKey.Checked := False;
  ObjectModified := False;
end;

procedure TFrameEditorField.SetData(const Value: TColumnDef);
begin
  AssignColumnDef(FData,Value);
  DisplayData;
end;

procedure TFrameEditorField.ResetData;
begin
  ResetDataDisplay;
  ObjectModified := False;
  pgTypes.ActivePage := tsSubType;
end;

procedure TFrameEditorField.chkPrimaryKeyClick(Sender: TObject);
begin
  fData.Pk := chkPrimaryKey.Checked;
end;

procedure TFrameEditorField.edDefaultChange(Sender: TObject);
begin
  fData.Default := edDefault.Text;
  ObjectModified := True;
end;

procedure TFrameEditorField.edTableKeyPress(Sender: TObject; var Key: Char);
begin
  if Assigned(Datamodule) then
    Key := CheckNameKey(Key,Datamodule.Connection.SQLDialect);
end;

constructor TFrameEditorField.Create(AOwner: TComponent);
var n:integer;
begin
  inherited;
  cmbDataType.Items.Clear;
  for n:=0 to 11 do
    cmbDataType.Items.Add(sDataType[n]);
  cmbDataType.ItemIndex := 0;
  cmbDataTypeChange(cmbDataType);
  cmbDataType.DropDownCount := length(sDataType);
end;

procedure TFrameEditorField.SetnPanel(const Value: TPanelType);
begin
  FnPanel := Value;
    case FnPanel of
      ptNum:
      begin
        pNumeric.Visible := True;
        pBlob.Visible := False;
        pChar.Visible := False;
      end;
      ptChar :
      begin
        pChar.Visible := True;
        pNumeric.Visible := False;
        pBlob.Visible := False;
      end;
      ptBlob :
      begin
        pBlob.Visible := True;
        pNumeric.Visible := False;
        pChar.Visible := False;
      end;
    else
      begin
        pBlob.Visible := False;
        pNumeric.Visible := False;
        pChar.Visible := False;
      end;
    end;

end;

procedure TFrameEditorField.SetOnObjectModified(const Value: TNotifyEvent);
begin
  FOnObjectModified := Value;
end;


procedure TFrameEditorField.cmbDataTypeCloseUp(Sender: TObject);
begin
  cmbDataTypeChange(Sender);
end;

procedure TFrameEditorField.cmbDomainCloseUp(Sender: TObject);
begin
  cmbDomainChange(Sender);
end;

function TFrameEditorField.Validate: boolean;
begin
  Result := False;
  if edColumn.Text = '' then
  begin
    LMMessageDlg(52, mtError, [mbOK], 0);
    edColumn.SetFocus;
    Exit;
  end;

  if pgTypes.ActivePage = tsSubType then // raw data type...
  begin
    // do nothing....
  end
  else
  if pgTypes.ActivePage = tsSubDomain then // raw data type...
  begin
    if cmbDomain.Text = '' then
    begin
      LMMessageDlg(53, mtError, [mbOK], 0);
      pgTypes.ActivePage := tsSubDomain;
      cmbDomain.SetFocus;
      Exit;
    end;
  end
  else
  if pgTypes.ActivePage = tsComputed then // raw data type...
  begin
    if Trim(edComputed.Text) = '' then
    begin
      LMMessageDlg(54, mtError, [mbOK], 0);
      pgTypes.ActivePage := tsComputed;
      edComputed.SetFocus;
      Exit;
    end;
  end;
  Result := True;
end;

end.
