{************************************************************************}
{  This Unit was modified to be interfaced in IBObjects
   by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com
   }
{  This software is distributed under the License is distributed on      }
{    an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either              }
{    express or implied. See the ORIGINAL Copyright                      }
unit frmEditorOptionsU;
interface
uses
{$IFDEF LINUX}
  QGraphics, QControls, QForms, QDialogs, QExtCtrls, QStdCtrls, IBCXDummies,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ColorGrd,
{$ENDIF}
  SysUtils, Classes, SynEditHighlighter, SynHighlighterSQL, SynEdit, IBOConsoleForm;

type
  TfrmEditorOptions = class(TIBOConsoleForm)
    ColorGrid1: TColorGrid;
    lstElements: TListBox;
    GroupBox1: TGroupBox;
    chkBold: TCheckBox;
    chkItalic: TCheckBox;
    chkUnderline: TCheckBox;
    Label1: TLabel;
    Label2: TLabel;
    SynEditor: TSynEdit;
    Panel1: TPanel;
    GroupBox2: TGroupBox;
    chkDefaultFG: TCheckBox;
    chkDefaultBG: TCheckBox;
    Button1: TButton;
    Button2: TButton;
    SynSQLSyn1: TSynSQLSyn;
    Panel2: TPanel;
    ButtonChangeFont: TButton;
    Label3: TLabel;
    FontDialog1: TFontDialog;
    procedure FormShow(Sender: TObject);
    procedure lstElementsClick(Sender: TObject);
    procedure ColorGrid1Click(Sender: TObject);
    procedure ColorGrid1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure chkDefaultFGClick(Sender: TObject);
    procedure chkStyleClick(Sender: TObject);
    procedure ButtonChangeFontClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmEditorOptions: TfrmEditorOptions;

implementation

uses IBOCGlobal;

{$R *.DFM}

const
  Colors: array[1..16] of TColor = (clBlack, clMaroon, clGreen, clOlive,
    clNavy, clPurple, clTeal, clDkGray, clLtGray, clRed, clLime,
    clYellow, clBlue, clFuchsia, clAqua, clWhite);

function ColorToIndex(AColor: TColor): integer;
var
  i: integer;
begin
  Result := -1;
  for i := Low(Colors) to High(Colors) do
    if Colors[i] = AColor then
    begin
      Result := i - 1;
      break;
    end;
end;

function IndexToColor(AIndex: integer): TColor;
begin
  Result := Colors[AIndex + 1];
end;

procedure TfrmEditorOptions.FormShow(Sender: TObject);
var   i : Integer;
begin
  lstElements.Items.Clear;
  if SynEditor.Highlighter = nil then exit;
  for i := 0 to SynEditor.Highlighter.AttrCount - 1 do
    if (SynEditor.Highlighter.Attribute[i].Name <> 'Reserved word (PL/SQL)')
       and (SynEditor.Highlighter.Attribute[i].Name <> 'SQL*Plus command')
    then
      lstElements.Items.Add(SynEditor.Highlighter.Attribute[i].Name);
  lstElements.ItemIndex := 0;
  lstElementsClick(lstElements);
  SynEditor.Font.Name := gAppSettings[SYNTAX_FONT_NAME];
  SynEditor.Font.Charset := gAppSettings[SYNTAX_FONT_CHARSET];
  Label3.Font.Name := SynEditor.Font.Name;
  Label3.Font.Charset := SynEditor.Font.Charset;
  Label3.Caption := SynEditor.Font.Name;
  FontDialog1.Font.Name := SynEditor.Font.Name;
  FontDialog1.Font.Charset := SynEditor.Font.Charset;
end;

procedure TfrmEditorOptions.lstElementsClick(Sender: TObject);
var   Attr: TSynHighlighterAttributes;
begin
  Attr := TSynHighlighterAttributes.Create('');
  try
    if SynEditor.Highlighter <> nil then
      Attr.Assign(SynEditor.Highlighter.Attribute[lstElements.ItemIndex]);
{$IFDEF MSWINDOWS}
    ColorGrid1.ForegroundEnabled := true;
    ColorGrid1.BackgroundEnabled := true;
    ColorGrid1.ForegroundIndex := ColorToIndex(Attr.Foreground);
    ColorGrid1.BackgroundIndex := ColorToIndex(Attr.Background);
    if ColorToIndex(Attr.Foreground) <= 0 then
    begin
      chkDefaultFG.Checked := true;
      ColorGrid1.ForegroundEnabled := false;
    end
    else
      chkDefaultFG.Checked := false;
    if (ColorToIndex(Attr.Background) = 15) or
       (ColorToIndex(Attr.Background) = -1) then
    begin
      chkDefaultBG.Checked := true;
      ColorGrid1.BackgroundEnabled := false;
    end
    else
      chkDefaultBG.Checked := false;
{$ENDIF}
    chkBold.Checked := (fsBold in Attr.Style);
    chkItalic.Checked := (fsItalic in Attr.Style);
    chkUnderLine.Checked := (fsUnderline in Attr.Style);
  finally
    Attr.Free;
  end;
end;

procedure TfrmEditorOptions.ColorGrid1Click(Sender: TObject);
begin
{$IFDEF MSWINDOWS}
  if SynEditor.Highlighter = nil then exit;
    if chkDefaultFG.Checked then
      SynEditor.Highlighter.Attribute[lstElements.ItemIndex].Foreground := clWindowText
    else
    begin
      SynEditor.Highlighter.Attribute[lstElements.ItemIndex].Foreground := IndexToColor(ColorGrid1.ForegroundIndex);
    end;
    if chkDefaultBG.Checked then
      SynEditor.Highlighter.Attribute[lstElements.ItemIndex].Background := clWindow
    else
      SynEditor.Highlighter.Attribute[lstElements.ItemIndex].Background := IndexToColor(ColorGrid1.BackgroundIndex);
{$ENDIF}
end;

procedure TfrmEditorOptions.ColorGrid1MouseDown(Sender: TObject; Button: TMouseButton;   Shift: TShiftState; X, Y: Integer);
begin
{$IFDEF MSWINDOWS}
  if Button = mbLeft then
  begin
    chkDefaultFG.Checked := false;
    ColorGrid1.ForegroundEnabled := true;
  end
  else
  if Button = mbRight then
  begin
    chkDefaultBG.Checked := false;
    ColorGrid1.BackgroundEnabled := true;
  end;
{$ENDIF}
end;

procedure TfrmEditorOptions.chkDefaultFGClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}
  if Sender = chkDefaultFG then
    if chkDefaultFG.Checked then
      ColorGrid1.ForegroundEnabled := false
    else
      ColorGrid1.ForegroundEnabled := true
    else
    if chkDefaultBG.Checked then
      ColorGrid1.BackgroundEnabled := false
    else
      ColorGrid1.BackgroundEnabled := true;
  ColorGrid1Click(nil);
  {$ENDIF}
end;

procedure TfrmEditorOptions.chkStyleClick(Sender: TObject);
var   Style: TFontStyles;
begin
  if SynEditor.Highlighter <> nil then
  begin
    Style := [];
    if chkBold.Checked then
      Include(Style, fsBold);
    if chkItalic.Checked then
      Include(Style, fsItalic);
    if chkUnderLine.Checked then
      Include(Style, fsUnderline);
    SynEditor.Highlighter.Attribute[lstElements.ItemIndex].Style := Style;
  end;
end;

procedure TfrmEditorOptions.ButtonChangeFontClick(Sender: TObject);
//var i:integer;
begin
  if FontDialog1.Execute then
  begin
//    gAppSettings[SYNTAX_FONT_NAME] := FontDialog1.Font.Name;
//    gAppSettings[SYNTAX_FONT_CHARSET]  := FontDialog1.Font.CharSet;
    Label3.Font.Name := FontDialog1.Font.Name;
    Label3.Font.Charset := FontDialog1.Font.Charset;
    Label3.Caption := FontDialog1.Font.Name;
    SynEditor.Font.Name := FontDialog1.Font.Name;
    SynEditor.Font.Charset := FontDialog1.Font.Charset;
  end;
end;

procedure TfrmEditorOptions.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

end.
