{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 *
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 *
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 *
 * Contributor(s): Krzysztof Golko.
}

{****************************************************************
*
*  f r m u C o m m D i a g
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Description:  This unit provides all the necessary functions
*                and interface to perform network communication
*                diagnostics.
*
*****************************************************************
* Revisions:
*
*****************************************************************}

unit frmuCommDiag;

interface

uses
{$IFDEF LINUX}
  QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QExtCtrls,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Winsock,
{$ENDIF}
  ScktComp, IdComponent, IdRawBase, IdRawClient, IdIcmpClient, IdTCPConnection,
  IdTCPClient,  IdBaseComponent,  IdException,
  SysUtils, Classes, IB_Components, IBOConnectionInfo, IB_session,
  zluibcClasses, zluCommDiag, IBOConsoleForm;

type
  TfrmCommDiag = class(TIBOConsoleForm)
    btnSelDB: TButton;
    cbDBServer: TComboBox;
    cbNetBEUIServer: TComboBox;
    cbProtocol: TComboBox;
    cbSPXServer: TComboBox;
    cbService: TComboBox;
    cbTCPIPServer: TComboBox;
    edtDatabase: TEdit;
    edtPassword: TEdit;
    edtUsername: TEdit;
    gbDBServerInfo: TGroupBox;
    gbDatabaseInfo: TGroupBox;
    gbNetBEUIServerInfo: TGroupBox;
    gbNovellServerInfo: TGroupBox;
    gbTCPIPServerInfo: TGroupBox;
    lblDBResults: TLabel;
    lblDatabase: TLabel;
    lblNetBEUIServer: TLabel;
    lblNetBeuiResults: TLabel;
    lblPassword: TLabel;
    lblProtocol: TLabel;
    lblSPXResults: TLabel;
    lblSPXServer: TLabel;
    lblServerName: TLabel;
    lblService: TLabel;
    lblUsername: TLabel;
    lblWinSockResults: TLabel;
    lblWinsockServer: TLabel;
    memDBResults: TMemo;
    memNetBeuiResults: TMemo;
    memSPXResults: TMemo;
    memTCPIPResults: TMemo;
    pgcDiagnostics: TPageControl;
    rbLocalServer: TRadioButton;
    rbRemoteServer: TRadioButton;
    tabDBConnection: TTabSheet;
    tabNetBEUI: TTabSheet;
    tabSPX: TTabSheet;
    tabTCPIP: TTabSheet;
    pnlButtonBar: TPanel;
    btnTest: TButton;
    btnCancel: TButton;
    ICMP: TIdIcmpClient;
    inTCP: TIdTCPClient;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure btnCancelClick(Sender: TObject);
    procedure btnSelDBClick(Sender: TObject);
    procedure btnTestClick(Sender: TObject);
    procedure cbDBServerClick(Sender: TObject);
    procedure rbLocalServerClick(Sender: TObject);
    procedure rbRemoteServerClick(Sender: TObject);
    procedure edtDatabaseChange(Sender: TObject);
    procedure ICMPReply(ASender: TComponent;
      const AReplyStatus: TReplyStatus);
  private
    { Private declarations }
    FProtocols: TStringList;
    FServers: TStringList;
    function VerifyInputData(): boolean;
    procedure PingServer;
    procedure TestDBConnect;
    procedure TestNetBEUI;
    procedure TestPort(Port : String);
    procedure TestSPX;
  public
    { Public declarations }
  end;

  function DoDiagnostics(const CurrSelServer: TibcServerNode) : Integer;
  function ServiceRunning(const CurrSelServer : TibcServerNode) : Boolean;

const
  Port21     = 0;                      // constants to identify TCP/IP service tests
  PortFTP    = 1;
  Port3050   = 2;
  Portgds_db = 3;
  Ping       = 4;

var
  frmCommDiag : TfrmCommDiag;

implementation

uses
   IBOCGlobal, IBOCPersistent, zluContextHelp, IBOServices, frmuMessage;

{$R *.DFM}

//  Input:  TibcServerNode - Specifies the currenly selected server
//  Return: Integer - Determines success or failure.
function DoDiagnostics(const CurrSelServer: TibcServerNode): integer;
var
  frmCommDiag: TfrmCommDiag;
begin
  frmCommDiag:= TfrmCommDiag.Create(Application.MainForm);
  try
    // determine if a server is currently selected
    if Assigned(CurrSelServer) then
    begin
      // if a local server is selected then check the local radio button
      if CurrSelServer.Server.Protocol = cpLocal then
      begin
        frmCommDiag.rbLocalServer.Checked := true;
      end
      else
      begin
        // otherwise select the specified protocol of the remote server from the protocol combo box
        // and check the remote radio button
        frmCommDiag.cbDBServer.Text := CurrSelServer.Servername;
        case CurrSelServer.Server.Protocol of
          cpTCP_IP: frmCommDiag.cbProtocol.ItemIndex := frmCommDiag.cbProtocol.Items.IndexOf('TCP/IP');
          cpNETBeui: frmCommDiag.cbProtocol.ItemIndex := frmCommDiag.cbProtocol.Items.IndexOf('NetBEUI');
          cpNovell: frmCommDiag.cbProtocol.ItemIndex := frmCommDiag.cbProtocol.Items.IndexOf('SPX');
        end;
        frmCommDiag.rbRemoteServer.Checked := true;
      end;
    end
    else                               // if no server selected then assume Local Server
      frmCommDiag.rbLocalServer.Checked := true;
{$ifdef initlangfile} WriteAllCaptionAndText('',frmCommDiag,gTransIniFile); {$endif}
    frmCommDiag.ShowModal;             // show form as modal
    if frmCommDiag.ModalResult = mrOK then
    begin
      result := SUCCESS;
    end
    else
      result := FAILURE;
  finally
    // deallocate memory
    frmCommDiag.Free;
  end;
end;

{****************************************************************
*
*  D o D i a g n o t i c s
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  TibcServerNode - Specifies the currenly selected server
*
*  Return: Integer - Determines success or failure.
*
*  Description:  Responsible for creating the form.
*
*****************************************************************
* Revisions:
*
*****************************************************************}

function ServiceRunning(const CurrSelServer: TibcServerNode) : Boolean;
var
{$IFDEF MSWINDOWS}
  lPipe   : TibcPipes;
  lSPX    : TibcSPX;
  lStr    : String;
{$ENDIF}
  lSocket : TIdTCPClient;
begin
  Result  := True;
  // kris - to clear comp warning
  lSocket := nil;
  lPipe := nil;
  lSPX := nil;
  try
    case CurrSelServer.Server.Protocol of
      cpTCP_IP :
      begin
        lSocket := TIdTCPClient.Create(Nil);
        lSocket.Host := CurrSelServer.Servername;
        lSocket.Port := 3050;         // gds_db
        try
          lSocket.Connect;
        except
          Result := False;
        end;
      end;
{$IFDEF MSWINDOWS}
      cpNETBeui :
      begin
        lPipe := TibcPipes.Create;
        lPipe.Server := CurrSelServer.Servername;
        lPipe.Path := '\pipe\interbas\server\gds_db';
        lPipe.Tries := 5;
        lStr := '';

        // test pipe
        Result := lPipe.TestPipe(lStr, True);
      end;
      cpNovell :
      begin

      end;
{$ENDIF}
    end;
  finally
{$IFDEF MSWINDOWS}
    lPipe.Free;
    lSPX.Free;
{$ENDIF}
    lSocket.Free
  end;
end;

// if user presses the Cancel button then set the result to mrCancel
procedure TfrmCommDiag.btnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

{****************************************************************
*
*  b t n S e l D B C l i c k
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  Tobject
*
*  Return: None
*
*  Description:  Shows an open file dialog box that allows
*                a user to browse for a local database file.
*
*****************************************************************
* Revisions:
*
*****************************************************************}

procedure TfrmCommDiag.btnSelDBClick(Sender: TObject);
var
  lOpenDialog: TOpenDialog;
begin
  lOpenDialog := nil;
  try
  begin
    // create and show open file dialog box
    lOpenDialog := TOpenDialog.Create(self);
    // specify defaul extension and filters
    lOpenDialog.DefaultExt := 'fdb';
    lOpenDialog.Filter := DlgDbFilters;
    // if OK is pressed then extract selected filename
    if lOpenDialog.Execute then
    begin
      edtDatabase.Text := lOpenDialog.Filename;
    end;
  end
  finally
    // deallocate memory
    lOpenDialog.free;
  end;
end;

// if the local radio button is selected then enable and/or disable
// the appropriate controls
procedure TfrmCommDiag.rbLocalServerClick(Sender: TObject);
begin
  if rbLocalServer.Checked then
  begin
    cbDBServer.Text := '';
    cbDBServer.Enabled := false;
    cbProtocol.ItemIndex := -1;
    cbProtocol.Enabled := false;
    cbDBServer.Color := clSilver;
    cbProtocol.Color := clSilver;
    btnSelDB.Enabled := true;
    edtDatabase.Text := '';
    edtUsername.Text := '';
    edtPassword.Text := '';
  end
  else
  begin
    cbDBServer.Enabled := true;
    cbProtocol.Enabled := true;
    cbDBServer.Color := clWindow;
    cbProtocol.Color := clWindow;
    btnSelDB.Enabled := false;
    edtDatabase.Text := '';
    edtUsername.Text := '';
    edtPassword.Text := '';
  end
end;

// if the remote radio button is selected then enable and/or disable
// the appropriate controls
procedure TfrmCommDiag.rbRemoteServerClick(Sender: TObject);
begin
  if rbRemoteServer.Checked then
  begin
    cbDBServer.Enabled := true;
    cbProtocol.Enabled := true;
    cbDBServer.Color := clWindow;
    cbProtocol.Color := clWindow;
    btnSelDB.Enabled := false;
    edtDatabase.Text := '';
    edtUsername.Text := '';
    edtPassword.Text := '';
  end
  else
  begin
    cbDBServer.Enabled := false;
    cbProtocol.Enabled := false;
    cbDBServer.Color := clSilver;
    cbProtocol.Color := clSilver;
    btnSelDB.Enabled := false;
    edtDatabase.Text := '';
    edtUsername.Text := '';
    edtPassword.Text := '';
  end
end;

{****************************************************************
*
*  V e r i f y I n p u t D a t a ( )
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  None
*
*  Return: Boolean - Indicates whether all the necessary
*                    data has been entered in order to
*                    perform the specified task
*
*  Description:  Verifies whther all the necessary data has been
*                enetered in order to performt he specified
*                task.  If not an error message will be displayed
*                and the control responsible for the violation
*                will be given focus.
*
*****************************************************************
* Revisions:
*
*****************************************************************}

function TfrmCommDiag.VerifyInputData(): boolean;
begin
  result := true;

  // if the db connection tab is currently active
  if pgcDiagnostics.ActivePage = tabDBConnection then
  begin
    // if the remote radio button is checked
    if rbRemoteServer.Checked then
    begin
      // then make sure a server has been supplied
      if (cbDBServer.Text = '') or (cbDBServer.Text = ' ') then
      begin
        DisplayMsg(ERR_SERVER_NAME,'');
        cbDBServer.SetFocus;
        result := false;
        Exit;
      end;
      // also make sure a network protocol has been selected
      if (cbProtocol.Text = '') or (cbProtocol.Text = ' ') then
      begin
        DisplayMsg(ERR_PROTOCOL,'');
        cbProtocol.SetFocus;
        result := false;
        Exit;
      end;
    end;

    // ensure that a database has been specified
    if (edtDatabase.Text = '') or (edtDatabase.Text = ' ') then
    begin
      DisplayMsg(ERR_DB_ALIAS,'');
      edtDatabase.SetFocus;
      result := false;
      Exit;
    end;

    // ensure that a username has been specified
    if (edtUsername.Text = '') or (edtUsername.Text = ' ') then
    begin
      DisplayMsg(ERR_USERNAME,'');
      edtUsername.SetFocus;
      result := false;
      Exit;
    end;

    // ensure that a password has been specified
    if (edtPassword.Text = '') or (edtPassword.Text = ' ') then
    begin
      DisplayMsg(ERR_PASSWORD,'');
      edtPassword.SetFocus;
      result := false;
      Exit;
    end;
  end

  // otherwise, if the NetBEUI tab is active
  else if pgcDiagnostics.ActivePage = tabNetBEUI then
  begin
    // ensure that a server is specified
    if (cbNetBEUIServer.Text = '') or (cbNetBEUIServer.Text = ' ') then
    begin
      DisplayMsg(ERR_SERVER_NAME,'');
      cbNetBEUIServer.SetFocus;
      result := false;
      Exit;
    end;
  end

  // otherwise if the SPX tab is active
  else if pgcDiagnostics.ActivePage = tabSPX then
  begin
    // ensure that a server is specified
    if (cbSPXServer.Text = '') or (cbSPXServer.Text = ' ') then
    begin
      DisplayMsg(ERR_SERVER_NAME,'');
      cbSPXServer.SetFocus;
      result := false;
      Exit;
    end;
  end

  // otherwise if the TCP/IP tab is active
  else if pgcDiagnostics.ActivePage = tabTCPIP then
  begin
    // ensure that a server is specified
    if (cbTCPIPServer.Text = '') or (cbTCPIPServer.Text = ' ') then
    begin
      DisplayMsg(ERR_SERVER_NAME,'');
      cbTCPIPServer.SetFocus;
      result := false;
      Exit;
    end;

    // also ensure that a service has been selected
    if (cbService.Text = '') or (cbService.Text = ' ') then
    begin
      DisplayMsg(ERR_SERVICE,'');
      cbService.SetFocus;
      result := false;
      Exit;
    end;
  end;
end;

{****************************************************************
*
*  b t n T e s t C l i c k
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  TObject
*
*  Return: None
*
*  Description:  Determines which tab is active and performs
*                the appropriate network disgnostic.
*
*****************************************************************
* Revisions:
*
*****************************************************************}

procedure TfrmCommDiag.btnTestClick(Sender: TObject);
begin
  // if all the necessary data has been supplied then proceed
  if VerifyInputData() then
  begin
    // if DB connection is the active page
    if pgcDiagnostics.ActivePage = tabDBConnection then
    begin
      memDBResults.Lines.Clear;        // then clear the database results
      TestDBConnect;                   // perform the database test
    end;

    // if TCP/IP is the active page
    if pgcDiagnostics.ActivePage = tabTCPIP then
    begin
      memTCPIPResults.Lines.Clear;     // then clear the TCP/IP results
      case cbService.ItemIndex of     // determine which service is selected
        Port21     : TestPort('21');   // and perform the TCP/IP test
        PortFTP    : TestPort('ftp');
        Port3050   : TestPort('3050');
        Portgds_db : TestPort('gds_db');
        Ping       : PingServer;
      end;
    end;

{$IFDEF MSWINDOWS}
    // if NetBEUI is the active page
    if pgcDiagnostics.ActivePage = tabNetBEUI then
    begin
      memNetBEUIResults.Lines.Clear;   // then clear the NetBEUI results
      TestNetBEUI;                     // perform the NetBEUI test
    end;

    // if SPX is the active page
    if pgcDiagnostics.ActivePage = tabSPX then
    begin
      memSPXResults.Lines.Clear;       // then clear the SPX results
      TestSPX;
    end;
{$ENDIF}
  end;
end;

procedure TfrmCommDiag.PingServer;
var
  i: integer;
begin
  memTCPIPResults.Clear;
  ICMP.Host := cbTCPIPServer.Text;
  try
    for i := 1 to 4 do
    begin
      ICMP.Ping;
      Application.ProcessMessages;
    end;
  except
    memTCPIPResults.Lines.add('Unknown host ' + cbTCPIPServer.Text + '.');
  end;
end;

{****************************************************************
*
*  T e s t D B C o n n e c t
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  None
*
*  Return: None
*
*  Description:  Creates a Database object and performs a
*                database connect test using a specified protocol.
*
*****************************************************************
* Revisions:
*
*****************************************************************}

procedure TfrmCommDiag.TestDBConnect;
var
  lDatabase : TIB_Connection;
  lProto    : TIB_Protocol;
  iSuccess  : Boolean;
  lDBInfo   : TIB_ConnectionInfo;
begin
  lDatabase := Nil;                    // initialize variables
  lDBInfo   := Nil;
  iSuccess  := True;
  lProto    := cpLocal;

  Screen.Cursor := crHourGlass;

  // if the local radio button is checked then set protocol to local
  if rbLocalServer.Checked or ((cbProtocol.ItemIndex < 0) and
     (rbLocalServer.Checked = False)) then
    lProto:=cpLocal
  else
  begin
    // otherwise determine the specified protocol
    case cbProtocol.ItemIndex of
      0 : lProto:=cpTCP_IP;
      1 : lProto:=cpNetBEUI;
      2 : lProto:=cpNovell;
    end;
  end;

  try
    // create the database object
    lDatabase := TIB_Connection.Create(Self);
    lDBInfo   := TIB_ConnectionInfo.Create(Self);
    try
(*      // setup database path according to the specified protocol
      case lProto of
        cpTCP_IP      : lDatabase.DatabaseName := Format('%s:%s',[UpperCase(), edtDatabase.Text]);
        cpNetBEUI : lDatabase.DatabaseName := Format('\\%s\%s',[UpperCase(cbDBServer.Text), edtDatabase.Text]);
        cpNovell       : lDatabase.DatabaseName := Format('%s@%s',[UpperCase(cbDBServer.Text), edtDatabase.Text]);
        cpLocal     : lDatabase.DatabaseName := edtDatabase.Text;
      end;*)

      // supply parameters to the database
      lDatabase.LoginPrompt:=False;
      lDatabase.Username:=edtUsername.Text;
      lDatabase.Password:=edtPassword.Text;
      lDatabase.Protocol:=lProto;
      lDatabase.Server := cbDBServer.Text;
      lDatabase.Database := edtDatabase.Text;
      lDatabase.Protocol:=lProto;
      // connect to database
      lDatabase.Connected:=True;

      // show database name
      lDBInfo.Database := lDatabase;
      memDBResults.Lines.Add(GetPrompt(84));
      memDBResults.Lines.Add(lDatabase.DatabaseName);
      memDBResults.Lines.Add(Format('%s : %s', [getLangText('Version'),lDBInfo.Version]));
      memDBResults.Lines.Add('');

      // test attach - if connected then attach was successful
      if lDatabase.VerifyConnection then
        memDBResults.Lines.Add(GetPrompt(85));

      try
        // test detach - detach from database
        lDatabase.Connected:=False;
        // if not connected then detach was successful
        if not lDatabase.Connected then
          memDBResults.Lines.Add(getprompt(86));
      except
        on E : EIB_ISCError do
        begin
          // if an error occurs while detaching then show message
          memDBResults.Lines.Add(getprompt(87));
          memDBResults.Lines.Add(getLangText('Error')+' - ' + E.Message);
          memDBResults.Lines.Add('');
          iSuccess:=False;             // set success flag to false
        end;
      end;
    except
      on E : EIB_ISCError do
      begin
        // if an error occurs while attaching then show message
        memDBResults.Lines.Add(GetPrompt(88));
        memDBResults.Lines.Add(getLangText('Error')+' - ' +E.Message);
        iSuccess:=False;               // set success flag to false
      end;
    end;
  finally
    with memDBResults.Lines do
    begin
      if iSuccess then                 // show appropriate message
      begin                            // depending on Success flag
        Add('');
        Add(getprompt(89));
      end
      else
        Add(getprompt(90));
    end;
    // deallocate memory
    lDatabase.Free;
    lDBInfo.Free;
    Screen.Cursor := crDefault;
  end;
end;

{****************************************************************
*
*  T e s t P o r t
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  String - name or number of port service
*
*  Return: None
*
*  Description:  Creates a socket object and performs a
*                port/service test using TCP/IP.
*
*****************************************************************
* Revisions:
*
*****************************************************************}

procedure TfrmCommDiag.TestPort(Port : String);
var
  iSuccess : Boolean;                  // test successful
begin
  inTCP.Host := cbTCPIPServer.Text;
  if Port = 'gds_db' then
    inTCP.Port := 3050
  else
    if Port = 'ftp' then
      inTCP.Port := 21
    else
      inTCP.Port := StrToInt(Port);
  iSuccess := True;
  try
    with memTCPIPResults.Lines do
    begin
      Add(Format(GetPrompt(91),[cbTCPIPServer.Text]));
      Add(GetPrompt(92));
      Add('');

      try
        // connect to server via the specified service/port
        inTCP.Connect;
      except
        on E : EIdSocksRequestFailed do
        begin
          // if a socket error occurs then set success flag to false
          Add(GetPrompt(93));
          iSuccess:=False;
        end;
        on E : EIdSocksRequestServerFailed do
        begin
          // if a socket error occurs then set success flag to false
          Add(GetPrompt(94));
          iSuccess:=False;
        end;
        on E : EIdSocksServerRespondError do
        begin
          // if a socket error occurs then set success flag to false
          Add(GetPrompt(95));
          iSuccess:=False;
        end;
        on E : Exception do
        begin
          // if a socket error occurs then set success flag to false
          Add(Format(GetPrompt(96),[cbTCPIPServer.Text]));
          Add(Format(GetPrompt(97),[Port]) + E.Message + '.');
          iSuccess:=False;
        end;
      end;

      // if the connectin is successful
      if inTCP.Connected then
      begin
        Add(format(GetPrompt(98),[cbTCPIPServer.Text]));
        Add(format(GetPrompt(99),[Port]));
        inTCP.Disconnect;
      end;
    end;
  finally
    with memTCPIPResults.Lines do
    begin
      Add('');
      if iSuccess then
        Add(GetPrompt(100))
      else
        Add(GetPrompt(101));
    end;
  end;
end;

// when the user presses the enter key then execute test task
procedure TfrmCommDiag.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if ord(key)=13 then
  begin
    btnTestClick(sender);
    key:=char(0);
  end;
end;

{****************************************************************
*
*  F o r m C r e a t e
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  Tobject
*
*  Return: None
*
*  Description: Loads registered remote servers into the
*               combo boxes (also stores their protocols).
*
*****************************************************************
* Revisions:
*
*****************************************************************}

procedure TfrmCommDiag.FormCreate(Sender: TObject);
var
  lCount : Integer ;
  lServerAliases : TStringList;
  sProps: TibcServerProps;
begin
  inherited;
  HelpContext := FEATURES_DIAGNOSTICS;
  FServers := Nil;
  FProtocols := Nil;
  lServerAliases :=Nil;
  pgcDiagnostics.ActivePageIndex := 0;
  try
    FServers := TStringList.Create;
    FProtocols := TStringList.Create;
    lServerAliases := TStringList.Create;

    // if server entries are found
    begin
      // get all server aliases
      gPersistentInfo.GetServerAliases(lServerAliases);
      // loop through list of aliases to get server names
      for lCount := 0 to lServerAliases.Count - 1 do
      begin
        gPersistentInfo.GetSerVerProps(lServerAliases[lCount], sProps);
          // Only add remote servers (and their protocol) to stringlists
        if sProps.ServerName <> 'Local Server' then
        begin
          FServers.Add(sProps.ServerName);
          FProtocols.Add(IntToStr(integer(sProps.Protocol)));
          cbDBServer.Items.Add(sProps.ServerName);
          cbTCPIPServer.Items.Add(sProps.ServerName);
          cbNetBEUIServer.Items.Add(sProps.ServerName);
          cbSPXServer.Items.Add(sProps.ServerName);
        end;
      end;
    end;

    // add remote servers to all server combo boxes
    for lCount := 0 to FServers.Count - 1 do
    begin
      cbDBServer.Items.Add(FServers.Strings[lCount]);
      cbTCPIPServer.Items.Add(FServers.Strings[lCount]);
      cbNetBEUIServer.Items.Add(FServers.Strings[lCount]);
      cbSPXServer.Items.Add(FServers.Strings[lCount]);
    end; 
  finally
    lServerAliases.Free;
  end;
{$IFDEF LINUX}
  tabNetBEUI.Visible := false;
  tabSPX.Visible := false;
{$ENDIF}
end;

// deallocate registry and stringlists when form is closed
procedure TfrmCommDiag.FormDestroy(Sender: TObject);
begin
  FProtocols.Free;
end;

{****************************************************************
*
*  T e s t N e t B E U I
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  None
*
*  Return: None
*
*  Description:  Creates a pipes object and performs a
*                NetBEUI test using named pipes.
*
*****************************************************************
* Revisions:
*
*****************************************************************}

procedure TfrmCommDiag.TestNetBEUI;
{$IFDEF MSWINDOWS}
var
  lPipe : TibcPipes;                   // named pipe
  lStr : String;
begin
  lPipe := Nil;                        // initialize variables
  lStr:='';
  Screen.Cursor := crHourGlass;
  try
    lPipe := TibcPipes.Create;         // create pipe

    // specify server name
    lPipe.Server := cbNetBEUIServer.Text;
    // specify pipe name
    lPipe.Path := '\pipe\interbas\server\gds_db';
    // specify number of attempts
    lPipe.Tries := 5;

    // test pipe
    lPipe.TestPipe(lStr, False);

    // assign results to NetBEUI results memo
    memNetBEUIResults.SetTextBuf(PChar(lStr));
  finally
    // deallocate memery
    lPipe.Free;
    Screen.Cursor := crDefault;
  end;
{$ENDIF}
{$IFDEF LINUX}
begin
{$ENDIF}
end;

{****************************************************************
*
*  T e s t S P X
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  None
*
*  Return: None
*
*  Description:  Creates a SPX object and performs a
*                test server connect using SPX.
*
*****************************************************************
* Revisions:
*
*****************************************************************}

procedure TfrmCommDiag.TestSPX;
{$IFDEF MSWINDOWS}
var
  lspx : TibcSPX;
  lStr : String;
begin
  lspx := Nil;                         // initialize variables
  lStr:='';

  Screen.Cursor := crHourGlass;

  try
    lspx := TibcSPX.Create;

    // specify servername
    lspx.ServerName:=cbSPXServer.Text;
    // test SPX connection
    lspx.TestSPX(lStr);

    // assign results to SPX results memo
    memSPXResults.SetTextBuf(PChar(lStr));
  finally
    // deallocate memory
    lspx.Free;
    Screen.Cursor := crDefault;
  end;
{$ENDIF}
{$IFDEF LINUX}
begin
{$ENDIF}
end;

// assigns appropriate protocol for a selected server in the DB connection tab
procedure TfrmCommDiag.cbDBServerClick(Sender: TObject);
begin
  cbProtocol.ItemIndex :=
    StrToInt(FProtocols[FServers.IndexOf(cbDBServer.Text)]);
end;

procedure TfrmCommDiag.edtDatabaseChange(Sender: TObject);
begin
  edtDatabase.Hint := edtDatabase.Text;
end;

procedure TfrmCommDiag.ICMPReply(ASender: TComponent;
  const AReplyStatus: TReplyStatus);
var
  sTime: string;
begin
  // TODO: check for error on ping reply (ReplyStatus.MsgType?)
  if (AReplyStatus.MsRoundTripTime = 0) then
    sTime := '<1'
  else
    sTime := '=';

  if memTCPIPResults.Lines.Count = 0 then
  begin
    memTCPIPResults.Lines.Add(format(GetPrompt(102),[ICMP.Host,AReplyStatus.FromIpAddress,ICMP.BufferSize]));
    memTCPIPResults.Lines.Add('');
  end;

  memTCPIPResults.Lines.Add(Format(GetPrompt(103),
    [AReplyStatus.BytesReceived, AReplyStatus.FromIpAddress,
     AReplyStatus.SequenceId, AReplyStatus.TimeToLive, sTime,
     AReplyStatus.MsRoundTripTime]));
end;

end.
