{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 * 
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 * 
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 * 
 * Contributor(s): Krzysztof Golko, Sergey Gavrilev.
}

unit frmuDBConnect;

interface

uses
{$IFDEF LINUX}
  QForms, QExtCtrls, QStdCtrls, QControls,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Forms, ExtCtrls, StdCtrls, Controls,
{$ENDIF}
  dmuNode, SysUtils, Classes, zluibcClasses, IBOConsoleForm, IB_Schema;

type
  TfrmDBConnect = class(TIBOConsoleForm)
    lblDatabaseName: TLabel;
    bvlLine1: TBevel;
    lblUsername: TLabel;
    lblPassword: TLabel;
    lblRole: TLabel;
    Label1: TLabel;
    cbDialect: TComboBox;
    btnConnect: TButton;
    btnCancel: TButton;
    edtRole: TEdit;
    edtPassword: TEdit;
    edtUsername: TEdit;
    cbCaseSensitive: TCheckBox;
    cbCharacterSet: TComboBox;
    Label2: TLabel;
    lblDatabase: TLabel;
    procedure btnCancelClick(Sender: TObject);
    procedure btnConnectClick(Sender: TObject);
    procedure edtRoleChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    function VerifyInputData(): boolean;
  public
    { Public declarations }
  end;

function IBCDBConnect(CurrSelDatabase: TibcDatabaseNode; const CurrSelServer: TibcServerNode; const SilentConnect: boolean): boolean;

implementation

uses
  IBOServices, frmuMessage, IBOCGlobal, zluContextHelp, IB_Session;

{$IF Declared(Forms)}
{$R *.dfm}
{$IFEND}

{$IF Declared(QForms)}
{$R *.xfm}
{$IFEND}

function DoDBConnect(const CurrSelServer: TibcServerNode; CurrSelDatabase: TibcDatabaseNode): boolean;
begin
  // check if not already connected
  if CurrSelDatabase.DataModule.Connection.Connected then
  begin
    Result := TRUE;
    Exit;
  end;

  try
    CurrSelDatabase.DataModule.Connection.Params.Clear;
      // check if a database file has been specified
    if CurrSelDatabase.DatabaseFiles.Count > 0 then
    begin
      // construct UNC path to database file
      CurrSelDatabase.DataModule.Connection.Server := CurrSelServer.ServerName;
      CurrSelDatabase.DataModule.Connection.Database := CurrSelDatabase.DatabaseFiles.Strings[0];
      CurrSelDatabase.DataModule.Connection.Protocol := CurrSelServer.Server.Protocol;
      case CurrSelServer.Server.Protocol of
        cpLocal:
        begin
          CurrSelDatabase.DataModule.Connection.Server := '';
        end;
      end;
    end;

    // clear database parameters and submit login details
    CurrSelDatabase.DataModule.Connection.Username:=CurrSelDatabase.UserName;
    CurrSelDatabase.DataModule.Connection.Password:=CurrSelDatabase.Password;
    CurrSelDatabase.DataModule.Connection.SQLRole := CurrSelDatabase.Role;

    if (CurrSelDatabase.CharacterSet <> '') and (CurrSelDatabase.CharacterSet <> 'None') then
      CurrSelDatabase.DataModule.Connection.CharSet := CurrSelDatabase.CharacterSet;

//lm    if CurrSelDatabase.PatchBlankCharset then
//lm      CurrSelDatabase.DataModule.Connection.CharSet := '';

    Screen.Cursor := crHourGlass;
    CurrSelDatabase.DataModule.Connection.Connected := true;
    Screen.Cursor := crDefault;
  except                               // if an exception occurs then trap it
    on E:Exception do                   // and show error message
    begin
      Screen.Cursor := crDefault;
      DisplayMsg(ERR_DB_CONNECT, E.Message);
    end;
  end;
  Result := CurrSelDatabase.DataModule.Connection.Connected;
end;

{  Input:  CurrSelDatabase - The specified database[  CurrSelServer   - The specified server
   SilentConnect - Indicates whether or not to prompt the user for login information or
                   to use the default login information.
  Return: Boolean - Indicates the success/failure of the operation
  Description:  Connects to the specified database of the specified server.
                If SilentConnect is false the user is prompted for login information. }
function IBCDBConnect(CurrSelDatabase: TibcDatabaseNode;
  const CurrSelServer: TibcServerNode;
  const SilentConnect: boolean): boolean;
var
  frmDBConnect: TfrmDBConnect;
begin
  frmDBConnect := nil;
  CurrSelDatabase.UserName := CurrSelServer.UserName;
  CurrSelDatabase.Password := CurrSelServer.Password;
  if CurrSelServer.SqlRole <> '' then
    CurrSelDatabase.Role := CurrSelServer.SqlRole;
  CurrSelDatabase.DataModule.Connection.SQLDialect := 3;
  if CurrSelDatabase.CaseSensitiveRole then
    CurrSelDatabase.DataModule.Connection.SQLDialect := 3;

  // check if a SilentConnect is specified
  if not SilentConnect then
  begin
    try
      frmDBConnect:= TfrmDBConnect.Create(Application.MainForm);

      frmDBConnect.lblDatabase.Caption := CurrSelDatabase.NodeName;
      frmDBConnect.edtUsername.Text := CurrSelDatabase.UserName;
      frmDBConnect.edtRole.Text := CurrSelDatabase.Role;
      frmDBConnect.cbDialect.ItemIndex := 2; // default to dialect 3
      try
        if not CurrSelDatabase.DataModule.Connection.Connected then
        begin
          CurrSelDatabase.DataModule.Connection.Connect;
        end;
        CurrSelDatabase.DataModule.GetCharSetNames(frmDBConnect.cbCharacterSet.Items);
        frmDBConnect.cbCharacterSet.ItemIndex := frmDBConnect.cbCharacterSet.Items.IndexOf(CurrSelDatabase.CharacterSet);
      except
        //Lorenzo: eat exception to get a canche to show window amd connect
      end;
      Result := FALSE;
      while frmDBConnect.ShowModal = mrOK do
      begin
        // set username, password and role
        CurrSelDatabase.UserName := frmDBConnect.edtUsername.Text;
        CurrSelDatabase.Password := frmDBConnect.edtPassword.Text;
        CurrSelDatabase.CharacterSet := frmDBConnect.cbCharacterSet.Text;
        CurrSelDatabase.Role := frmDBConnect.edtRole.Text;
        // get SQL dialect
        CurrSelDatabase.DataModule.Connection.SQLDialect := frmDBConnect.cbDialect.ItemIndex + 1;
        // upgrade SQL dialect if case sensitive role name
        if frmDBConnect.cbCaseSensitive.Checked then
          CurrSelDatabase.DataModule.Connection.SQLDialect := 3;

        if DODBConnect(CurrSelServer, CurrSelDatabase) then
        begin
          Result := true;
          Break;
        end;
      end
    finally
      frmDBConnect.Free;
    end;
  end
  else
  begin
    Result := DoDBConnect(CurrSelServer, CurrSelDatabase);
  end;

  if Result then
  begin
    gAppSettings[DEFAULT_DIALECT] := CurrSelDatabase.DataModule.Connection.SQLDialect;
    gAppSettings[CHARACTER_SET] := CurrSelDatabase.CharacterSet;
  end;
end;

procedure TfrmDBConnect.btnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TfrmDBConnect.btnConnectClick(Sender: TObject);
begin
  if VerifyInputData() then
    ModalResult := mrOK;
end;

{****************************************************************
*
*  V e r i f y I n p u t D a t a ( )
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  None
*
*  Return: Boolean - Indicates the success/failure of the operation
*
*  Description:  Performs some basic validation on data entered by
*                the user
*
*****************************************************************
* Revisions:
*
*****************************************************************}
function TfrmDBConnect.VerifyInputData(): boolean;
begin
  result := true;

  // if username was not specified
  if (edtUsername.Text = '') or (edtUsername.Text = ' ') then
  begin
    DisplayMsg(ERR_USERNAME,'');       // show error message
    edtUsername.SetFocus;              // give focus to control
    result := false;
    Exit;
  end;

  // if password was not specified
  if (edtPassword.Text = '') or (edtPassword.Text = ' ') then
  begin
    DisplayMsg(ERR_PASSWORD,'');       // show error message
    edtPassword.SetFocus;              // give focus to control
    result := false;
    Exit;
  end;
end;

procedure TfrmDBConnect.edtRoleChange(Sender: TObject);
begin
  inherited;
  cbCaseSensitive.Enabled := (edtRole.Text <> '');
end;

procedure TfrmDBConnect.FormCreate(Sender: TObject);
begin
  inherited;
  HelpContext := DATABASE_CONNECT;
  with cbDialect do begin
    Clear;
    Items.Add('1');
    Items.Add('2');
    Items.Add('3');
  end;
  with cbCharacterSet do
  begin
    // This is temporary, list of vailable character sets is set in IBX
    // and in another place in IBOConsole, it should be provided by IBX
    Items.Add('None');
    Items.Add('ASCII');
    Items.Add('BIG_5');
    Items.Add('CYRL');
    Items.Add('DOS437');
    Items.Add('DOS850');
    Items.Add('DOS852');
    Items.Add('DOS857');
    Items.Add('DOS860');
    Items.Add('DOS861');
    Items.Add('DOS863');
    Items.Add('DOS865');
    Items.Add('EUCJ_0208');
    Items.Add('GB_2312');
    Items.Add('ISO8859_1');
    Items.Add('KSC_5601');
    Items.Add('NEXT');
    Items.Add('OCTETS');
    Items.Add('SJIS_0208');
    Items.Add('UNICODE_FSS');
    Items.Add('WIN1250');
    Items.Add('WIN1251');
    Items.Add('WIN1252');
    Items.Add('WIN1253');
    Items.Add('WIN1254');
  end;
end;

procedure TfrmDBConnect.FormShow(Sender: TObject);
begin
  if start then
  begin
//    if Assigned(Datamodule)
  end
end;

end.
