{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 * 
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 * 
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 * 
 * Contributor(s): Krzysztof Golko.
}

unit frmuDBConnections;

interface

uses
{$IFDEF LINUX}
  QForms, QStdCtrls, QControls, QComCtrls,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Forms, ExtCtrls, StdCtrls, ComCtrls, Controls,
{$ENDIF}
  Classes, zluibcClasses,  IBDatabase, SysUtils, IBDatabaseInfo, zluContextHelp,
  IBOServices, IB_Components, IBOConnectionInfo, frmuMessage, IB_Session, IBOConsoleForm;

type
  TfrmDBConnections = class(TIBOConsoleForm)
    lvConnections: TListView;
    btnOK: TButton;
    procedure btnOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

function ViewDBConnections(const CurrSelServer: TibcServerNode; const CurrDatabase: TIB_Connection): boolean;

implementation

uses
  IBOCGlobal, zluUtility;

{$IF Declared(Forms)}
{$R *.dfm}
{$IFEND}

{$IF Declared(QForms)}
{$R *.xfm}
{$IFEND}

{****************************************************************
*
*  V i e w D B C o n n e c t i o n s
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input: TIB_Connection -  Database for which a list of connections
*                        is requested
*
*  Return: FAILURE on database login failure, SUCCESS otherwise
*
*  Description:  Displays the DB Connections form and fills in the
*                list of connected users.
*
*****************************************************************
* Revisions:
*
*****************************************************************}
function ViewDBConnections(const CurrSelServer: TibcServerNode; const CurrDatabase: TIB_Connection): boolean;
var
  frmDBConnections: TfrmDBConnections;
  lIBDBInfo: TIB_ConnectionInfo;
  lUserName: TListItem;
  i: integer;
  lDatabase : TIB_Connection;
begin
  lDatabase := nil;
  lIBDBInfo := nil;
  frmDBConnections := nil;
  try
    Screen.Cursor := crHourGlass;
    frmDBConnections := TfrmDBConnections.Create(Application);
    lDatabase := TIB_Connection.Create(frmDBConnections);
    lIBDBInfo := TIB_ConnectionInfo.Create(frmDBConnections);
    try
      lDatabase.DatabaseName := CurrDatabase.DatabaseName;
      lDatabase.LoginPrompt := false;
      lDatabase.Username := CurrSelServer.UserName;
      lDatabase.Password := CurrSelServer.Password;
      lDatabase.Connected := true;
      Application.ProcessMessages;
      result := true;
    except
      on E:EIB_Error do
      begin
        DisplayMsg(ERR_DB_CONNECT,E.Message);
        result := false;
        exit;
      end;
    end;

    lIBDBInfo.Database := lDatabase;

    for i := 1 to lIBDBInfo.UserNames.Count - 1 do
    begin
      lUserName := frmDBConnections.lvConnections.Items.Add;
      lUserName.Caption := lIBDBInfo.UserNames[i];
    end;

{$ifdef initlangfile} WriteAllCaptionAndText('',frmDBConnections,gTransIniFile); {$endif}
    frmDBConnections.ShowModal;
    result := true;

  finally
    if lDatabase.Connected then
      lDatabase.Connected := false;
    Application.ProcessMessages;
    lDatabase.Free;
    lIBDBInfo.Free;
    frmDBConnections.Free;
    Screen.Cursor := crDefault;
  end;
end;

{****************************************************************
*
*  b t n O K C l i c k
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input: ignored
*
*  Description:  Closes the form with a modal result of mrOK.
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBConnections.btnOKClick(Sender: TObject);
begin
  ModalResult := mrOK;
end;

procedure TfrmDBConnections.FormCreate(Sender: TObject);
begin
  HelpContext := DATABASE_ACTIVITY;
end;

end.
