{************************************************************************
  This Unit was created  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  Under the GPL License from Free Software Foundation
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

************************************************************************}
unit frmuDBFind;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, BrowseFolder, TreeScan, ExtCtrls, ComCtrls, Buttons, StdCtrls,
  IBOConsoleForm, IBOCGlobal;

type
  TFormDBFind = class(TIBOConsoleForm)
    DirTreeView: TTreeView;
    Panel1: TPanel;
    Panel2: TPanel;
    LabelFound: TLabel;
    EditStartPath: TEdit;
    ButtonBrowseFolder: TButton;
    ButtonStartSearch: TButton;
    BitBtnOk: TBitBtn;
    BitBtnCancel: TBitBtn;
    Panel3: TPanel;
    Panel4: TPanel;
    LabelStartIn: TLabel;
    CheckBoxGdb: TCheckBox;
    CheckBoxFDB: TCheckBox;
    SpeedButtonExpandAll: TSpeedButton;
    SpeedButtonCollapseAll: TSpeedButton;
    EditMask: TEdit;
    CheckBoxCustom: TCheckBox;
    ButtonStop: TButton;
    LabelSelected: TLabel;
    procedure TreeScanner1FileFound(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ButtonBrowseFolderClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ButtonStartSearchClick(Sender: TObject);
    procedure SpeedButtonExpandAllClick(Sender: TObject);
    procedure SpeedButtonCollapseAllClick(Sender: TObject);
    procedure ButtonStopClick(Sender: TObject);
    procedure DirTreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure BitBtnOkClick(Sender: TObject);
  private
    { Private declarations }
    fCurrent:TTreeNode;
    fDirList:TStringList;
    filecount:integer;
    TreeScanner1: TTreeScanner;
    BrowseFolder1: TBrowseFolder;
    fLocFiles:String;
    fSelectedFile:String;
    function GetTreeDir(const p: string): TTreeNode;
  public
    { Public declarations }
    property SelectedFile:String read fSelectedFile;
  end;

var
  FormDBFind: TFormDBFind;

implementation

{$R *.dfm}

function TrimLastPD(const s:String):string;
begin
  Result := s;
  if Result = '' then exit;
  if Result[Length(Result)] = PathDelim then
    Delete(Result,Length(Result),1);
end;

function GetLastDir(const s:String):string;
var n,i:integer;
begin
  Result := s;
  if Result = '' then exit;
  for i := Length(result) downto 1 do
  if (result[i]=PathDelim) then
  begin
    n:=i;
    Result := copy(Result,n+1,MaxInt);
    break;
  end;
end;

function SearchChild(const x:TTreeNode; const s:string):TTreeNode;
var
  n:Integer;
begin
  Result := Nil;
  if Assigned(x) then
  begin
    for n:=0 to x.Count-1 do
    if x.Item[n].Text = s then
    begin
      Result := x.Item[n];
      break;
    end;
  end;
end;

function TFormDBFind.GetTreeDir(const p:string):TTreeNode;
var sdir,spath:string;
    n:integer;
    nparent,x:TTreeNode;
begin
  if fDirList.Find(p,n) then
  begin
    Result := TTreeNode(fDirList.Objects[n]);
  end
  else
  begin
    Result := Nil;
    if p = DirTreeView.Items[0].Text then
      Result := DirTreeView.Items[0]
    else
    begin
      nparent := DirTreeView.Items[0];
      spath := copy(p,length(nparent.Text)+1,MaxInt);
      while assigned(nparent) and (spath<>'') do
      begin
        if (spath<>'') and (spath[1] = PathDelim) then Delete(spath,1,1);
        n := pos(PathDelim,spath);
        if n>0 then
        begin
          sdir := copy(spath,1,n-1);
          delete(spath,1,n); //spath := copy(spath,n+1,MaxInt);
        end
        else
        begin
          sdir := spath;
          spath := '';
        end;
        if (sdir<>'') then
          x := SearchChild(nparent, sdir)
        else
          x:=nil;
        if not assigned(x) then
        begin
          x := DirTreeView.Items.AddChild(nparent,sdir);
          fDirList.AddObject(p,x);
        end;
        nparent := x;
      end;
      if Assigned(nparent) and (spath='') then
        Result := nparent;
    end;
  end;
end;

procedure TFormDBFind.TreeScanner1FileFound(Sender: TObject);
var p,f:string;
    fDir:TTreeNode;
begin
  inc(filecount);
  p:=TrimLastPD(ExtractFilePath(TreeScanner1.FileFound));
  f:=ExtractFileName(TreeScanner1.FileFound);
  fDir:= GetTreeDir(p);
  if Assigned(fDir) then
    DirTreeView.Items.AddChild(fDir,f);
  LabelFound.Caption := fLocFiles+' '+IntToStr(filecount);
end;

procedure TFormDBFind.FormCreate(Sender: TObject);
begin
  fDirList:=TStringList.Create;
  fDirList.Duplicates := dupIgnore;
  fCurrent := DirTreeView.Items[0];
  EditStartPath.Text := '';
  DirTreeView.Items.Clear;

  TreeScanner1 := TTreeScanner.Create(Self);
  TreeScanner1.OnFileFound := TreeScanner1FileFound;

  BrowseFolder1:= TBrowseFolder.Create(Self);
  BrowseFolder1.Flags := [bfFileSysDirsOnly, bfStatusText];
  BrowseFolder1.ShowPathInStatusArea := False;
  BrowseFolder1.SyncCustomButton := False;

  LabelStartIn.Caption := GetLangText('In');
  SpeedButtonExpandAll.Caption := GetLangText('Expand All');
  SpeedButtonCollapseAll.Caption := GetLangText('Collapse All');
  ButtonStartSearch.Caption := GetLangText('Find');
  fLocFiles := GetLangText('Files');
  BitBtnOk.Caption := GetLangText('Ok');
  BitBtnCancel.Caption := GetLangText('Cancel');

end;

procedure TFormDBFind.ButtonBrowseFolderClick(Sender: TObject);
begin
  if BrowseFolder1.Execute then
  begin
    ButtonStartSearchClick(Sender);
  end;
end;

procedure TFormDBFind.FormDestroy(Sender: TObject);
begin
  fDirList.Free;
  if Self = FormDBFind then FormDBFind := nil;
end;

procedure TFormDBFind.ButtonStartSearchClick(Sender: TObject);
begin
    fDirList.Clear;
    DirTreeView.Items.Clear;
    filecount := 0;
    EditStartPath.Text := BrowseFolder1.Directory;
    if EditStartPath.Text = '' then exit;
    DirTreeView.Items.Add(nil,EditStartPath.Text);
    TreeScanner1.InitialDirectory := EditStartPath.Text;
    if CheckBoxFDB.Checked then
    begin
      TreeScanner1.FileMask := '*.fdb';
      TreeScanner1.ScanTree;
    end;
    if CheckBoxGdb.Checked then
    begin
      TreeScanner1.FileMask := '*.gdb';
      TreeScanner1.ScanTree;
    end;
    if CheckBoxCustom.Checked and (EditMask.Text<>'') then
    begin
      TreeScanner1.FileMask := EditMask.Text;
      TreeScanner1.ScanTree;
    end;
end;

procedure TFormDBFind.SpeedButtonExpandAllClick(Sender: TObject);
begin
  if DirTreeView.Items.Count>0 then
    DirTreeView.Items[0].Expand(True);
end;

procedure TFormDBFind.SpeedButtonCollapseAllClick(Sender: TObject);
begin
  if DirTreeView.Items.Count>0 then
    DirTreeView.Items[0].Collapse(True);
end;

procedure TFormDBFind.ButtonStopClick(Sender: TObject);
begin
  if TreeScanner1.SearchInProgress then
  begin
    TreeScanner1.StopScan;
  end;
end;

procedure TFormDBFind.DirTreeViewChange(Sender: TObject; Node: TTreeNode);
var s:string;
    x: TTreeNode;
begin
  x:= node;
  s:='';
  while Assigned(x) do
  begin
    if s<>'' then
      s:= TrimLastPD(x.Text) + PathDelim + s
    else
      s:= TrimLastPD(x.Text);
    x := x.Parent;
  end;
  if FileExists(s) then
    fSelectedFile := s
  else
    fSelectedFile := '';
  LabelSelected.Caption := fSelectedFile;
end;

procedure TFormDBFind.BitBtnOkClick(Sender: TObject);
begin
  if fSelectedFile<>'' then ModalResult := mrOk;
end;

end.
