{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 *
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 *
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 *
 * Contributor(s): Krzysztof Golko.
}

unit frmuDBProperties;

interface

uses
{$IFDEF LINUX}
  Types, Qt, QForms, QExtCtrls, QStdCtrls, QGrids, QComCtrls, QControls,
  QDialogs, QGraphics,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Forms, ExtCtrls, StdCtrls, ComCtrls, Dialogs, Grids,
  Graphics, Controls,
{$ENDIF}
  Classes, SysUtils, zluibcClasses, dmuNode,
  IBOCPersistent, IBDatabaseInfo, zluContextHelp, IB_Session, IBOConsoleForm,
  IBOServices, frmuMessage, Db, IB_Components, IB_Header, IBOConnectionInfo;

type
  TfrmDBProperties = class(TIBOConsoleForm)
    TabAlias: TTabSheet;
    TabGeneral: TTabSheet;
    cbOptions: TComboBox;
    edtAliasName: TEdit;
    edtFilename: TEdit;
    gbSummaryInfo: TGroupBox;
    lblAliasName: TLabel;
    lblDBOwner: TLabel;
    lblDBPages: TLabel;
    lblFilename: TLabel;
    lblOptions: TLabel;
    lblPageSize: TLabel;
    lvSecondaryFiles: TListView;
    pgcMain: TPageControl;
    sgOptions: TStringGrid;
    btnSelFilename: TButton;
    pnlOptionName: TPanel;
    lblServerName: TLabel;
    btnApply: TButton;
    btnCancel: TButton;
    Button1: TButton;
    lblServer: TLabel;
    lblOwner: TLabel;
    lblPages: TLabel;
    lblPgSize: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure cbOptionsChange(Sender: TObject);
    procedure cbOptionsDblClick(Sender: TObject);
    procedure cbOptionsExit(Sender: TObject);
    procedure cbOptionsKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure edtAliasNameChange(Sender: TObject);
    procedure sgOptionsDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
    procedure sgOptionsSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
    procedure btnSelFilenameClick(Sender: TObject);
    procedure edtFilenameChange(Sender: TObject);
    procedure SetDefaults (const readOnly, sweep, synch, dialect: String);
    procedure edtFilenameExit(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    FApplyChanges: boolean;
    FOriginalAlias: string;
    FOriginalReadOnly: string;
    FOriginalSweepInterval: string;
    FOriginalSynchMode: string;
    FOriginalSQLDialect: String;
    FAliaschanged: boolean;
    FErrorState: boolean;
    function  VerifyInputData(): boolean;
{$IFDEF MSWINDOWS}
    procedure SetErrorState(const Value: boolean);
{$ENDIF}
  public
    sOriginalForcedWrites: string;
    sOriginalReadOnly: string;
    sOriginalSweepInterval: string;
    sOriginalSQLDialect: string;
    bOriginalConnectStatus: boolean;
    CurrSelDatabase: TibcDatabaseNode;
    CurrSelServer: TibcServerNode;
    property ErrorState: boolean read FErrorState write SetErrorState;
end;

function EditDBProperties(const CurrSelServer: TibcServerNode; CurrSelDatabase: TibcDatabaseNode): integer;

implementation

uses
  IBOCGlobal, zluUtility, frmuMain, LMMessageDialog;

{$IF Declared(Forms)}
{$R *.dfm}
{$IFEND}

{$IF Declared(QForms)}
{$R *.xfm}
{$IFEND}

const
  OPTION_NAME_COL = 0;
  OPTION_VALUE_COL = 1;
  FORCED_WRITES_ROW = 0;
  SWEEP_INTERVAL_ROW = 1;
  READ_ONLY_ROW = 3;
  SQL_DIALECT_ROW = 2;
  SWEEP_INTERVAL_MIN = 0;
  SWEEP_INTERVAL_MAX = 200000;
  SQL_DIALECT1 = '1';
  SQL_DIALECT2 = '2';
  SQL_DIALECT3 = '3';

var
  FORCED_WRITES_TRUE:string='Enabled';
  FORCED_WRITES_FALSE:string = 'Disabled';
  READ_ONLY_TRUE:string = 'True';
  READ_ONLY_FALSE:string = 'False';


{****************************************************************
*
*  F o r m C r e a t e
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input: Sender - The object that initiated the event
*
*  Return:
*
*  Description:  This procedure creates an instance of the TfrmDBProperties
*                class and fills in some properties
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBProperties.FormCreate(Sender: TObject);
begin
  inherited;
  HelpContext := DATABASE_PROPERTIES;
  FORCED_WRITES_TRUE:=GetLangText('Enabled');
  FORCED_WRITES_FALSE:=GetLangText('Disabled');
  READ_ONLY_TRUE:=GetLangText('True');
  READ_ONLY_FALSE:=GetLangText('False');

  FApplyChanges := false;
  FAliasChanged := false;
  sgOptions.DefaultRowHeight := cbOptions.Height;
  sgOptions.RowCount := 4;
  sgOptions.Cells[OPTION_NAME_COL,FORCED_WRITES_ROW] :=  GetLangText('Forced Writes');
  sgOptions.Cells[OPTION_NAME_COL,SWEEP_INTERVAL_ROW] := GetLangText('Sweep Interval');
  sgOptions.Cells[OPTION_NAME_COL,SQL_DIALECT_ROW] :=    GetLangText('Database Dialect');
  sgOptions.Cells[OPTION_NAME_COL,READ_ONLY_ROW] :=      GetLangText('Read Only');

  cbOptions.Visible := True;
  pnlOptionName.Visible := True;
  btnApply.Enabled := false;
end;

{****************************************************************
*
*  F o r m S h o w
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input: Sender - The object that initiated the event
*
*  Return:
*
*  Description:  Assigns initial values of editable form items
*                for use in determining if changes are made and
*                if the user enters valid combinations of input.
*                Sets up the combobox to prevent blank string grid
*                cells from occurring.
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBProperties.FormShow(Sender: TObject);
begin
  FOriginalAlias := edtAliasName.Text;
  pnlOptionName.Caption := GetLangText('Forced Writes');
  cbOptions.Style := csDropDown;
  cbOptions.Items.Add(FORCED_WRITES_TRUE);
  cbOptions.Items.Add(FORCED_WRITES_FALSE);
  cbOptions.ItemIndex := cbOptions.Items.IndexOf(FOriginalSynchMode);
  cbOptions.Tag := FORCED_WRITES_ROW;
  btnApply.Enabled := false;
  FAliasChanged := false;
end;

{****************************************************************
*
*  b t n A p p l y C l i c k
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input: Sender - The object that initiated the event
*
*  Return:
*
*  Description:  This procedure verifies user entries and closes the
*                form when the user clicks on the Apply button
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBProperties.btnApplyClick(Sender: TObject);
var
  i: integer;
  lConfigService: TIBOConfigService;
  dbProps: TibcDatabaseProps;

begin
  if VerifyInputData() then
  begin
    Screen.Cursor := crHourGlass;
    try
      // save alias and database file information
      lConfigService := TIBOConfigService.Create(self);

      CurrSelDatabase.DatabaseFiles.Clear;
      CurrSelDatabase.DatabaseFiles.Add(edtFilename.Text);

      for i := 0 to lvSecondaryFiles.Items.Count-1 do
        CurrSelDatabase.DatabaseFiles.Add(lvSecondaryFiles.Items[i].Caption);

      // kris Kylix
      if CurrSelDatabase.NodeName <> edtAliasName.Text then
        gPersistentInfo.RenameDatabaseAlias(CurrSelServer.NodeName, CurrSelDatabase.NodeName, edtAliasName.Text);
      gPersistentInfo.GetDatabaseProps(CurrSelServer.NodeName, edtAliasName.Text, dbProps);
      dbProps.DatabaseFiles := edtFilename.Text;
//lm      dbProps.PatchBlankCharset := CheckBox1.Checked;
      gPersistentInfo.StoreDatabaseProps(CurrSelServer.NodeName, edtAliasName.Text, dbProps);
      // kris end

      CurrSelDatabase.NodeName := edtAliasName.Text;
      frmMain.RenameTreeNode(CurrSelDatabase, edtAliasName.Text);

      // Set properties if general tab was shown

      if TabGeneral.TabVisible then
      begin
        try  // try to connect to configuration service
          lConfigService.DatabaseName := CurrSelDatabase.DataModule.Connection.DatabaseName;
          lConfigService.LoginPrompt := false;
          lConfigService.ServerName := CurrSelServer.Servername;
          lConfigService.Protocol := CurrSelServer.Server.Protocol;
          lConfigService.Params.Add(Format('isc_spb_user_name=%s', [CurrSelDatabase.UserName]));
          lConfigService.Params.Add(Format('isc_spb_password=%s', [CurrSelDatabase.Password]));
          lConfigService.Attach();
        except
          on E:EIB_ISCError do
          begin
            DisplayMsg(ERR_SERVER_LOGIN, E.Message);
            if (E.ERRCODE = isc_lost_db_connection) or
               (E.ERRCODE = isc_unavailable) or
               (E.ERRCODE = isc_network_error) then
              frmMain.ErrorState:= True;
            ErrorState:= True;
            Screen.Cursor := crDefault;
            Exit;
          end;
        end;

        if lConfigService.Active then // if attached successfully
        begin
          try
            // Toggle Read-Only first if changing from Read_Only
            if ((sgOptions.Cells[OPTION_VALUE_COL,READ_ONLY_ROW] <> sOriginalReadOnly) and
             (sOriginalReadOnly = READ_ONLY_TRUE))   then
            begin
              CurrSelDatabase.DataModule.Connection.Connected := False;  // need to disconnect from database
              if not lConfigService.Active then
                lConfigService.Attach();

              if lConfigService.Active then  // if attached successfully
              begin
                lConfigService.SetReadOnly(False);  // toggle original value
                CurrSelDatabase.DataModule.Connection.Connected := bOriginalConnectStatus;
              end;
            end; // end if read-only changed

            // Set sweep interval if changed
            if sgOptions.Cells[OPTION_VALUE_COL,SWEEP_INTERVAL_ROW] <> sOriginalSweepInterval then
            begin
              lConfigService.SetSweepInterval(StrToInt(sgOptions.Cells[OPTION_VALUE_COL,SWEEP_INTERVAL_ROW]));
              while (lConfigService.IsServiceRunning) and (not gApplShutdown) do
                Application.ProcessMessages;
            end;

            // Set SQL Dialect if changed
            if sgOptions.Cells[OPTION_VALUE_COL,SQL_DIALECT_ROW] <> sOriginalSQLDialect then
            begin
              try
                lConfigService.SetDBSqlDialect (StrToInt(sgOptions.Cells[OPTION_VALUE_COL,SQL_DIALECT_ROW]));
                while (lConfigService.IsServiceRunning) and (not gApplShutdown) do
                  Application.ProcessMessages;
              except
                on E : EIB_ISCError do
                begin
                  DisplayMsg(ERR_SERVICE, E.Message);
                  if (E.ERRCODE = isc_lost_db_connection) or
                     (E.ERRCODE = isc_unavailable) or
                     (E.ERRCODE = isc_network_error) then
                    frmMain.ErrorState:= True;
                  ErrorState:= True;
                  Screen.Cursor := crDefault;
                  exit;
                end;
              end;
            end;

            // Set forced writes if changed
            if sgOptions.Cells[OPTION_VALUE_COL,FORCED_WRITES_ROW] <> sOriginalForcedWrites then
            begin
              lConfigService.SetAsyncMode(sOriginalForcedWrites = FORCED_WRITES_TRUE);  // toggle original value
              while (lConfigService.IsServiceRunning) and (not gApplShutdown) do
                Application.ProcessMessages;
            end;

            // Toggle read only if changed
            if ((sgOptions.Cells[OPTION_VALUE_COL,READ_ONLY_ROW] <> sOriginalReadOnly) and
             (sOriginalReadOnly = READ_ONLY_FALSE)) then
            begin
              CurrSelDatabase.DataModule.Connection.Connected := False;  // need to disconnect from database
              try
                if not lConfigService.Active then
                  lConfigService.Attach();
              except
                on E : EIB_ISCError do
                begin
                  DisplayMsg(ERR_SERVER_LOGIN, E.Message);
                  if (E.ERRCODE = isc_lost_db_connection) or
                     (E.ERRCODE = isc_unavailable) or
                     (E.ERRCODE = isc_network_error) then
                  begin
                    frmMain.ErrorState:= True;
                    ErrorState:= True;
                  end
                  else
                    CurrSelDatabase.DataModule.Connection.Connected := True;
                  Screen.Cursor := crDefault;
                  Exit;
                end;
              end;
              try
                if lConfigService.Active then               // if attached successfully
                begin
                  lConfigService.SetReadOnly(True);         // toggle original value
                  CurrSelDatabase.DataModule.Connection.Connected := bOriginalConnectStatus;
                end;
              except
                on E : EIB_ISCError do
                begin
                  DisplayMsg(ERR_SERVER_LOGIN, E.Message);
                  if (E.ERRCODE = isc_lost_db_connection) or
                     (E.ERRCODE = isc_unavailable) or
                     (E.ERRCODE = isc_network_error) then
                  begin
                    frmMain.ErrorState:= True;
                    ErrorState:= True;
                  end
                  else
                    // reconnect to database if an exception occurs
                    CurrSelDatabase.DataModule.Connection.Connected := True;
                  Screen.Cursor := crDefault;
                  Exit;
                end;
              end;
            end; // end if read-only changed
          except
            on E : EIB_ISCError do
            begin
              DisplayMsg(ERR_MODIFY_DB_PROPERTIES, E.Message);
              Screen.Cursor := crDefault;
              Exit;
            end;
          end;

        end; // end successful service start
        if lConfigService.Active then
          lConfigService.Detach();  // finally detach
      end;  // end if connected
    finally
      Screen.Cursor := crDefault;
    end;
  end;  // end if VerifyData
end;

{****************************************************************
*
*  b t n C a n c e l C l i c k
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input: Sender - The object that initiated the event
*
*  Return:
*
*  Description:  This procedure returns a ModalResult of mrCancel
*                whent the user presses the Cancel button, btnCancel
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBProperties.btnCancelClick(Sender: TObject);
begin
  Cursor := crHourGlass;
  btnApply.Click;
  Cursor := crHourGlass;
  ModalResult := mrOK;
end;

{****************************************************************
*
*  c b O p t i o n s C h a n g e
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input: Sender - The object that initiated the event
*
*  Return:
*
*  Description:  This procedure handles changes to the text of the
*                options combo box.  It calls the function NoteChanges
*                to look for and prepare the form to accept changes
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBProperties.cbOptionsChange(Sender: TObject);
begin
  FApplyChanges := True;
  btnApply.Enabled := True;
end;

{****************************************************************
*
*  c b O p t i o n s D b l C l i c k
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input: Sender - The object that initiated the event
*
*  Return:
*
*  Description:  Flips through the items in the combo box,
*                assigning the next value or the first one when the
*                last item is reached.  Notifies the form that changes
*                may have been made via NoteChanges.
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBProperties.cbOptionsDblClick(Sender: TObject);
begin
  if (sgOptions.Col = OPTION_VALUE_COL) or (sgOptions.Col = OPTION_NAME_COL) then
  begin
    if cbOptions.ItemIndex = cbOptions.Items.Count - 1 then
      cbOptions.ItemIndex := 0
    else
      cbOptions.ItemIndex := cbOptions.ItemIndex + 1;

    if sgOptions.Col = OPTION_VALUE_COL then
      sgOptions.Cells[sgOptions.Col,sgOptions.Row] := cbOptions.Items[cbOptions.ItemIndex];
  end;
  FApplyChanges := True;
  btnApply.Enabled := True;
end;

{****************************************************************
*
*  c b O p t i o n s E x i t
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input: Sender - The object that initiated the event
*
*  Return:
*
*  Description:  This procedure adjusts the appearance of the form
*                when the user selects another object on the form
*                while cbOptions has focus.
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBProperties.cbOptionsExit(Sender: TObject);
var
  lR     : Trect;
  iIndex : Integer;
begin
  iIndex := cbOptions.Items.IndexOf(cbOptions.Text);

  if (iIndex = -1) and (sgOptions.Row <> SWEEP_INTERVAL_ROW) then
  begin
    LMMessageDlg(16, mtError, [mbOK], 0);

    cbOptions.ItemIndex := 0;
    //Size and position the combo box to fit the cell
    lR := sgOptions.CellRect(OPTION_VALUE_COL, sgOptions.Row);
    lR.Left := lR.Left + sgOptions.Left;
    lR.Right := lR.Right + sgOptions.Left;
    lR.Top := lR.Top + sgOptions.Top;
    lR.Bottom := lR.Bottom + sgOptions.Top;
    cbOptions.Left := lR.Left + 1;
    cbOptions.Top := lR.Top + 1;
    cbOptions.Width := (lR.Right + 1) - lR.Left;
    cbOptions.Height := (lR.Bottom + 1) - lR.Top;
    cbOptions.Visible := True;
    cbOptions.SetFocus;
  end
  else if (sgOptions.Row = SWEEP_INTERVAL_ROW) then
  begin
    sgOptions.Cells[OPTION_VALUE_COL,sgOptions.Row] := cbOptions.Text;
  end
  else if (sgOptions.Col <> OPTION_NAME_COL) then
  begin
    sgOptions.Cells[sgOptions.Col,sgOptions.Row] := cbOptions.Items[iIndex];
  end
  else
  begin
    sgOptions.Cells[OPTION_VALUE_COL,sgOptions.Row] := cbOptions.Items[iIndex];
  end;
end;

{****************************************************************
*
*  c b O p t i o n s K e y D o w n
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input: Sender - The object that initiated the event
*
*  Return:
*
*  Description: Enables the user to use the keyboard to select
*               items from the combo box.
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBProperties.cbOptionsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
{$IFDEF LINUX}
const
  VK_DOWN = KEY_DOWN;
{$ENDIF}
begin
  if (Key = VK_DOWN) then
    cbOptions.DroppedDown := true;
end;

{****************************************************************
*
*  e d t A l i a s N a m e C h a n g e
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input: Sender - The object that initiated the event
*
*  Return:
*
*  Description:  Notifies the form that changes may have been made.
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBProperties.edtAliasNameChange(Sender: TObject);
begin
  FAliasChanged := true;
  FApplyChanges := True;
  btnApply.Enabled := True;
  edtAliasName.Hint := edtAliasName.Text;
end;

{****************************************************************
*
*  s g O p t i o n s D r a w C e l l
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input: Sender - The object that initiated the event
*
*  Return:
*
*  Description:
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBProperties.sgOptionsDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
const
  INDENT = 2;
var
  lLeft: integer;
  lText: string;
begin
  with sgOptions.canvas do
  begin
    if ACol = OPTION_VALUE_COL then
    begin
      font.color := clBlue;
      if brush.color = clHighlight then
        font.color := clWhite;
      lText := sgOptions.Cells[ACol,ARow];
      lLeft := Rect.Left + INDENT;
      TextRect(Rect, lLeft, Rect.top + INDENT, lText);
    end;
  end;
end;

{****************************************************************
*
*  s g O p t i o n s S e l e c t C e l l
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input: Sender - The object that initiated the event
*
*  Return:
*
*  Description:  This procedure prepares the combobox cbOptions
*                and inserts it into the selected cell.
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBProperties.sgOptionsSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
var
  lR, lName : TRect;
begin
  cbOptionsExit(Sender);
  cbOptions.Items.Clear;

  case ARow of
    FORCED_WRITES_ROW:
    begin
      cbOptions.Style := csDropDown;
      cbOptions.Items.Add(FORCED_WRITES_TRUE);
      cbOptions.Items.Add(FORCED_WRITES_FALSE);
      cbOptions.Tag := FORCED_WRITES_ROW;
    end;
    SWEEP_INTERVAL_ROW:
    begin
{$IFNDEF NOT_YET_IN_KYLIX}
      cbOptions.Style := csSimple;
{$ENDIF}      
      cbOptions.Text := sgOptions.Cells[OPTION_VALUE_COL,SWEEP_INTERVAL_ROW];
      cbOptions.Tag := SWEEP_INTERVAL_ROW;
    end;
    READ_ONLY_ROW:
    begin
      cbOptions.Style := csDropDown;
      cbOptions.Items.Add(READ_ONLY_TRUE);
      cbOptions.Items.Add(READ_ONLY_FALSE);
      cbOptions.Tag := READ_ONLY_ROW;
    end;
    SQL_DIALECT_ROW:
    begin
      cbOptions.Style := csDropDown;
      cbOptions.Items.Add(SQL_DIALECT1);
      cbOptions.Items.Add(SQL_DIALECT2);
      cbOptions.Items.Add(SQL_DIALECT3);
      cbOptions.ItemIndex := StrToInt(FOriginalSQLDialect)-1;
      cbOptions.Tag := SQL_DIALECT_ROW;
    end;
  end;

  pnlOptionName.Caption := sgOptions.Cells[OPTION_NAME_COL, ARow];

  if ACol = OPTION_NAME_COL then
    cbOptions.ItemIndex := cbOptions.Items.IndexOf(sgOptions.Cells[ACOL+1,ARow])
  else if ACol = OPTION_VALUE_COL then
  begin
    cbOptions.ItemIndex := cbOptions.Items.IndexOf(sgOptions.Cells[ACol,ARow]);
    if (cbOptions.ItemIndex = -1) or (ARow = SWEEP_INTERVAL_ROW) then
      cbOptions.Text := sgOptions.Cells[ACol,ARow];
  end;

  if ACol = OPTION_NAME_COL then
  begin
    lName := sgOptions.CellRect(ACol, ARow);
    lR := sgOptions.CellRect(ACol + 1, ARow);
  end
  else
  begin
    lName := sgOptions.CellRect(ACol - 1, ARow);
    lR := sgOptions.CellRect(ACol, ARow);
  end;

  // lName := sgOptions.CellRect(ACol, ARow);
  lName.Left := lName.Left + sgOptions.Left;
  lName.Right := lName.Right + sgOptions.Left;
  lName.Top := lName.Top + sgOptions.Top;
  lName.Bottom := lName.Bottom + sgOptions.Top;
  pnlOptionName.Left := lName.Left + 1;
  pnlOptionName.Top := lName.Top + 1;
  pnlOptionName.Width := (lName.Right + 1) - lName.Left;
  pnlOptionName.Height := (lName.Bottom + 1) - lName.Top;
  pnlOptionName.Visible := True;

  // lR := sgOptions.CellRect(ACol, ARow);
  lR.Left := lR.Left + sgOptions.Left;
  lR.Right := lR.Right + sgOptions.Left;
  lR.Top := lR.Top + sgOptions.Top;
  lR.Bottom := lR.Bottom + sgOptions.Top;
  cbOptions.Left := lR.Left + 1;
  cbOptions.Top := lR.Top + 1;
  cbOptions.Width := (lR.Right + 1) - lR.Left;
  cbOptions.Height := (lR.Bottom + 1) - lR.Top;
  cbOptions.Visible := True;
  cbOptions.SetFocus;
end;

{****************************************************************
*
*  V e r i f y I n p u t D a t a
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input: none
*
*  Return:  Returns TRUE if all data is valid.  Returns FALSE if
*           any data (Sweep Interval particularly) is invalid,
*           or if an invalid combination of values has been provided.
*
*  Description:  This function verifies that valid values have been
*                provided by the user.
*
*****************************************************************
* Revisions:
*
*****************************************************************}

function TfrmDBProperties.VerifyInputData(): boolean;
begin
  result := true;  // only if no exceptions raised

  if FAliasChanged and gPersistentInfo.DatabaseAliasExists(CurrSelServer.ServerName, edtAliasName.Text) then
  begin
    DisplayMsg(ERR_ALIAS_EXISTS, '');
    edtAliasName.text := FOriginalAlias;
    result := false;
    FAliasChanged := false;
  end;
  
  if TabGeneral.Visible then
  try
    if (StrToInt(sgOptions.Cells[OPTION_VALUE_COL,SWEEP_INTERVAL_ROW]) < SWEEP_INTERVAL_MIN) or
       (StrToInt(sgOptions.Cells[OPTION_VALUE_COL,SWEEP_INTERVAL_ROW]) > SWEEP_INTERVAL_MAX) then
      raise ERangeError.Createfmt(GetPrompt(120),[SWEEP_INTERVAL_MIN,SWEEP_INTERVAL_MAX]);
    if ((FOriginalReadOnly = READ_ONLY_TRUE) and
       (sgOptions.Cells[OPTION_VALUE_COL,READ_ONLY_ROW] = READ_ONLY_TRUE) and
       ((sgOptions.Cells[OPTION_VALUE_COL,SWEEP_INTERVAL_ROW] <> FOriginalSweepInterval) or
       (sgOptions.Cells[OPTION_VALUE_COL,READ_ONLY_ROW] = FOriginalReadOnly))) then
      raise EPropReadOnly.Create(GetPrompt(121));
    Exit;
  except
    on E:EConvertError do
    begin
      DisplayMsg(ERR_INVALID_PROPERTY_VALUE, GetLangText('Sweep Interval')+': ' + E.Message );
      result := false;
    end;
    on E:ERangeError do
    begin
      DisplayMsg(ERR_NUMERIC_VALUE, E.Message );
      result := false;
    end;
    on E:EPropReadOnly do
    begin
      DisplayMsg(ERR_INVALID_PROPERTY_VALUE,E.Message);
      result := false;
    end;
  end;
end;

{*******************************************************************
*
* E d i t D B P r o p e r t i e s ( )
*
********************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   April 28, 1999
*
*  Input:  CurrSelServer : TibcServerNode, the current server node
           CurrSelDatabase : TibcDatabaseNode, the current database
*
*  Return: integer - a status code indicating the success/failure
*                    of the operation.
*
*  Description: This procedure creates and displays the database
*               properties form.  The user can then view and make
*               changes to the properties.  When the user closes
*               the form, the function then tests for any changes and
*               applies them to the database.  Finally, the
*               form and all supporting IB objects are destroyed.
*
********************************************************************
* Revisions:
*
********************************************************************}
function EditDBProperties(const CurrSelServer: TibcServerNode; CurrSelDatabase: TibcDatabaseNode): integer;
var
  frmDBProperties: TfrmDBProperties;
  lSubKeys: TStringList;
  lIBDBInfo: TIB_ConnectionInfo;
  lConfigService: TIBOConfigService;
  lListItem: TListItem;
  qryDBProperties: TIB_Cursor;
  sOriginalForcedWrites: string;
  sOriginalReadOnly: string;
  sOriginalSweepInterval: string;
  sOriginalSQLDialect: string;
  bOriginalConnectStatus: boolean;
begin
  frmDBProperties := TfrmDBProperties.Create(Application.MainForm);
  lSubKeys := TStringList.Create();
  lIBDBInfo := TIB_ConnectionInfo.Create(frmDBProperties);
  lConfigService := TIBOConfigService.Create(frmDBProperties);
  qryDBProperties := TIB_Cursor.Create(frmDBProperties);
  try
//lm    frmDBProperties.CheckBox1.Checked := CurrSelDatabase.PatchBlankCharset;
    frmDBProperties.edtAliasName.Text := CurrSelDatabase.NodeName;
    if CurrSelDatabase.DatabaseFiles.Count > 0 then
       frmDBProperties.edtFilename.Text := CurrSelDatabase.DatabaseFiles.Strings[0]
    else
       frmDBProperties.edtFilename.Text := '';
    frmDBProperties.lblServer.Caption := CurrSelServer.NodeName;

    if CurrSelServer.Server.Protocol <> cpLocal then
      frmDBProperties.btnSelFilename.Enabled := false;

    bOriginalConnectStatus := CurrSelDatabase.DataModule.Connection.Connected;
    if not CurrSelDatabase.DataModule.Connection.Connected then
      frmDBProperties.TabGeneral.TabVisible := false
    else
    begin  // retrieve database properties
//lm      frmDBProperties.CheckBox1.Enabled := false;
      frmDBProperties.edtFileName.Enabled := false;
      frmDBProperties.btnSelFilename.Enabled := false;
      lIBDBInfo.Database := CurrSelDatabase.DataModule.Connection;                       // assign selected database to db info object
      frmDBProperties.lblPgSize.Caption := IntToStr(lIBDBInfo.PageSize);  // get page size from ib info object
      frmDBProperties.lblPages.Caption := IntToStr(lIBDBInfo.Allocation); // get number of pages allocated
      sOriginalSweepInterval := IntToStr(lIBDBInfo.SweepInterval);
      frmDBProperties.sgOptions.Cells[OPTION_VALUE_COL,SWEEP_INTERVAL_ROW] := sOriginalSweepInterval;

      if lIBDBInfo.ForcedWrites <> 0 then    // True
        sOriginalForcedWrites := FORCED_WRITES_TRUE
      else                                   // False
        sOriginalForcedWrites := FORCED_WRITES_FALSE;

      frmDBProperties.sgOptions.Cells[OPTION_VALUE_COL,FORCED_WRITES_ROW] := sOriginalForcedWrites;

      if lIBDBInfo.ReadOnly <> 0 then        // True
        sOriginalReadOnly := READ_ONLY_TRUE
      else                                   // False
        sOriginalReadOnly := READ_ONLY_FALSE;

      frmDBProperties.sgOptions.Cells[OPTION_VALUE_COL,READ_ONLY_ROW] := sOriginalReadOnly;

      sOriginalSQLDialect := IntToStr(lIBDBInfo.DBSQLDialect);
      frmDBProperties.sgOptions.Cells[OPTION_VALUE_COL,SQL_DIALECT_ROW] := sOriginalSQLDialect;

      //if not CurrSelDatabase.DataModule.Connection.DefaultTransaction.InTransaction then
      //  CurrSelDatabase.DataModule.Connection.DefaultTransaction.StartTransaction;

      // Set the defaults for the database properties
      frmDBProperties.SetDefaults (sOriginalReadOnly, sOriginalSweepInterval, sOriginalForcedWrites, sOriginalSQLDialect);
      with qryDBProperties do
      begin
        Close;
        IB_Connection := CurrSelDatabase.DataModule.Connection;
        IB_Transaction := CurrSelDatabase.DataModule.Connection.DefaultTransaction;
        SQL.Clear;
        SQL.Text :=('SELECT RDB$FILE_NAME, RDB$FILE_START FROM RDB$FILES ' +
                'WHERE RDB$SHADOW_NUMBER IS NULL OR RDB$SHADOW_NUMBER < 1 ' +
                'ORDER BY RDB$FILE_SEQUENCE ASC');
        try
          Open;
          First;
          while not eof do
          begin
            lListItem := frmDBProperties.lvSecondaryFiles.Items.Add;
            lListItem.Caption := qryDBProperties.Fields[0].AsString;
            lListItem.SubItems.Add(qryDBProperties.Fields[1].AsString);
            Next;
          end;
        except
          on e:EIB_ISCError do
          begin
            lListItem := frmDBProperties.lvSecondaryFiles.Items.Add;
            lListItem.Caption := GetLangText('Not Available');
            lListItem.SubItems.Add(GetLangText('Not Available'));
            DisplayMsg(ERR_GET_TABLE_DATA,E.Message + ' '+GetPrompt(118));
          end;
        end;
        Close;
        //if not CurrSelDatabase.DataModule.Connection.DefaultTransaction.InTransaction then
        //  CurrSelDatabase.DataModule.Connection.DefaultTransaction.StartTransaction;
        IB_Transaction := CurrSelDatabase.DataModule.Connection.DefaultTransaction;
        SQL.Clear;
        SQL.Text :=('SELECT RDB$OWNER_NAME FROM RDB$RELATIONS ' +
                'WHERE RDB$RELATION_NAME = ''RDB$DATABASE'' ');
        try
          Open;
          First;
          frmDBProperties.lblOwner.Caption := Fields[0].AsString;
        except
          on E:EIB_ISCError do
          begin
            frmDBProperties.lblOwner.Caption := GetLangText('Not Available');
            DisplayMsg(ERR_GET_TABLE_DATA,E.Message + ' '+GetPrompt(119));
          end;
        end;
        Close;
      end; // with qryDBProperties
    end; // retrieve database properties
    frmDBProperties.CurrSelDatabase := CurrSelDatabase;
    frmDBProperties.CurrSelServer := CurrSelServer;
    frmDBProperties.sOriginalForcedWrites := sOriginalForcedWrites;
    frmDBProperties.sOriginalReadOnly := sOriginalReadOnly;
    frmDBProperties.sOriginalSweepInterval := sOriginalSweepInterval;
    frmDBProperties.sOriginalSQLDialect := sOriginalSQLDialect;
    frmDBProperties.bOriginalConnectStatus := bOriginalConnectStatus;
{$ifdef initlangfile} WriteAllCaptionAndText('',frmDBProperties,gTransIniFile); {$endif}
    frmDBProperties.ShowModal;
{$ifdef initlangfile} WriteAllCaptionAndText('',frmDBProperties,gTransIniFile); {$endif}
    Application.ProcessMessages;
    result := SUCCESS;
    
  finally
    Screen.Cursor := crDefault;
    qryDBProperties.Free;
    lConfigService.Free;
    lIBDBInfo.Free;
    frmDBProperties.Free;
    lSubKeys.Free;
  end;
end;

procedure TfrmDBProperties.btnSelFilenameClick(Sender: TObject);
var
  lOpenDialog: TOpenDialog;
begin
  lOpenDialog := nil;
  try
  begin
    lOpenDialog := TOpenDialog.Create(self);
    lOpenDialog.Title := GetLangText('Select Database');
    lOpenDialog.DefaultExt := 'fdb';
    lOpenDialog.Filter := DlgDbFilters;
{$IFDEF MSWINDOWS}
    lOpenDialog.Options := [ofHideReadOnly,ofNoNetworkButton, ofEnableSizing];
{$ENDIF}
{$IFDEF LINUX}
    lOpenDialog.Options := [ofEnableSizing];
{$ENDIF}
    if lOpenDialog.Execute then
    begin
      // get filename
      edtFilename.Text := lOpenDialog.FileName;
      // if no dbalias is specified then make it the name of the file
      if (edtAliasName.Text = '') or (edtAliasName.Text = ' ') then
      begin
        edtAliasName.Text := ExtractFileName(edtFilename.Text);
        if (edtAliasName.Text = '') or (edtAliasName.Text = ' ') then
        begin
          edtAliasName.Text := ExtractFileName(edtFilename.Text);
        end;
      end;
    end;
  end
  finally
    lOpenDialog.free;
  end;
end;

procedure TfrmDBProperties.edtFilenameChange(Sender: TObject);
begin
  FApplyChanges := True;
  btnApply.Enabled := True;
  edtFilename.Hint := edtFilename.Text;
end;

procedure TfrmDBProperties.SetDefaults(const readOnly, sweep, synch,
  dialect: String);
begin

  FOriginalReadOnly := readOnly;
  FOriginalSweepInterval := Sweep;
  FOriginalSynchMode := synch;
  FOriginalSQLDialect := dialect;

end;

procedure TfrmDBProperties.edtFilenameExit(Sender: TObject);
begin
  inherited;
  if not (IsValidDBName(edtFilename.text)) then
     DisplayMsg(WAR_REMOTE_FILENAME, Format(getLangText('File')+': %s', [edtFileName.text]));
end;

procedure TfrmDBProperties.Button1Click(Sender: TObject);
begin
  inherited;
  ModalResult := mrCancel;
end;

procedure TfrmDBProperties.SetErrorState(const Value: boolean);
begin
  FErrorState := Value;
end;

end.
