{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 * 
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 * 
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 * 
 * Contributor(s): Krzysztof Golko.
}

unit frmuDBStatistics;

interface

uses
{$IFDEF LINUX}
  Types, Qt, QForms, QStdCtrls, QExtCtrls, QControls, QGrids,
  QDialogs, QGraphics,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls,
  ComCtrls, Grids,
{$ENDIF}
  SysUtils, Classes, zluibcClasses, IBOServices, IB_Session, IB_Header, dmuNode, IBOConsoleForm;

type
  TfrmDBStatistics = class(TIBOConsoleForm)
    sgOptions: TStringGrid;
    pnlOptionName: TPanel;
    cbOptions: TComboBox;
    lblOptions: TLabel;
    lblDatabaseName: TLabel;
    bvlLine1: TBevel;
    btnOK: TButton;
    btnCancel: TButton;
    lblDatabase: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure cbOptionsChange(Sender: TObject);
    procedure cbOptionsDblClick(Sender: TObject);
    procedure cbOptionsExit(Sender: TObject);
    procedure cbOptionsKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure sgOptionsDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
    procedure sgOptionsSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
  private
    FErrorState: boolean;
    { Private declarations }
    function VerifyInputData(): boolean;
{$IFDEF MSWINDOWS}
    procedure SetErrorState(const Value: boolean);
{$ENDIF}
  public
    { Public declarations }
    property ErrorState: boolean read FErrorState write SetErrorState;
  end;

function DoDBStatistics(const SourceServerNode: TibcServerNode;
                        const CurrSelDatabase: TibcDatabaseNode): integer;

implementation

uses
  IBOCGlobal, zluUtility, zluContextHelp, frmuMessage,
  frmuMain, frmuTextViewer, LMMessageDialog;

{$IF Declared(Forms)}
{$R *.dfm}
{$IFEND}

{$IF Declared(QForms)}
{$R *.xfm}
{$IFEND}

const
  OPTION_NAME_COL = 0;
  OPTION_VALUE_COL = 1;

  STATISTICS_OPTION_ROW = 0;

{****************************************************************
*
*  D o D B S t a t i s t i c s ( )
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  TibcServerNode  - currently selected server
*          TibcDatabseNode - currently selected database (the
*                            database to be validated)
*
*  Return: Integer - indicates success or failure
*
*  Description:
*
*****************************************************************
* Revisions:
*
*****************************************************************}
function DoDBStatistics(const SourceServerNode: TibcServerNode;
  const CurrSelDatabase: TibcDatabaseNode): integer;
var
  frmDBStatistics: TfrmDBStatistics;
  lDBStatisticsData: TStringList;
  lDBStatistics: TIBOStatisticalService;
  lDBStatisticsOptions: TStatOptions;

begin
  lDBStatisticsData := TStringList.Create();
  lDBStatistics := TIBOStatisticalService.Create(Application.MainForm);
  try
    try
      // assign server details
      lDBStatistics.LoginPrompt := false;
      lDBStatistics.ServerName := SourceServerNode.Server.ServerName;
      lDBStatistics.Protocol := SourceServerNode.Server.Protocol;
      lDBStatistics.Params.Clear;
      lDBStatistics.Params.Assign(SourceServerNode.Server.Params);
      lDBStatistics.Attach();            // try to attach to server
    except                               // if an exception occurs then trap it
      on E:EIB_ISCError do                   // and display an error message
      begin
        DisplayMsg(ERR_SERVER_LOGIN, E.Message);
        result := FAILURE;
        if (E.ERRCODE = isc_lost_db_connection) or
           (E.ERRCODE = isc_unavailable) or
           (E.ERRCODE = isc_network_error) then
          frmMain.ErrorState:= True;
        Exit;
      end;
    end;

    // if successfully attached to server
    if lDBStatistics.Active = true then
    begin
      frmDBStatistics := TfrmDBStatistics.Create(Application.MainForm);
      try
{$ifdef initlangfile} WriteAllCaptionAndText('',frmDBStatistics,gTransIniFile); {$endif}
//{$IFNDEF NOT_YET_IN_KYLIX}
        frmDBStatistics.lblDatabase.Caption := MinimizeName(CurrSelDatabase.NodeName,
          frmDBStatistics.lblDatabase.Canvas,
          (frmDBStatistics.lblDatabase.ClientRect.Left - frmDBStatistics.lblDatabase.ClientRect.Right));
(*{$ELSE}
        frmDBStatistics.lblDatabase.Caption := CurrSelDatabase.NodeName;
{$ENDIF}   *)

        frmDBStatistics.lblDatabase.Hint := CurrSelDatabase.NodeName;
{$ifdef initlangfile} WriteAllCaptionAndText('',frmDBStatistics,gTransIniFile); {$endif}
        frmDBStatistics.ShowModal;

        if (frmDBStatistics.ModalResult = mrOK) and
           (not frmDBStatistics.ErrorState) then
        begin
          // repaint screen
          Application.ProcessMessages;
          Screen.Cursor := crHourGlass;

          // assign database details
          lDBStatistics.DatabaseName := CurrSelDatabase.DatabaseFiles.Strings[0];

          lDBStatisticsOptions := [];
          // determine which options have been selected
          if frmDBStatistics.sgOptions.Cells[1,STATISTICS_OPTION_ROW] = 'Data Pages' then
            Include(lDBStatisticsOptions, DataPages)
          else
            if frmDBStatistics.sgOptions.Cells[1,STATISTICS_OPTION_ROW] = 'Database Log' then
              Include(lDBStatisticsOptions, DbLog)
            else
              if frmDBStatistics.sgOptions.Cells[1,STATISTICS_OPTION_ROW] = 'Header Page' then
                Include(lDBStatisticsOptions, HeaderPages)
              else
                if frmDBStatistics.sgOptions.Cells[1,STATISTICS_OPTION_ROW] = 'Index Pages' then
                  Include(lDBStatisticsOptions, IndexPages)
                else
                  if frmDBStatistics.sgOptions.Cells[1,STATISTICS_OPTION_ROW] = 'System Relations' then
                    Include(lDBStatisticsOptions, SystemRelations);

          // assign validation options
          lDBStatistics.Options := lDBStatisticsOptions;

          // start service
          try
            lDBStatistics.ServiceStart;
            IBCShowServiceInfo(lDBStatistics, 'Database Statistics');
            lDBStatistics.Detach;
          except
            on E: EIB_ISCError do
            begin
              DisplayMsg(E.ERRCODE, E.Message);
              if (E.ERRCODE = isc_lost_db_connection) or
                 (E.ERRCODE = isc_unavailable) or
                 (E.ERRCODE = isc_network_error) then
                frmMain.ErrorState:= True;
            end;
          end;
        end;
      except
        on E: Exception do
        begin
          DisplayMsg(ERR_SERVER_SERVICE,E.Message + CRLF + 'Database statistics cannot be displayed.');
          result := FAILURE;
        end;
      end;
      result := SUCCESS;
    end
    else
      result := FAILURE;
  finally
    Screen.Cursor := crDefault;
    if lDBStatistics.Active then
      lDBStatistics.Detach();
    lDBStatisticsData.Free;
    lDBStatistics.Free;
  end;
end;

{****************************************************************
*
*  F o r  m C r e a t e
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  TObject - Object that initiated the event
*
*  Return: None
*
*  Description: This procedure is responsible for populating
*               the string grid when the form is created.
*
*****************************************************************
* Revisions:
*
*****************************************************************}

procedure TfrmDBStatistics.FormCreate(Sender: TObject);
begin
  inherited;
  HelpContext := DATABASE_STATISTICS;
  sgOptions.DefaultRowHeight := cbOptions.Height;
  cbOptions.Visible := True;
  pnlOptionName.Visible := True;

  sgOptions.RowCount := 1;

  sgOptions.Cells[OPTION_NAME_COL,STATISTICS_OPTION_ROW] := GetLangText('Show data for:');
  sgOptions.Cells[OPTION_VALUE_COL,STATISTICS_OPTION_ROW] := GetLangText('All Options');

  pnlOptionName.Caption := GetLangText('Show data for:');
  cbOptions.Items.Add(GetLangText('All Options'));
  cbOptions.Items.Add(GetLangText('Data Pages'));
  cbOptions.Items.Add(GetLangText('Database Log'));
  cbOptions.Items.Add(GetLangText('Header Page'));
  cbOptions.Items.Add(GetLangText('Index Pages'));
  cbOptions.Items.Add(GetLangText('System Relations'));
  cbOptions.ItemIndex := 0;
end;

procedure TfrmDBStatistics.btnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TfrmDBStatistics.btnOKClick(Sender: TObject);
begin
  if VerifyInputData() then
    ModalResult := mrOK;
end;

procedure TfrmDBStatistics.cbOptionsChange(Sender: TObject);
begin
  {
  sgOptions.Cells[sgOptions.Col,sgOptions.Row] :=
    cbOptions.Items[cbOptions.ItemIndex];
  cbOptions.Visible := false;
  sgOptions.SetFocus;
  }
end;

{****************************************************************
*
*  c b O p t i o n s D b l C l i c k
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  TObject - object that initiated the event
*
*  Return: None
*
*
*  Description: This procedure rotates through a list of values
*               when the option name or value is double-clicked.
*
*****************************************************************
* Revisions:
*
*****************************************************************}

procedure TfrmDBStatistics.cbOptionsDblClick(Sender: TObject);
begin
  if (sgOptions.Col = OPTION_VALUE_COL) or (sgOptions.Col = OPTION_NAME_COL) then
  begin
    if cbOptions.ItemIndex = cbOptions.Items.Count - 1 then
      cbOptions.ItemIndex := 0
    else
      cbOptions.ItemIndex := cbOptions.ItemIndex + 1;

    if sgOptions.Col = OPTION_VALUE_COL then
      sgOptions.Cells[sgOptions.Col,sgOptions.Row] := cbOptions.Items[cbOptions.ItemIndex];

    // cbOptions.Visible := True;
    // sgOptions.SetFocus;
  end;
end;

procedure TfrmDBStatistics.cbOptionsExit(Sender: TObject);
var
  lR     : TRect;
  iIndex : Integer;
begin
  iIndex := cbOptions.Items.IndexOf(cbOptions.Text);

  if (iIndex = -1) then
  begin
    LMMessageDlg(16, mtError, [mbOK], 0);

    cbOptions.ItemIndex := 0;
    // Size and position the combo box to fit the cell
    lR := sgOptions.CellRect(OPTION_VALUE_COL, sgOptions.Row);
    lR.Left := lR.Left + sgOptions.Left;
    lR.Right := lR.Right + sgOptions.Left;
    lR.Top := lR.Top + sgOptions.Top;
    lR.Bottom := lR.Bottom + sgOptions.Top;
    cbOptions.Left := lR.Left + 1;
    cbOptions.Top := lR.Top + 1;
    cbOptions.Width := (lR.Right + 1) - lR.Left;
    cbOptions.Height := (lR.Bottom + 1) - lR.Top;
    cbOptions.Visible := True;
    cbOptions.SetFocus;
  end
  else if (sgOptions.Col <> OPTION_NAME_COL) then
  begin
    sgOptions.Cells[sgOptions.Col,sgOptions.Row] := cbOptions.Items[iIndex];
  end
  else
  begin
    sgOptions.Cells[OPTION_VALUE_COL,sgOptions.Row] := cbOptions.Items[iIndex];
  end;
end;

procedure TfrmDBStatistics.cbOptionsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
{$IFDEF LINUX}
const
  VK_DOWN = KEY_DOWN;
{$ENDIF}
begin
  if (Key = VK_DOWN) then
    cbOptions.DroppedDown := true;
end;

{****************************************************************
*
*  s g O p t i o n s D r a w C e l l
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  TObject - Object that initiated the event
*          Integer - currently selected column
*          Integer - currently selected row
*          TRect   - coordinates
*          TGridDrawState - drawing state of grid
*
*  Return: None
*
*  Description: This procedure draws contents to a specified cell in
*               the Option string grid.
*
*****************************************************************
* Revisions:
*
*****************************************************************}

procedure TfrmDBStatistics.sgOptionsDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
const
  INDENT = 2;
var
  lLeft: integer;
  lText: string;
begin
  with sgOptions.canvas do
  begin
    if (ACol = OPTION_VALUE_COL) then
    begin
      font.color := clBlue;
      if brush.color = clHighlight then
        font.color := clWhite;
      lText := sgOptions.Cells[ACol,ARow];
      lLeft := Rect.Left + INDENT;
      TextRect(Rect, lLeft, Rect.top + INDENT, lText);
    end;
  end;
end;

{****************************************************************
*
*  s g O p t i o n s S e l e c t C e l l
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  TObject - Object that initiated the event
*          Integer - currently selected column
*          Integer - currently selected row
*          Boolean - indicates whether call can be selected
*
*  Return: None
*
*  Description: This procedure shows the combo box and populates
*               it when the user selects a row in the value
*               column of the options grid.
*
*****************************************************************
* Revisions:
*
*****************************************************************}

procedure TfrmDBStatistics.sgOptionsSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
var
  lR, lName : TRect;
begin
  cbOptions.Items.Clear;
  cbOptions.Items.Add(GetLangText('All Options'));
  cbOptions.Items.Add(GetLangText('Data Pages'));
  cbOptions.Items.Add(GetLangText('Database Log'));
  cbOptions.Items.Add(GetLangText('Header Page'));
  cbOptions.Items.Add(GetLangText('Index Pages'));
  cbOptions.Items.Add(GetLangText('System Relations'));

  pnlOptionName.Caption := sgOptions.Cells[OPTION_NAME_COL, ARow];

  if ACol = OPTION_NAME_COL then
    cbOptions.ItemIndex := cbOptions.Items.IndexOf(sgOptions.Cells[ACol+1,ARow])
  else if ACol = OPTION_VALUE_COL then
    cbOptions.ItemIndex := cbOptions.Items.IndexOf(sgOptions.Cells[ACol,ARow]);

  if ACol = OPTION_NAME_COL then
  begin
    lName := sgOptions.CellRect(ACol, ARow);
    lR := sgOptions.CellRect(ACol + 1, ARow);
  end
  else
  begin
    lName := sgOptions.CellRect(ACol - 1, ARow);
    lR := sgOptions.CellRect(ACol, ARow);
  end;

  // lName := sgOptions.CellRect(ACol, ARow);
  lName.Left := lName.Left + sgOptions.Left;
  lName.Right := lName.Right + sgOptions.Left;
  lName.Top := lName.Top + sgOptions.Top;
  lName.Bottom := lName.Bottom + sgOptions.Top;
  pnlOptionName.Left := lName.Left + 1;
  pnlOptionName.Top := lName.Top + 1;
  pnlOptionName.Width := (lName.Right + 1) - lName.Left;
  pnlOptionName.Height := (lName.Bottom + 1) - lName.Top;
  pnlOptionName.Visible := True;

  // lR := sgOptions.CellRect(ACol, ARow);
  lR.Left := lR.Left + sgOptions.Left;
  lR.Right := lR.Right + sgOptions.Left;
  lR.Top := lR.Top + sgOptions.Top;
  lR.Bottom := lR.Bottom + sgOptions.Top;
  cbOptions.Left := lR.Left + 1;
  cbOptions.Top := lR.Top + 1;
  cbOptions.Width := (lR.Right + 1) - lR.Left;
  cbOptions.Height := (lR.Bottom + 1) - lR.Top;
  cbOptions.Visible := True;
  cbOptions.SetFocus;
end;

function TfrmDBStatistics.VerifyInputData(): boolean;
begin
  result := true;
end;

procedure TfrmDBStatistics.SetErrorState(const Value: boolean);
begin
  FErrorState := Value;
end;

end.
