{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 * 
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 * 
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 *
 * Contributor(s): Krzysztof Golko.
}

unit frmuDBTransactions;

interface

uses
{$IFDEF LINUX}
  QForms, QStdCtrls, QExtCtrls, QComCtrls, QControls, QDialogs,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Graphics, Controls, Forms, Dialogs, StdCtrls, ComCtrls, ExtCtrls, ImgList,
{$ENDIF}
  dmuNode, SysUtils, Classes, zluibcClasses, IBOServices, IB_SEssion, IB_Header, IBOConsoleForm;

type
  TfrmDBTransactions = class(TIBOConsoleForm)
    edtPath: TEdit;
    lblConnectPath: TLabel;
    lblStatus: TLabel;
    lvTransactions: TListView;
    memAdvice: TMemo;
    pgcMain: TPageControl;
    tabAdvice: TTabSheet;
    tabTransactions: TTabSheet;
    lblDatabase: TLabel;
    btnOK: TButton;
    btnCancel: TButton;
    rgOptions: TRadioGroup;
    lblRepairStatus: TLabel;
    FValidation: TIBOValidationService;
    lblDatabaseName: TLabel;
    procedure btnCancelClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure lvTransactionsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure pgcMainChange(Sender: TObject);
    procedure pgcMainChanging(Sender: TObject; var AllowChange: Boolean);
    procedure rgOptionsClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FGlobalAction: TTransactionGlobalAction;
    FCurrentRecord : Integer;
    FErrorState: boolean;
    procedure GetLimboTransactions(const SourceServerNode : TibcServerNode; const CurrSelDatabase : TibcDatabaseNode);
    procedure SetErrorState(const Value: boolean);
  public
    { Public declarations }
    property ErrorState: boolean read FErrorState write SetErrorState;
  end;

function DoDBTransactions(const SourceServerNode: TibcServerNode; const CurrSelDatabase: TibcDatabaseNode): integer;

implementation

uses
  IBOCGlobal, zluUtility, zluContextHelp, frmuMessage, frmuMain,
  LMMessageDialog;

{$IF Declared(Forms)}
{$R *.dfm}
{$IFEND}

{$IF Declared(QForms)}
{$R *.xfm}
{$IFEND}

{****************************************************************
*
*  D o D B T r a n s a c t i o n s ( )
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  SourceServerNode - The currently selected server
*          CurrSelDatabase  - The currently selected database
*
*  Return: Integer - Determines whether the operation was a
*                    success or a failure
*
*  Description: This function checks for limbo transactions,
*               if any exist the form is shown, otherwise
*               a message is displayed and control is returned
*               to the main
*
*****************************************************************
* Revisions:
*
*****************************************************************}
function DoDBTransactions(const SourceServerNode: TibcServerNode; const CurrSelDatabase: TibcDatabaseNode): integer;
var
  frmDBTransactions : TfrmDBTransactions;
begin
  frmDBTransactions := Nil;
  try
    frmDBTransactions := TfrmDBTransactions.Create(Application.MainForm);

    Screen.Cursor := crHourGlass;
    frmDBTransactions.lblDatabaseName.Caption:=CurrSelDatabase.NodeName;
    // get a list of limbo transactions
    frmDBTransactions.GetLimboTransactions(SourceServerNode, CurrSelDatabase);

    // if the record count is not 0 then there are limbo transactions
    if frmDBTransactions.FCurrentRecord <> 0 then
      frmDBTransactions.ShowModal     // show the form as a modal dialog box
    else                               // if there were no limbo transactions
    begin                              // then display message
      DisplayMsg(INF_NO_PENDING_TRANSACTIONS, '');
      Result := FAILURE;
      Exit;
    end;

    if (frmDBTransactions.ModalResult = mrOK) and
       (not frmDBTransactions.ErrorState) then
      result:=SUCCESS
    else
      result:=FAILURE;
  finally
    // deallocate memory
    frmDBTransactions.Free;
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmDBTransactions.btnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

{****************************************************************
*
*  G e t L i m b o T r a n s a c t i o n s
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  SourceServerNode - The currently selected server
*          CurrSelDatabase  - The currently selected database
*
*  Return: None
*
*  Description: This procedure populates the listview with
*               a list of limbo transactions and they're IDs.
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBTransactions.GetLimboTransactions(const SourceServerNode : TibcServerNode; const CurrSelDatabase : TibcDatabaseNode);
var
  lListItem : TListItem;
begin
  try
    // assign server details
    FValidation.ServerName := SourceServerNode.Server.ServerName;
    FValidation.Protocol := SourceServerNode.Server.Protocol;
    FValidation.Params.Clear;
    FValidation.Params.Assign(SourceServerNode.Server.Params);
    // assign database details
    case SourceServerNode.Server.Protocol of
      cpTCP_IP : FValidation.DatabaseName := Format('%s:%s',[SourceServerNode.ServerName,CurrSelDatabase.DatabaseFiles.Strings[0]]);
      cpNetBeui : FValidation.DatabaseName := Format('\\%s\%s',[SourceServerNode.ServerName,CurrSelDatabase.DatabaseFiles.Strings[0]]);
      cpNovell : FValidation.DatabaseName := Format('%s@%s',[SourceServerNode.ServerName,CurrSelDatabase.DatabaseFiles.Strings[0]]);
      cpLocal : FValidation.DatabaseName := CurrSelDatabase.DatabaseFiles.Strings[0];
    end;
    // attach to server and start service
    FValidation.Options := [LimboTransactions];
    FValidation.Attach();
    FValidation.ServiceStart;
  except
    on E: EIB_ISCError do
    begin
      DisplayMsg(E.ERRCODE, E.Message);
      if (E.ERRCODE = isc_lost_db_connection) or
         (E.ERRCODE = isc_unavailable) or
         (E.ERRCODE = isc_network_error) then
        frmMain.ErrorState:= True;
      ErrorState:= True;
      Exit;
    end;
  end;

  // get limbo transactions and populate LimboTransactionsInfo array
  FValidation.FetchLimboTransactionInfo;

  while (FValidation.IsServiceRunning) and (not gApplShutdown) do
  begin
    Application.ProcessMessages;
  end;

  if FValidation.Active then
    FValidation.Detach();

  // get number of records and populate listview
  FCurrentRecord := 0;
  while (FCurrentRecord < FValidation.LimboTransactionInfoCount) and
        (FValidation.LimboTransactionInfo[FCurrentRecord].ID > 0) do
  begin
    lListItem := lvTransactions.Items.Add;

    if FValidation.LimboTransactionInfo[FCurrentRecord].MultiDatabase then
      lListItem.Caption := GetLangtext('Multi-Database Transaction')
    else
      lListItem.Caption := GetLangtext('Transaction');

    lListItem.SubItems.Add(IntToStr(FValidation.LimboTransactionInfo[FCurrentRecord].ID));
    case FValidation.LimboTransactionInfo[FCurrentRecord].Advise of
      CommitAdvise   : lListItem.SubItems.Add(GetLangtext('Commit'));
      RollBackAdvise : lListItem.SubItems.Add(GetLangtext('Rollback'));
      UnknownAdvise  : lListItem.SubItems.Add(GetLangtext('Unknown'));
    end;
    lListItem.SubItems.Add(GetLangtext('In Limbo'));

    Inc(FCurrentRecord);
  end;
end;

{****************************************************************
*
*  p g c M a i n C h a n g e
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  Sender - The object that initiated the event
*
*  Return: None
*
*  Description: This procedure shows details about the
*               currently selected transaction if the Advice
*               tab is selected
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBTransactions.pgcMainChange(Sender: TObject);
var
  i         : Integer;
  lListItem : TListItem;
  lStr      : String;
begin
  // determine which page is active
  lListItem := lvTransactions.Selected;
  if pgcMain.ActivePage = tabAdvice then
  begin
    // find transaction record
    i := 0;
    while (i < FValidation.LimboTransactionInfoCount) and
          (FValidation.LimboTransactionInfo[i].ID <> 0) and
          (FValidation.LimboTransactionInfo[i].ID <> StrToInt(lListItem.SubItems[0]))  do
      Inc(i);

    // make it the current record
    FCurrentRecord := i;

    // if there an item is currently selected in the list
    if (lListItem <> Nil) and (lListItem.Caption <> '') then
    begin
      // populate memo
      with memAdvice.Lines do
      begin
        Clear;
        Add(lListItem.Caption);
        Add(Format(GetPrompt(106), [FValidation.LimboTransactionInfo[i].HostSite]));

        case FValidation.LimboTransactionInfo[i].State of
          LimboState    : lStr := GetPrompt(107);
          CommitState   : lStr := GetPrompt(108);
          RollBackState : lStr := GetPrompt(109);
          UnknownState  : lStr := GetPrompt(110);
        end;
        Add(Format(GetPrompt(111), [FValidation.LimboTransactionInfo[i].ID, lStr]));

        Add(Format(GetPrompt(112), [FValidation.LimboTransactionInfo[i].RemoteSite]));
        Add(Format(GetPrompt(113), [FValidation.LimboTransactionInfo[i].RemoteDatabasePath]));

        Add('');

        case FValidation.LimboTransactionInfo[i].Advise of
          CommitAdvise   : lStr := GetLangtext('Commit');
          RollBackAdvise : lStr := GetLangtext('Rollback');
          UnknownAdvise  : lStr := GetLangtext('Unknown');
        end;

        Add(Format(GetPrompt(114), [lStr]));

      end;
    end
    else
    begin
      // show message in memo if no item is selected
      memAdvice.Lines.Clear;
      memAdvice.Lines.Add(GetPrompt(115));
    end;
  end
  else
  begin
    // give the listview control focus
    lvTransactions.SetFocus;
  end;
end;

procedure TfrmDBTransactions.lvTransactionsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
var
  i : Integer;
begin
  // find currently selected record
  i := 0;
  while (i < FValidation.LimboTransactionInfoCount) and
        (FValidation.LimboTransactionInfo[i].ID <> StrToInt(Item.SubItems[0])) and
        (FValidation.LimboTransactionInfo[i].ID <> 0) do
    Inc(i);

  FCurrentRecord := i;

  // show its database path
  if FCurrentRecord < FValidation.LimboTransactionInfoCount then
    edtPath.Text := FValidation.LimboTransactionInfo[i].RemoteDatabasePath
  else
    edtPath.Text := '';
end;

{****************************************************************
*
*  b t n O K C l i c k
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  Sender - The object that initiated the event
*
*  Return: None
*
*  Description: This procedure will try to fix any limbo
*               transactions.
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmDBTransactions.btnOKClick(Sender: TObject);
var
  lCnt: integer;
begin
  try
    FValidation.GlobalAction := FGlobalAction;
    Screen.Cursor := crHourGlass;
    { Fix:  Service only allows 1 transaction to be fixed at a time }
    for lCnt := 0 to lvTransactions.Items.Count-1 do
    begin
      Application.ProcessMessages;    
      lblRepairStatus.Caption := Format(GetPrompt(116),[lvTransactions.Items[lCnt].Subitems[0]]);
      lblRepairStatus.Visible := true;
      FValidation.Attach;
      FValidation.FixLimboTransactionErrors;
      while FValidation.IsServiceRunning do
        Application.ProcessMessages;
      lvTransactions.Items[lCnt].SubItems[2] := GetLangText('Fixed');
      FValidation.Detach;
      FValidation.Attach;
      FValidation.ServiceStart;
      Application.ProcessMessages;      
      FValidation.FetchLimboTransactionInfo;
      while FValidation.IsServiceRunning do
        Application.ProcessMessages;
      FValidation.Detach;      
    end;
  except
    on E:EIB_ISCError do
    begin
      DisplayMsg(E.ERRCODE, E.Message);
      if (E.ERRCODE = isc_lost_db_connection) or
         (E.ERRCODE = isc_unavailable) or
         (E.ERRCODE = isc_network_error) then
      begin
        frmMain.ErrorState:= True;
        ErrorState:= True;
      end
      else
      if Fvalidation.Active then
        FValidation.Detach;
    end;
  end;
  ModalResult := mrOK;
  Screen.Cursor := crDefault;
  if Fvalidation.Active then
    FValidation.Detach;
  LMMessageDlg(17, mtInformation, [mbOK], 0);
end;

procedure TfrmDBTransactions.pgcMainChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
  inherited;
  if not Assigned(lvTransactions.Selected) then
  begin
    LMMessageDlg(117,mtInformation,[mbok],0);
    AllowChange := false;
  end;
end;

procedure TfrmDBTransactions.rgOptionsClick(Sender: TObject);
begin
  inherited;
  case rgOptions.ItemIndex of
    0: FGlobalAction := NoGlobalAction;
    1: FGlobalAction := CommitGlobal;
    2: FGlobalAction := RollbackGlobal;
    3: FGlobalAction := RecoverTwoPhaseGlobal;
  end;
end;

procedure TfrmDBTransactions.FormCreate(Sender: TObject);
begin
  inherited;
  FGlobalAction := NoGlobalAction;
  HelpContext := TRANSACTIONS_RECOVERY;
end;

procedure TfrmDBTransactions.SetErrorState(const Value: boolean);
begin
  FErrorState := Value;
end;

end.
