{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 *
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 *
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 *
 * Contributor(s): Jeff Overcash, Krzysztof Golko.

08.08.2002, Kalwa:
- CHG: TPNGImage is now TPNGObject (http://pngdelphi.sourceforge.net)

12.August.2002, Lorenzo:
- Added Code for exporting image to clipboard and file

}

unit frmuDisplayBlob;

interface

uses
{$IFDEF LINUX}
  QForms, QExtCtrls, QControls, QClipbrd, StdCtrls, ExtCtrls, Classes,
  Controls, Dialogs, ExtDlgs, QMenus,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Graphics, Controls, Forms, Dialogs, ExtCtrls, Clipbrd,
{$ENDIF}
  SysUtils, Classes, IBCustomDataset, DB, StdCtrls, menus, ExtDlgs,
{$IFDEF useGraphicEx}
  GraphicEx,
{$ELSE}
  pngimage,
{$ENDIF}
  jpeg,
{$IFDEF  _JvGIF_}
  JvGIF,
{$ENDIF}
  IBOConsoleForm, JvDialogs;

type
  TfrmDisplayBlob = class(TIBOConsoleForm)
    Bevel1: TBevel;
    Image: TImage;
    Label1: TLabel;
    PopupMenu1: TPopupMenu;
    Copy1: TMenuItem;
    SaveToFile1: TMenuItem;
    SavePictureDialog1: TSavePictureDialog;
    JvSaveDialog1: TJvSaveDialog;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Copy1Click(Sender: TObject);
    procedure SaveToFile1Click(Sender: TObject);
  private
    { Private declarations }
    fField: TBlobField;
  public
    { Public declarations }
  end;

  function GraphicFromStream(Stream: TStream):TGraphic;
  function LoadImageFromField(const Image: TImage; const Field: TField; var emessage:string): boolean;
  function GraphicFromClipboard:TGraphic;
  function GraphicFilter(X:TGraphic):String;
  function GraphicExt(X:TGraphic):String;
  function WriteImageToField(const Image: TImage; const Field: TField; var emessage:string): boolean;
  function GraphicType(X:TGraphic):String;

function IBCDisplayBlob (const Owner: TComponent; const Field: TBlobField): boolean;

implementation

{$IF Declared(Forms)}
{$R *.DFM}
{$IFEND}
{$IF Declared(QForms)}
{$R *.xfm}
{$IFEND}

uses
  frmuMessage;

function TryType(gClass:TGraphicClass; Stream: TStream):TGraphic;
begin
  Result:=gClass.Create;
  try
    Stream.Position := 0;
    Result.LoadFromStream(Stream);
  except
    on e:Exception do
    try
      try
        Result.Free;
      except
      end;
    finally
      Result:=Nil;
    end;
  end;
end;

function GraphicFromStream(Stream: TStream):TGraphic;
begin
  Result := Nil;
  if not Assigned(Result) then Result := TryType(TJPEGImage,Stream);
  if not Assigned(Result) then Result := TryType(TMetafile,Stream);
  if not Assigned(Result) then Result := TryType(TIcon,Stream);
{$IFDEF  _JvGIF_}
  if not Assigned(Result) then Result := TryType(TJvJvGIF,Stream);
{$ENDIF}
  if not Assigned(Result) then Result := TryType(Graphics.TBitmap,Stream);
{$IFNDEF useGraphicEx}
  if not Assigned(Result) then Result := TryType(TPNGObject,Stream);
{$ELSE}
  if not Assigned(Result) then Result := TryType(TGIFGraphic,Stream);
  if not Assigned(Result) then Result := TryType(TPNGGraphic,Stream);
  if not Assigned(Result) then Result := TryType(TTIFFGraphic,Stream);
  if not Assigned(Result) then Result := TryType(TSGIGraphic,Stream);
  if not Assigned(Result) then Result := TryType(TEPSGraphic,Stream);
  if not Assigned(Result) then Result := TryType(TTargaGraphic,Stream);
  if not Assigned(Result) then Result := TryType(TPCXGraphic,Stream);
  if not Assigned(Result) then Result := TryType(TPCDGraphic,Stream);
  if not Assigned(Result) then Result := TryType(TPPMGraphic,Stream);
  if not Assigned(Result) then Result := TryType(TCUTGraphic,Stream);
  if not Assigned(Result) then Result := TryType(TRLAGraphic,Stream);
  if not Assigned(Result) then Result := TryType(TPSDGraphic,Stream);
  if not Assigned(Result) then Result := TryType(TPSPGraphic,Stream);
  if not Assigned(Result) then Result := TryType(TAutodeskGraphic,Stream);
{$ENDIF}
end;

function GraphicType(X:TGraphic):String;
begin
  Result := '';
  if X is TJPEGImage       then Result := 'JPEG' else
  if X is TMetafile        then Result := 'Metafile' else
  if X is TIcon            then Result := 'Icon' else
{$IFDEF  _JvGIF_}
  if X is TJvGIFImage        then Result := 'GIF ' else
{$ENDIF}
{$IFNDEF useGraphicEx}
  if X is TPNGObject       then Result := 'Png' else
{$ELSE}
  if X is TGIFGraphic      then Result := 'GIF' else
  if X is TPNGGraphic      then Result := 'PNG' else
  if X is TTIFFGraphic     then Result := FileFormatList.GetDescription(TTIFFGraphic    ) else
  if X is TSGIGraphic      then Result := FileFormatList.GetDescription(TSGIGraphic     ) else
  if X is TEPSGraphic      then Result := FileFormatList.GetDescription(TEPSGraphic     ) else
  if X is TTargaGraphic    then Result := FileFormatList.GetDescription(TTargaGraphic   ) else
  if X is TPCXGraphic      then Result := FileFormatList.GetDescription(TPCXGraphic     ) else
  if X is TPCDGraphic      then Result := FileFormatList.GetDescription(TPCDGraphic     ) else
  if X is TPPMGraphic      then Result := FileFormatList.GetDescription(TPPMGraphic     ) else
  if X is TCUTGraphic      then Result := FileFormatList.GetDescription(TCUTGraphic     ) else
  if X is TRLAGraphic      then Result := FileFormatList.GetDescription(TRLAGraphic     ) else
  if X is TPSDGraphic      then Result := FileFormatList.GetDescription(TPSDGraphic     ) else
  if X is TPSPGraphic      then Result := FileFormatList.GetDescription(TPSPGraphic     ) else
  if X is TAutodeskGraphic then Result := FileFormatList.GetDescription(TAutodeskGraphic) else
{$ENDIF}
  if X is TBitmap          then Result := 'Bitmaps';
end;

function GraphicFilter(X:TGraphic):String;
begin
  if X is TJPEGImage       then Result := 'JPEG Image File (*.jpg)|*.jpg' else
{$IFDEF  _JvGIF_}
  if X is TJvGIFImage        then Result := 'CompuServe GIF Image (*.gif)|*.gif' else
{$ENDIF}
  if X is TMetafile        then Result := 'Metafiles (*.wmf)|*.wmf' else
  if X is TIcon            then Result := 'Icons (*.ico)|*.ico' else
{$IFDEF useGraphicEx}
  if X is TGIFGraphic      then Result := 'CompuServe GIF Image (*.gif)|*.gif' else
  if X is TPNGGraphic      then Result := 'PNG Image File (*.png)|*.png' else
  if X is TTIFFGraphic     then Result := FileFormatList.GetGraphicFilter([],fstExtension,[foIncludeExtension],TTIFFGraphic    ) else
  if X is TSGIGraphic      then Result := FileFormatList.GetGraphicFilter([],fstExtension,[foIncludeExtension],TSGIGraphic     ) else
  if X is TEPSGraphic      then Result := FileFormatList.GetGraphicFilter([],fstExtension,[foIncludeExtension],TEPSGraphic     ) else
  if X is TTargaGraphic    then Result := FileFormatList.GetGraphicFilter([],fstExtension,[foIncludeExtension],TTargaGraphic   ) else
  if X is TPCXGraphic      then Result := FileFormatList.GetGraphicFilter([],fstExtension,[foIncludeExtension],TPCXGraphic     ) else
  if X is TPCDGraphic      then Result := FileFormatList.GetGraphicFilter([],fstExtension,[foIncludeExtension],TPCDGraphic     ) else
  if X is TPPMGraphic      then Result := FileFormatList.GetGraphicFilter([],fstExtension,[foIncludeExtension],TPPMGraphic     ) else
  if X is TCUTGraphic      then Result := FileFormatList.GetGraphicFilter([],fstExtension,[foIncludeExtension],TCUTGraphic     ) else
  if X is TRLAGraphic      then Result := FileFormatList.GetGraphicFilter([],fstExtension,[foIncludeExtension],TRLAGraphic     ) else
  if X is TPSDGraphic      then Result := FileFormatList.GetGraphicFilter([],fstExtension,[foIncludeExtension],TPSDGraphic     ) else
  if X is TPSPGraphic      then Result := FileFormatList.GetGraphicFilter([],fstExtension,[foIncludeExtension],TPSPGraphic     ) else
  if X is TAutodeskGraphic then Result := FileFormatList.GetGraphicFilter([],fstExtension,[foIncludeExtension],TAutodeskGraphic) else
{$ELSE}
  if X is TPNGObject       then Result := 'PNG Image File (*.png)|*.png' else
{$ENDIF}
  if X is TBitmap          then Result := 'Bitmaps (*.bmp)|*.bmp';

end;

function GraphicExt(X:TGraphic):String;
begin
  Result := '';
  if X is TJPEGImage       then Result := 'jpg' else
  if X is TMetafile        then Result := 'wmf' else
  if X is TIcon            then Result := 'Ico' else
{$IFDEF useGraphicEx}
  if X is TGIFGraphic      then Result := 'gif' else
  if X is TPNGGraphic      then Result := 'png' else
  if X is TTIFFGraphic     then Result := FileFormatList.GetExtension(TTIFFGraphic    ) else
  if X is TSGIGraphic      then Result := FileFormatList.GetExtension(TSGIGraphic     ) else
  if X is TEPSGraphic      then Result := FileFormatList.GetExtension(TEPSGraphic     ) else
  if X is TTargaGraphic    then Result := FileFormatList.GetExtension(TTargaGraphic   ) else
  if X is TPCXGraphic      then Result := FileFormatList.GetExtension(TPCXGraphic     ) else
  if X is TPCDGraphic      then Result := FileFormatList.GetExtension(TPCDGraphic     ) else
  if X is TPPMGraphic      then Result := FileFormatList.GetExtension(TPPMGraphic     ) else
  if X is TCUTGraphic      then Result := FileFormatList.GetExtension(TCUTGraphic     ) else
  if X is TRLAGraphic      then Result := FileFormatList.GetExtension(TRLAGraphic     ) else
  if X is TPSDGraphic      then Result := FileFormatList.GetExtension(TPSDGraphic     ) else
  if X is TPSPGraphic      then Result := FileFormatList.GetExtension(TPSPGraphic     ) else
  if X is TAutodeskGraphic then Result := FileFormatList.GetExtension(TAutodeskGraphic) else
{$ELSE}
{$IFDEF  _JvGIF_}
  if X is TJvGIFImage        then Result := 'gif' else
{$ENDIF}
  if X is TPNGObject       then Result := 'png' else
{$ENDIF}
  if X is TBitmap          then Result := 'bmp' else

end;



function TryTypeFromClipboard(gClass:TGraphicClass):TGraphic;
begin
  Result:=gClass.Create;
  try
    if Clipboard.HasFormat( CF_BITMAP ) then Result.Assign(Clipboard);
  except
    try
      try
        Result.Free;
      except
      end;
    finally
      Result:=Nil;
    end;
  end;
end;

function GraphicFromClipboard:TGraphic;
begin
  Result := TryTypeFromClipboard(Graphics.TBitmap);
  if not Assigned(Result) then Result := TryTypeFromClipboard(TMetafile);
  if not Assigned(Result) then Result := TryTypeFromClipboard(TIcon);
  if not Assigned(Result) then Result := TryTypeFromClipboard(TJPEGImage);
{$IFDEF useGraphicEx}
//  if not Assigned(Result) then Result := TryTypeFromClipboard(TJPEGImage);
{$ELSE}
{$IFDEF  _JvGIF_}
  if not Assigned(Result) then Result := TryTypeFromClipboard(TJvGIFImage);
{$ENDIF}
  if not Assigned(Result) then Result := TryTypeFromClipboard(TPNGObject);
{$ENDIF}
end;


function LoadImageFromField(const Image: TImage; const Field: TField; var emessage:string): boolean;
var bs:TStream;
begin
  Result := true;
  emessage := '';
  begin
    try
      bs := Field.DataSet.CreateBlobStream(Field,bmRead);
      try
        Image.Picture.Graphic := GraphicFromStream(bs);
      finally
        bs.free;
      end;
    except
      on E: Exception do
      begin
        emessage := E.Message;
        Result := false;
      end;
    end;
  end;
end;

function WriteImageToField(const Image: TImage; const Field: TField; var emessage:string): boolean;
var bs:TStream;
begin
  Result := true;
  emessage := '';
  begin
    try
      bs := Field.DataSet.CreateBlobStream(Field,bmWrite);
      try
        Image.Picture.Graphic.SaveToStream(bs);
      finally
        bs.free;
      end;
    except
      on E: Exception do
      begin
        emessage := E.Message;
        Result := false;
      end;
    end;
  end;
end;


function IBCDisplayBlob (const Owner: TComponent; const Field: TBlobField): boolean;
var
    emessage:String;
begin
  Result := true;
  if not Field.IsNull then
    with TfrmDisplayBlob.Create(Owner) do
    begin
      fField:=Field;
      Caption := Field.DisplayName;
      if not LoadImageFromField(Image,Field,emessage) then
      begin
        Label1.Caption := emessage;
        Result := false;
      end
      else
      begin
        if Assigned(Image.Picture) and Assigned(Image.Picture.Graphic) then
          Label1.Caption := GraphicType(Image.Picture.Graphic)
        else
          Label1.Caption := '??';
        Show;
      end;
    end;
end;

procedure TfrmDisplayBlob.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TfrmDisplayBlob.Copy1Click(Sender: TObject);
begin
  if Assigned(Image.Picture.Graphic) then
    Clipboard.Assign(Image.Picture);
end;

procedure TfrmDisplayBlob.SaveToFile1Click(Sender: TObject);
begin
  if Assigned(Image.Picture.Graphic) then
  begin
      SavePictureDialog1.Filter := GraphicFilter(Image.Picture.Graphic);
      SavePictureDialog1.DefaultExt := GraphicExt(Image.Picture.Graphic);
      if SavePictureDialog1.Execute then
        Image.Picture.SaveToFile(SavePictureDialog1.FileName);
  end
  else
  if Assigned(fField) then
  begin
    if JvSaveDialog1.Execute then
      fField.SaveToFile(SavePictureDialog1.FileName);
  end;
end;

end.
