{*******************************************************************}
{  This Unit was created to by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.
                                                                         }

{******************************************************************}
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{******************************************************************} 

unit frmuEditorConstraint;

interface

uses
{$IFDEF LINUX}
  QControls, QForms, QDialogs, QComCtrls, QStdCtrls, QMenus,
  Types, QTypes, QImgList, QExtCtrls,
  QStdActns, QActnList, SynEdit, SynEditPrint, IBCXDummies,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Controls, Forms, Dialogs, ComCtrls, StdCtrls,
  Menus, ImgList, StdActns, ActnList, Graphics, Spin, DBGrids,
{$ENDIF}
  SysUtils, Classes, Grids, IBODataset, IBOCGlobal, IB_COmponents,
  ExtCtrls, FrameFieldsList, dmuNode, types, SynEditHighlighter,
  SynHighlighterSQL, SynEdit, IBOConsoleForm;

type
  TConstraintType = (ctNew, ctFK, ctCheck, ctPK, ctUnique);

  TfrmEditorConstraint = class(TIBOConsoleForm)
    btnOK: TButton;
    btnCancel: TButton;
    btnHelp: TButton;
    pgConstraint: TPageControl;
    tsFK: TTabSheet;
    tsCheck: TTabSheet;
    Label4: TLabel;
    cmbTables: TComboBox;
    Label5: TLabel;
    Label6: TLabel;
    edCheckName: TEdit;
    Label9: TLabel;
    edFKName: TEdit;
    chkOnDelete: TCheckBox;
    chkOnUpdate: TCheckBox;
    cmbOnDelete: TComboBox;
    cmbOnUpdate: TComboBox;
    tsPK: TTabSheet;
    Label7: TLabel;
    edPKName: TEdit;
    PkFrame: TFrameIndexList;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Panel4: TPanel;
    Panel5: TPanel;
    Panel6: TPanel;
    RefFrame: TFrameIndexList;
    ForeignFrame: TFrameIndexList;
    edCheck: TSynEdit;
    SynSQLSyn1: TSynSQLSyn;
    procedure cmbTablesChange(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure tsFKResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure RefFrameSpeedButtonAddClick(Sender: TObject);
    procedure RefFrameSpeedButtonRemoveClick(Sender: TObject);
    procedure tsFKShow(Sender: TObject);
    procedure tsCheckShow(Sender: TObject);
    procedure tsPKShow(Sender: TObject);
    procedure pgConstraintChanging(Sender: TObject;
      var AllowChange: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure edFKNameChange(Sender: TObject);
    procedure edCheckNameChange(Sender: TObject);
    procedure edPKNameChange(Sender: TObject);
    procedure edFKNameKeyPress(Sender: TObject; var Key: Char);
    procedure edCheckNameKeyPress(Sender: TObject; var Key: Char);
    procedure edPKNameKeyPress(Sender: TObject; var Key: Char);
  private
    FConstraintType: TConstraintType;
    FNewObject: boolean;
    FConstraintName: String;
    procedure SetConstraintType(const Value: TConstraintType);
    procedure SetNewObject(const Value: boolean);
    function ValidateData: boolean;
    function GetCreateSQL: String;
    procedure SetConstraintName(const Value: String);
  private
    FTableName: String;
    FSQLDialect : Integer;
    FCompileText: TStringList;
    FDataModule: TDmN;
    fErrors:boolean;
    FObjectModified: boolean;
    
    { Private declarations }
    procedure SetDataModule(const Value: TDmN);
    procedure DoCompile;
    procedure InitFieldLists;
    procedure SetObjectModified(const Value: boolean);
    property ConstraintType:TConstraintType read FConstraintType write SetConstraintType;
    property ConstraintName:String read FConstraintName write SetConstraintName;
  public
    { Public declarations }
    property TableName : String read FTableName write FTableName;
    property DataModule:TDmN read FDataModule write SetDataModule;
    procedure NewConstraint(aDataModule:TDmN; aTable:String; aType:TConstraintType);
    procedure LoadConstraint(ConstraintName:String);
    property ObjectModified:boolean read FObjectModified write SetObjectModified;
    property NewObject:boolean read FNewObject write SetNewObject;
  end;

implementation

uses IBOAdminUtils, zluUtility, frmuMessage, frmuWindowList,
  LMMessageDialog;


{$R *.DFM}


const
  pki = 2;
  fki = 0;
  chki = 1;


procedure TfrmEditorConstraint.cmbTablesChange(Sender: TObject);
begin
  DataModule.GetColumnList(ForeignFrame.Fields,cmbTables.Text);
  ForeignFrame.ResetFields;
end;

function TfrmEditorConstraint.ValidateData:boolean;
begin
  result := True;
  case pgConstraint.ActivePage.PageIndex of
    pki : //Primary key
      begin
        if edPKName.Text = '' then
        begin
          if ConstraintType = ctPK then
             LMMessageDlg(19, mtError, [mbOK], 0)
          else
             LMMessageDlg(20, mtError, [mbOK], 0);
          result := False;
          Exit;
        end;

        if PkFrame.lvIdxColumns.Count = 0 then
        begin
          if ConstraintType = ctPK then
            LMMessageDlg(21, mtError, [mbOK], 0)
          else
            LMMessageDlg(22, mtError, [mbOK], 0);
          result := False;
          Exit;
        end;
      end;
    fki : //foreign Key
      begin
        if edFKName.Text = '' then
        begin
          LMMessageDlg(23, mtError, [mbOK], 0);
          result := False;
          Exit;
        end;

        if ForeignFrame.lvIdxColumns.Items.Count = 0 then
        begin
          LMMessageDlg(24, mtError, [mbOK], 0);
          result := False;
          Exit;
        end;

        if RefFrame.lvIdxColumns.Items.Count = 0 then
        begin
          LMMessageDlg(25, mtError, [mbOK], 0);
          result := False;
          Exit;
        end;

      end;

    chki : //check constraint
      begin
        if Trim(edCheckName.Text) = '' then
        begin
          LMMessageDlg(26, mtError, [mbOK], 0);
          result := False;
          Exit;
        end;

        if Trim(edCheck.Text) = '' then
        begin
          LMMessageDlg(27, mtError, [mbOK], 0);
          result := False;
          Exit;
        end;
      end;
  end;
end;

function TfrmEditorConstraint.GetCreateSQL:String;
var
  Tmp : String;
  Tmp1 : String;
  Tmp2 : String;
  idx : Integer;
begin
  Result := '';
  case pgConstraint.ActivePage.PageIndex of
    pki : //Primary key
    begin
      if ConstraintType = ctUnique then
      begin
        Tmp := '';
        for idx := 0 to PkFrame.lvIdxColumns.Items.Count - 1 do
        begin
          Tmp := Tmp + QuoteIdentifier(FSQLDialect, PkFrame.lvIdxColumns.Items[idx]) + ', ';
        end;
        Tmp := Copy(Tmp, 1, Length(Tmp) - 2);
        Result := 'constraint ' + QuoteIdentifier(FSQLDialect,ConstraintName) + ' unique (' + Tmp + ')';
      end
      else
      if ConstraintType = ctPk then
      begin
        Tmp := '';
        for idx := 0 to PkFrame.lvIdxColumns.Items.Count - 1 do
        begin
          Tmp := Tmp + QuoteIdentifier(FSQLDialect, PkFrame.lvIdxColumns.Items[idx]) + ', ';
        end;

        Tmp := Copy(Tmp, 1, Length(Tmp) - 2);

        Result := 'constraint ' + QuoteIdentifier(FSQLDialect,ConstraintName) + ' primary key (' + Tmp + ')';
      end;
    end;

    fki : //foreign Key
      begin
        Tmp := '';
        for idx := 0 to RefFrame.lvIdxColumns.Items.Count - 1 do
        begin
          Tmp := Tmp + QuoteIdentifier(FSQLDialect,RefFrame.lvIdxColumns.Items[idx]) + ', ';
        end;
        Tmp := Copy(Tmp, 1, Length(Tmp) - 2);

        Tmp1 := '';
        for idx := 0 to ForeignFrame.lvIdxColumns.Items.Count - 1 do
        begin
          Tmp1 := Tmp1 + QuoteIdentifier(FSQLDialect,ForeignFrame.lvIdxColumns.Items[idx]) + ', ';
        end;
        Tmp1 := Copy(Tmp1, 1, Length(Tmp1) - 2);

        Tmp2 := '';
        if chkOnDelete.Checked then
        begin
          Tmp2 := Tmp2 + ' on delete ' + cmbOnDelete.Text;
        end;
        if chkOnUpdate.Checked then
        begin
          Tmp2 := Tmp2 + ' on update ' + cmbOnUpdate.Text;
        end;

        Result := 'constraint ' + QuoteIdentifier(FSQLDialect,ConstraintName) + ' foreign key (' + Tmp + ') references ' +
        QuoteIdentifier(FSQLDialect,cmbTables.Text) + '(' + Tmp1 + ')' + Tmp2;
      end;

    chki : //check constraint
      begin
        Result := 'constraint ' + QuoteIdentifier(FSQLDialect,ConstraintName) + ' ' + Trim(edCheck.Text);
      end;
  end;
end;

procedure TfrmEditorConstraint.btnOKClick(Sender: TObject);
var
  AlterSQL : String;
  DropSQL : String;
begin
  if ValidateData then
  begin
    AlterSQL := 'alter table ' + QuoteIdentifier(FSQLDialect,FTableName) + ' add ' +GetCreateSQL;
    DropSQL :=  'alter table ' + QuoteIdentifier(FSQLDialect,FTableName) +
                ' drop constraint ' + QuoteIdentifier(FSQLDialect,ConstraintName) + ';';
    FCOmpileText.Clear;
    if not NewObject then
      FCOmpileText.Add(DropSQL);
    FCOmpileText.Add(AlterSQL);
    fErrors := False;
    DoCompile;
    if not fErrors then
    begin
      OpenWindows.RefreshObject(DataModule,NODE_TABLE,TableName);
      ModalResult := mrOK;
    end;
  end;
end;

procedure TfrmEditorConstraint.FormCreate(Sender: TObject);
begin
  fErrors := False;
  FCompileText:=TStringList.Create;
end;

procedure TfrmEditorConstraint.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift = [ssCtrl]) and (Key = VK_TAB) then
    pgConstraint.SelectNextPage(True);
end;

procedure TfrmEditorConstraint.tsFKResize(Sender: TObject);
var H:integer;
begin
  H := tsFK.ClientHeight - Panel2.Height - Panel4.Height - Panel6.Height;
  Panel3.Height := H div 2;
end;

procedure TfrmEditorConstraint.FormShow(Sender: TObject);
begin
  tsFKResize(Sender);
end;

procedure TfrmEditorConstraint.SetDataModule(const Value: TDmN);
begin
  FDataModule := Value;
  if Assigned(FDataModule) then
  begin
    FSQLDialect := FDataModule.Connection.SQLDialect;
  end;
end;

procedure TfrmEditorConstraint.InitFieldLists;
var
  Idx:integer;
  s,s1:String;
  lObjectList: TStringList;
begin
  lObjectList:=TStringList.Create;
  try
    DataModule.GetColumnList(PkFrame.Fields,TableName);
    SetLength(RefFrame.Fields,Length(PkFrame.Fields));
    for Idx := Low(PkFrame.Fields) to High(PkFrame.Fields) do
    begin
      RefFrame.Fields[idx] := PkFrame.Fields[idx];
    end;
    RefFrame.RestoreFields;
    PkFrame.RestoreFields;
    cmbTables.Clear;
    DataModule.GetTableList(lObjectList, False);
    for Idx := 1 to lObjectList.Count - 1 do
    begin
      s:= lObjectList[idx];
      s1:= GetNextField(s,DEL);
      cmbTables.Items.Add(s1);
    end;
  finally
    lObjectList.Free;
  end;
end;

procedure TfrmEditorConstraint.NewConstraint(aDataModule:TDmN; aTable: String; aType:TConstraintType);
var s:String;
    pre:String;
    sType:string;
    t:String;
    n:integer;
begin
  NewObject := True;
  ConstraintType := aType;
  TableName := aTable;
  DataModule := aDataModule;
//  Caption := 'New Constraint for ' + TableName + '';
  InitFieldLists;
  case ConstraintType of
    ctFK:     begin pre := 'FK_'; sType := GetLangType(NODE_REFERENTIAL_CONSTRAINT); end;
    ctCheck:  begin pre := 'CH_'; sType := GetLangType(NODE_CHECK_CONSTRAINT); end;
    ctPK:     begin pre := 'PK_'; sType := GetLangType(NODE_UNIQUE_CONSTRAINT); end;
    ctUnique: begin pre := 'UK_'; sType := GetLangType(NODE_UNIQUE_CONSTRAINT); end;
  else
    exit;
  end;
  Caption := GetLangText('New')+' '+sType+' - ' + TableName ;
  n:=1;
  t:=copy(TableName,1,31-7);
  s:= pre+t+'_'+inttostr(n);
  while DataModule.ConstraintNameExist(s) and (n<21) do
  begin
    inc(n);
    s:= pre+t+'_'+inttostr(n);
  end;
  ConstraintName := s;
  case ConstraintType of
    ctFK:
    begin
      edFKName.Text    := ConstraintName;
      pgConstraint.ActivePage := tsFK;
    end;
    ctCheck:
    begin
      edCheckName.Text := ConstraintName;
      pgConstraint.ActivePage := tsCheck;
    end;
    ctPK:
    begin
      edPKName.Text    := ConstraintName;
      pgConstraint.ActivePage := tsPK;
    end;
    ctUnique:
    begin
       edPKName.Text := ConstraintName;
       tsPK.Caption  := 'Unique Key';
       pgConstraint.ActivePage := tsFK;
    end;
  else
    exit;
  end;
end;

procedure TfrmEditorConstraint.DoCompile;
var error:string;
begin
  if DataModule.ExecuteDDL(fCompileText,error) then
  begin
    DataModule.CommitDDL;
  end
  else
  begin
    DisplayMsg(ERR_DDL_EXECUTION,error);
    DataModule.RollbackDDL;
    fErrors := True;
  end;
end;

procedure TfrmEditorConstraint.LoadConstraint(ConstraintName: String);
var Data: TConstraintDef;
    Idx:TIndexDef;
    Ref: TRefConstraintDef;
    rPK:TStringDynArray;
    sType:string;
begin
  NewObject := True;
  InitFieldLists;
  DataModule.GetConstraintData(Data,TableName,ConstraintName);
  if Data.cType = 'FOREIGN KEY' then
  begin
//    Caption := 'Foreign Key '+ConstraintName+ ' for ' + TableName;
    ConstraintType := ctFK;
    pgConstraint.ActivePage := tsFK;
    edFKName.Text := Data.Name;
    DataModule.GetReferentialConstData(Ref,TableName,ConstraintName);
    DataModule.GetIndexData(Idx,Ref.IndexName);
    RefFrame.IncludeFields(Idx.Fields);
    cmbTables.ItemIndex := cmbTables.Items.IndexOf(Ref.RelationName);
    cmbTablesChange(nil);
    DataModule.GetPKList(Ref.RelationName,rPK);
    ForeignFrame.IncludeFields(rPK);
    cmbOnUpdate.ItemIndex := cmbOnUpdate.Items.IndexOf(Ref.Update_Rule);
    chkOnUpdate.Checked := Ref.Update_Rule <> '';
    cmbOnDelete.ItemIndex := cmbOnDelete.Items.IndexOf(Ref.DeleteRule);
    chkOnDelete.Checked := Ref.DeleteRule <> '';
  end
  else
  if Data.cType = 'PRIMARY KEY' then
  begin
//    Caption := 'Primary Key for ' + TableName;
    ConstraintType := ctPK;
    pgConstraint.ActivePage := tsPK;
    edPKName.Text := Data.Name;
    DataModule.GetIndexData(Idx,Data.IndexName);
    PkFrame.IncludeFields(Idx.Fields);
  end
  else
  if Data.cType = 'CHECK' then
  begin
//    Caption := 'Check Constraint '+ConstraintName+ ' for ' + TableName;
    ConstraintType := ctCheck;
    pgConstraint.ActivePage := tsCheck;
    edCheckName.Text := Data.Name;
    edCheck.Lines.Text := DataModule.GetCheckConstSource(TableName,Data.Name)
  end;
  case ConstraintType of
    ctFK:     begin sType := GetLangType(NODE_REFERENTIAL_CONSTRAINT); end;
    ctCheck:  begin sType := GetLangType(NODE_CHECK_CONSTRAINT); end;
    ctPK:     begin sType := GetLangType(NODE_UNIQUE_CONSTRAINT); end;
    ctUnique: begin sType := GetLangType(NODE_UNIQUE_CONSTRAINT); end;
  else
    exit;
  end;
  Caption := sType+' - ' + TableName ;
  NewObject := False;
end;

procedure TfrmEditorConstraint.SetObjectModified(const Value: boolean);
begin
  FObjectModified := Value;
end;


procedure TfrmEditorConstraint.RefFrameSpeedButtonAddClick(Sender: TObject);
begin
  RefFrame.SpeedButtonAddClick(Sender);
  ObjectModified := true;
end;

procedure TfrmEditorConstraint.RefFrameSpeedButtonRemoveClick(
  Sender: TObject);
begin
  RefFrame.SpeedButtonRemoveClick(Sender);
  ObjectModified := true;
end;

procedure TfrmEditorConstraint.SetConstraintType(
  const Value: TConstraintType);
begin
  FConstraintType := Value;
end;

procedure TfrmEditorConstraint.tsFKShow(Sender: TObject);
begin
  ConstraintType := ctFK;
end;

procedure TfrmEditorConstraint.tsCheckShow(Sender: TObject);
begin
  ConstraintType := ctCheck;
end;

procedure TfrmEditorConstraint.tsPKShow(Sender: TObject);
begin
  ConstraintType := ctPK;
end;

procedure TfrmEditorConstraint.pgConstraintChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
  AllowChange := ConstraintType = ctNew;
end;

procedure TfrmEditorConstraint.SetNewObject(const Value: boolean);
begin
  FNewObject := Value;
  edFKName.Enabled := FNewObject;
  edCheckName.Enabled := FNewObject;
  edPKName.Enabled := FNewObject;
end;

procedure TfrmEditorConstraint.FormDestroy(Sender: TObject);
begin
  FCompileText.Free;
end;

procedure TfrmEditorConstraint.SetConstraintName(const Value: String);
begin
  FConstraintName := Value;
  ObjectModified := true;
end;

procedure TfrmEditorConstraint.edFKNameChange(Sender: TObject);
begin
  if ConstraintType in [ctNew,ctFK] then
    ConstraintName := edFKName.Text;
end;

procedure TfrmEditorConstraint.edCheckNameChange(Sender: TObject);
begin
  if ConstraintType in [ctNew,ctCheck] then
    ConstraintName := edCheckName.Text;
end;

procedure TfrmEditorConstraint.edPKNameChange(Sender: TObject);
begin
  if ConstraintType in [ctNew,ctPK] then
    ConstraintName := edPKName.Text;
end;

procedure TfrmEditorConstraint.edFKNameKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Assigned(Datamodule) then
    Key := CheckNameKey(Key,Datamodule.Connection.SQLDialect);
end;

procedure TfrmEditorConstraint.edCheckNameKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Assigned(Datamodule) then
    Key := CheckNameKey(Key,Datamodule.Connection.SQLDialect);
end;

procedure TfrmEditorConstraint.edPKNameKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Assigned(Datamodule) then
    Key := CheckNameKey(Key,Datamodule.Connection.SQLDialect);
end;

end.
