{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{******************************************************************}
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{******************************************************************} 

unit frmuEditorException;

interface

uses
{$IFDEF LINUX}
  QControls, QForms, QDialogs, QComCtrls, QStdCtrls, QMenus,
  Types, QTypes, QImgList, QExtCtrls,
  QStdActns, QActnList, SynEdit, SynEditPrint, IBCXDummies,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Controls, Forms, Dialogs, ComCtrls, StdCtrls,
  Menus, ImgList, StdActns, ActnList, Graphics, Spin, Printers,
  ExtCtrls, ClipBrd, Buttons, 
{$ENDIF}
  SysUtils, Classes,
  DB,  Grids, DBGrids, DBCtrls, dmuNode, IBOConsoleForm,
  IBODataset, SynEdit, SynEditHighlighter,
  SynHighlighterSQL, IB_Components;

type
  TfrmExceptions = class(TIBOConsoleEditor)
    pgObjectEditor: TPageControl;
    tsObject: TTabSheet;
    tsDocoView: TTabSheet;
    Label1: TLabel;
    edExceptionName: TEdit;
    Label2: TLabel;
    edExceptionText: TEdit;
    qryException: TIBOQuery;
    tsDDL: TTabSheet;
    stsEditor: TStatusBar;
    edDescription: TSynEdit;
    edDDL: TSynEdit;
    ActionList1: TActionList;
    ActionCompile: TAction;
    SpeedButton1: TSpeedButton;
    Action1: TAction;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure edExceptionNameChange(Sender: TObject);
    procedure edExceptionTextChange(Sender: TObject);
    procedure pgObjectEditorChanging(Sender: TObject;
      var AllowChange: Boolean);
    procedure pgObjectEditorChange(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ActionCompileExecute(Sender: TObject);
    procedure ActionCompileUpdate(Sender: TObject);
    procedure edDescriptionStatusChange(Sender: TObject;
      Changes: TSynStatusChanges);
    procedure edExceptionNameKeyPress(Sender: TObject; var Key: Char);
    procedure edDescriptionChange(Sender: TObject);
  private
    { Private declarations }
    FMessageModified: boolean;
    FNameModified: boolean;
    procedure SaveDescription;
    procedure LoadDescription;
    procedure GetDDL;
    procedure SetMessageModified(const Value: boolean);
    procedure SetNameModified(const Value: boolean);
  protected
    procedure SignalNewObject; override;
    procedure SetDataModule(const Value: TDmN); override;
    procedure SignalObjectModified; override;
    function AlterObject:boolean; override;
    function CompileNewObject:boolean; override;
    procedure DoLoadObject; Override;
  public
    { Public declarations }
    property NameModified:boolean read FNameModified write SetNameModified;
    property MessageModified:boolean read FMessageModified write SetMessageModified;
  end;


implementation

uses IBOCGlobal, IBOAdminUtils, dmuMain, frmuMain,  IBOExtract,
  frmuWindowList, frmuMessage, LMMessageDialog;

{$R *.DFM}


procedure TfrmExceptions.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
  OpenWindows.Remove(Self);
end;

procedure TfrmExceptions.FormCreate(Sender: TObject);
begin
  pgObjectEditor.ActivePage := tsObject;
  ObjectType := NODE_EXCEPTION;
end;

procedure TfrmExceptions.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := InternalCloseQuery;
end;

procedure TfrmExceptions.edExceptionNameChange(Sender: TObject);
begin
  NewName := edExceptionName.Text;
  ObjectModified := True;
  NameModified := true;
end;

procedure TfrmExceptions.edExceptionTextChange(Sender: TObject);
begin
  ObjectModified := True;
  MessageModified := true;
end;

procedure TfrmExceptions.pgObjectEditorChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
  if IsNewObject then
  begin
    LMMessageDlg(29, mtWarning, [mbOK], 0);
    AllowChange := False;
  end
  else
  begin
    if pgObjectEditor.ActivePage = tsDocoView then
    begin
      if edDescription.Modified then
      begin
        if LMMessageDlg(30, mtConfirmation, [mbYes, mbNo], 0) = mrYes then
          SaveDescription;
      end;
    end;
  end;
end;

procedure TfrmExceptions.GetDDL;
begin
  if Assigned(DataModule) then
  begin
    DataModule.IBOExtract.ExtractObject(eoException,ObjectName);
    edDDL.Lines.Clear;
    edDDL.Lines.Text:=DataModule.IBOExtract.Items.Text;
    edDDL.Modified := False;
  end;
end;

procedure TfrmExceptions.pgObjectEditorChange(Sender: TObject);
begin
  if pgObjectEditor.ActivePage = tsObject then
  begin
    stsEditor.Panels[0].Text := '';
    stsEditor.Panels[1].Text := '';
    stsEditor.Panels[2].Text := '';
    edExceptionName.SetFocus;
  end
  else
  if pgObjectEditor.ActivePage = tsDocoView then
  begin
    edDescription.SetFocus;
  end
  else
  if pgObjectEditor.ActivePage = tsDDL then
  begin
    edDDL.SetFocus;
    if not IsNewObject then
      GetDDL;
  end;
end;

procedure TfrmExceptions.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift = [ssCtrl]) and (Key = VK_TAB) then
    pgObjectEditor.SelectNextPage(True);
end;

procedure TfrmExceptions.SetDataModule(const Value: TDmN);
begin
  inherited SetDataModule(Value);
  if Assigned(DataModule) then
  begin
    qryException.IB_Connection := DataModule.Connection;
    qryException.IB_Transaction := DataModule.DDLTransaction;
    stsEditor.Panels[3].Text := DataModule.Connection.DatabaseName;
  end
  else
  begin
    qryException.IB_Connection := Nil;
    qryException.IB_Transaction := Nil;
    stsEditor.Panels[3].Text := '';
  end
end;

procedure TfrmExceptions.SaveDescription;
begin
  if Assigned(Datamodule) then
  begin
    DataModule.ObjectDescriptionUpdate(NODE_EXCEPTIONS,ObjectName,edDescription.Text);
    edDescription.Modified := False;
    frmMain.ObjectListRefresh(DataModule, NODE_EXCEPTIONS);
  end;
end;

procedure TfrmExceptions.LoadDescription;
begin
  if Assigned(Datamodule) then
  begin
    edDescription.Text := DataModule.GetObjectDescription(NODE_EXCEPTIONS,ObjectName);
    edDescription.Modified := False;
  end;
end;

procedure TfrmExceptions.SetMessageModified(const Value: boolean);
begin
  FMessageModified := Value;
end;

procedure TfrmExceptions.SetNameModified(const Value: boolean);
begin
  FNameModified := Value;
end;

procedure TfrmExceptions.ActionCompileExecute(Sender: TObject);
begin
  Compile;
end;

procedure TfrmExceptions.ActionCompileUpdate(Sender: TObject);
begin
  ActionCompile.Enabled := ObjectModified;
end;

procedure TfrmExceptions.edDescriptionStatusChange(Sender: TObject;
  Changes: TSynStatusChanges);
begin
  if pgObjectEditor.ActivePage = tsDocoView then
    stsEditor.Panels[0].Text := Format('%d:%d',[edDescription.CaretY,edDescription.CaretX]);
end;

procedure TfrmExceptions.edExceptionNameKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Assigned(Datamodule) then
    Key := CheckNameKey(Key,Datamodule.Connection.SQLDialect);
end;

function TfrmExceptions.AlterObject: boolean;
var
  CompileText : String;
  Error:String;
begin
  if edExceptionText.Text = '' then
  begin
    LMMessageDlg(31, mtError, [mbOK], 0);
    Result := False;
    Exit;
  end;
  Result := False;
  if NameModified then
  begin
    CompileText := 'drop exception ' + QuoteIdentifier(DataModule.Connection.SQLDialect,ObjectName) + ';';
    Result := DataModule.ExecuteDDL(CompileText,Error);
    if result then
       CompileText := 'create exception ' +
                      QuoteIdentifier(DataModule.Connection.SQLDialect,NewName) + ' '+
                      AnsiQuotedStr(edExceptionText.Text, '''')+';';
    Result := DataModule.ExecuteDDL(CompileText,Error);
  end
  else
  if MessageModified then
  begin
    CompileText := 'alter exception ' +
                   QuoteIdentifier(DataModule.Connection.SQLDialect,ObjectName) + ' ' +
                   AnsiQuotedStr(edExceptionText.Text, '''')+';';
    Result := DataModule.ExecuteDDL(CompileText,Error);
  end;
  CompileErrors := Error;
end;

function TfrmExceptions.CompileNewObject: boolean;
var
  CompileText : String;
  Error:String;
begin
  Result := False;
  if not Assigned(DataModule) then exit;
  if edExceptionText.Text = '' then
  begin
    LMMessageDlg(31, mtError, [mbOK], 0);
    Result := False;
    Exit;
  end;
  CompileText := 'create exception ' + QuoteIdentifier(DataModule.Connection.SQLDialect,NewName) + ' '
                                     + AnsiQuotedStr(edExceptionText.Text, '''')+';';
  Result := DataModule.ExecuteDDL(CompileText,Error);
  CompileErrors := error;
end;

procedure TfrmExceptions.DoLoadObject;
begin
  inherited;
  Caption := 'Exception - ' + ObjectName;
  try
    qryException.BeginBusy(False);
    qryException.SQL.Clear;
    qryException.SQL.Text :=('select rdb$exception_name, rdb$message '+
                             'from rdb$exceptions where rdb$exception_name = '''
                             + ObjectName + ''';');
    qryException.Open;
    edExceptionName.Text := qryException.FieldByName('rdb$exception_name').AsString;
    edExceptionText.Text := qryException.FieldByName('rdb$message').AsString;
    qryException.Close;
    //qryException.IB_Transaction.Commit;
    ObjectModified := False;
    LoadDescription;
  finally
    qryException.EndBusy;
  end;
  edDDL.Modified := False;
  edDescription.Modified := False;
  NameModified := False;
  MessageModified := False;
end;

procedure TfrmExceptions.SignalNewObject;
begin
  inherited;
  Caption := 'Exception - ' + ObjectName;
  edExceptionName.Text := ObjectName;
  edExceptionText.Text := '';
  ActiveControl := edExceptionText;
  edDDL.Modified := False;
  edDescription.Modified := False;
end;

procedure TfrmExceptions.SignalObjectModified;
begin
    if ObjectModified then
    begin
      stsEditor.Panels[0].Text := '*';
    end
    else
    begin
      stsEditor.Panels[0].Text := '';
      NameModified := False;
      MessageModified := False;
    end;
end;

procedure TfrmExceptions.edDescriptionChange(Sender: TObject);
begin
  EditDescription := edDescription.Text;
end;

end.
