{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{******************************************************************}
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{******************************************************************}

unit frmuEditorField;

interface

uses
{$IFDEF LINUX}
  QControls, QForms, QDialogs, QComCtrls, QStdCtrls, QMenus,
  Types, QTypes, QImgList, QExtCtrls,
  QStdActns, QActnList, SynEdit, SynEditPrint, IBCXDummies,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Controls, Forms, Dialogs, ComCtrls, StdCtrls,
  Menus, ImgList, StdActns, ActnList, Graphics, Spin, ExtCtrls,
{$ENDIF}
  SysUtils, Classes, DB, dmuNode, SynEdit, IBOConsoleForm,
  frameEditorField;

type
  TfrmEditorField = class(TIBOConsoleEditor)
    btnHelp: TButton;
    btnCancel: TButton;
    btnOK: TButton;
    edTable: TEdit;
    Label5: TLabel;
    Panel1: TPanel;
    stsEditor: TStatusBar;
    ActionList1: TActionList;
    ActionHelp: TAction;
    ActionOK: TAction;
    ActionCancel: TAction;
    Panel4: TPanel;
    FrameFields: TFrameEditorField;
    procedure btnHelpClick(Sender: TObject);
    procedure ActionHelpUpdate(Sender: TObject);
    procedure ActionOKExecute(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormShow(Sender: TObject);
    procedure edTableChange(Sender: TObject);
    procedure ActionCancelUpdate(Sender: TObject);
    procedure ActionCancelExecute(Sender: TObject);
    procedure ActionOKUpdate(Sender: TObject);
    procedure edTableKeyPress(Sender: TObject; var Key: Char);
    procedure FrameFieldsedColumnChange(Sender: TObject);
    procedure FrameFieldsedComputedChange(Sender: TObject);
    procedure FrameFieldstsComputedShow(Sender: TObject);
    procedure FrameFieldsedDefaultStatusChange(Sender: TObject;
      Changes: TSynStatusChanges);
    procedure FrameFieldsedDefaultChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    fData:TColumnDef;
    fLoading:boolean;
    fStart:boolean;
    FIsNewTable: boolean;
    FTableName: String;
    { Private declarations }
    procedure SetTableName(const Value: String);
    function DoCreateTable :boolean;
    function GetNullValue(ColumnName: String): String;
    procedure SetIsNewTable(const Value: boolean);
    procedure FrameFieldsOnObjectModified(Sender: TObject);
    procedure LoadField(aTableName, aFieldName: String);
  protected
    procedure SignalNewObject; override;
    procedure SetDataModule(const Value: TDmN); override;
    procedure SetObjectModified(const Value: boolean); override;
    function AlterObject:boolean; override;
    function CompileNewObject:boolean; override;
    procedure DoLoadObject; Override;
  public
    { Public declarations }
    { Table Part }
    procedure NewField(aTableName:String);
    property IsNewTable:boolean read FIsNewTable write SetIsNewTable;
    procedure NewTable;
    property TableName:String read FTableName write SetTableName;
    { Field part }
  end;

implementation

uses
  frmuArrayDialog, dmuMain, frmuMain, zluUtility, IBOCGlobal, IBOAdminUtils,
  frmuMessage, frmuWindowList, LMMessageDialog;

{$R *.DFM}

const
  PG_DOMAIN = 0;
  PG_RAW_TYPE = 1;
  PG_COMPUTED = 2;


procedure TfrmEditorField.btnHelpClick(Sender: TObject);
begin
//  Application.HelpCommand(HELP_CONTEXT, IDH_Add_Column_Dialog);
end;

procedure TfrmEditorField.SetDataModule(const Value: TDmN);
begin
  inherited SetDataModule(Value);
  FrameFields.DataModule := Value;
  if Assigned(DataModule) then
  begin
    stsEditor.Panels[2].Text := DataModule.Connection.DatabaseName;
  end
  else
  begin
    stsEditor.Panels[2].Text := '';
  end;
end;

procedure TfrmEditorField.LoadField(aTableName, aFieldName: String);
begin
  if Assigned(Datamodule) then
  begin
    fLoading := True;
    try
      if Datamodule.GetColumnData(fData,aTableName, aFieldName) = SUCCESS then
      begin
        FrameFields.Data := fData;
        Caption := GetLangType(NODE_COLUMN)+' - '+ fData.ColumnName;
        ObjectModified := False;
      end;
    finally
    end;
  end;
end;

procedure TfrmEditorField.SetObjectModified(const Value: boolean);
begin
  inherited SetObjectModified(Value);
  if fLoading then exit;
  if not ObjectModified then
  begin
  end;
  if ObjectModified then
    stsEditor.Panels[1].Text := '*'
  else
    stsEditor.Panels[1].Text := '';
end;

procedure TfrmEditorField.ActionHelpUpdate(Sender: TObject);
begin
  ActionHelp.Enabled := False;
end;

procedure TfrmEditorField.ActionOKExecute(Sender: TObject);
begin
  if IsNewTable then
  begin
    if (edTable.Text = '') then
    begin
      LMMessageDlg(32, mtError, [mbOK], 0);
      edTable.SetFocus;
      Exit;
    end
  end;

  if not FrameFields.Validate then exit;

  if IsNewTable then
  begin
    //if we get to here then validation has worked...
    if DoCreateTable then
    begin
      ObjectModified := False;
      ModalResult := mrOK;
    end;
  end
  else
    if Compile then
    begin
      ModalResult := mrOK;
      OpenWindows.RefreshObject(DataModule,NODE_TABLE,TableName);
    end;

end;


procedure TfrmEditorField.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := InternalCloseQuery;
end;

procedure TfrmEditorField.NewField(aTableName: String);
begin
  TableName := aTableName;
  NewObject;
  Caption := GetLangText('New')+' '+GetLangType(NODE_COLUMN)+' - ' + TableName;
  edTable.Text := TableName;
  edTable.Enabled := False;
end;

procedure TfrmEditorField.SetTableName(const Value: String);
begin
  fTableName := Value;
  if not IsNewTable then
    edTable.Text := fTableName;
end;

procedure TfrmEditorField.SignalNewObject;
begin
  Caption := GetLangText('New')+' '+GetLangType(NODE_COLUMN)+' - ' + TableName;
  edTable.Text := TableName;
  edTable.Enabled := False;
end;

procedure TfrmEditorField.FormShow(Sender: TObject);
begin
  if fLoading then
  begin
    fLoading := False;
    ObjectModified := False;
  end;
  if fStart then
  begin
    fStart:= false;
    ActiveControl := FrameFields.edColumn;
  end;
  if IsNewTable then
    Caption := GetLangText('New')+' '+GetLangType(NODE_TABLE);
end;

function TfrmEditorField.GetNullValue(ColumnName:String): String;
var c:boolean;
begin
  result := '';
  repeat
    c := InputQuery(GetLangText('Value for')+' '+ColumnName, GetLangText('Replace NULL with'), result);
  until (result <> '') or not c;
end;

const
  SqlClearNotNull =
    'update RDB$RELATION_FIELDS set '+
    'RDB$NULL_FLAG = NULL '+
    'where (RDB$FIELD_NAME = %s) and '+
    '(RDB$RELATION_NAME = %s) ';

  SqlSetNotNull =
    'update RDB$RELATION_FIELDS set '+
    'RDB$NULL_FLAG = 1 '+
    'where (RDB$FIELD_NAME = %s) and '+
    '(RDB$RELATION_NAME = %s)';

  sqlAlterName = 'ALTER TABLE %s ALTER COLUMN %s TO %s';
  sqlAlterType = 'ALTER TABLE %s ALTER COLUMN %s TYPE %s';
  sqlFillNull = 'update %s set %s = %s where %s is null';
  sqlAlterDefaut = 'ALTER TABLE %s ALTER COLUMN %s DEFAULT %s';


function TfrmEditorField.AlterObject:boolean;
var AlterSQL:String;
    lNameChange:boolean;
    lTypeChange:boolean;
    lNotNullChange:boolean;
    lSqlDialect:integer;
    newdef, error:String;
    nullValue:String;
    lNewData:TColumnDef;

    function ExecSql:boolean;
    begin
      Result := DataModule.ExecuteDDL(AlterSQL,error);
      CompileErrors := error;
    end;

begin
  Result := True;
  lNewData := FrameFields.Data;
  NewName := lNewData.ColumnName;
  lNewData.ColumnName := NewName;
  lSqlDialect := DataModule.Connection.SQLDialect;
  lNameChange := fData.ColumnName <> NewName;
  lTypeChange := CompareColumnDataType(fData,lNewData);

  lNotNullChange := fData.NotNull <> lNewData.NotNull;

  AlterSQL := '';

  if lNameChange then
  begin
    AlterSQL := Format(sqlAlterName,[QuoteIdentifier(lSQLDialect,TableName),
                                     QuoteIdentifier(lSQLDialect,fData.ColumnName),
                                     QuoteIdentifier(lSQLDialect,NewName)]);
    Result := ExecSql;
  end;

  if Result and lTypeChange then
  begin
    newdef := GetDataType(lNewData,lSQLDialect);
    AlterSQL := Format(sqlAlterType,[QuoteIdentifier(lSQLDialect,TableName),
                                     QuoteIdentifier(lSQLDialect,NewName),
                                     newdef]);
    Result := ExecSql;
  end;


  if Result and lNotNullChange then
  begin
    if lNewData.NotNull then
    begin
      nullValue := '';
      case LMMessageDlg(33,mtWarning,[mbYes,mbNo,mbCancel],0) of
        mrYes:
        begin
          nullValue := GetNullValue(NewName);
          if nullValue = '' then exit;
        end;
        mrNo:;
        mrCancel: exit;
      end;
      AlterSql := Format(SqlSetNotNull,[AnsiQuotedStr(NewName,''''),
                                        AnsiQuotedStr(TableName,'''')]);
      Result := ExecSql;
      if Result and (nullValue <> '') then
      begin
        AlterSql := Format(sqlFillNull,[QuoteIdentifier(lSQLDialect,TableName),
                                         QuoteIdentifier(lSQLDialect,NewName),
                                         AnsiQuotedStr(nullValue,''''),
                                         QuoteIdentifier(lSQLDialect,NewName)]);
        Result := ExecSql;
      end;
    end
    else
      case LMMessageDlg(34, mtWarning,[mbYes,mbNo,mbCancel],0) of
        mrYes:
        begin
          AlterSql := Format(SqlClearNotNull,[AnsiQuotedStr(NewName,''''),
                                        AnsiQuotedStr(TableName,'''')]);
          Result := ExecSql;
        end;
        else exit;
      end;
  end;

end;

function TfrmEditorField.CompileNewObject:boolean;
var error:string;
var AlterSQL:string;
    lNewData:TColumnDef;
begin
  lNewData := FrameFields.Data;
  NewName := lNewData.ColumnName;
  lNewData.ColumnName := NewName;

  AlterSQL := 'alter table '+ QuoteIdentifier(DataModule.Connection.SQLDialect,TableName)
              + ' add ' + GetSqlFieldDef(lNewData,DataModule.Connection.SQLDialect,True,True) + ';';

  Result :=  DataModule.ExecuteDDL(AlterSQL,error);
  CompileErrors := error;
end;

function TfrmEditorField.DoCreateTable:boolean;
var error:string;
var AlterSQL:string;
    lNewData:TColumnDef;
begin
  Result := False;
  lNewData := FrameFields.Data;
  NewName := lNewData.ColumnName; 
  lNewData.ColumnName := NewName; 
  if lNewData.Pk then
  AlterSQL := 'create table '+ QuoteIdentifier(DataModule.Connection.SQLDialect,TableName)
              + ' ( ' + GetSqlFieldDef(lNewData,DataModule.Connection.SQLDialect,True,True) + ', '+
              ' constraint '+QuoteIdentifier(DataModule.Connection.SQLDialect,
                             'PK_'+TableName)+
                             ' primary key ('+
                                QuoteIdentifier(DataModule.Connection.SQLDialect,
                                NewName)
                             +'));'
  else
  AlterSQL := 'create table '+ QuoteIdentifier(DataModule.Connection.SQLDialect,TableName)
              + ' ( ' + GetSqlFieldDef(lNewData,DataModule.Connection.SQLDialect,True,True) + ');';

  if DataModule.ExecuteDDL(AlterSQL,error)
  then
  begin
    frmMain.ObjectListRefresh(DataModule, NODE_TABLES, TableName);
    Result := True;
  end
  else
  begin
    DisplayMsg(ERR_DDL_EXECUTION,Error);
  end;
  if DataModule.AutoDDL then
  begin
    if Result then
      DataModule.CommitDDL
    else
      DataModule.RollbackDDL;
  end;
end;


procedure TfrmEditorField.edTableChange(Sender: TObject);
begin
  if IsNewTable then
  begin
    TableName := edTable.Text;
    NewName := TableName;
  end;
end;

procedure TfrmEditorField.NewTable;
begin
  ObjectType := NODE_TABLE;
  ResetColumnDef(fData);
  FrameFields.Data := fData;
  edTable.Text := '';
  edTable.Enabled := True;
  FIsNewTable:=True;
  Caption := GetLangText('New')+' '+GetLangType(NODE_TABLE);
  TableName := '';
  FrameFields.chkPrimaryKey.Visible := True;
end;

procedure TfrmEditorField.ActionCancelUpdate(Sender: TObject);
begin
  ActionCancel.Enabled := True; //ObjectModified;
end;

procedure TfrmEditorField.ActionCancelExecute(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TfrmEditorField.ActionOKUpdate(Sender: TObject);
begin
  ActionOK.Enabled := ObjectModified and (TableName <> '');
end;

procedure TfrmEditorField.edTableKeyPress(Sender: TObject; var Key: Char);
begin
  if Assigned(Datamodule) then
    Key := CheckNameKey(Key,Datamodule.Connection.SQLDialect);
end;

procedure TfrmEditorField.SetIsNewTable(const Value: boolean);
begin
  FIsNewTable := Value;
end;

procedure TfrmEditorField.FrameFieldsedColumnChange(Sender: TObject);
begin
  FrameFields.edColumnChange(Sender);
  if not IsNewTable then
  begin
    NewName := FrameFields.Data.ColumnName;
  end;
end;

procedure TfrmEditorField.FrameFieldsedComputedChange(Sender: TObject);
begin
  FrameFields.edComputedChange(Sender);
  if FrameFields.pgTypes.ActivePage = FrameFields.tsComputed then
    stsEditor.Panels[0].Text := Format('%d:%d',[FrameFields.edComputed.CaretY,FrameFields.edComputed.CaretX]);
end;

procedure TfrmEditorField.FrameFieldstsComputedShow(Sender: TObject);
begin
  if FrameFields.pgTypes.ActivePage = FrameFields.tsComputed then
    stsEditor.Panels[0].Text := Format('%d:%d',[FrameFields.edComputed.CaretY,FrameFields.edComputed.CaretX]);
end;

procedure TfrmEditorField.FrameFieldsedDefaultStatusChange(Sender: TObject;
  Changes: TSynStatusChanges);
begin
  if FrameFields.pgTypes.ActivePage = FrameFields.tsDefault then
    stsEditor.Panels[0].Text := Format('%d:%d',[FrameFields.edDefault.CaretY,FrameFields.edDefault.CaretX]);
end;

procedure TfrmEditorField.FrameFieldsedDefaultChange(Sender: TObject);
begin
  FrameFields.edDefaultChange(Sender);
end;

procedure TfrmEditorField.FormCreate(Sender: TObject);
begin
  FrameFields.OnObjectModified := FrameFieldsOnObjectModified;
  FrameFields.chkPrimaryKey.Visible := False;
  ObjectType := NODE_COLUMN;
end;

procedure TfrmEditorField.FrameFieldsOnObjectModified(Sender: TObject);
begin
  ObjectModified := CompareColumnDef(fdata,FrameFields.Data);
end;

procedure TfrmEditorField.DoLoadObject;
begin
  inherited;
  LoadField(TableName,ObjectName);
  FrameFields.Data := fData;
  FrameFields.tsDefault.TabVisible := False;
end;

end.
