{************************************************************************
{  This Unit was created to by Lorenzo Mengoni - (Ancona, Italy)
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

}
{******************************************************************}
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{******************************************************************}
unit frmuEditorFilter;

interface

uses
{$IFDEF LINUX}
  QControls, QForms, QDialogs, QComCtrls, QStdCtrls, QMenus,
  Types, QTypes, QImgList, QExtCtrls,
  QStdActns, QActnList, SynEdit, SynEditPrint, IBCXDummies,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Controls, Forms, Dialogs, ComCtrls, StdCtrls,
  Menus, ImgList, StdActns, ActnList, Graphics, Spin, Buttons,  
{$ENDIF}
  SysUtils, Variants, Classes, SynEdit, dmuNode, IBOConsoleForm;

type
  TFrmEditorFilter = class(TIBOConsoleForm)
    objControl: TPageControl;
    tabFilters: TTabSheet;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    edFilterModule: TEdit;
    edFilterEntry: TEdit;
    edFilterInput: TEdit;
    edFilterOutput: TEdit;
    tsMetadata: TTabSheet;
    reMetaData: TSynEdit;
    tsDescription: TTabSheet;
    edDescription: TSynEdit;
    stsEditor: TStatusBar;
    ActionList1: TActionList;
    actReset: TAction;
    ActionCompile: TAction;
    SpeedButton1: TSpeedButton;
    Action1: TAction;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure edDescriptionStatusChange(Sender: TObject;
      Changes: TSynStatusChanges);
    procedure ActionCompileUpdate(Sender: TObject);
    procedure ActionCompileExecute(Sender: TObject);
  private
    FObjectModified: boolean;
    FNewObject: boolean;
    FDropClose: boolean;
    FObjectName: String;
    FDataModule: TDmN;
    FFilterData: TStringList;
    FRefreshList:boolean;
    FErrors:boolean;
    procedure SetDataModule(const Value: TDmN);
    procedure SetDropClose(const Value: boolean);
    procedure SetNewObject(const Value: boolean);
    procedure SetObjectModified(const Value: boolean);
    procedure SetObjectName(const Value: String);
    procedure SaveDescription;
    procedure LoadDescription;
    procedure GetDDL;
    procedure LoadObject;
    function InternalCloseQuery : Boolean;
    procedure DoExtractMetadata;
    procedure DoCompile;
    { Private declarations }
  public
    { Public declarations }
    procedure NewFilter;
    property ObjectModified:boolean read FObjectModified write SetObjectModified;
    property NewObject:boolean read FNewObject write SetNewObject;
    property DropClose:boolean read FDropClose write SetDropClose;
    property ObjectName:String read FObjectName write SetObjectName;
    property DataModule:TDmN read FDataModule write SetDataModule;
  end;


implementation

uses IBOCGlobal, zluUtility, IBOExtract, frmuWindowList, LMMessageDialog;

{$R *.dfm}

{ TFrmEditorFilter }

procedure TFrmEditorFilter.GetDDL;
begin
  DoExtractMetadata;
end;

procedure TFrmEditorFilter.DoCompile;
begin
  ObjectModified := False;
end;

function TFrmEditorFilter.InternalCloseQuery: Boolean;
begin
    Result := True;
    if ObjectModified then
    begin
      case LMMessageDlg(10,[GetTypeArticle(NODE_BLOB_FILTER),GetLangType(NODE_BLOB_FILTER), FObjectName], mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
        mrYes:
          begin
            DoCompile;
            if FErrors then
              Result := False
            else
              Result := True;
          end;
        mrCancel :
          Result := False;
        mrNo :
          begin
            Result := True;
            ObjectModified := False;
          end;
      end;
    end;
    if edDescription.Modified then
    begin
      case LMMessageDlg(35,[GetLangType(NODE_BLOB_FILTER), FObjectName], mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
        mrYes:     SaveDescription;
        mrCancel : Result := False;
        mrNo :
          begin
            Result := True;
            edDescription.Modified := False;
          end;  
      end;
    end;

end;

procedure TFrmEditorFilter.LoadDescription;
begin
  if Assigned(Datamodule) then
  begin
    edDescription.Text := DataModule.GetObjectDescription(NODE_BLOB_FILTERS,FObjectName);
    edDescription.Modified := False;
  end;
end;

procedure TFrmEditorFilter.LoadObject;
var
  lLine: String;
begin
  if not Assigned(FFilterData) then
    FFilterData := TStringList.Create;

  if (FFilterData.Count = 0) or FRefreshList = true then
  begin
    FFilterData.Clear;
    FRefreshList := false;
    DataModule.GetFilterData(FFilterData, FObjectName);
  end;

  lLine := FFilterData.Strings[0];
  edFilterModule.Text := GetNextField(lLine,DEL);
  edFilterEntry.Text := GetNextField(lLine,DEL);
  edFilterInput.Text := GetNextField(lLine,DEL);
  edFilterOutput.Text := GetNextField(lLine,DEL);
end;


procedure TFrmEditorFilter.DoExtractMetadata;
begin
  FRefreshList := false;
  Screen.Cursor := crHourGlass;
  try
    DataModule.IBOExtract.ExtractObject(eoBLOBFilter, FObjectName);
    reMetadata.Text := DataModule.IBOExtract.Items.Text;
{$IFNDEF NOT_YET_IN_KYLIX}
    reMetadata.Perform(EM_SCROLLCARET, 0, 0);
{$ENDiF}
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TFrmEditorFilter.NewFilter;
begin
  FObjectName := 'new_filter';
  FNewObject := True;
  Caption := 'Blob Filter - '+FObjectName;
  reMetaData.ClearAll;
  edDescription.ClearAll;
  ObjectModified := True;
end;

procedure TFrmEditorFilter.SaveDescription;
begin
  if Assigned(Datamodule) then
  begin
    DataModule.ObjectDescriptionUpdate(NODE_BLOB_FILTERS,FObjectName,edDescription.Text);
    edDescription.Modified := False;
  end;
end;

procedure TFrmEditorFilter.SetDataModule(const Value: TDmN);
begin
  FDataModule := Value;
end;

procedure TFrmEditorFilter.SetDropClose(const Value: boolean);
begin
  FDropClose := Value;
end;

procedure TFrmEditorFilter.SetNewObject(const Value: boolean);
begin
  FNewObject := Value;
end;

procedure TFrmEditorFilter.SetObjectModified(const Value: boolean);
begin
  if FObjectModified <> Value then
  begin
    FObjectModified := Value;
    if FObjectModified then
    begin
      stsEditor.Panels[0].Text := '*';
    end
    else
    begin
      stsEditor.Panels[0].Text := '';
    end;
  end;
end;

procedure TFrmEditorFilter.SetObjectName(const Value: String);
begin
  FObjectName := Value;
  Caption := 'Blob Filter - '+FObjectName;
  LoadObject;
  LoadDescription;
  GetDDL;
  ObjectModified := False;
end;

procedure TFrmEditorFilter.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := CaFree;
  OpenWindows.Remove(Self);
end;

procedure TFrmEditorFilter.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := InternalCloseQuery;
end;

procedure TFrmEditorFilter.edDescriptionStatusChange(Sender: TObject;
  Changes: TSynStatusChanges);
begin
  if objControl.ActivePage = tsDescription then
    stsEditor.Panels[0].Text := Format('%d:%d',[edDescription.CaretY,edDescription.CaretX]);
end;

procedure TFrmEditorFilter.ActionCompileUpdate(Sender: TObject);
begin
  ActionCompile.Enabled := ObjectModified;
end;

procedure TFrmEditorFilter.ActionCompileExecute(Sender: TObject);
begin
  DoCompile;
end;

end.
