{******************************************************************}
{  This Unit was created to by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com
}
{******************************************************************}
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{******************************************************************} 
{
15.August.2002, Lorenzo:
- Bugfix in extraction of index definition data in existing indexes 
}

unit frmuEditorIndex;

interface

uses
{$IFDEF LINUX}
  QControls, QForms, QDialogs, QComCtrls, QStdCtrls, QMenus,
  Types, QTypes, QImgList, QExtCtrls,
  QStdActns, QActnList, SynEdit, SynEditPrint, IBCXDummies,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Controls, Forms, Dialogs, ComCtrls, StdCtrls,
  Menus, ImgList, StdActns, ActnList, Graphics, Spin,
  Buttons, ExtCtrls,
{$ENDIF}
  SysUtils, Classes, Db, dmuNode, types, IBOConsoleForm, FrameFieldsList,
  IBODataset, IB_Components;

type
  TfrmEditorIndex = class(TIBOConsoleEditor)
    btnOK: TButton;
    btnCancel: TButton;
    Label6: TLabel;
    edIdxName: TEdit;
    chkUnique: TCheckBox;
    Label2: TLabel;
    cmbOrder: TComboBox;
    chkActive: TCheckBox;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    IndexFrame: TFrameIndexList;
    StatusBar1: TStatusBar;
    Panel5: TPanel;
    ActionList1: TActionList;
    ActionCompile: TAction;
    procedure FormCreate(Sender: TObject);
    procedure edIdxNameChange(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnCancelClick(Sender: TObject);
    procedure chkActiveClick(Sender: TObject);
    procedure chkUniqueClick(Sender: TObject);
    procedure cmbOrderChange(Sender: TObject);
    procedure edIdxNameKeyPress(Sender: TObject; var Key: Char);
    procedure ActionCompileUpdate(Sender: TObject);
    procedure ActionCompileExecute(Sender: TObject);
  private
    { Private declarations }
    FTableName: String;
    FSQLDialect : Integer;
    FCompileText: String;
    fNewData:TIndexDef;
    fOldData:TIndexDef;
    FInActiveChanged: boolean;
    { Private declarations }
    procedure ListChange(Sender: TObject);
    procedure SetInActiveChanged(const Value: boolean);
    function CreateIndexSql: String;
    procedure SetCompileText(const Value: String);
  protected
    procedure SetObjectModified(const Value : Boolean); override;
    procedure SetDataModule(const Value: TDmn); override;
    procedure DoLoadObject; override;
    function CompileNewObject: boolean; override;
    function AlterObject: boolean; override;
    procedure SignalNewObject; override;
    procedure SignalObjectModified; override;
    procedure SignalObjectNameChanged; override;
  public
    { Public declarations }
    property TableName : String read FTableName write FTableName;
    property CompileText : String read FCompileText write SetCompileText;
    property InActiveChanged:boolean read FInActiveChanged write SetInActiveChanged;
  end;

implementation

uses IBOAdminUtils, zluUtility, IBOCGlobal, frmuMessage, frmuWindowList,
  LMMessageDialog;


{$R *.DFM}

function TfrmEditorIndex.CreateIndexSql:String;
var
    Tmp:String;
    idx:integer;
    sOrder:string;
begin
    if fNewData.IndexType = 0 then
      sOrder := 'ASCENDING'
    else
      sOrder := 'DESCENDING';

    if chkUnique.Checked then
      Result := 'create unique ' + sOrder + ' index ' +
                  QuoteIdentifier(FSQLDialect, NewName) +
                  ' on ' + QuoteIdentifier(FSQLDialect, fNewData.Table) + '('
    else
      Result := 'create ' + sOrder + ' index ' +
                  QuoteIdentifier(FSQLDialect ,NewName) + ' on ' +
                  QuoteIdentifier(FSQLDialect, fNewData.Table) + '(';
    Tmp := '';
    for idx := Low(fNewData.Fields) to high(fNewData.fields) do
    begin
      Tmp := Tmp + QuoteIdentifier(FSQLDialect, fNewData.fields[idx]) + ', ';
    end;
    Tmp := Copy(Tmp, 1, Length(Tmp) - 2);
    Result := Result + Tmp + ')';
end;

procedure TfrmEditorIndex.FormCreate(Sender: TObject);
begin
//  Columns:=TStringList.Create;
  IndexFrame.OnChange := ListChange;
  Objecttype := Node_index;
end;

procedure TfrmEditorIndex.SetObjectModified(const Value: Boolean);
begin
  inherited SetObjectModified(Value);
    if ObjectModified then
    begin
      StatusBar1.Panels[1].Text := '*';
    end
    else
    begin
      StatusBar1.Panels[1].Text := '';
      InActiveChanged := False;
    end;
end;

procedure TfrmEditorIndex.edIdxNameChange(Sender: TObject);
begin
  NewName := edIdxName.Text;
  ObjectModified := True;
end;

procedure TfrmEditorIndex.SetCompileText(const Value: String);
begin
  FCompileText := Value;
end;


procedure TfrmEditorIndex.SetDataModule(const Value: TDmn);
begin
  inherited SetDataModule(Value);
  if Assigned(DataModule) then
  begin
    FSQLDialect := DataModule.Connection.SQLDialect;
    StatusBar1.Panels[2].Text := DataModule.Connection.DatabaseName;
  end
  else
  begin
    FSQLDialect := 0;
    StatusBar1.Panels[2].Text := '';
  end;
end;


procedure TfrmEditorIndex.ListChange(Sender: TObject);
var n:integer;
begin
  SetLength(fnewData.fields,IndexFrame.lvIdxColumns.Count);
  for n := 0 to IndexFrame.lvIdxColumns.Count-1 do
  begin
    fnewData.fields[n]:= IndexFrame.lvIdxColumns.Items[n];
  end;
  ObjectModified := True;
end;

procedure TfrmEditorIndex.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := InternalCloseQuery;
end;

procedure TfrmEditorIndex.btnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TfrmEditorIndex.chkActiveClick(Sender: TObject);
begin
  if chkActive.Checked then
  begin
    fNewData.Inactive := 0;
  end
  else
  begin
    fNewData.Inactive := 1;
  end;
  InActiveChanged := True;
end;

procedure TfrmEditorIndex.chkUniqueClick(Sender: TObject);
begin
  if chkUnique.Checked then
  begin
    fNewData.Unique := 1;
  end
  else
  begin
    fNewData.Unique := 0;
  end;
  ObjectModified := True;
end;

procedure TfrmEditorIndex.SetInActiveChanged(const Value: boolean);
begin
  FInActiveChanged := Value;
end;

procedure TfrmEditorIndex.cmbOrderChange(Sender: TObject);
begin
  fNewData.IndexType := cmbOrder.ItemIndex;
  ObjectModified := True;
end;

procedure TfrmEditorIndex.edIdxNameKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Assigned(Datamodule) then
    Key := CheckNameKey(Key,Datamodule.Connection.SQLDialect);
end;

procedure TfrmEditorIndex.ActionCompileUpdate(Sender: TObject);
begin
  ActionCompile.Enabled := ObjectModified;
end;

procedure TfrmEditorIndex.ActionCompileExecute(Sender: TObject);
begin
  if Compile then
  begin
    OpenWindows.RefreshObject(DataModule,NODE_TABLE,TableName);
    ModalResult := mrOk;
  end;
end;

function TfrmEditorIndex.AlterObject: boolean;
var AlterSQL:TStringList;
    Tmp:String;
begin
  if IndexFrame.lvIdxColumns.Items.Count = 0 then
  begin
    LMMessageDlg(36, mtError, [mbOK], 0);
    Result := False;
    Exit;
  end;
  AlterSQL:=TStringList.Create;
  try
    if InActiveChanged and not ObjectModified then
    begin
      if fNewData.Inactive = 1 then
        AlterSQL.Add('alter index ' + QuoteIdentifier(FSQLDialect, fNewData.Name ) + ' inactive')
      else
        AlterSQL.Add('alter index ' + QuoteIdentifier(FSQLDialect, fNewData.Name) + ' active');
    end
    else
    if ObjectModified then
    begin
      AlterSQL.Add('drop index ' + QuoteIdentifier(FSQLDialect, fNewData.Name ));
      AlterSQL.Add(CreateIndexSql);
      if fNewData.Inactive = 1 then
        AlterSQL.Add('alter index ' + QuoteIdentifier(FSQLDialect, fNewData.Name ) + ' inactive')
    end;

    Result := DataModule.ExecuteDDL(AlterSQL,Tmp);
    CompileErrors := tmp;
  finally
    AlterSQL.Free;
  end;
end;

function TfrmEditorIndex.CompileNewObject: boolean;
var
  Tmp:String;
begin
  if IndexFrame.lvIdxColumns.Items.Count = 0 then
  begin
    LMMessageDlg(36, mtError, [mbOK], 0);
    Result := False;
    Exit;
  end;
  Result := DataModule.ExecuteDDL(CreateIndexSql,Tmp);
  CompileErrors := tmp;
end;

procedure TfrmEditorIndex.DoLoadObject;
begin
  inherited;
  if Assigned(DataModule) then
    if DataModule.GetIndexData(fOldData,ObjectName) = SUCCESS then
    begin
      AssignIndexDef(fNewData,fOldData);
      edIdxName.Text := ObjectName;
      fNewData.Table := fOldData.Table;
      Caption := GetLangType(NODE_INDEX)+' - '+ObjectName;
      chkUnique.Checked := fOldData.Unique = 1;
      chkActive.Checked := fOldData.Inactive = 0;
      cmbOrder.ItemIndex := fOldData.IndexType;
      DataModule.GetColumnList(Indexframe.Fields,fOldData.Table);
      Indexframe.IncludeFields(fOldData.Fields);
      ObjectModified := False;
    end;
end;

procedure TfrmEditorIndex.SignalNewObject;
var s:String;
    pre:String;
    t:String;
    n:integer;
begin
  inherited;
  Caption := GetLangText('New')+' '+GetLangType(NODE_INDEX)+' - ' + TableName ;
  chkActive.Enabled := False;
  if Assigned(DataModule) then
  begin
    fNewData.Table := TableName;
    DataModule.GetColumnList(Indexframe.Fields,TableName);
    Indexframe.RestoreFields;
    ObjectModified := False;
    pre := 'IDX_';
    n:=1;
    t:=copy(TableName,1,31-8);
    s:= pre+t+'_'+inttostr(n);
    while DataModule.IndexNameExist(s) and (n<20) do
    begin
      inc(n);
      s:= pre+t+'_'+inttostr(n);
    end;
    edIdxName.Text := S;
    try
      if CanFocus then
        ActiveControl := edIdxName;
    except
    end;
  end;
end;


procedure TfrmEditorIndex.SignalObjectModified;
begin
  inherited;

end;

procedure TfrmEditorIndex.SignalObjectNameChanged;
begin
  inherited;

end;

end.
