{************************************************************************
{  This Unit was created to by Lorenzo Mengoni - (Ancona, Italy)
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

}
{******************************************************************}
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{******************************************************************} 

unit frmuEditorRole;

interface

uses
{$IFDEF LINUX}
  QControls, QForms, QDialogs, QComCtrls, QStdCtrls, QMenus,
  Types, QTypes, QImgList, QExtCtrls,
  QStdActns, QActnList, SynEdit, SynEditPrint, IBCXDummies,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Controls, Forms, Dialogs, ComCtrls, StdCtrls,
  Menus, ImgList, StdActns, ActnList, Graphics, Spin, Grids, DBGrids,
  DBCtrls, ExtCtrls, ClipBrd, Buttons, 
{$ENDIF}
  SysUtils, Classes, DB, IBODataset, IBOCGlobal, IB_Components, IBOConsoleForm,
  dmuNode, SynEdit;

type
  TfrmRole = class(TIBOConsoleEditor)
    pgObjectEditor: TPageControl;
    tsRole: TTabSheet;
    Label1: TLabel;
    edRoleName: TEdit;
    tsDDLView: TTabSheet;
    stsEditor: TStatusBar;
    ActionList1: TActionList;
    ActionCompile: TAction;
    edMetadata: TSynEdit;
    SpeedButton1: TSpeedButton;
    Action1: TAction;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure pgObjectEditorChanging(Sender: TObject;
      var AllowChange: Boolean);
    procedure pgObjectEditorChange(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure ActionCompileExecute(Sender: TObject);
    procedure ActionCompileUpdate(Sender: TObject);
    procedure edRoleNameChange(Sender: TObject);
    procedure edRoleNameKeyPress(Sender: TObject; var Key: Char);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    procedure GetDDL;
  protected
    procedure DoLoadObject; override;
    procedure SignalObjectModified;  override;
    function CompileNewObject:boolean; override;
    function AlterObject:boolean; override;
  public
    function InternalCloseQuery : Boolean; override;
  end;



implementation

uses
  IB_Schema, IBOAdminUtils, IBOExtract, frmuMain, frmuMessage,
  dmuMain, LMMessageDialog;

{$R *.DFM}

procedure TfrmRole.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TfrmRole.GetDDL;
begin
  if Assigned(DataModule) then
  begin
    DataModule.IBOExtract.ExtractObject(eoRole,ObjectName);
    edMetadata.Lines.Clear;
    edMetadata.Lines.Text := DataModule.IBOExtract.Items.Text;
    edMetadata.Modified := False;
  end;
end;


procedure TfrmRole.FormCreate(Sender: TObject);
begin
  pgObjectEditor.ActivePage := tsRole;
  ActiveControl := edRoleName;
  ObjectType := NODE_ROLE;
end;

procedure TfrmRole.pgObjectEditorChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
  if IsNewObject and (pgObjectEditor.ActivePage = tsRole) then
  begin
    LMMessageDlg(37, mtInformation, [mbOK], 0);
    AllowChange := False;
  end;
end;

procedure TfrmRole.pgObjectEditorChange(Sender: TObject);
begin
  if pgObjectEditor.ActivePage = tsRole then
  begin
    //blank the panels...
    stsEditor.Panels[0].Text := '';
    stsEditor.Panels[1].Text := '';
    stsEditor.Panels[2].Text := '';
  end
  else
  if pgObjectEditor.ActivePage = tsDDLView then
  begin
    edMetadata.SetFocus;
    if not IsNewObject then
      GetDDL;
  end;
end;

procedure TfrmRole.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift = [ssCtrl]) and (Key = VK_TAB) then
    pgObjectEditor.SelectNextPage(True);
end;


function TfrmRole.InternalCloseQuery: Boolean;
begin
  Result := inherited InternalCloseQuery;
end;

procedure TfrmRole.DoLoadObject;
begin
  edRoleName.Text := ObjectName;
  GetDDL;
end;

procedure TfrmRole.SignalObjectModified;
begin
  if ObjectModified then
    stsEditor.Panels[0].Text := '*'
  else
    stsEditor.Panels[0].Text := '';
end;

procedure TfrmRole.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := InternalCloseQuery;
end;

procedure TfrmRole.ActionCompileExecute(Sender: TObject);
begin
  Compile;
end;

procedure TfrmRole.ActionCompileUpdate(Sender: TObject);
begin
  (Sender As TAction).Enabled := ObjectModified;
end;

procedure TfrmRole.edRoleNameChange(Sender: TObject);
begin
  NewName := edRoleName.Text;
  ObjectModified := True;
end;

procedure TfrmRole.edRoleNameKeyPress(Sender: TObject; var Key: Char);
begin
  if Assigned(Datamodule) then
    Key := CheckNameKey(Key,Datamodule.Connection.SQLDialect);
end;

procedure TfrmRole.FormShow(Sender: TObject);
begin
  if Assigned(DataModule) then
  begin
    stsEditor.Panels[3].Text := DataModule.Connection.DatabaseName;
  end
  else
  begin
    stsEditor.Panels[3].Text := '';
  end
end;


function TfrmRole.AlterObject: boolean;
var errors:String;
begin
  Result := DataModule.DropObject(ObjectType,ObjectName,errors);
  if result then
    Result := DataModule.ExecuteDDL('create role ' + QuoteIdentifier(DataModule.Connection.SQLDialect, NewName),errors);
  CompileErrors := errors;
end;

function TfrmRole.CompileNewObject: boolean;
var errors:String;
begin
  Result := DataModule.ExecuteDDL('create role ' + QuoteIdentifier(DataModule.Connection.SQLDialect, NewName),errors);
  if result then ObjectName := NewName;
  CompileErrors := errors;
end;

end.
