{******************************************************************}
{  This Unit was created to by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.
}
{******************************************************************}
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{******************************************************************} 

{
13.August.2002, Lorenzo:
- updated Code for handling trigger description

25.December.2003, Lorenzo:
- added use of EnumTriggerTypes to populate cmbTrigType items
}


unit frmuEditorTrigger;

interface

uses
{$IFDEF LINUX}
  QControls, QForms, QDialogs, QComCtrls, QStdCtrls, QMenus,
  Types, QTypes, QImgList, QExtCtrls,
  QStdActns, QActnList, SynEdit, SynEditPrint, IBCXDummies,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Controls, Forms, Dialogs, ComCtrls, StdCtrls,
  Menus, ImgList, StdActns, ActnList, Graphics, Spin, ExtCtrls,
  Buttons,
{$ENDIF}
  SysUtils, Classes, dmuNode, SynEdit, IBOConsoleForm, IBOExtract;

type
  TParamRec = record
    pName:String;
    pType:String;
  end;
  TParamArray = array of TParamRec;

  TfrmEditorTrigger = class(TIBOConsoleEditor)
    Label1: TLabel;
    edTriggerName: TEdit;
    cmbTrigType: TComboBox;
    Label4: TLabel;
    edPosition: TEdit;
    UpDown1: TUpDown;
    cmbActive: TComboBox;
    Label2: TLabel;
    Label3: TLabel;
    edSource: TSynEdit;
    Label5: TLabel;
    Panel1: TPanel;
    Panel2: TPanel;
    Label6: TLabel;
    EditTable: TEdit;
    edDescription: TSynEdit;
    Label7: TLabel;
    stsEditor: TStatusBar;
    Panel3: TPanel;
    ActionList1: TActionList;
    ActionCompile: TAction;
    SpeedButton1: TSpeedButton;
    Action1: TAction;
    procedure FormCreate(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure edTriggerNameChange(Sender: TObject);
    procedure edTriggerNameKeyPress(Sender: TObject; var Key: Char);
    procedure edSourceStatusChange(Sender: TObject;
      Changes: TSynStatusChanges);
    procedure ActionCompileUpdate(Sender: TObject);
    procedure ActionCompileExecute(Sender: TObject);
    procedure edSourceChange(Sender: TObject);
    procedure edDescriptionChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    //FTriggerName: String;
    FTableName: String;
    FErrors:boolean;
    procedure SetTableName(const Value: String);
  protected
    procedure SetDataModule(const Value: TDmN); override;
    procedure DoLoadObject; override;
    function CompileNewObject: boolean; override;
    function AlterObject: boolean; override;
    procedure SignalNewObject; override;
    procedure SignalObjectModified; override;
    procedure SignalObjectNameChanged; override;
  public
    { Public declarations }
    property TableName:String read FTableName write SetTableName;
    //property TriggerName:String read FTriggerName write SetTriggerName;
    //procedure NewTrigger(aDataModule:TDmN; aTableName:String);
  end;


implementation

uses frmuMain, frmuMessage, IBOAdminUtils, dmuMain, IBOCGlobal,
  frmuWindowList, LMMessageDialog;


{$R *.DFM}

procedure TfrmEditorTrigger.FormCreate(Sender: TObject);
begin
  cmbActive.ItemIndex := 0;
  ObjectType := NODE_TRIGGER;
  EnumTriggerTypes(cmbTrigType.Items);

end;

procedure TfrmEditorTrigger.SetTableName(const Value: String);
begin
  FTableName := Value;
end;

procedure TfrmEditorTrigger.edTriggerNameChange(Sender: TObject);
begin
  NewName := edTriggerName.Text;
end;

procedure TfrmEditorTrigger.SetDataModule(const Value: TDmN);
begin
  inherited SetDataModule(Value);
  if Assigned(DataModule) then
  begin
    stsEditor.Panels[2].Text := DataModule.Connection.DatabaseName;
  end
  else
  begin
    stsEditor.Panels[2].Text := '';
  end
end;


procedure TfrmEditorTrigger.edTriggerNameKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Assigned(Datamodule) then
    Key := CheckNameKey(Key,Datamodule.Connection.SQLDialect);
end;

procedure TfrmEditorTrigger.edSourceStatusChange(Sender: TObject;
  Changes: TSynStatusChanges);
begin
  stsEditor.Panels[0].Text := Format('%d:%d',[edSource.CaretY,edSource.CaretX]);
end;

procedure TfrmEditorTrigger.ActionCompileUpdate(Sender: TObject);
begin
  ActionCompile.Enabled := ObjectModified;
end;

procedure TfrmEditorTrigger.ActionCompileExecute(Sender: TObject);
begin
  if Compile then
  begin
    Caption := 'Trigger - '+ObjectName;
    OpenWindows.RefreshObject(DataModule,NODE_TABLE,TableName);
  end;
end;

function TfrmEditorTrigger.AlterObject: boolean;
var
  AlterSQL:String;
  error:String;
  source:String;
  lSqlDialect:integer;
begin
  lSqlDialect := DataModule.Connection.SQLDialect;
  AlterSQL := '';

  if cmbTrigType.Text = '' then
  begin
    LMMessageDlg(45, mtError, [mbOK], 0);
    Result := False;
    Exit;
  end;

  source:= Trim(edSource.Lines.Text);

  if not ((Length(source)>1) and
     (Upcase(source[1]) = 'A') and
     (Upcase(source[2]) = 'S'))
  then
     source := 'as' + CRLF + source;
  source := source + '^';
    AlterSQL := AlterSQL + 'set term ^; '+CRLF;
    AlterSQL := AlterSQL + 'alter trigger ' + QuoteIdentifier(lSQLDialect,NewName) ;
    AlterSQL := AlterSQL + ' ' + cmbActive.Text + ' ';
    AlterSQL := AlterSQL + cmbTrigType.Text + ' position ' + edPosition.Text + CRLF + source+ CRLF;
    AlterSQL := AlterSQL + 'set term ; ^ ';

  Result := DataModule.ExecuteDDLScript(AlterSQL,error);
  CompileErrors := error;
  if edDescription.Modified then
  try
    DataModule.ObjectDescriptionUpdate(NODE_TRIGGER,NewName,edDescription.Text);
    edDescription.Modified := False;
  except
    edDescription.Modified := False;
  end;
end;

function TfrmEditorTrigger.CompileNewObject: boolean;
var
  AlterSQL:String;
  error:String;
  source:String;
  lSqlDialect:integer;
begin
  lSqlDialect := DataModule.Connection.SQLDialect;
  AlterSQL := '';

  if cmbTrigType.Text = '' then
  begin
    LMMessageDlg(45, mtError, [mbOK], 0);
    Result := False;
    Exit;
  end;

  source:= Trim(edSource.Lines.Text);

  if not ((Length(source)>1) and
     (Upcase(source[1]) = 'A') and
     (Upcase(source[2]) = 'S'))
  then
     source := 'as' + CRLF + source;
  source := source + '^';
    AlterSQL := AlterSQL + 'set term ^; '+CRLF;
    AlterSQL := AlterSQL + 'create trigger ' + QuoteIdentifier(lSQLDialect,NewName) + ' for ';
    AlterSQL := AlterSQL + QuoteIdentifier(lSQLDialect,TableName) + ' ' + cmbActive.Text + ' ';
    AlterSQL := AlterSQL + cmbTrigType.Text + ' position ' + edPosition.Text + CRLF + source+ CRLF;
    AlterSQL := AlterSQL + 'set term ; ^ ';

  Result := DataModule.ExecuteDDLScript(AlterSQL,error);
  CompileErrors := error;
  if edDescription.Modified then
  try
    DataModule.ObjectDescriptionUpdate(NODE_TRIGGER,NewName,edDescription.Text);
    edDescription.Modified := False;
  except
    edDescription.Modified := False;
  end;
end;

procedure TfrmEditorTrigger.DoLoadObject;
var
  TriggerType,
  Active,
  Table,
  FireOrder,
  Description,
  Source: String;
begin
  edTriggerName.Text := ObjectName;
  Caption := 'Trigger - '+ObjectName;
  if DataModule.GetTriggerData(ObjectName,TriggerType,Active,Table,FireOrder,Source,Description) = SUCCESS then
  begin
    cmbTrigType.ItemIndex := StrToIntDef(TriggerType,0)-1;
    UpDown1.Position := StrToIntDef(FireOrder,-1);
    TableName := Table;
    EditTable.Text := Table;
    cmbActive.ItemIndex := StrToIntDef(Active,-1);
    edSource.Lines.Text := Source;
    edDescription.Lines.Text := Description;
    edDescription.Modified := False;
    edTriggerName.ReadOnly := True;
  end;
end;

procedure TfrmEditorTrigger.SignalNewObject;
var s:String;
    pre:String;
    t:String;
    n:integer;
begin
  inherited;
  EditTable.Text := TableName;
  edTriggerName.ReadOnly := False;
  pre := 'TRIG_';
  n:=1;
  t:=copy(TableName,1,31-8);
  s:= pre+t+'_'+inttostr(n);
  while DataModule.TriggerNameExist(s) and (n<20) do
  begin
    inc(n);
    s:= pre+t+'_'+inttostr(n);
  end;
  edTriggerName.Text := S;
  NewName :=s;
  edSource.Lines.Add('/* code here */');
  edDescription.Modified := False;
  //edSource.Lines.Add('POST_EVENT '+AnsiQuotedStr(FTriggerName,#39)+';');
end;

procedure TfrmEditorTrigger.SignalObjectModified;
begin
  inherited;

end;

procedure TfrmEditorTrigger.SignalObjectNameChanged;
begin
  inherited;

end;

procedure TfrmEditorTrigger.btnOKClick(Sender: TObject);
var
  AlterSQL:String;
  error:String;
  source:String;
  lSqlDialect:integer;
begin
  lSqlDialect := DataModule.Connection.SQLDialect;
  AlterSQL := '';

  if NewName = '' then
  begin
    LMMessageDlg(46, mtError, [mbOK], 0);
    Exit;
  end;

  if cmbTrigType.Text = '' then
  begin
    LMMessageDlg(45, mtError, [mbOK], 0);
    Exit;
  end;

  source:= Trim(edSource.Lines.Text);

  if not ((Length(source)>1) and
     (Upcase(source[1]) = 'A') and
     (Upcase(source[2]) = 'S'))
  then
     source := 'as' + CRLF + source;
  source := source + '^';
  if isNewObject then
  begin
    AlterSQL := AlterSQL + 'set term ^; '+CRLF;
    AlterSQL := AlterSQL + 'create trigger ' + QuoteIdentifier(lSQLDialect,NewName) + ' for ';
    AlterSQL := AlterSQL + QuoteIdentifier(lSQLDialect,TableName) + ' ' + cmbActive.Text + ' ';
    AlterSQL := AlterSQL + cmbTrigType.Text + ' position ' + edPosition.Text + CRLF + source+ CRLF;
    AlterSQL := AlterSQL + 'set term ; ^ ';
  end
  else
  begin
    AlterSQL := AlterSQL + 'set term ^; '+CRLF;
    AlterSQL := AlterSQL + 'alter trigger ' + QuoteIdentifier(lSQLDialect,NewName) ;
    AlterSQL := AlterSQL + ' ' + cmbActive.Text + ' ';
    AlterSQL := AlterSQL + cmbTrigType.Text + ' position ' + edPosition.Text + CRLF + source+ CRLF;
    AlterSQL := AlterSQL + 'set term ; ^ ';
  end;

  if DataModule.ExecuteDDLScript(AlterSQL,error) then
  begin
    if edDescription.Modified then
    try
      DataModule.ObjectDescriptionUpdate(NODE_TRIGGER,NewName,edDescription.Text);
      edDescription.Modified := False;
    except
      edDescription.Modified := False;
    end;
    ModalResult := mrOK;
  end
  else
  begin
    DisplayMsg(ERR_DDL_EXECUTION,Error);
    FErrors := True;
  end;

end;

procedure TfrmEditorTrigger.edSourceChange(Sender: TObject);
begin
  ObjectModified := True;
end;

procedure TfrmEditorTrigger.edDescriptionChange(Sender: TObject);
begin
  EditDescription := edDescription.Text;
end;

procedure TfrmEditorTrigger.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := CaFree;
end;

end.

