{******************************************************************}
{  This Unit was created to by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.
}
{******************************************************************}
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{******************************************************************} 

unit frmuEditorUDF;

interface

uses
{$IFDEF LINUX}
  QControls, QForms, QDialogs, QComCtrls, QStdCtrls, QMenus,
  Types, QTypes, QImgList, QExtCtrls,
  QStdActns, QActnList, SynEdit, SynEditPrint, IBCXDummies,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Controls, Forms, Dialogs, ComCtrls, StdCtrls,
  Menus, ImgList, StdActns, ActnList, Graphics, Spin, Grids, DBGrids, DBCtrls,
   Printers, ClipBrd, ExtCtrls, Buttons,
{$ENDIF}
  SysUtils, Classes, DB, dmuNode, IBODataset, SynEdit, IBOConsoleForm;

type
  TfrmUDFEditor = class(TIBOConsoleForm)
    pgObjectEditor: TPageControl;
    tsObject: TTabSheet;
    tsDocoView: TTabSheet;
    qryUtil: TIBOQuery;
    tsDDL: TTabSheet;
    stsEditor: TStatusBar;
    Panel2: TPanel;
    lvUDFParams: TListView;
    Panel1: TPanel;
    Label5: TLabel;
    Label4: TLabel;
    Label3: TLabel;
    Label2: TLabel;
    Label1: TLabel;
    Label6: TLabel;
    cmbRtnType: TComboBox;
    edLibraryName: TEdit;
    edEntryPoint: TEdit;
    edUDFName: TEdit;
    edRtnParam: TEdit;
    edDescription: TSynEdit;
    edMetadata: TSynEdit;
    SpeedButton1: TSpeedButton;
    ActionList1: TActionList;
    ActionCompile: TAction;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure pgObjectEditorChanging(Sender: TObject;
      var AllowChange: Boolean);
    procedure pgObjectEditorChange(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure lvUDFParamsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cmbRtnTypeChange(Sender: TObject);
    procedure ActionCompileUpdate(Sender: TObject);
    procedure ActionCompileExecute(Sender: TObject);
    procedure edUDFNameKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FErrors : Boolean;
    FObjectName: String;
    FNewObject: Boolean;
    FObjectModified: boolean;
    FDataModule: TDmN;

    procedure SetDataModule(const Value: TDmN);
    procedure LoadDescription;
    procedure SaveDescription;
    procedure GetDDL;
    procedure LoadUDF;
    function InternalCloseQuery : Boolean;
    procedure SetObjectName(Value : String);
    procedure SetObjectModified(Value : Boolean);
    procedure DoCompile;  //override;
    function CanObjectNewInputParam : Boolean;  //override;
    procedure DoObjectNewInputParam;  //override;
    function CanObjectDropInputParam : Boolean;  //override;
    procedure DoObjectDropInputParam;  //override;
    function ParamText(Index: Integer): String;
    function ReturnType: String;
  public
    { Public declarations }
    procedure NewUDF;
    property ObjectName:String read FObjectName write SetObjectName;
    property ObjectModified:boolean read FObjectModified write SetObjectModified;
    property NewObject:Boolean read FNewObject;
    property DataModule:TDmN read FDataModule write SetDataModule;

  end;


implementation

uses
  frmuUDFInputParam, IBOCGlobal, frmuMain, IBOExtract,
  frmuWindowList, IBOAdminUtils, dmuMain, LMMessageDialog;

{$R *.DFM}

procedure TfrmUDFEditor.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
  OpenWindows.Remove(self);
end;

procedure TfrmUDFEditor.FormCreate(Sender: TObject);
begin
  pgObjectEditor.ActivePage := tsObject;
end;

procedure TfrmUDFEditor.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := InternalCloseQuery;
end;


procedure TfrmUDFEditor.pgObjectEditorChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
  if FNewObject then
  begin
    LMMessageDlg(29, mtWarning, [mbOK], 0);
    AllowChange := False;
  end
  else
  begin
    if pgObjectEditor.ActivePage = tsDocoView then
    begin
      if edDescription.Modified then
      begin
        if LMMessageDlg(30, mtConfirmation, [mbYes, mbNo], 0) = mrYes then
          SaveDescription;
      end;
    end;
  end;
end;

procedure TfrmUDFEditor.pgObjectEditorChange(Sender: TObject);
begin
  if pgObjectEditor.ActivePage = tsObject then
  begin
    stsEditor.Panels[0].Text := '';
    stsEditor.Panels[1].Text := '';
    stsEditor.Panels[2].Text := '';
    edUDFName.SetFocus;
  end
  else
  if pgObjectEditor.ActivePage = tsDocoView then
  begin
    edDescription.SetFocus;
  end
  else
  if pgObjectEditor.ActivePage = tsDDL then
  begin
    edMetadata.SetFocus;
    if not FNewObject then
      GetDDL;
  end;
end;

procedure TfrmUDFEditor.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift = [ssCtrl]) and (Key = VK_TAB) then
    pgObjectEditor.SelectNextPage(true);
end;

function TfrmUDFEditor.InternalCloseQuery: Boolean;
begin
    Result := True;
    if ObjectModified then
    begin
      case LMMessageDlg(10,[GetTypeArticle(NODE_FUNCTION),GetLangType(NODE_FUNCTION),FObjectName], mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
        mrYes:
          begin
            DoCompile;
            if FErrors then
              Result := False
            else
              Result := True;
          end;
        mrCancel :
          Result := False;
        mrNo :
          begin
            Result := True;
            ObjectModified := False;
          end;
      end;
    end;
    if edDescription.Modified then
    begin
      case LMMessageDlg(35,[GetLangType(NODE_FUNCTION),FObjectName], mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
        mrYes:
          begin
            SaveDescription;
          end;
        mrCancel :
          Result := False;
        mrNo :
          begin
            Result := True;
            edDescription.Modified := False;
          end;  
      end;
    end;
end;

procedure TfrmUDFEditor.SetObjectModified(Value: Boolean);
begin
  FObjectModified := Value;
  if FObjectModified then
    stsEditor.Panels[1].Text := '*' 
  else
    stsEditor.Panels[1].Text := '';
end;

procedure TfrmUDFEditor.SetObjectName(Value: String);
begin
  FObjectName := Value;
  Caption := GetLangType(NODE_FUNCTION)+' - ' + FObjectName;
  edUDFName.Text := FObjectName;
  LoadUDF;
  LoadDescription;
  ObjectModified := False;
  fNewObject := False;
  OpenWindows.UpdateWindowList(Caption,self);
end;

procedure TfrmUDFEditor.LoadUDF;
var
  RtnPosition : Integer;
  L : TListItem;
  
begin
  try
    qryUtil.BeginBusy(False);
    qryUtil.SQL.Clear;
    qryUtil.SQL.Add('select * from rdb$functions where rdb$function_name = ' + AnsiQuotedStr(FObjectName, '''') + ';');
    qryUtil.Open;

    //now fill the controls.....

    edUDFName.Text := qryUtil.FieldByName('rdb$function_name').AsString;
    edEntryPoint.Text := qryUtil.FieldByName('rdb$entrypoint').AsString;
    edLibraryName.Text := qryUtil.FieldByName('rdb$module_name').AsString;
    RtnPosition := qryUtil.FieldByName('rdb$return_argument').AsInteger;

    qryUtil.Close;

    qryUtil.SQL.Clear;
    qryUtil.SQL.Add('select * from rdb$function_arguments where rdb$function_name = ' + AnsiQuotedStr(FObjectName, '''') + ' order by rdb$argument_position asc;');
    qryUtil.Open;

    While Not qryUtil.EOF do
    begin
      if qryUtil.FieldByName('rdb$argument_position').AsInteger = RtnPosition then
      begin
        if DataModule.Connection.SQLDialect = 3 then
        begin
          edRtnParam.Text := ConvertFieldType(qryUtil.FieldByName('rdb$field_type').AsInteger,
                                              qryUtil.FieldByName('rdb$field_length').AsInteger,
                                              qryUtil.FieldByName('rdb$field_scale').AsInteger,
                                              qryUtil.FieldByName('rdb$field_sub_type').AsInteger,
                                              qryUtil.FieldByName('rdb$field_precision').AsInteger,
                                              DataModule.Connection.SQLDialect);
        end
        else
        begin
          edRtnParam.Text := ConvertFieldType(qryUtil.FieldByName('rdb$field_type').AsInteger,
                                              qryUtil.FieldByName('rdb$field_length').AsInteger,
                                              qryUtil.FieldByName('rdb$field_scale').AsInteger,
                                              -1,
                                              -1,
                                              DataModule.Connection.SQLDialect);
        end;
        cmbRtnType.ItemIndex := qryUtil.FieldByName('rdb$mechanism').AsInteger;
      end
      else
      begin
        L := lvUDFParams.Items.Add;

        if DataModule.Connection.SQLDialect = 3 then
        begin
          L.Caption := ConvertFieldType(qryUtil.FieldByName('rdb$field_type').AsInteger,
                                        qryUtil.FieldByName('rdb$field_length').AsInteger,
                                        qryUtil.FieldByName('rdb$field_scale').AsInteger,
                                        qryUtil.FieldByName('rdb$field_sub_type').AsInteger,
                                        qryUtil.FieldByName('rdb$field_precision').AsInteger,
                                        DataModule.Connection.SQLDialect);
        end
        else
        begin
          L.Caption := ConvertFieldType(qryUtil.FieldByName('rdb$field_type').AsInteger,
                                        qryUtil.FieldByName('rdb$field_length').AsInteger,
                                        qryUtil.FieldByName('rdb$field_scale').AsInteger,
                                        -1,
                                        -1,
                                        DataModule.Connection.SQLDialect);
        end;
        case qryUtil.FieldByName('rdb$mechanism').AsInteger of
          0 : L.SubItems.Add(GetLangText('By Value'));
          1 : L.SubItems.Add(GetLangText('By Reference'));
        end;
      end;
      qryUtil.Next;
    end;
    qryUtil.Close;
  finally
    qryUtil.EndBusy;
  end;
end;

procedure TfrmUDFEditor.NewUDF;
begin
  FObjectName := 'new_udf';
  Caption := GetLangType(NODE_FUNCTION)+' - ' + FObjectName ;
  edUDFName.Text := FObjectName;
  ActiveControl := edUDFName;
  ObjectModified := True;
  FNewObject := True;
end;

function TfrmUDFEditor.ParamText(Index: Integer): String;
begin
  Result := lvUDFParams.Items.Item[Index].Caption;
  if lvUDFParams.Items.Item[Index].SubItems[0] = 'By Value' then
    Result := Result + ' by value';
end;

function TfrmUDFEditor.ReturnType: String;
begin
  Result := edRtnParam.Text;
  if cmbRtnType.Text = 'value' then
    Result := Result + ' by value';
end;


procedure TfrmUDFEditor.DoCompile;
var
  Doco : String;
  CompileText : String;
  FNewFlag:boolean;
  DTmp:String;
  Idx:integer;

begin
  FNewFlag := False;
  Doco := edDescription.Text;

  Refresh;
  if edUDFName.Text = '' then
  begin
    LMMessageDlg(8,[GetTypeArticle(NODE_FUNCTION),GetLangType(NODE_FUNCTION)], mtError, [mbOK], 0);
    edUDFName.SetFocus;
    Exit;
  end;

  if edEntryPoint.Text = '' then
  begin
    LMMessageDlg(47, mtError, [mbOK], 0);
    edEntryPoint.SetFocus;
    Exit;
  end;

  if edLibraryName.Text = '' then
  begin
    LMMessageDlg(48, mtError, [mbOK], 0);
    edLibraryName.SetFocus;
    Exit;
  end;

  if edRtnParam.Text = '' then
  begin
    LMMessageDlg(49, mtError, [mbOK], 0);
    edRtnParam.SetFocus;
    Exit;
  end;

  CompileText := '';

  try
    qryUtil.IB_Connection := DataModule.Connection;
    qryUtil.IB_Transaction := DataModule.DDLTransaction;
    qryUtil.SQL.Text := 'select rdb$function_name '+
                    'from rdb$functions where rdb$function_name = '
                    + AnsiQuotedStr(FObjectName, '''') + ';';
    qryUtil.Open;
    If not (qryUtil.BOF and qryUtil.EOF) then
      FNewFlag := False
    else
      FNewFlag := True;
    qryUtil.Close;
    qryUtil.SQL.Clear;

    if Not FNewFlag then
    begin
      DTmp := 'drop external function ' + QuoteIdentifier(DataModule.Connection.SQLDialect,FObjectName)  + ';';
      qryUtil.SQL.Text := DTmp;
      qryUtil.ExecSQL;
    end;

    //build the statement from the owner form......
    DTmp := 'declare external function ' + QuoteIdentifier(DataModule.Connection.SQLDialect,edUDFName.Text) + ' ';

    //add input params
    for Idx := 0 to lvUDFParams.Items.Count - 1 do
    begin
      if Idx < (lvUDFParams.Items.Count - 1) then
        DTmp := DTmp + ', '+ ParamText(Idx)
      else
        DTmp := DTmp + ParamText(Idx);
    end;

    DTmp := DTmp + ' ';

    //add out put params
    DTmp := DTmp + 'returns ' + ReturnType;

    //add entry point and module name....

    DTmp := DTmp + ' entry_point ''' + edEntryPoint.Text + ''' module_name ''' + edLibraryName.Text + ''';';

    qryUtil.SQL.Text := DTmp;
    qryUtil.ExecSQL;

  except
    on e:Exception do
    begin
      LMMessageDlg(e.Message, mtError, [mbOk], 0);
      FErrors := True;
    end;
  end;

  if FErrors then  Exit;

  ObjectName := edUDFName.Text;
  LoadUDF;
  if Not FNewFlag then
  begin
    edDescription.Text := Doco;
    SaveDescription;
  end
  else
    LoadDescription;
  FNewObject := False;
  ObjectModified := False;
  frmMain.ObjectListRefresh(DataModule, NODE_FUNCTIONS);

end;

function TfrmUDFEditor.CanObjectNewInputParam: Boolean;
begin
  Result := pgObjectEditor.ActivePage = tsObject;
end;

procedure TfrmUDFEditor.DoObjectNewInputParam;
var
  frmUDFAddInput: TfrmUDFAddInput;
  I : TListItem;

begin
  frmUDFAddInput := TfrmUDFAddInput.Create(Self);
  try
    with frmUDFAddInput do
    begin
      if ShowModal = mrOK then
      begin
        I := lvUDFParams.Items.Add;
        I.Caption := LowerCase(edParameter.Text);
        case cmbRtnType.ItemIndex of
          0 : I.SubItems.Add(GetLangText('By Value'));
          1 : I.SubItems.Add(GetLangText('By Reference'));
        end;
      end;
    end;
  finally
    frmUDFAddInput.Free;
  end;
end;

function TfrmUDFEditor.CanObjectDropInputParam: Boolean;
begin
  Result := (pgObjectEditor.ActivePage = tsObject) and
            (lvUDFParams.Selected <> nil);
end;

procedure TfrmUDFEditor.DoObjectDropInputParam;
begin
  if LMMessageDlg(50, mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    lvUDFParams.Selected.Delete;
end;

procedure TfrmUDFEditor.lvUDFParamsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_INSERT:
      begin
        if CanObjectNewInputParam then
          DoObjectNewInputParam;
      end;
    VK_DELETE:
      begin
        if CanObjectDropInputParam then
          DoObjectDropInputParam;
      end;
  end;
end;

procedure TfrmUDFEditor.SetDataModule(const Value: TDmN);
begin
  FDataModule := Value;
  if not Assigned(FDataModule) then
  begin
    qryUtil.IB_Connection := nil;
    stsEditor.Panels[3].Text := GetLangText('No Connection');
  end
  else
  begin
    qryUtil.IB_Connection := FDataModule.Connection;
    qryUtil.IB_Transaction := FDataModule.Transaction;
    stsEditor.Panels[3].Text := FDataModule.Connection.DatabaseName;
  end;
end;

procedure TfrmUDFEditor.SaveDescription;
begin
  if Assigned(Datamodule) then
  begin
    DataModule.ObjectDescriptionUpdate(NODE_FUNCTIONS,FObjectName,edDescription.Text);
    edDescription.Modified := False;
    frmMain.ObjectListRefresh(DataModule, NODE_FUNCTIONS);
  end;
end;

procedure TfrmUDFEditor.LoadDescription;
begin
  if Assigned(Datamodule) then
  begin
    edDescription.Text := DataModule.GetObjectDescription(NODE_FUNCTIONS,FObjectName);
    edDescription.Modified := False;
  end;
end;

procedure TfrmUDFEditor.GetDDL;
begin
  if Assigned(DataModule) then
  begin
    DataModule.IBOExtract.ExtractObject(eoFunction,FObjectName);
    edMetadata.Lines.Clear;
    edMetadata.Lines.Text := DataModule.IBOExtract.Items.Text;
    edMetadata.Modified := False;
  end;
end;

procedure TfrmUDFEditor.cmbRtnTypeChange(Sender: TObject);
begin
  ObjectModified := True;
end;

procedure TfrmUDFEditor.ActionCompileUpdate(Sender: TObject);
begin
  ActionCompile.Enabled := ObjectModified;
end;

procedure TfrmUDFEditor.ActionCompileExecute(Sender: TObject);
begin
  DoCompile;
end;

procedure TfrmUDFEditor.edUDFNameKeyPress(Sender: TObject; var Key: Char);
begin
  if Assigned(Datamodule) then
    Key := CheckNameKey(Key,Datamodule.Connection.SQLDialect);
end;

end.
