{******************************************************************}
{  This Unit was created to by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.
}
{******************************************************************}
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }
{                                                                  }
{******************************************************************}
{
  Part of this code is coming from IBMarathon from Gimbal Software
  and modified by Lorenzo Mengoni - Italy - lorenzo@mengoni.com
}

{
15.August.2002, Lorenzo:
- Added Compile function and related logic
15.February.2005, Lorenzo:
- bug fix in procedure TfrmViewEditor.LoadView;
}


unit frmuEditorView;

interface

uses
{$IFDEF LINUX}
  QControls, QForms, QDialogs, QComCtrls, QStdCtrls, QMenus,
  Types, QTypes, QImgList, QExtCtrls,
  QStdActns, QActnList, SynEdit, SynEditPrint, IBCXDummies,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Controls, Forms, Dialogs, ComCtrls, StdCtrls,
  Menus, ImgList, StdActns, ActnList, Graphics, Spin, Grids, DBGrids,
  DBCtrls,  ExtCtrls, ClipBrd, Printers, Tabs, Buttons, 
{$ENDIF}
  SysUtils, Classes, DB, IB_Components,  SynEdit, IBODataset,
  dmuNode, IBOExtract, IB_TransactionBar, IBOConsoleForm;

type
  TfrmViewEditor = class(TIBOConsoleEditor)
    stsEditor: TStatusBar;
    pgObjectEditor: TPageControl;
    tsTableView: TTabSheet;
    tsDependenciesView: TTabSheet;
    tsTriggerView: TTabSheet;
    tsData: TTabSheet;
    tvTriggers: TTreeView;
    pnlDataView: TPanel;
    navDataView: TDBNavigator;
    dsTableData: TDataSource;
    tsSQL: TTabSheet;
    lvFieldList: TListView;
    tsDoco: TTabSheet;
    qryTable: TIBOQuery;
    qryTriggers: TIBOQuery;
    tblTableData: TIBOQuery;
    grdDataView: TDBGrid;
    tsGrants: TTabSheet;
    btnRefresh: TSpeedButton;
    qryUtil: TIBOQuery;
    pnlMessages: TPanel;
    lstResults: TListBox;
    edEditor: TSynEdit;
    Panel1: TPanel;
    Label1: TLabel;
    cmbTriggerDisplay: TComboBox;
    Panel2: TPanel;
    rbDependent: TRadioButton;
    rbDependedOn: TRadioButton;
    pnlDependents: TPanel;
    tvDependents: TTreeView;
    pnlDependencies: TPanel;
    tvDependencies: TTreeView;
    edDescription: TSynEdit;
    lvPermissions: TListView;
    Panel3: TPanel;
    Image1: TImage;
    Label2: TLabel;
    Panel4: TPanel;
    SpeedButtonCompile: TSpeedButton;
    ActionList1: TActionList;
    ActionCompile: TAction;
    IB_TransactionBar1: TIB_TransactionBar;
    Panel5: TPanel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure tvTriggersMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pgObjectEditorChange(Sender: TObject);
    procedure tvTriggersGetImageIndex(Sender: TObject; Node: TTreeNode);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure edEditorChange(Sender: TObject);
    procedure edEditorStatusChange(Sender: TObject;
      Changes: TSynStatusChanges);
    procedure rbDependentClick(Sender: TObject);
    procedure rbDependedOnClick(Sender: TObject);
    procedure edDescriptionStatusChange(Sender: TObject;
      Changes: TSynStatusChanges);
    procedure ActionCompileUpdate(Sender: TObject);
    procedure edDescriptionChange(Sender: TObject);
    procedure ActionCompileExecute(Sender: TObject);
  private
    { Private declarations }
    FCharSet: TFontCharset;
    procedure FillFieldList;
    procedure SetCharSet(const Value: TFontCharset);
    procedure LoadDescription;
    procedure LoadView;
    procedure UpdateTriggers;
    function GetViewEditName: String;
  protected
    procedure SetDataModule(const Value: TDmN); override;
    procedure DoLoadObject; override;
    function CompileNewObject: boolean; override;
    function AlterObject: boolean; override;
    procedure SignalNewObject; override;
    procedure SignalObjectModified; override;
    procedure SignalObjectNameChanged; override;
  public
    { Public declarations }
    property CharSet:TFontCharset read FCharSet write SetCharSet;
  end;


implementation

uses IBOCGlobal, IBOAdminUtils, frmuMain, frmuMessage;

{$R *.DFM}

const
   PG_SQL            = 0;
   PG_STRUCT         = 1;
   PG_DEPEND         = 2;
   PG_TRIGGER        = 3;
   PG_DATA           = 4;
   PG_DOCO           = 5;
   PG_GRANTS         = 6;

procedure TfrmViewEditor.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  inherited;
  Action := caFree;
end;

procedure TfrmViewEditor.FormCreate(Sender: TObject);
begin
  pnlMessages.Visible := False;
  cmbTriggerDisplay.ItemIndex := 0;
  ObjectType := NODE_VIEW;
end;

procedure TfrmViewEditor.tvTriggersMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  T : TTreeNode;
begin
  if Button = mbRight then
  begin
    T := tvTriggers.GetNodeAt(x, Y);
    tvTriggers.Selected := T;
  end;
end;

procedure TfrmViewEditor.UpdateTriggers;
var
  AbsRoot : TTreeNode;
  Root : TTreeNode;

  BefInsRoot : TTreeNode;
  AftInsRoot : TTreeNode;
  BefUpdRoot : TTreeNode;
  AftUpdRoot : TTreeNode;
  BefDelRoot : TTreeNode;
  AftDelRoot : TTreeNode;

begin
  try
    tvTriggers.Items.BeginUpdate;
    qryTriggers.BeginBusy(False);

    tvTriggers.Items.Clear;

    AbsRoot := tvTriggers.Items.GetFirstNode;
    Root := tvTriggers.Items.AddChild(AbsRoot, 'Triggers for [' + ObjectName + ']');

    BefInsRoot := tvTriggers.Items.AddChild(Root, 'Before Insert');
    AftInsRoot := tvTriggers.Items.AddChild(Root, 'After Insert');
    BefUpdRoot := tvTriggers.Items.AddChild(Root, 'Before Update');
    AftUpdRoot := tvTriggers.Items.AddChild(Root, 'After Update');
    BefDelRoot := tvTriggers.Items.AddChild(Root, 'Before Delete');
    AftDelRoot := tvTriggers.Items.AddChild(Root, 'After Delete');

    case cmbTriggerDisplay.ItemIndex of
      0 :
        begin
          qryTriggers.Close;
          qryTriggers.SQL.Clear;
          qryTriggers.SQL.Add('select rdb$trigger_name '+
                              'from rdb$triggers '+
                              'where rdb$trigger_type = 1 '+
                              'and rdb$relation_name = ' +
                              AnsiQuotedStr(ObjectName, '''') +
                              ' and rdb$trigger_name not in '+
                              '(select rdb$trigger_name from '+
                              'rdb$check_constraints) '+
                              'order by rdb$trigger_name asc;');
          qryTriggers.Open;
          While not qryTriggers.EOF do
          begin
            tvTriggers.Items.AddChild(BefInsRoot, qryTriggers.FieldByName('rdb$trigger_name').AsString);
            qryTriggers.Next;
          end;

          qryTriggers.Close;
          qryTriggers.SQL.Clear;
          qryTriggers.SQL.Add('select rdb$trigger_name from rdb$triggers '+
                              'where rdb$trigger_type = 2 and rdb$relation_name = ' +
                              AnsiQuotedStr(ObjectName, '''') +
                              ' and rdb$trigger_name not in '+
                              '(select rdb$trigger_name from rdb$check_constraints) order by rdb$trigger_name asc;');
          qryTriggers.Open;
          While not qryTriggers.EOF do
          begin
            tvTriggers.Items.AddChild(AftInsRoot, qryTriggers.FieldByName('rdb$trigger_name').AsString);
            qryTriggers.Next;
          end;

          qryTriggers.Close;
          qryTriggers.SQL.Clear;
          qryTriggers.SQL.Add('select rdb$trigger_name from rdb$triggers where rdb$trigger_type = 3 and rdb$relation_name = ' + AnsiQuotedStr(ObjectName, '''') + ' and rdb$trigger_name not in (select rdb$trigger_name from rdb$check_constraints) order by rdb$trigger_name asc;');
          qryTriggers.Open;
          While not qryTriggers.EOF do
          begin
            tvTriggers.Items.AddChild(BefUpdRoot, qryTriggers.FieldByName('rdb$trigger_name').AsString);
            qryTriggers.Next;
          end;

          qryTriggers.Close;
          qryTriggers.SQL.Clear;
          qryTriggers.SQL.Add('select rdb$trigger_name from rdb$triggers where rdb$trigger_type = 4 and rdb$relation_name = ' + AnsiQuotedStr(ObjectName, '''') + ' and rdb$trigger_name not in (select rdb$trigger_name from rdb$check_constraints) order by rdb$trigger_name asc;');
          qryTriggers.Open;
          While not qryTriggers.EOF do
          begin
            tvTriggers.Items.AddChild(AftUpdRoot, qryTriggers.FieldByName('rdb$trigger_name').AsString);
            qryTriggers.Next;
          end;

          qryTriggers.Close;
          qryTriggers.SQL.Clear;
          qryTriggers.SQL.Add('select rdb$trigger_name from rdb$triggers where rdb$trigger_type = 5 and rdb$relation_name = ' + AnsiQuotedStr(ObjectName, '''') + ' and rdb$trigger_name not in (select rdb$trigger_name from rdb$check_constraints) order by rdb$trigger_name asc;');
          qryTriggers.Open;
          While not qryTriggers.EOF do
          begin
            tvTriggers.Items.AddChild(BefDelRoot, qryTriggers.FieldByName('rdb$trigger_name').AsString);
            qryTriggers.Next;
          end;

          qryTriggers.Close;
          qryTriggers.SQL.Clear;
          qryTriggers.SQL.Add('select rdb$trigger_name from rdb$triggers where rdb$trigger_type = 6 and rdb$relation_name = ' + AnsiQuotedStr(ObjectName, '''') + ' and rdb$trigger_name not in (select rdb$trigger_name from rdb$check_constraints) order by rdb$trigger_name asc;');
          qryTriggers.Open;
          While not qryTriggers.EOF do
          begin
            tvTriggers.Items.AddChild(AftDelRoot, qryTriggers.FieldByName('rdb$trigger_name').AsString);
            qryTriggers.Next;
          end;
        end;
      1 :
        begin
          qryTriggers.Close;
          qryTriggers.SQL.Clear;
          qryTriggers.SQL.Add('select rdb$trigger_name, rdb$trigger_sequence from rdb$triggers where rdb$trigger_type = 1 and rdb$relation_name = ' + AnsiQuotedStr(ObjectName, '''') + ' and rdb$trigger_name not in (select rdb$trigger_name from rdb$check_constraints) order by rdb$trigger_sequence asc;');
          qryTriggers.Open;
          While not qryTriggers.EOF do
          begin
            tvTriggers.Items.AddChild(BefInsRoot, qryTriggers.FieldByName('rdb$trigger_name').AsString);
            qryTriggers.Next;
          end;

          qryTriggers.Close;
          qryTriggers.SQL.Clear;
          qryTriggers.SQL.Add('select rdb$trigger_name, rdb$trigger_sequence from rdb$triggers where rdb$trigger_type = 2 and rdb$relation_name = ' + AnsiQuotedStr(ObjectName, '''') + ' and rdb$trigger_name not in (select rdb$trigger_name from rdb$check_constraints) order by rdb$trigger_sequence asc;');
          qryTriggers.Open;
          While not qryTriggers.EOF do
          begin
            tvTriggers.Items.AddChild(AftInsRoot, qryTriggers.FieldByName('rdb$trigger_name').AsString);
            qryTriggers.Next;
          end;

          qryTriggers.Close;
          qryTriggers.SQL.Clear;
          qryTriggers.SQL.Add('select rdb$trigger_name, rdb$trigger_sequence from rdb$triggers where rdb$trigger_type = 3 and rdb$relation_name = ' + AnsiQuotedStr(ObjectName, '''') + ' and rdb$trigger_name not in (select rdb$trigger_name from rdb$check_constraints) order by rdb$trigger_sequence asc;');
          qryTriggers.Open;
          While not qryTriggers.EOF do
          begin
            tvTriggers.Items.AddChild(BefUpdRoot, qryTriggers.FieldByName('rdb$trigger_name').AsString);
            qryTriggers.Next;
          end;

          qryTriggers.Close;
          qryTriggers.SQL.Clear;
          qryTriggers.SQL.Add('select rdb$trigger_name, rdb$trigger_sequence from rdb$triggers where rdb$trigger_type = 4 and rdb$relation_name = ' + AnsiQuotedStr(ObjectName, '''') + ' and rdb$trigger_name not in (select rdb$trigger_name from rdb$check_constraints) order by rdb$trigger_sequence asc;');
          qryTriggers.Open;
          While not qryTriggers.EOF do
          begin
            tvTriggers.Items.AddChild(AftUpdRoot, qryTriggers.FieldByName('rdb$trigger_name').AsString);
            qryTriggers.Next;
          end;

          qryTriggers.Close;
          qryTriggers.SQL.Clear;
          qryTriggers.SQL.Add('select rdb$trigger_name, rdb$trigger_sequence from rdb$triggers where rdb$trigger_type = 5 and rdb$relation_name = ' + AnsiQuotedStr(ObjectName, '''') + ' and rdb$trigger_name not in (select rdb$trigger_name from rdb$check_constraints) order by rdb$trigger_sequence asc;');
          qryTriggers.Open;
          While not qryTriggers.EOF do
          begin
            tvTriggers.Items.AddChild(BefDelRoot, qryTriggers.FieldByName('rdb$trigger_name').AsString);
            qryTriggers.Next;
          end;

          qryTriggers.Close;
          qryTriggers.SQL.Clear;
          qryTriggers.SQL.Add('select rdb$trigger_name, rdb$trigger_sequence from rdb$triggers where rdb$trigger_type = 6 and rdb$relation_name = ' + AnsiQuotedStr(ObjectName, '''') + ' and rdb$trigger_name not in (select rdb$trigger_name from rdb$check_constraints) order by rdb$trigger_sequence asc;');
          qryTriggers.Open;
          While not qryTriggers.EOF do
          begin
            tvTriggers.Items.AddChild(AftDelRoot, qryTriggers.FieldByName('rdb$trigger_name').AsString);
            qryTriggers.Next;
          end;
        end;
    end;
    qryTriggers.Close;
  finally
    qryTriggers.EndBusy;
    tvTriggers.Items.EndUpdate;
  end;
  Root.Expand(True);
  if Screen.ActiveForm = Self then
    tvTriggers.SetFocus;
end;


procedure TfrmViewEditor.FillFieldList;
var
  L : TListItem;
  Idx : Integer;
  DataTypeText : String;
  Dimensions : Integer;

begin
  //field list...
  lvFieldList.Items.BeginUpdate;

  lvFieldList.Items.Clear;
  qryTable.Close;
  qryTable.SQL.Clear;
  if DataModule.Connection.SQLDialect = 3 then
  begin
    qryTable.SQL.Add('select a.rdb$field_name, a.rdb$null_flag as tnull_flag, ' +
                     'b.rdb$null_flag as fnull_flag, a.rdb$field_source, a.rdb$default_source, ' +
                     'b.rdb$computed_source, b.rdb$field_length, b.rdb$field_precision, a.rdb$description, ' +
                     'b.rdb$dimensions, ' +
                     'b.rdb$field_scale, b.rdb$field_type, b.rdb$field_sub_type from ' +
                     'rdb$relation_fields a, rdb$fields b where ' +
                     'a.rdb$field_source = b.rdb$field_name and a.rdb$relation_name = ' +
                     AnsiQuotedStr(ObjectName, '''') + ' ' +
                     ' order by a.rdb$field_position asc;');
  end
  else
  begin
    qryTable.SQL.Add('select a.rdb$field_name, a.rdb$null_flag as tnull_flag, ' +
                     'b.rdb$null_flag as fnull_flag, a.rdb$field_source, a.rdb$default_source, ' +
                     'b.rdb$computed_source, b.rdb$field_length, a.rdb$description, ' +
                     'b.rdb$dimensions, ' +
                     'b.rdb$field_scale, b.rdb$field_type, b.rdb$field_sub_type from ' +
                     'rdb$relation_fields a, rdb$fields b where ' +
                     'a.rdb$field_source = b.rdb$field_name and a.rdb$relation_name = ' +
                     AnsiQuotedStr(ObjectName, '''') + ' ' +
                     ' order by a.rdb$field_position asc;');
  end;
  qryTable.Open;
  While not qryTable.EOF do
  begin
    L := lvFieldList.Items.Add;
    L.ImageIndex := 6;
    L.Caption := qryTable.FieldByName('rdb$field_name').AsString;

    if DataModule.Connection.SQLDialect = 3 then
    begin
      DataTypeText := ConvertFieldType(qryTable.FieldByName('rdb$field_type').AsInteger,
                                       qryTable.FieldByName('rdb$field_length').AsInteger,
                                       qryTable.FieldByName('rdb$field_scale').AsInteger,
                                       qryTable.FieldByName('rdb$field_sub_type').AsInteger,
                                       qryTable.FieldByName('rdb$field_precision').AsInteger,
                                       DataModule.Connection.SQLDialect);
    end
    else
    begin
      DataTypeText := ConvertFieldType(qryTable.FieldByName('rdb$field_type').AsInteger,
                                       qryTable.FieldByName('rdb$field_length').AsInteger,
                                       qryTable.FieldByName('rdb$field_scale').AsInteger,
                                       -1,
                                       -1,
                                       DataModule.Connection.SQLDialect);
    end;


    if qryTable.FieldByName('rdb$dimensions').AsInteger > 0 then
    begin
      DataTypeText := DataTypeText + '[';
      Dimensions := qryTable.FieldByName('rdb$dimensions').AsInteger;
      for Idx := 0 to Dimensions do
      begin
        qryUtil.Close;
        qryUtil.SQL.Clear;
        qryUtil.SQL.Add('select rdb$lower_bound, rdb$upper_bound from rdb$field_dimensions where ' +
                                'rdb$dimension = ' + IntToStr(Idx)  + 'and rdb$field_name = ' + AnsiQuotedStr(qryTable.FieldByName('rdb$field_source').AsString, ''''));
        qryUtil.Open;
        if not (qryUtil.EOF and qryUtil.BOF) then
        begin
          DataTypeText := DataTypeText + qryUtil.FieldByName('rdb$lower_bound').AsString + ':' + qryUtil.FieldByName('rdb$upper_bound').AsString + ', ';
        end;
      end;
      DataTypeText := Trim(DataTypeText);
      if DataTypeText[Length(DataTypeText)] = ',' then
        DataTypeText := Copy(DataTypeText, 1, Length(DataTypeText) - 1);
      DataTypeText := DataTypeText + ']';
    end;
    L.SubItems.Add(DataTypeText);

    if qryTable.FieldByName('rdb$field_type').AsInteger = 261 then
      L.SubItems.Add(qryTable.FieldByName('rdb$field_sub_type').AsString)
    else
      L.SubItems.Add('');
    L.SubItems.Add(qryTable.FieldByName('rdb$field_source').AsString);

    if qryTable.FieldByName('tnull_flag').IsNull and qryTable.FieldByName('fnull_flag').IsNull then
      L.SubItems.Add('False')
    else
      L.SubItems.Add('True');

    L.SubItems.Add(Trim(qryTable.FieldByName('rdb$description').AsString));
    L.SubItems.Add(Trim(qryTable.FieldByName('rdb$default_source').AsString));
    L.SubItems.Add(Trim(qryTable.FieldByName('rdb$computed_source').AsString));
    qryTable.Next;
  end;
  qryTable.Close;
end;

procedure TfrmViewEditor.pgObjectEditorChange(Sender: TObject);
begin
  try
    qryTable.BeginBusy(False);
    Case pgObjectEditor.ActivePage.PageIndex of
      PG_STRUCT:
        begin
          stsEditor.Panels[0].Text := '';
          stsEditor.Panels[1].Text := '';
          FillFieldList;
          lvFieldList.Items.EndUpdate;
          ActiveControl := lvFieldList;
        end;

      PG_DEPEND:
        begin
          stsEditor.Panels[0].Text := '';
          stsEditor.Panels[1].Text := '';
          rbDependedOn.Checked := true;
          DataModule.GetDependencies(tvDependencies,tvDependents,ObjectName,NODE_VIEWS);
        end;

      PG_TRIGGER:
        begin

          stsEditor.Panels[0].Text := '';
          stsEditor.Panels[1].Text := '';

          //triggers
          UpdateTriggers;
        end;

      PG_DATA:
        begin
          //data
          stsEditor.Panels[0].Text := '';
          stsEditor.Panels[1].Text := '';
          tblTableData.SQL.Clear;
          tblTableData.SQL.Add('select * from ' + QuoteIdentifier( DataModule.Connection.SQLDialect,ObjectName) + ';');
          tblTableData.Open;
        end;

      PG_DOCO:
        begin
          LoadDescription;
        end;

      PG_GRANTS:
        begin
          Datamodule.GetPermissions(lvPermissions,ObjectName,NODE_VIEWS);
          stsEditor.Panels[0].Text := '';
          stsEditor.Panels[1].Text := '';
        end;
    end;
  finally
    qryTable.EndBusy;
  end;
end;


procedure TfrmViewEditor.tvTriggersGetImageIndex(Sender: TObject;
  Node: TTreeNode);
begin
  case Node.Level of
    0 :
      begin
        Node.ImageIndex := 0;
        Node.SelectedIndex := 0;
      end;
    1 :
      begin
        Node.ImageIndex := 1;
        Node.SelectedIndex := 1;
      end;
    2 :
      begin
        Node.ImageIndex := 4;
        Node.SelectedIndex := 4;
      end;
  end;
end;

procedure TfrmViewEditor.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift = [ssCtrl]) and (Key = VK_TAB) then
    pgObjectEditor.SelectNextPage(true);
end;

procedure TfrmViewEditor.LoadView;
begin
  pgObjectEditor.ActivePage := tsSQL;
  pgObjectEditorChange(pgObjectEditor);
  try
    if edEditor.CanFocus then
      ActiveControl := edEditor;
  except
    on e:Exception do
    begin
      // Eat cannot focus a disabled window 
    end;
  end;
  DataModule.IBOExtract.ExtractObject(eoView,ObjectName);
  edEditor.Lines.Clear;
  edEditor.Lines.Text := DataModule.IBOExtract.Items.Text;
  edEditor.Modified := False;
end;


procedure TfrmViewEditor.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := InternalCloseQuery;
end;


procedure TfrmViewEditor.edEditorChange(Sender: TObject);
begin
  ObjectModified := True;
  case pgObjectEditor.ActivePage.PageIndex of
    PG_STRUCT :
      begin
      end;

    PG_DEPEND :
      begin
        stsEditor.Panels[0].Text := '';
        stsEditor.Panels[1].Text := '';
      end;

    PG_TRIGGER :
      begin
        stsEditor.Panels[0].Text := '';
        stsEditor.Panels[1].Text := '';
      end;

    PG_DATA :
      begin
        stsEditor.Panels[0].Text := '';
        stsEditor.Panels[1].Text := '';
      end;

    PG_DOCO :
      begin

      end;

    PG_GRANTS :
      begin
        stsEditor.Panels[0].Text := '';
        stsEditor.Panels[1].Text := '';
      end;
  end;
end;

procedure TfrmViewEditor.edEditorStatusChange(Sender: TObject;
  Changes: TSynStatusChanges);
begin
  if pgObjectEditor.ActivePage = tsSQL then
    stsEditor.Panels[0].Text := Format('%d:%d',[edEditor.CaretY,edEditor.CaretX]);
end;

procedure TfrmViewEditor.SetCharSet(const Value: TFontCharset);
begin
  FCharSet := Value;
end;

procedure TfrmViewEditor.SetDataModule(const Value: TDmN);
begin
  inherited SetDataModule(Value);
  if DataModule = Nil then
  begin
    qryTable.IB_Connection := nil;
    qryUtil.IB_Connection := nil;
    qryTriggers.IB_Connection := nil;
    stsEditor.Panels[2].Text := 'No Connection';
    IB_TransactionBar1.IB_Transaction := Nil;
  end
  else
  begin
    qryTable.IB_Connection := DataModule.Connection;
    qryTable.IB_Transaction := DataModule.Transaction;

    qryUtil.IB_Connection := DataModule.Connection;
    qryUtil.IB_Transaction := DataModule.Transaction;

    qryTriggers.IB_Connection := DataModule.Connection;
    qryTriggers.IB_Transaction := DataModule.Transaction;

    tblTableData.IB_Connection := DataModule.Connection;
    tblTableData.IB_Transaction := DataModule.Transaction;

    stsEditor.Panels[2].Text := DataModule.Connection.DatabaseName;

    IB_TransactionBar1.IB_Transaction := DataModule.Transaction;
  end;
end;

procedure TfrmViewEditor.rbDependentClick(Sender: TObject);
begin
  pnlDependents.Visible := true;
  pnlDependencies.Visible := false;
end;

procedure TfrmViewEditor.rbDependedOnClick(Sender: TObject);
begin
  pnlDependents.Visible := false;
  pnlDependencies.Visible := true;
end;

procedure TfrmViewEditor.LoadDescription;
begin
  if Assigned(Datamodule) then
  begin
    edDescription.Text := DataModule.GetObjectDescription(NODE_VIEWS,ObjectName);
    edDescription.Modified := False;
  end;
end;

procedure TfrmViewEditor.edDescriptionStatusChange(Sender: TObject;
  Changes: TSynStatusChanges);
begin
  if pgObjectEditor.ActivePage = tsDoco then
    stsEditor.Panels[0].Text := Format('%d:%d',[edDescription.CaretY,edDescription.CaretX]);
end;

procedure TfrmViewEditor.ActionCompileUpdate(Sender: TObject);
begin
  ActionCompile.Enabled := ObjectModified;
end;

function TfrmViewEditor.GetViewEditName:String;
var Tmp:String;
    UTmp:String;
    p1,p2:integer;
    endChar:Char;
begin
  result := '';
  Tmp := StringReplace(edEditor.Text,CRLF,' ',[rfReplaceAll]);
  Tmp := StringReplace(Tmp,#10,' ',[rfReplaceAll]);
  if DataModule.Connection.SQLDialect = 1 then
    Tmp := StringReplace(Tmp,'  ',' ',[rfReplaceAll]);
  UTmp:= UpperCase(tmp);
  p1:= pos('CREATE',utmp);
  if p1>0 then
  begin
    tmp := Trim(copy(tmp,p1+6,maxint));
    utmp := Trim(copy(utmp,p1+6,maxint));
    p2:= pos('VIEW',utmp);
    if p2>0 then
    begin
      tmp := Trim(copy(tmp,p2+4,maxint));
      endChar := ' ';
      if (DataModule.Connection.SQLDialect = 3) and (tmp<>'') and (tmp[1]='"') then
      begin
        delete(tmp,1,1);
        endChar := '"';
      end;
      p1 := pos(endChar,tmp);
      if p1>0 then
      begin
        tmp := copy(tmp,1,p1-1);
        Result := tmp;
      end;
    end;
  end;
end;

function TfrmViewEditor.AlterObject: boolean;
var Tmp:String;
    Script:String;
begin
    Newname := GetViewEditName;
    if NewName = '' then
    begin
      CompileErrors := 'The '+sObjectType+' must have a name.';
      Result := False;
      Exit;
    end;
    Script := '';
    if (NewName<>'') and (NewName=ObjectName) then
      Script := 'drop view ' + QuoteIdentifier(DataModule.Connection.SQLDialect,ObjectName)+';'+CRLF;
    Script := Script + edEditor.Text;
    Result := DataModule.ExecuteDDLScript(Script,Tmp);
    CompileErrors := tmp;
end;

function TfrmViewEditor.CompileNewObject: boolean;
var Tmp:String;
    Script:String;
begin
    Newname := GetViewEditName;
    if NewName = '' then
    begin
      CompileErrors := 'The new '+sObjectType+' must have a name.';
      Result := False;
      Exit;
    end;
    Script := edEditor.Text;
    Result := DataModule.ExecuteDDLScript(Script,Tmp);
    CompileErrors := tmp;
end;

procedure TfrmViewEditor.DoLoadObject;
begin
  inherited;
  Caption := 'View - ' + ObjectName ;
  LoadView;
  LoadDescription;

end;

procedure TfrmViewEditor.SignalNewObject;
var
  Tmp : String;
begin
  Caption := 'View - ' + ObjectName ;
  tmp := 'create view new_view' + CRLF;
  tmp := Tmp + '( /*result columns here*/' + CRLF;
  tmp := Tmp + ')' + CRLF;
  tmp := Tmp + 'as' + CRLF;
  tmp := Tmp + '/*select statement here*/' + CRLF;
  edEditor.Text := AdjustLineBreaks(tmp);
end;

procedure TfrmViewEditor.SignalObjectModified;
begin
  inherited;
  if ObjectModified then
  begin
    stsEditor.Panels[1].Text := '*';
    newName := 'tmp_name_000';
  end
  else
    stsEditor.Panels[1].Text := '';

end;

procedure TfrmViewEditor.SignalObjectNameChanged;
begin
  inherited;

end;

procedure TfrmViewEditor.edDescriptionChange(Sender: TObject);
begin
  EditDescription := edDescription.Text;
end;

procedure TfrmViewEditor.ActionCompileExecute(Sender: TObject);
begin
  Compile;
end;

end.
