{************************************************************************}
{  This Unit was modified to be interfaced in IBObjects
   by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com
   }
{  This software is distributed under the License is distributed on      }
{    an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either              }
{    express or implied. See the ORIGINAL Copyright                      }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 * 
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 * 
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 * 
 * Contributor(s): Krzysztof Golko.
}

unit frmuIBOConsoleOptions;

interface

uses
{$IFDEF LINUX}
  Types, Qt, QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QExtCtrls, QGrids, QComCtrls,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Grids, ComCtrls,
{$ENDIF}
  SysUtils, Classes, IBOConsoleForm;

type
  TfrmOptions = class(TForm)
    btnApply: TButton;
    btnOK: TButton;
    pgControl: TPageControl;
    TabSheet2: TTabSheet;
    btnCancel: TButton;
    DDLRadioGroup: TRadioGroup;
    CheckBox1: TCheckBox;
    Label1: TLabel;
    ButtonChangeFont: TButton;
    FontDialog1: TFontDialog;
    Label2: TLabel;
    ComboBoxLanguage: TComboBox;
    LabelLanguage: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure DDLRadioGroupClick(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure ButtonChangeFontClick(Sender: TObject);
    procedure ComboBoxLanguageChange(Sender: TObject);
    procedure ComboBoxLanguageCloseUp(Sender: TObject);
  private
    { Private declarations }
    function LangFileExists(const LangId:string): boolean;
  public
    { Public declarations }
  end;

var
  frmOptions: TfrmOptions;

implementation

uses IBOCGlobal, frmuMain, frmuMessage, LMMessageDialog;
{$IF Declared(Forms)}
{$R *.DFM}
{$IFEND}
{$IF Declared(QForms)}
{$R *.xfm}
{$IFEND}

procedure TfrmOptions.FormCreate(Sender: TObject);
var n:integer;
begin
  inherited;
{$ifdef initlangfile}    WriteAllCaptionAndText('',Self,gTransIniFile); {$endif}
  if Assigned(gGUILanguageFile) then
    GetAllCaptionAndText('',self,gGUILanguageFile);
  pgControl.ActivePageIndex := 0;
  if gAppSettings[AUTO_COMMIT_DDL] then
    DDLRadioGroup.ItemIndex := 0
  else
    DDLRadioGroup.ItemIndex := 1;

  CheckBox1.Checked := gAppSettings[FORCE_UPPERCASE_IDENTIFIER];

  FontDialog1.Font.Name :=  gAppSettings[FONT_NAME];
  FontDialog1.Font.Size := 10;
  FontDialog1.Font.Color := clBlack;
  FontDialog1.Font.Charset := gAppSettings[FONT_CHARSET];

  Label2.Font := FontDialog1.Font;
  Label2.Caption := FontDialog1.Font.Name;
  Label1.Font := FontDialog1.Font;
  for n := 0 to NUM_LANGUAGES-1 do
  if LangFileExists(gLanguagesArray[n].Id) then
  begin
    ComboBoxLanguage.Items.Add(gLanguagesArray[n].Caption);
  end;
  ComboBoxLanguage.ItemIndex := ComboBoxLanguage.Items.IndexOf(GetLanguageString(gAppSettings[GUI_LANGUAGE]));
end;

function TfrmOptions.LangFileExists(const LangId:string): boolean;
var
  langFile:string;
begin
  result :=False;
  if (langId='en') then Result := true
  else
  if (langId <> '') then
  begin
    langFile := ChangeFileExt(Application.ExeName,'.lang.'+langId);
    Result := FileExists(langFile);
  end;
end;

procedure TfrmOptions.btnApplyClick(Sender: TObject);
var i:integer;
    langId:string;
begin
  inherited;
  case DDLRadioGroup.ItemIndex of
    0: gAppSettings[AUTO_COMMIT_DDL] := True;
    1: gAppSettings[AUTO_COMMIT_DDL] := False;
  end;
  gAppSettings[FORCE_UPPERCASE_IDENTIFIER] := CheckBox1.Checked;

  for I := 0 to Screen.CustomFormCount-1 do
  if Screen.CustomForms[I] <> self then
  begin
    if Screen.CustomForms[I] is TIBOConsoleForm then
      TIBOConsoleForm(Screen.CustomForms[I]).UseFont(gAppSettings[FONT_NAME],gAppSettings[FONT_CHARSET] );
    try
      SetAllFontsNameCharset(Screen.CustomForms[I],gAppSettings[FONT_NAME],gAppSettings[FONT_CHARSET] );
    except
    end;
  end;

  langId := GetLanguageId(ComboBoxLanguage.Text);
  if LangFileExists(langId) then
  begin
    gAppSettings[GUI_LANGUAGE] := langId;
    if langId = 'en' then
      LMMessageDlg('In order to redisplay IBOConsole in English it''s necessary to close and restart IBOConsole',mtInformation,[mbok],0);
  end
  else
  begin
    DisplayMsg (ERR_FOPEN, ChangeFileExt(Application.ExeName,'.lang.'+langId));
  end;

  btnApply.Enabled := false;
end;

procedure TfrmOptions.btnOKClick(Sender: TObject);
begin
  inherited;
  if btnApply.Enabled then
    btnApply.Click;
  Close;
end;

procedure TfrmOptions.FormShow(Sender: TObject);
begin
  inherited;
  btnApply.Enabled := false;
end;

procedure TfrmOptions.btnCancelClick(Sender: TObject);
begin
  inherited;
  ModalResult := mrCancel;
end;

procedure TfrmOptions.DDLRadioGroupClick(Sender: TObject);
begin
  btnApply.Enabled := true;
end;

procedure TfrmOptions.CheckBox1Click(Sender: TObject);
begin
  btnApply.Enabled := true;
end;

procedure TfrmOptions.ButtonChangeFontClick(Sender: TObject);
begin
  if FontDialog1.Execute then
  begin
    gAppSettings[FONT_NAME] := FontDialog1.Font.Name;
    gAppSettings[FONT_CHARSET]  := FontDialog1.Font.CharSet;
    Label2.Font := FontDialog1.Font;
    Label2.Caption := FontDialog1.Font.Name;
    Label1.Font := FontDialog1.Font;
    btnApply.Enabled := true;
  end;
end;

procedure TfrmOptions.ComboBoxLanguageChange(Sender: TObject);
begin
  btnApply.Enabled := true;
end;

procedure TfrmOptions.ComboBoxLanguageCloseUp(Sender: TObject);
begin
  btnApply.Enabled := true;
end;

end.
