{******************************************************************}
{  This Unit was created to by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.
}
unit frmuLookUpFilterUpas;

interface

uses
{$IFDEF LINUX}
  QControls, QForms, QDialogs, QComCtrls, QStdCtrls, QMenus,
  Types, QTypes, QImgList, QExtCtrls,
  QStdActns, QActnList, SynEdit, SynEditPrint, IBCXDummies,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Controls, Forms, Dialogs, ComCtrls, StdCtrls,
  Menus, ImgList, StdActns, ActnList, Graphics, Spin, Buttons, ExtCtrls,
  Grids, DBGrids, 
{$ENDIF}
  Types, SysUtils, Variants, Classes, DB, IBODataset, dmuNode, IBOConsoleForm;

type
  TFormLookUp = class(TForm)
    DBGrid1: TDBGrid;
    Panel1: TPanel;
    DataSource1: TDataSource;
    IBOQuery1: TIBOQuery;
    ActionList1: TActionList;
    ActionClearAll: TAction;
    ActionCancel: TAction;
    ActionOk: TAction;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ActionClearAllExecute(Sender: TObject);
    procedure ActionClearAllUpdate(Sender: TObject);
    procedure ActionOkExecute(Sender: TObject);
    procedure ActionCancelExecute(Sender: TObject);
  private
    { Private declarations }
    fDatamodule:TDmn;
    fTable:String;
    fField:String;
    FSql: String;
    FMultipleValues: boolean;
    procedure SetSql(const Value: String);
    procedure SetMultipleValues(const Value: boolean);
  public
    { Public declarations }
    constructor CreateLookup(AOwner: TComponent; aDatamodule:TDmn; aTable,aField:String); virtual;
    property Sql:String read FSql write SetSql;
    property MultipleValues:boolean read FMultipleValues write SetMultipleValues;
  end;

var
  FormLookUp: TFormLookUp;

implementation

uses IBOAdminUtils, IBOCGlobal;

{$R *.dfm}

{ TForm1 }

constructor TFormLookUp.CreateLookup(AOwner: TComponent; aDatamodule: TDmn;
  aTable, aField: String);
begin
  inherited Create(AOwner);
  fDatamodule:=aDatamodule;
  fTable:=aTable;
  fField:=aField;
  Caption := fField;
  IBOQuery1.IB_Connection := fDatamodule.Connection;
  IBOQuery1.IB_Transaction := fDatamodule.Transaction;
  IBOQuery1.SQL.Text := format('select distinct %s from %s where %s is not null',
  [
    QuoteIdentifier(fDatamodule.Connection.SQLDialect,fField),
    QuoteIdentifier(fDatamodule.Connection.SQLDialect,fTable),
    QuoteIdentifier(fDatamodule.Connection.SQLDialect,fField)
  ]);
  IBOQuery1.Prepare;
  if IBOQuery1.FieldCount > 0 then
    if IBOQuery1.fields[0] is TNumericfield then
      TNumericfield(IBOQuery1.fields[0]).DisplayFormat := '#,###.###';
end;

procedure TFormLookUp.FormResize(Sender: TObject);
begin
  if DBGrid1.Columns.Count > 0 then
    DBGrid1.Columns[0].Width := DBGrid1.ClientWidth - 18;
end;

procedure MoveMouse(X,Y:integer);
var
  Pt : TPoint;
begin
 {Get the point}
  Pt.x := X ;
  Pt.y := Y ;
 {Convert Pt to Mickeys}
  Pt.x := Round(Pt.x * (65535 / Screen.Width));
  Pt.y := Round(Pt.y * (65535 / Screen.Height));
 {Simulate the mouse move}
  Mouse_Event(MOUSEEVENTF_ABSOLUTE or
              MOUSEEVENTF_MOVE,
              Pt.x, Pt.y, 0, 0);
end;


procedure TFormLookUp.FormShow(Sender: TObject);
var
  R,Ro:TRect;
  x,y:integer;
begin

  R := Bounds(Left, Top, Width, Height);
  if not (Types.IntersectRect(Ro, Monitor.BoundsRect, R)
     and PtInRect(Monitor.BoundsRect,R.BottomRight))
  then
  begin
    if Width > Monitor.Width then  Width:= Monitor.Width;
    x := (Monitor.Width - Width) div 2;
    if x < 0 then x:=0;
    if Height > Monitor.Height then Height:=Monitor.Height;
    y := (Monitor.Height - Height) div 2;
    if y < 0 then y:=0;
    R := Bounds(x, y, Width, Height);
    BoundsRect := R;
  end;

  SetAllFontsNameCharset(self,gAppSettings[FONT_NAME],gAppSettings[FONT_CHARSET]);
  ActionClearAll.Caption := GetLangText('Clear All');
  ActionCancel.Caption := GetLangText('Cancel');
  ActionOk.Caption := GetLangText('Ok');

  if not IBOQuery1.Active then
  begin
    Screen.Cursor := crHourGlass;
    IBOQuery1.DisableControls;
    try
      IBOQuery1.Open;
    finally
      IBOQuery1.EnableControls;
      Screen.Cursor := crDefault;
    end;
  end;
  MoveMouse(Left + Width div 2,Top + Height div 2);
end;

procedure TFormLookUp.ActionClearAllExecute(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  IBOQuery1.DisableControls;
  try
    DBGrid1.SelectedRows.Clear;
  finally
    IBOQuery1.EnableControls;
    Screen.Cursor := crDefault;
  end;
end;

procedure TFormLookUp.ActionClearAllUpdate(Sender: TObject);
begin
  ActionClearAll.Enabled := IBOQuery1.Active;
end;

procedure TFormLookUp.ActionOkExecute(Sender: TObject);
var i:integer;
    s:String;
begin
  fSql:='';
  FMultipleValues := False;
  if Assigned(DBGrid1.SelectedRows) and (DBGrid1.SelectedRows.Count>0) then
  begin
    Screen.Cursor := crHourGlass;
    IBOQuery1.DisableControls;
    try
      s:='';
      FMultipleValues := DBGrid1.SelectedRows.Count>1;
      if FMultipleValues then
      begin
        for i:=0 to DBGrid1.SelectedRows.Count-1 do
        begin
          IBOQuery1.GotoBookmark(pointer(DBGrid1.SelectedRows.Items[i]));
          begin
            if s<>'' then s:=s+', ';
            s:=s+QuotedStr(IBOQuery1.Fields[0].AsString);
          end;
        end;
      end
      else
      begin
        IBOQuery1.GotoBookmark(pointer(DBGrid1.SelectedRows.Items[0]));
        s:= IBOQuery1.Fields[0].AsString;
      end;
      fSql:=s;
    finally
      IBOQuery1.EnableControls;
      Screen.Cursor := crDefault;
    end;
  end;
  ModalResult := mrOk;
end;

procedure TFormLookUp.ActionCancelExecute(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TFormLookUp.SetSql(const Value: String);
begin
  FSql := Value;
end;

procedure TFormLookUp.SetMultipleValues(const Value: boolean);
begin
  FMultipleValues := Value;
end;

end.
