{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 *
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 *
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 *
 * Contributor(s): Krzysztof Golko.

08.08.2002, Kalwa:
- all images are transparent now (Win2k, WinXP)
- set btnOk.Cancel := True, the window now can be closed using the ESC key.
- increased memDetailMsg size, most messages now fit into the memo without
  scrolling.
}

unit frmuMessage;

interface

uses
{$IFDEF LINUX}
  QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QExtCtrls, 
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls,
{$ENDIF}
  SysUtils, Classes, IBOConsoleForm;

type
  TfrmMessage = class(TIBOConsoleForm)
    imgWarning: TImage;
    imgInformation: TImage;
    imgError: TImage;
    lblSummaryMsg: TLabel;
    lblDetailMsg: TLabel;
    btnOK: TButton;
    memDetailMsg: TMemo;
    procedure btnOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    function GetMessage(MsgNo: integer): string;
  end;

Const
  {Global Errors - (0001-2999}
  ERR_SYSTEM_INIT = 0001;
  ERR_USERNAME = 0002;
  ERR_PASSWORD = 0003;
  ERR_PASSWORD_MISSMATCH = 0004;
  ERR_ADD_USER = 0005;
  ERR_MODIFY_USER = 0006;
  ERR_DELETE_USER = 0007;
  ERR_GET_USERS = 0008;
  ERR_GET_USER_INFO = 0009;
  ERR_SOURCE_DB = 0010;
  ERR_DESTINATION_DB = 0011;
  ERR_SAME_SOURCE_DESTINATION = 0012;
  ERR_DB_FILE = 0013;
  ERR_SERVER_NAME = 0014;
  ERR_PROTOCOL = 0015;
  ERR_BACKUP_DB = 0016;
  ERR_RESTORE_DB = 0017;
  ERR_GET_TABLES = 0018;
  ERR_GET_VIEWS = 0019;
  ERR_SERVICE = 0020;
  ERR_NUMERIC_VALUE = 0025;
  ERR_GET_TABLE_DATA = 0026;
  ERR_DB_ALIAS = 0027;
  ERR_GET_ROLES = 0028;
  ERR_SERVER_LOGIN = 0029;
  ERR_DB_CONNECT = 0030;
  ERR_DB_DISCONNECT = 0031;
  ERR_GET_PROCEDURES = 0032;
  ERR_GET_FUNCTIONS = 0033;
  ERR_GET_GENERATORS = 0034;
  ERR_GET_EXCEPTIONS = 0035;
  ERR_GET_BLOB_FILTERS = 0036;
  ERR_GET_COLUMNS = 0037;
  ERR_GET_INDICES = 0038;
  ERR_GET_REFERENTIAL_CONST = 0039;
  ERR_GET_UNIQUE_CONST = 0040;
  ERR_GET_CHECK_CONST = 0041;
  ERR_GET_TRIGGERS = 0042;
  ERR_GET_DDL = 0043;
  ERR_INVALID_PROPERTY_VALUE = 0044;
  ERR_GET_DEPENDENCIES = 0045;
  ERR_GET_DB_PROPERTIES = 0046;
  ERR_DB_SIZE = 0047;
  ERR_ISQL_ERROR = 0048;
  ERR_SERVER_SERVICE = 0049;
  ERR_EXTERNAL_EDITOR = 0050;
  ERR_SERVER_ALIAS = 0051;
  ERR_BACKUP_ALIAS = 0052;
  ERR_DB_SHUTDOWN = 0053;
  ERR_MODIFY_DB_PROPERTIES = 0054;
  ERR_DROP_DATABASE = 0055;
  ERR_FILE_OPEN = 0056;
  ERR_INV_EDITOR = 0057;
  ERR_EDITOR_MISSING = 0058;
  ERR_BAD_FORMAT = 0059;
  ERR_INV_DIALECT = 0060;
  ERR_FOPEN = 0061;
  ERR_TEXT_NOT_FOUND = 0062;
  ERR_PRINT = 0063;
  ERR_NO_PATH = 0064;
  ERR_NO_FILES = 0065;
  ERR_GET_DOMAINS = 0066;
  ERR_EXT_TOOL_ERROR = 0067;
  ERR_PROPERTIES = 0068;
  ERR_ALIAS_EXISTS = 0069;
  ERR_CANNOT_START_LOCALIB = 0070;
  ERR_SERVER_REGISTER = 0071;
  ERR_SERVER_UNREGISTER = 0072;
  ERR_DATABASE_REGISTER = 0073;
  ERR_DATABASE_UNREGISTER = 0074;
  ERR_DDL_EXECUTION = 0075;

  {Global Warnings - (3000-5999}
  WAR_DB_UNAVAILABLE = 3000;
  WAR_NO_PERMISSION = 3001;
  WAR_SERVER_REGISTERED = 3002;
  WAR_DUPLICATE_DB_ALIAS = 3003;
  WAR_BACKUP_FILE_REGISTERED = 3004;
  WAR_DIALECT_MISMATCH = 3005;
  WAR_REMOTE_FILENAME = 3006;
  WAR_PERSISTENT_UPDATE = 3007;

  {Global Messages - (6000-9999}
  INF_ADD_USER_SUCCESS = 6000;
  INF_BACKUP_DB_SUCCESS = 6001;
  INF_RESTORE_DB_SUCCESS = 6002;
  INF_NO_PENDING_TRANSACTIONS = 6003;
  INF_RESTART_SERVER = 6004;
  INF_DATABASE_SHUTDOWN = 6005;
  INF_DATABASE_RESTARTED = 6006;
  INF_SQL_SCRIPT = 6007;
  INF_DATABASE_SWEEP = 6008;

function DisplayMsg(const MsgNo: integer; MsgText: string): boolean;

implementation

uses zluUtility, IB_Header, IBOCGlobal;

{$IF Declared(Forms)}
{$R *.DFM}
{$IFEND}

{$IF Declared(QForms)}
{$R *.xfm}
{$IFEND}

{****************************************************************
*
*  D i s p l a y M s g ()
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  MsgNo   - Message No
*          MsgText - The actual error message text
*
*  Return: None
*
*  Description:  Prepare and show message dialog depending on
*                the MsgNo and MsgText
*
*****************************************************************
* Revisions:
*
*****************************************************************}
function DisplayMsg(const MsgNo: integer; MsgText: string): boolean;
var
  frmMessage: TfrmMessage;
begin
  frmMessage := TfrmMessage.Create(Application.MainForm);
  try
    MsgText := RemoveControlChars(MsgText);

    case MsgNo of
      1..2999:
      begin
        frmMessage.Caption := GetLangText('Error');
        frmMessage.imgError.Visible := true;
        frmMessage.imgWarning.Visible := false;
        frmMessage.imgInformation.Visible := false;
        frmMessage.memDetailMsg.Text :=  MsgText;
      end;
      3000..5999:
      begin
        frmMessage.Caption := GetLangText('Warning');
        frmMessage.imgError.Visible := false;
        frmMessage.imgWarning.Visible := true;
        frmMessage.imgInformation.Visible := false;
        frmMessage.memDetailMsg.Text :=  MsgText;
      end;
      6000..9999:
      begin
        frmMessage.Caption := GetLangText('Information');
        frmMessage.imgError.Visible := false;
        frmMessage.imgWarning.Visible := false;
        frmMessage.imgInformation.Visible := true;
        frmMessage.memDetailMsg.Text :=  MsgText;
      end;
      335544321..336920607:
      begin
        frmMessage.Caption := GetLangText('Error');
        frmMessage.imgError.Visible := true;
        frmMessage.imgWarning.Visible := false;
        frmMessage.imgInformation.Visible := false;
        frmMessage.memDetailMsg.Text :=  MsgText;
      end;
    end;

    frmMessage.lblSummaryMsg.Caption := frmMessage.GetMessage(MsgNo);

    if (MsgText = '') or
       (frmMessage.lblSummaryMsg.Caption = MsgText) then
    begin
      frmMessage.lblDetailMsg.Visible := false;
      frmMessage.memDetailMsg.Visible := false;
      frmMessage.btnOK.Top := frmMessage.memDetailMsg.Top;
      frmMessage.Height := frmMessage.btnOK.Top + frmMessage.btnOK.Height + 35;
      if frmMessage.btnOK.Top > frmMessage.ClientHeight then
        frmMessage.btnOK.Top := frmMessage.ClientHeight - frmMessage.btnOK.Height-1;
      frmMessage.Update;
      Application.ProcessMessages;
    end
    else
    begin
      frmMessage.btnOK.Top := 32;
      Application.ProcessMessages;
    end;

    if ((frmMessage.ShowModal) = mrAbort) then
      result := true
    else
      result := false;
  finally
    frmMessage.Free;
  end;
end;

function GetDefaultMessage(MsgNo: integer): string;
begin
    case MsgNo of
      //****** Errors ******
      ERR_SYSTEM_INIT:             Result := 'Initialization failure.';
      ERR_USERNAME:                Result := 'Invalid username. Please enter a valid username.';
      ERR_PASSWORD:                Result := 'Invalid password. Please enter a valid password.';
      ERR_PASSWORD_MISSMATCH:      Result := 'The password does not match the confirmation password.';
      ERR_ADD_USER:                Result := 'Unable to add user.';
      ERR_MODIFY_USER:             Result := 'Unable to modify user account.';
      ERR_DELETE_USER:             Result := 'Unable to delete user.';
      ERR_GET_USERS:               Result := 'Unable to retrieve user list.';
      ERR_GET_USER_INFO:           Result := 'Unable to retrieve user information.';
      ERR_SOURCE_DB:               Result := 'Invalid source database name. Please enter a valid database name.';
      ERR_DESTINATION_DB:          Result := 'Invalid destination database name. Please enter a valid database name.';
      ERR_SAME_SOURCE_DESTINATION: Result := 'The source and destination must be different.';
      ERR_DB_FILE:                 Result := 'Invalid database file or the file does not exist.';
      ERR_SERVER_NAME:             Result := 'Invalid server name. Please enter a valid server name.';
      ERR_PROTOCOL:                Result := 'Invalid network protocol. Please select a network protocol from the list.';
      ERR_BACKUP_DB:               Result := 'Database backup failed.';
      ERR_RESTORE_DB:              Result := 'Database restore failed.';
      ERR_GET_TABLES:              Result := 'Unable to retrieve a list of tables.';
      ERR_GET_VIEWS:               Result := 'Unable to retrieve a list of views.';
      ERR_SERVICE:                 Result := 'Invalid service. Please select a service from the list.';
      ERR_NUMERIC_VALUE:           Result := 'Invalid numeric value. Please enter a valid numeric value.';
      ERR_GET_TABLE_DATA:          Result := 'Unable to retrieve data.';
      ERR_DB_ALIAS:                Result := 'Invalid database alias. Please enter a valid database name.';
      ERR_GET_ROLES:               Result := 'Unable to retrieve a list of roles.';
      ERR_SERVER_LOGIN:            Result := 'Error logging into the requested server.';
      ERR_DB_CONNECT:              Result := 'Error connecting to the requested database.';
      ERR_DB_DISCONNECT:           Result := 'Error disconnecting from database.';
      ERR_GET_PROCEDURES:          Result := 'Unable to retrieve a list of procedures.';
      ERR_GET_FUNCTIONS:           Result := 'Unable to retrieve a list of functions.';
      ERR_GET_GENERATORS:          Result := 'Unable to retrieve a list of generators.';
      ERR_GET_EXCEPTIONS:          Result := 'Unable to retrieve a list of exceptions.';
      ERR_GET_BLOB_FILTERS:        Result := 'Unable to retrieve a list of blob filters.';
      ERR_GET_COLUMNS:             Result := 'Unable to retrieve a list of columns.';
      ERR_GET_INDICES:             Result := 'Unable to retrieve a list of indices.';
      ERR_GET_REFERENTIAL_CONST:   Result := 'Unable to retrieve a list of referential constraints.';
      ERR_GET_UNIQUE_CONST:        Result := 'Unable to retrieve a list of unique constraints.';
      ERR_GET_CHECK_CONST:         Result := 'Unable to retrieve a list of check constraints.';
      ERR_GET_TRIGGERS:            Result := 'Unable to retrieve a list of triggers.';
      ERR_GET_DDL:                 Result := 'An error occured while attempting to extract metadata information.';
      ERR_INVALID_PROPERTY_VALUE:  Result := 'Invalid Property Value.';
      ERR_GET_DEPENDENCIES:        Result := 'An error occured while attempting to extract dependency information.';
      ERR_GET_DB_PROPERTIES:       Result := 'An error occured while attempting to retrieve database properties.';
      ERR_DB_SIZE:                 Result := 'Invalid database file size.';
      ERR_ISQL_ERROR:              Result := 'SQL Error';
      ERR_SERVER_SERVICE:          Result := 'Service Error';
      ERR_EXTERNAL_EDITOR:         Result := 'External Editor Error';
      ERR_SERVER_ALIAS:            Result := 'Invalid server alias. Please enter a valid server alias.';
      ERR_BACKUP_ALIAS:            Result := 'Invalid backup alias. Please enter a valid backup alias.';
      ERR_DB_SHUTDOWN:             Result := 'Database shutdown unsuccessful.';
      ERR_MODIFY_DB_PROPERTIES:    Result := 'Unable to modify database properties.';
      ERR_DROP_DATABASE:           Result := 'An error occured while attempting to drop the database.';
      ERR_FILE_OPEN:               Result := 'An error occured while attempting to open file.';
      ERR_INV_EDITOR:              Result := 'The editor specified is invalid.';
      ERR_EDITOR_MISSING:          Result := 'The external editor is not specified.';
      ERR_BAD_FORMAT:              Result := 'Unable to display blob.  The format is not graphical.';
      ERR_INV_DIALECT:             Result := 'Unable to change the client dialect.';
      ERR_FOPEN:                   Result := 'Error occured opening file.';
      ERR_TEXT_NOT_FOUND:          Result := 'Search string not found.';
      ERR_PRINT:                   Result := 'Unable to print.  Make sure your printer is installed and working.';
      ERR_NO_PATH:                 Result := 'No path was specified for the backup file or database.';
      ERR_NO_FILES:                Result := 'No files were specified for backup or restore.';
      ERR_GET_DOMAINS:             Result := 'Unable to retrieve a list of domains.';
      ERR_EXT_TOOL_ERROR:          Result := 'Unable to launch external tool';
      ERR_PROPERTIES:              Result := 'Unable to retrieve properties for the object';
      ERR_ALIAS_EXISTS:            Result := 'Invalid alias.  This alias already exists.';
      // kris start
      ERR_CANNOT_START_LOCALIB:    Result := 'Cannot start local interbase.';
      ERR_SERVER_REGISTER:         Result := 'An error enounter while trying to register a server.';
      ERR_SERVER_UNREGISTER:       Result := 'An error enounter while trying to unregister a server.';
      ERR_DATABASE_REGISTER:       Result := 'An error enounter while trying to register a database.';
      ERR_DATABASE_UNREGISTER:     Result := 'An error enounter while trying to unregister a database.';
      // kris end
      ERR_DDL_EXECUTION:           Result := 'An error occured while trying to excute a DDL statement.';

      //****** Warnings ******
      WAR_NO_PERMISSION:           Result := 'Insufficient rights for this operation.';
      WAR_SERVER_REGISTERED:       Result := 'The server is already registered.';
      WAR_DUPLICATE_DB_ALIAS:      Result := 'This database alias already exists.';
      WAR_BACKUP_FILE_REGISTERED:  Result := 'The backup file is already registered.';
      WAR_DIALECT_MISMATCH:        Result := 'The client dialect does not match the database dialect.';
      WAR_REMOTE_FILENAME:         Result := 'The file name specified may contain a server name.'+CRLF+'Some operations may not work correctly.';
      WAR_PERSISTENT_UPDATE:       Result := 'Cannot update persistent information.';

      {****** Information ******}
      INF_ADD_USER_SUCCESS:        Result := 'The user was added successfully.';
      INF_BACKUP_DB_SUCCESS:       Result := 'Database backup completed.';
      INF_RESTORE_DB_SUCCESS:      Result := 'Database restore completed.';
      INF_NO_PENDING_TRANSACTIONS: Result := 'No pending transactions were found.';
      INF_RESTART_SERVER:          Result := 'You must restart the server in order for the changes to go into effect.';
      INF_DATABASE_SHUTDOWN:       Result := 'Database shutdown completed successfully.';
      INF_DATABASE_RESTARTED:      Result := 'Database restart completed successfully.';
      INF_SQL_SCRIPT:              Result := 'SQL script done.';
      INF_DATABASE_SWEEP:          Result := 'Database sweep completed.';

      {****** InterBase Errors ******}
      isc_gbak_db_exists:          Result := 'To overwrite an existing database, set the Overwrite option to TRUE';
      isc_gfix_invalid_sw:         Result := 'An invalid option was specified for the operation.';
      isc_gfix_incmp_sw:           Result := 'The parameters for this operation are conflicting.';
      isc_gfix_retry:              Result := 'An operation was not specified.';
      isc_gfix_retry_db:           Result := 'A database was not specified for the operation';
      isc_gbak_page_size_missing:  Result := 'The page size must be specified.';
      isc_gsec_cant_open_db:       Result := 'The security database could not be opened.';
      isc_gsec_no_usr_name:        Result := 'User name missing.  A user name must be specified for all operations.';
      isc_gsec_err_add:            Result := 'An unknown error was encountered while attempting to add the user record.';
      isc_gsec_err_modify:         Result := 'An unknown error was encountered while attempting to modify the user record.';
      isc_gsec_err_find_mod:       Result := 'An unknown error was encountered while attempting to find/modify the user record.';
      isc_gsec_err_rec_not_found:  Result := 'The information for the user was not found.';
      isc_gsec_err_delete:         Result := 'An unknown error was encountered while attempting to delete the user record.';
      isc_gsec_err_find_del:       Result := 'An unknown error was encountered while attempting to find/delete the user record.';
      isc_gsec_err_find_disp:      Result := 'An unknown error was encountered while attempting to find/display the user record.';
      isc_sys_request:             Result := 'An unknown error attempting to open a file on the server.';
      else
        Result := GetLangText('Error')+' '+ IntToStr(MsgNo);
    end;
end;



function TfrmMessage.GetMessage(MsgNo: integer): string;
var s,id:string;
begin
  s:='';
  if assigned(gGUILanguageFile) then
  begin
    case MsgNo of
      //****** Errors ******
      ERR_SYSTEM_INIT:             id :=  'ERR_SYSTEM_INIT';
      ERR_USERNAME:                id :=  'ERR_USERNAME';
      ERR_PASSWORD:                id :=  'ERR_PASSWORD';
      ERR_PASSWORD_MISSMATCH:      id :=  'ERR_PASSWORD_MISSMATCH';
      ERR_ADD_USER:                id :=  'ERR_ADD_USER';
      ERR_MODIFY_USER:             id :=  'ERR_MODIFY_USER';
      ERR_DELETE_USER:             id :=  'ERR_DELETE_USER';
      ERR_GET_USERS:               id :=  'ERR_GET_USERS';
      ERR_GET_USER_INFO:           id :=  'ERR_GET_USER_INFO';
      ERR_SOURCE_DB:               id :=  'ERR_SOURCE_DB';
      ERR_DESTINATION_DB:          id :=  'ERR_DESTINATION_DB';
      ERR_SAME_SOURCE_DESTINATION: id :=  'ERR_SAME_SOURCE_DESTINATION';
      ERR_DB_FILE:                 id :=  'ERR_DB_FILE';
      ERR_SERVER_NAME:             id :=  'ERR_SERVER_NAME';
      ERR_PROTOCOL:                id :=  'ERR_PROTOCOL';
      ERR_BACKUP_DB:               id :=  'ERR_BACKUP_DB';
      ERR_RESTORE_DB:              id :=  'ERR_RESTORE_DB';
      ERR_GET_TABLES:              id :=  'ERR_GET_TABLES';
      ERR_GET_VIEWS:               id :=  'ERR_GET_VIEWS';
      ERR_SERVICE:                 id :=  'ERR_SERVICE';                 
      ERR_NUMERIC_VALUE:           id :=  'ERR_NUMERIC_VALUE';           
      ERR_GET_TABLE_DATA:          id :=  'ERR_GET_TABLE_DATA';          
      ERR_DB_ALIAS:                id :=  'ERR_DB_ALIAS';
      ERR_GET_ROLES:               id :=  'ERR_GET_ROLES';               
      ERR_SERVER_LOGIN:            id :=  'ERR_SERVER_LOGIN';            
      ERR_DB_CONNECT:              id :=  'ERR_DB_CONNECT';              
      ERR_DB_DISCONNECT:           id :=  'ERR_DB_DISCONNECT';           
      ERR_GET_PROCEDURES:          id :=  'ERR_GET_PROCEDURES';          
      ERR_GET_FUNCTIONS:           id :=  'ERR_GET_FUNCTIONS';
      ERR_GET_GENERATORS:          id :=  'ERR_GET_GENERATORS';          
      ERR_GET_EXCEPTIONS:          id :=  'ERR_GET_EXCEPTIONS';          
      ERR_GET_BLOB_FILTERS:        id :=  'ERR_GET_BLOB_FILTERS';
      ERR_GET_COLUMNS:             id :=  'ERR_GET_COLUMNS';             
      ERR_GET_INDICES:             id :=  'ERR_GET_INDICES';
      ERR_GET_REFERENTIAL_CONST:   id :=  'ERR_GET_REFERENTIAL_CONST';
      ERR_GET_UNIQUE_CONST:        id :=  'ERR_GET_UNIQUE_CONST';        
      ERR_GET_CHECK_CONST:         id :=  'ERR_GET_CHECK_CONST';         
      ERR_GET_TRIGGERS:            id :=  'ERR_GET_TRIGGERS';            
      ERR_GET_DDL:                 id :=  'ERR_GET_DDL';                 
      ERR_INVALID_PROPERTY_VALUE:  id :=  'ERR_INVALID_PROPERTY_VALUE';  
      ERR_GET_DEPENDENCIES:        id :=  'ERR_GET_DEPENDENCIES';
      ERR_GET_DB_PROPERTIES:       id :=  'ERR_GET_DB_PROPERTIES';       
      ERR_DB_SIZE:                 id :=  'ERR_DB_SIZE';                 
      ERR_ISQL_ERROR:              id :=  'ERR_ISQL_ERROR';              
      ERR_SERVER_SERVICE:          id :=  'ERR_SERVER_SERVICE';          
      ERR_EXTERNAL_EDITOR:         id :=  'ERR_EXTERNAL_EDITOR';         
      ERR_SERVER_ALIAS:            id :=  'ERR_SERVER_ALIAS';
      ERR_BACKUP_ALIAS:            id :=  'ERR_BACKUP_ALIAS';            
      ERR_DB_SHUTDOWN:             id :=  'ERR_DB_SHUTDOWN';             
      ERR_MODIFY_DB_PROPERTIES:    id :=  'ERR_MODIFY_DB_PROPERTIES';    
      ERR_DROP_DATABASE:           id :=  'ERR_DROP_DATABASE';           
      ERR_FILE_OPEN:               id :=  'ERR_FILE_OPEN';               
      ERR_INV_EDITOR:              id :=  'ERR_INV_EDITOR';
      ERR_EDITOR_MISSING:          id :=  'ERR_EDITOR_MISSING';          
      ERR_BAD_FORMAT:              id :=  'ERR_BAD_FORMAT';              
      ERR_INV_DIALECT:             id :=  'ERR_INV_DIALECT';             
      ERR_FOPEN:                   id :=  'ERR_FOPEN';                   
      ERR_TEXT_NOT_FOUND:          id :=  'ERR_TEXT_NOT_FOUND';          
      ERR_PRINT:                   id :=  'ERR_PRINT';
      ERR_NO_PATH:                 id :=  'ERR_NO_PATH';                 
      ERR_NO_FILES:                id :=  'ERR_NO_FILES';                
      ERR_GET_DOMAINS:             id :=  'ERR_GET_DOMAINS';             
      ERR_EXT_TOOL_ERROR:          id :=  'ERR_EXT_TOOL_ERROR';          
      ERR_PROPERTIES:              id :=  'ERR_PROPERTIES';              
      ERR_ALIAS_EXISTS:            id :=  'ERR_ALIAS_EXISTS';
      ERR_CANNOT_START_LOCALIB:    id :=  'ERR_CANNOT_START_LOCALIB';    
      ERR_SERVER_REGISTER:         id :=  'ERR_SERVER_REGISTER';         
      ERR_SERVER_UNREGISTER:       id :=  'ERR_SERVER_UNREGISTER';       
      ERR_DATABASE_REGISTER:       id :=  'ERR_DATABASE_REGISTER';
      ERR_DATABASE_UNREGISTER:     id :=  'ERR_DATABASE_UNREGISTER';     
      ERR_DDL_EXECUTION:           id :=  'ERR_DDL_EXECUTION';
      WAR_NO_PERMISSION:           id :=  'WAR_NO_PERMISSION';           
      WAR_SERVER_REGISTERED:       id :=  'WAR_SERVER_REGISTERED';       
      WAR_DUPLICATE_DB_ALIAS:      id :=  'WAR_DUPLICATE_DB_ALIAS';      
      WAR_BACKUP_FILE_REGISTERED:  id :=  'WAR_BACKUP_FILE_REGISTERED';  
      WAR_DIALECT_MISMATCH:        id :=  'WAR_DIALECT_MISMATCH';        
      WAR_REMOTE_FILENAME:         id :=  'WAR_REMOTE_FILENAME';
      WAR_PERSISTENT_UPDATE:       id :=  'WAR_PERSISTENT_UPDATE';       
      INF_ADD_USER_SUCCESS:        id :=  'INF_ADD_USER_SUCCESS';        
      INF_BACKUP_DB_SUCCESS:       id :=  'INF_BACKUP_DB_SUCCESS';       
      INF_RESTORE_DB_SUCCESS:      id :=  'INF_RESTORE_DB_SUCCESS';      
      INF_NO_PENDING_TRANSACTIONS: id :=  'INF_NO_PENDING_TRANSACTIONS'; 
      INF_RESTART_SERVER:          id :=  'INF_RESTART_SERVER';
      INF_DATABASE_SHUTDOWN:       id :=  'INF_DATABASE_SHUTDOWN';       
      INF_DATABASE_RESTARTED:      id :=  'INF_DATABASE_RESTARTED';      
      INF_SQL_SCRIPT:              id :=  'INF_SQL_SCRIPT';              
      INF_DATABASE_SWEEP:          id :=  'INF_DATABASE_SWEEP';          
      isc_gbak_db_exists:          id :=  'isc_gbak_db_exists';          
      isc_gfix_invalid_sw:         id :=  'isc_gfix_invalid_sw';
      isc_gfix_incmp_sw:           id :=  'isc_gfix_incmp_sw';           
      isc_gfix_retry:              id :=  'isc_gfix_retry';              
      isc_gfix_retry_db:           id :=  'isc_gfix_retry_db';           
      isc_gbak_page_size_missing:  id :=  'isc_gbak_page_size_missing';  
      isc_gsec_cant_open_db:       id :=  'isc_gsec_cant_open_db';       
      isc_gsec_no_usr_name:        id :=  'isc_gsec_no_usr_name';
      isc_gsec_err_add:            id :=  'isc_gsec_err_add';            
      isc_gsec_err_modify:         id :=  'isc_gsec_err_modify';         
      isc_gsec_err_find_mod:       id :=  'isc_gsec_err_find_mod';       
      isc_gsec_err_rec_not_found:  id :=  'isc_gsec_err_rec_not_found';  
      isc_gsec_err_delete:         id :=  'isc_gsec_err_delete';         
      isc_gsec_err_find_del:       id :=  'isc_gsec_err_find_del';
      isc_gsec_err_find_disp:      id :=  'isc_gsec_err_find_disp';      
      isc_sys_request:             id :=  'isc_sys_request';
      else                         id := '';
    end;
      s := gGUILanguageFile.ReadString('error_messages',id,'')
    end;
    if s = '' then s := GetDefaultMessage(MsgNo);
    result := s;
end;

procedure TfrmMessage.btnOKClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TfrmMessage.FormCreate(Sender: TObject);
begin
  inherited;
  memDetailMsg.Font.Color := clBtnText;
end;

end.
