{******************************************************************}
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{******************************************************************} 

unit frmuNewConstraint;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, DB, Grids, DBGrids, IBODataset, IBOCGlobal, IB_COmponents,
  ExtCtrls, FrameFieldsList, dmuNode, types;

type
  TConstraintType = (ctFK, ctCheck, ctPK);

  TfrmNewConstraint = class(TForm)
    btnOK: TButton;
    btnCancel: TButton;
    btnHelp: TButton;
    pgConstraint: TPageControl;
    tsFK: TTabSheet;
    tsCheck: TTabSheet;
    Label4: TLabel;
    cmbTables: TComboBox;
    Label5: TLabel;
    edCheck: TMemo;
    Label6: TLabel;
    edCheckName: TEdit;
    Label9: TLabel;
    edFKName: TEdit;
    chkOnDelete: TCheckBox;
    chkOnUpdate: TCheckBox;
    cmbOnDelete: TComboBox;
    cmbOnUpdate: TComboBox;
    tsPK: TTabSheet;
    Label7: TLabel;
    edPKName: TEdit;
    PkFrame: TFrameIndexList;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Panel4: TPanel;
    Panel5: TPanel;
    Panel6: TPanel;
    RefFrame: TFrameIndexList;
    ForeignFrame: TFrameIndexList;
    procedure cmbTablesChange(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure tsFKResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure edFKNameChange(Sender: TObject);
    procedure RefFrameSpeedButtonAddClick(Sender: TObject);
    procedure RefFrameSpeedButtonRemoveClick(Sender: TObject);
    procedure tsFKShow(Sender: TObject);
    procedure tsCheckShow(Sender: TObject);
    procedure tsPKShow(Sender: TObject);
    procedure pgConstraintChanging(Sender: TObject;
      var AllowChange: Boolean);
  private
    FConstraintType: TConstraintType;
    FNewObject: boolean;
    procedure SetConstraintType(const Value: TConstraintType);
    procedure SetNewObject(const Value: boolean);
  private
    FTableName: String;
    FSQLDialect : Integer;
    FCompileText: String;
    FDataModule: TDmN;
    fErrors:boolean;
    FObjectModified: boolean;
    
    { Private declarations }
    procedure SetCompileText(const Value: String);
    procedure SetDataModule(const Value: TDmN);
    procedure DoCompile;
    procedure InitFieldLists;
    procedure SetObjectModified(const Value: boolean);
    property ConstraintType:TConstraintType read FConstraintType write SetConstraintType;
  public
    { Public declarations }
    property TableName : String read FTableName write FTableName;
    property DataModule:TDmN read FDataModule write SetDataModule;
    property CompileText : String read FCompileText write SetCompileText;
    procedure NewConstraint(aDataModule:TDmN; aTable:String);
    procedure LoadConstraint(ConstraintName:String);
    property ObjectModified:boolean read FObjectModified write SetObjectModified;
    property NewObject:boolean read FNewObject write SetNewObject;
  end;

implementation

uses IBOAdminUtils, zluUtility, frmuMessage;


{$R *.DFM}



procedure TfrmNewConstraint.cmbTablesChange(Sender: TObject);
begin
  DataModule.GetColumnList(ForeignFrame.Fields,cmbTables.Text);
  ForeignFrame.ResetFields;
end;

procedure TfrmNewConstraint.btnOKClick(Sender: TObject);
var
  Tmp : String;
  Tmp1 : String;
  Tmp2 : String;
  idx : Integer;
  AlterSQL : String;
const
  pki = 2;
  fki = 0;
  chki = 1;
begin
  //validation first...

  case pgConstraint.ActivePage.PageIndex of
    pki : //Primary key
      begin
        if edPKName.Text = '' then
        begin
          MessageDlg('Primary Key Contraint must have a Name', mtError, [mbOK], 0);
          Exit;
        end;

        if PkFrame.lvIdxColumns.Count = 0 then
        begin
          MessageDlg('Please select field(s) to be used in Primary Key', mtError, [mbOK], 0);
          Exit;
        end;
      end;
    fki : //foreign Key
      begin
        if edFKName.Text = '' then
        begin
          MessageDlg('Foreign Key Contraint must have a Name', mtError, [mbOK], 0);
          Exit;
        end;

        if ForeignFrame.lvIdxColumns.Items.Count = 0 then
        begin
          MessageDlg('Please select field to be used in Foreign Key', mtError, [mbOK], 0);
          Exit;
        end;

        if RefFrame.lvIdxColumns.Items.Count = 0 then
        begin
          MessageDlg('Please select field to be referenced in Foreign Key', mtError, [mbOK], 0);
          Exit;
        end;

      end;

    chki : //check constraint
      begin
        if Trim(edCheckName.Text) = '' then
        begin
          MessageDlg('Check constraint must have a name', mtError, [mbOK], 0);
          Exit;
        end;

        if Trim(edCheck.Text) = '' then
        begin
          MessageDlg('Check constraint must contain an expression', mtError, [mbOK], 0);
          Exit;
        end;
      end;
  end;

  //now build up AlterSQL...

  case pgConstraint.ActivePage.PageIndex of
    pki : //Primary key
      begin
        Tmp := '';
        for idx := 0 to PkFrame.lvIdxColumns.Items.Count - 1 do
        begin
          Tmp := Tmp + QuoteIdentifier(FSQLDialect, PkFrame.lvIdxColumns.Items[idx]) + ', ';
        end;

        Tmp := Copy(Tmp, 1, Length(Tmp) - 2);

        AlterSQL := 'constraint ' + QuoteIdentifier(FSQLDialect,edPKName.Text) + ' primary key (' + Tmp + ')';
      end;

    fki : //foreign Key
      begin
        Tmp := '';
        for idx := 0 to RefFrame.lvIdxColumns.Items.Count - 1 do
        begin
          Tmp := Tmp + QuoteIdentifier(FSQLDialect,RefFrame.lvIdxColumns.Items[idx]) + ', ';
        end;
        Tmp := Copy(Tmp, 1, Length(Tmp) - 2);

        Tmp1 := '';
        for idx := 0 to ForeignFrame.lvIdxColumns.Items.Count - 1 do
        begin
          Tmp1 := Tmp1 + QuoteIdentifier(FSQLDialect,ForeignFrame.lvIdxColumns.Items[idx]) + ', ';
        end;
        Tmp1 := Copy(Tmp1, 1, Length(Tmp1) - 2);

        Tmp2 := '';
        if cmbOnDelete.Enabled then
        begin
          if chkOnDelete.Checked then
          begin
            Tmp2 := Tmp2 + ' on delete ' + cmbOnDelete.Text;
          end;
          if chkOnUpdate.Checked then
          begin
            Tmp2 := Tmp2 + ' on update ' + cmbOnUpdate.Text;
          end;
        end;

        AlterSQL := 'constraint ' + QuoteIdentifier(FSQLDialect,edFKName.Text) + ' foreign key (' + Tmp + ') references ' +
        QuoteIdentifier(FSQLDialect,cmbTables.Text) + '(' + Tmp1 + ')' + Tmp2;
      end;

    chki : //check constraint
      begin
        AlterSQL := 'constraint ' + QuoteIdentifier(FSQLDialect,edCheckName.Text) + ' check (' + Trim(edCheck.Text) + ')';
      end;
  end;

  FCOmpileText := 'alter table ' + QuoteIdentifier(FSQLDialect,FTableName) + ' add ' + AlterSQL + ';';

  fErrors := False;

  DoCompile;

  if not fErrors then
    ModalResult := mrOK;
end;

procedure TfrmNewConstraint.FormCreate(Sender: TObject);
begin
  fErrors := False;
  FCompileText := '';
end;

procedure TfrmNewConstraint.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift = [ssCtrl]) and (Key = VK_TAB) then
    pgConstraint.SelectNextPage(True);
end;

procedure TfrmNewConstraint.tsFKResize(Sender: TObject);
var H:integer;
begin
  H := tsFK.ClientHeight - Panel2.Height - Panel4.Height - Panel6.Height;
  Panel3.Height := H div 2;
end;

procedure TfrmNewConstraint.FormShow(Sender: TObject);
begin
  tsFKResize(Sender);
end;

procedure TfrmNewConstraint.SetCompileText(const Value: String);
begin
  FCompileText := Value;
end;

procedure TfrmNewConstraint.SetDataModule(const Value: TDmN);
begin
  FDataModule := Value;
end;

procedure TfrmNewConstraint.InitFieldLists;
var
  Idx:integer;
  s,s1:String;
  lObjectList: TStringList;
begin
  lObjectList:=TStringList.Create;
  try
    DataModule.GetColumnList(PkFrame.Fields,TableName);
    SetLength(RefFrame.Fields,Length(PkFrame.Fields));
    for Idx := Low(PkFrame.Fields) to High(PkFrame.Fields) do
    begin
      RefFrame.Fields[idx] := PkFrame.Fields[idx];
    end;
    RefFrame.RestoreFields;
    PkFrame.RestoreFields;
    cmbTables.Clear;
    DataModule.GetTableList(lObjectList, False);
    for Idx := 1 to lObjectList.Count - 1 do
    begin
      s:= lObjectList[idx];
      s1:= GetNextField(s,DEL);
      cmbTables.Items.Add(s1);
    end;
  finally
    lObjectList.Free;
  end;
end;

procedure TfrmNewConstraint.NewConstraint(aDataModule:TDmN; aTable: String);
begin
  NewObject := True;
  TableName := aTable;
  DataModule := aDataModule;
  Caption := 'New Constraint for ' + TableName + '';
  InitFieldLists;
end;

procedure TfrmNewConstraint.DoCompile;
var error:string;
begin
  if DataModule.ExecuteDDL(CompileText,error) then
  begin
    DataModule.CommitDDL;
  end
  else
  begin
    DisplayMsg(ERR_DDL_EXECUTION,error);
    DataModule.RollbackDDL;
    fErrors := True;
  end;
end;

procedure TfrmNewConstraint.LoadConstraint(ConstraintName: String);
var Data: TConstraintDef;
    Idx:TIndexDef;
    Ref: TRefConstraintDef;
    rPK:TStringDynArray;
begin
  NewObject := True;
  InitFieldLists;
  DataModule.GetConstraintData(Data,TableName,ConstraintName);
  if Data.cType = 'FOREIGN KEY' then
  begin
    Caption := 'Foreign Key '+ConstraintName+ ' for ' + TableName;
    pgConstraint.ActivePage := tsFK;
    edFKName.Text := Data.Name;
    DataModule.GetReferentialConstData(Ref,TableName,ConstraintName);
    DataModule.GetIndexData(Idx,Ref.IndexName);
    RefFrame.IncludeFields(Idx.Fields);
    cmbTables.ItemIndex := cmbTables.Items.IndexOf(Ref.RelationName);
    cmbTablesChange(nil);
    DataModule.GetPKList(Ref.RelationName,rPK);
    ForeignFrame.IncludeFields(rPK);
    cmbOnUpdate.ItemIndex := cmbOnUpdate.Items.IndexOf(Ref.Update_Rule);
    chkOnUpdate.Checked := Ref.Update_Rule <> '';
    cmbOnDelete.ItemIndex := cmbOnDelete.Items.IndexOf(Ref.DeleteRule);
    chkOnDelete.Checked := Ref.DeleteRule <> '';
  end
  else
  if Data.cType = 'PRIMARY KEY' then
  begin
    Caption := 'Primary Key for ' + TableName;
    pgConstraint.ActivePage := tsPK;
    edPKName.Text := Data.Name;
    DataModule.GetIndexData(Idx,Data.IndexName);
    PkFrame.IncludeFields(Idx.Fields);
  end
  else
  if Data.cType = 'CHECK' then
  begin
    Caption := 'Check Constraint '+ConstraintName+ ' for ' + TableName;
    pgConstraint.ActivePage := tsCheck;
    edCheckName.Text := Data.Name;
  end;
  NewObject := False;
end;

procedure TfrmNewConstraint.SetObjectModified(const Value: boolean);
begin
  FObjectModified := Value;
end;

procedure TfrmNewConstraint.edFKNameChange(Sender: TObject);
begin
  ObjectModified := true;
end;

procedure TfrmNewConstraint.RefFrameSpeedButtonAddClick(Sender: TObject);
begin
  RefFrame.SpeedButtonAddClick(Sender);
  ObjectModified := true;
end;

procedure TfrmNewConstraint.RefFrameSpeedButtonRemoveClick(
  Sender: TObject);
begin
  RefFrame.SpeedButtonRemoveClick(Sender);
  ObjectModified := true;
end;

procedure TfrmNewConstraint.SetConstraintType(
  const Value: TConstraintType);
begin
  FConstraintType := Value;
end;

procedure TfrmNewConstraint.tsFKShow(Sender: TObject);
begin
  ConstraintType := ctFK;
end;

procedure TfrmNewConstraint.tsCheckShow(Sender: TObject);
begin
  ConstraintType := ctCheck;
end;

procedure TfrmNewConstraint.tsPKShow(Sender: TObject);
begin
  ConstraintType := ctPK;
end;

procedure TfrmNewConstraint.pgConstraintChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
  AllowChange := FNewObject;
end;

procedure TfrmNewConstraint.SetNewObject(const Value: boolean);
begin
  FNewObject := Value;
end;

end.
