{******************************************************************}
{  This Unit was created to by Lorenzo Mengoni - (Ancona, Italy)
   for suggestion and contributions email to lorenzo@mengoni.com
}
{******************************************************************}
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html                          }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{******************************************************************} 

// 13.August.2002, Lorenzo:
// - updated Code for handling trigger description

unit frmuNewTrigger;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls, dmuNode, SynEdit;

type
  TParamRec = record
    pName:String;
    pType:String;
  end;
  TParamArray = array of TParamRec;

  TfrmNewTrigger = class(TForm)
    btnOK: TButton;
    btnCancel: TButton;
    Label1: TLabel;
    edTriggerName: TEdit;
    cmbTrigPos: TComboBox;
    Label4: TLabel;
    edPosition: TEdit;
    UpDown1: TUpDown;
    cmbActive: TComboBox;
    Label2: TLabel;
    Label3: TLabel;
    SynEdit1: TSynEdit;
    Label5: TLabel;
    Panel1: TPanel;
    Panel2: TPanel;
    Label6: TLabel;
    EditTable: TEdit;
    edDescription: TSynEdit;
    Label7: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure edTriggerNameChange(Sender: TObject);
  private
    FTriggerName: String;
    FTableName: String;
    FDataModule: TDmN;
    FErrors:boolean;
    FNewObject: boolean;
    procedure SetTableName(const Value: String);
    procedure SetTriggerName(const Value: String);
    procedure SetDataModule(const Value: TDmN);
    { Private declarations }
  public
    { Public declarations }
    property TableName:String read FTableName write SetTableName;
    property TriggerName:String read FTriggerName write SetTriggerName;
    property DataModule:TDmN read FDataModule write SetDataModule;
    procedure NewTrigger(aDataModule:TDmN; aTableName:String);
    procedure LoadTrigger(aDataModule:TDmN; aTableName,aTriggerName:String);
    property NewObject:boolean read FNewObject;
  end;


implementation

uses frmuMain, frmuMessage, IBOAdminUtils, dmuMain, IBOCGlobal;


{$R *.DFM}

procedure TfrmNewTrigger.FormCreate(Sender: TObject);

begin
  cmbActive.ItemIndex := 0;
end;

procedure TfrmNewTrigger.btnOKClick(Sender: TObject);
var
  AlterSQL:String;
  error:String;
  source:String;
  lSqlDialect:integer;
begin
  lSqlDialect := DataModule.Connection.SQLDialect;
  AlterSQL := '';

  if TriggerName = '' then
  begin
    MessageDlg('Trigger must have a name.', mtError, [mbOK], 0);
    Exit;
  end;

  if cmbTrigPos.Text = '' then
  begin
    MessageDlg('Trigger must be assigned a position.', mtError, [mbOK], 0);
    Exit;
  end;

  source:= Trim(SynEdit1.Lines.Text);

  if not ((Length(source)>1) and
     (Upcase(source[1]) = 'A') and
     (Upcase(source[2]) = 'S'))
  then
     source := 'as' + CRLF + source;
  source := source + '^';
  if NewObject then
  begin
    AlterSQL := AlterSQL + 'set term ^; '+CRLF;
    AlterSQL := AlterSQL + 'create trigger ' + QuoteIdentifier(lSQLDialect,FTriggerName) + ' for ';
    AlterSQL := AlterSQL + QuoteIdentifier(lSQLDialect,TableName) + ' ' + cmbActive.Text + ' ';
    AlterSQL := AlterSQL + cmbTrigPos.Text + ' position ' + edPosition.Text + CRLF + source+ CRLF;
    AlterSQL := AlterSQL + 'set term ; ^ ';
  end
  else
  begin
    AlterSQL := AlterSQL + 'set term ^; '+CRLF;
    AlterSQL := AlterSQL + 'alter trigger ' + QuoteIdentifier(lSQLDialect,FTriggerName) ;
    AlterSQL := AlterSQL + ' ' + cmbActive.Text + ' ';
    AlterSQL := AlterSQL + cmbTrigPos.Text + ' position ' + edPosition.Text + CRLF + source+ CRLF;
    AlterSQL := AlterSQL + 'set term ; ^ ';
  end;

  if DataModule.ExecuteDDLScript(AlterSQL,error) then
  begin
    DataModule.CommitDDL;
    if edDescription.Modified then
    try
      DataModule.ObjectDescriptionUpdate(NODE_TRIGGER,FTriggerName,edDescription.Text);
      edDescription.Modified := False;
    except
      edDescription.Modified := False;
    end;
    ModalResult := mrOK;
  end
  else
  begin
    DisplayMsg(ERR_DDL_EXECUTION,Error);
    DataModule.RollbackDDL;
    FErrors := True;
  end;

end;

procedure TfrmNewTrigger.SetTableName(const Value: String);
begin
  FTableName := Value;
end;

procedure TfrmNewTrigger.SetTriggerName(const Value: String);
begin
  FTriggerName := Value;
  edTriggerName.Text := Value;
end;

procedure TfrmNewTrigger.edTriggerNameChange(Sender: TObject);
begin
  FTriggerName := edTriggerName.Text;
end;

procedure TfrmNewTrigger.SetDataModule(const Value: TDmN);
begin
  FDataModule := Value;
end;

procedure TfrmNewTrigger.NewTrigger(aDataModule: TDmN; aTableName: String);
var s:String;
    pre:String;
    t:String;
    n:integer;
begin
  DataModule := aDataModule;
  TableName := aTableName;
  EditTable.Text := aTableName;
  fNewObject := True;
  pre := 'TRIG_';
  n:=1;
  t:=copy(TableName,1,31-8);
  s:= pre+t+'_'+inttostr(n);
  while DataModule.TriggerNameExist(s) and (n<20) do
  begin
    inc(n);
    s:= pre+t+'_'+inttostr(n);
  end;
  edTriggerName.Text := S;
  TriggerName :=s;
  SynEdit1.Lines.Add('/* code here */');
  edDescription.Modified := False;
  //SynEdit1.Lines.Add('POST_EVENT '+AnsiQuotedStr(FTriggerName,#39)+';');
end;

procedure TfrmNewTrigger.LoadTrigger(aDataModule: TDmN; aTableName,
  aTriggerName: String);
var
  TriggerType,
  Active,
  Table,
  FireOrder,
  Description,
  Source: String;
begin
  DataModule := aDataModule;
  TableName := aTableName;
  TriggerName := aTriggerName;
  Caption := 'Trigger - '+TriggerName;
  if DataModule.GetTriggerData(TriggerName,TriggerType,Active,Table,FireOrder,Source,Description) = SUCCESS then
  begin
    cmbTrigPos.ItemIndex := StrToIntDef(TriggerType,0)-1;
    UpDown1.Position := StrToIntDef(FireOrder,-1);
    TableName := Table;
    EditTable.Text := Table;
    cmbActive.ItemIndex := StrToIntDef(Active,-1);
    SynEdit1.Lines.Text := Source;
    edDescription.Lines.Text := Description;
    edDescription.Modified := False;
  end;
end;


end.

