{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 * 
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 * 
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 * 
 * Contributor(s): Krzysztof Golko
}

unit frmuServerLogin;

interface

uses
{$IFDEF LINUX}
  QForms, QExtCtrls, QStdCtrls, QControls,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Forms, ExtCtrls, StdCtrls, Controls,
  Messages,
{$ENDIF}
  IB_Session, IBOConsoleForm, SysUtils, Classes, zluibcClasses;

type
  TfrmServerLogin = class(TIBOConsoleForm)
    lblServerName: TLabel;
    lblUsername: TLabel;
    edtUsername: TEdit;
    lblPassword: TLabel;
    edtPassword: TEdit;
    btnLogin: TButton;
    btnCancel: TButton;
    bvlLine1: TBevel;
    lblServer: TLabel;
    lblRole: TLabel;
    edtRole: TEdit;
    cbCaseSensitive: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnLoginClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    function VerifyInputData(): boolean;
    class function DoLogin(var CurrSelServer: TibcServerNode): boolean;
  public
    { Public declarations }
  end;

function IBCServerLogin(var CurrSelServer: TibcServerNode; const SilentLogin: boolean): boolean;

implementation

uses
  IBOCGlobal, frmuMessage, zluContextHelp, IB_Header;

{$IF Declared(Forms)}
{$R *.DFM}
{$IFEND}
{$IF Declared(QForms)}
{$R *.xfm}
{$IFEND}

//  CallHelp := False;
//  Result := Application.HelpCommand(HELP_CONTEXT,SERVER_LOGIN);

class function TfrmServerLogin.DoLogin(var CurrSelServer: TibcServerNode): boolean;
begin
  // if a silent login is specified or the user has successfully entered the
  // proper login details
  try
    Application.ProcessMessages;

    CurrSelServer.AttachServer;
    // if the server successfully attached
    if CurrSelServer.Server.Active = true then
      result := true                   // set result to true
    else
      result := false;
  except                               // otherwise set result to false
    on E : EIB_ISCError do                 // if an exception occurs then trap it
    begin                              // and show error message
      Screen.Cursor := crDefault;      // change cursor to default
      case E.ERRCODE of
        isc_svcnotdef: raise Exception.Create('IBOConsole '+GetPrompt(126));
        else
          DisplayMsg (ERR_SERVER_LOGIN, E.Message);
      end;
      result := false;
    end;
  end;
end;

function IBCServerLogin(var CurrSelServer: TibcServerNode; const SilentLogin: boolean): boolean;
var
  frmServerLogin: TfrmServerLogin;
begin
  // check if a silent login was specified
  if not SilentLogin then
  begin
    // if no silent login was specified
    frmServerLogin:= TfrmServerLogin.Create(Application.MainForm);
    try
      // set server and user name
      frmServerLogin.lblServer.Caption := CurrSelServer.NodeName;
      frmServerLogin.edtUsername.Text := CurrSelServer.UserName;
      if frmServerLogin.cbCaseSensitive.Checked then
        frmServerLogin.edtRole.Text := CurrSelServer.SqlRole
      else
        frmServerLogin.edtRole.Text := UpperCase(CurrSelServer.SqlRole);

      // show form as modal dialog box
      Result := FALSE;
      while frmServerLogin.ShowModal = mrOk do
      begin
        CurrSelServer.UserName := frmServerLogin.edtUsername.Text;
        CurrSelServer.Password := frmServerLogin.edtPassword.Text;
        if frmServerLogin.cbCaseSensitive.Checked then
          CurrSelServer.SqlRole := frmServerLogin.edtRole.Text
        else
          CurrSelServer.SqlRole := UpperCase(frmServerLogin.edtRole.Text);
        if TfrmServerLogin.DoLogin(CurrSelServer) then
        begin
          Result := TRUE;
          Break;
        end;
      end;
    finally
      // deallocate memory
      frmServerLogin.Release;
    end;
  end
  else
    Result := TfrmServerLogin.DoLogin(CurrSelServer);
end;

procedure TfrmServerLogin.btnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TfrmServerLogin.btnLoginClick(Sender: TObject);
begin
  if VerifyInputData() then
    ModalResult := mrOK;
end;

{ Performs some basic validation on data entered by the user }
function TfrmServerLogin.VerifyInputData(): boolean;
begin
  result := true;

  // if no username is supplied
  if (edtUsername.Text = '') or (edtUsername.Text = ' ') then
  begin
    DisplayMsg(ERR_USERNAME,'');       // display an error message
    edtUsername.SetFocus;              // give focus to control
    result := false;
    Exit;
  end;

  // if no password is supplied
  if (edtPassword.Text = '') or (edtPassword.Text = ' ') then
  begin
    DisplayMsg(ERR_PASSWORD,'');       // display an error message
    edtPassword.SetFocus;              // give focus to control
    result := false;
    Exit;
  end;
end;

procedure TfrmServerLogin.FormShow(Sender: TObject);
begin
  if edtUserName.Text = '' then
    edtUserName.SetFocus
  else {if edtPassword.Text = '' then }
  begin
    edtPassword.SelectAll;
    edtPassword.SetFocus;
  end;
end;

procedure TfrmServerLogin.FormDestroy(Sender: TObject);
begin
  inherited;
  edtPassword.Clear;
end;

procedure TfrmServerLogin.FormCreate(Sender: TObject);
begin
  HelpContext := SERVER_LOGIN;
end;

end.
