{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 *
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 *
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 *
 * Contributor(s): Krzysztof Golko.
}

unit frmuTextViewer;

interface

uses
{$IFDEF LINUX}
  QControls, QForms, QDialogs, QComCtrls, QStdCtrls, QMenus,
  Types, QTypes, QImgList, QExtCtrls,
  QStdActns, QActnList, Menus, Dialogs, SynEditPrint, StdActns, Classes,
  ActnList, ImgList, Controls, SynEdit, ComCtrls, ToolWin, , SynEditPrint, IBCXDummies,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Menus, ImgList,
  SynEditPrint, StdActns, ActnList, ToolWin,
{$ENDIF}
  SynEditTypes, SynEdit,
  IBOServices, SysUtils, Classes, IBOConsoleForm;

type
  TfrmTextViewer = class(TIBOConsoleForm)
    ToolButton5: TToolButton;
    imgToolbarImages: TImageList;
    mnuEdCopy: TMenuItem;
    mnuEdFind: TMenuItem;
    mnuEdN1: TMenuItem;
    mnuEdit: TMenuItem;
    mnuFiExit: TMenuItem;
    mnuFiN1: TMenuItem;
    mnuFiPrint: TMenuItem;
    mnuFiSaveAs: TMenuItem;
    mnuFile: TMenuItem;
    mnuMain: TMainMenu;
    sbCopy: TToolButton;
    sbSaveAs: TToolButton;
    stbStatusBar: TStatusBar;
    tlbStandard: TToolBar;
    TextViewActions: TActionList;
    EditCopy1: TEditCopy;
    EditCut1: TEditCut;
    EditPaste1: TEditPaste;
    EditSelectAll1: TEditSelectAll;
    EditUndo1: TEditUndo;
    Cut1: TMenuItem;
    Paste1: TMenuItem;
    SelectAll1: TMenuItem;
    Undo1: TMenuItem;
    N1: TMenuItem;
    Font1: TMenuItem;
    EditFont: TAction;
    FontDialog1: TFontDialog;
    FindDialog1: TFindDialog;
    reEditor: TSynEdit;
    ToolButton1: TToolButton;
    ToolButton3: TToolButton;
    ToolButton2: TToolButton;
    ToolButton4: TToolButton;
    PrintDialog: TPrintDialog;
    SynEditPrint: TSynEditPrint;
    EditPopup: TPopupMenu;
    MenuItem1: TMenuItem;
    Copy1: TMenuItem;
    MenuItem2: TMenuItem;
    N15: TMenuItem;
    MenuItem3: TMenuItem;
    procedure FormShow(Sender: TObject);
    procedure mnuEdFindClick(Sender: TObject);
    procedure mnuFiExitClick(Sender: TObject);
    procedure mnuFiSaveAsClick(Sender: TObject);
    procedure EditFontExecute(Sender: TObject);
    procedure mnuFiPrintClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure EditUndo1Update(Sender: TObject);
    procedure FindDialog1Find(Sender: TObject);
    procedure reEditorStatusChange(Sender: TObject;
      Changes: TSynStatusChanges);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }
//    FFileName: string;
    FCancelled: boolean;
    f_Running:boolean;
//    procedure SetFileName(const sFileName: String);
    function ShowServiceInfo(const Service: TIBOControlAndQueryService; const sFormCaption: string; const readonly: boolean=true): integer;
    function ShowStrings(const Data: TStrings; const Title: String; const readonly: boolean=true): integer;
    procedure SetRunning(const Value: boolean);
  public
    { Public declarations }
    property Running:boolean read F_Running write SetRunning;
  published
  end;

procedure IBCShowServiceInfo(const Service: TIBOControlAndQueryService; const Title: string; const ReadOnly: boolean=true);
procedure IBCShowStrings(const Data: TStrings; const Title: String; const ReadOnly: boolean=true);

implementation

uses
  IBOCGlobal, zluContextHelp, frmuWindowList, frmuMessage, IB_Header,
  LMMessageDialog;

{$IF Declared(Forms)}
{$R *.DFM}
{$IFEND}
{$IF Declared(QForms)}
{$R *.xfm}
{$IFEND}
procedure IBCShowServiceInfo(const Service: TIBOControlAndQueryService; const Title: string; const ReadOnly: boolean=true);
var
  Viewer: TfrmTextViewer;
begin
  Viewer := TfrmTextViewer.Create(Application);
  Viewer.ShowServiceInfo(Service, Title, ReadOnly);
end;

procedure IBCShowStrings(const Data: TStrings; const Title: String; const ReadOnly: boolean=true);
var
  Viewer: TfrmTextViewer;
begin
  Viewer := TfrmTextViewer.Create(Application);
  Viewer.ShowStrings(Data, Title, ReadOnly);
end;

function TfrmTextViewer.ShowServiceInfo(const Service: TIBOControlAndQueryService;
        const sFormCaption: string; const readonly:boolean=true): integer;
begin
  Result := FAILURE;
  if running then exit;
  Running := True;
  try
    Caption := sFormCaption;           // set caption for form
    reEditor.Lines.Clear;
    reEditor.Readonly := readonly;
    if ReadOnly then
      stbStatusBar.Panels[1].Text := GetLangText('Read-Only');
    EditPaste1.Enabled := not Readonly;
    Show;
    if Assigned(OpenWindows) then
      OpenWindows.Add(sFormCaption, Self);
    reEditor.Lines.Add(GetLangText('Service started at')+' ' + DateTimeToStr(now));
    while not Service.Eof do
    begin
      if not FCancelled then
      begin
        reEditor.Lines.Add(service.GetNextLine);
        reEditor.TopLine := reEditor.Lines.Count - reEditor.LinesInWindow + 3;
      end
      else
      begin
        Result := FAILURE;
        Exit;
      end;
      Application.ProcessMessages;
    end;
    reEditor.Lines.Add(GetLangText('Service ended at')+' ' + DateTimeToStr(now));
    reEditor.TopLine := reEditor.Lines.Count - reEditor.LinesInWindow + 3;
    result := SUCCESS;
    reEditor.Modified := false;
  finally
    Running := False;
  end;

end;

procedure TfrmTextViewer.FormShow(Sender: TObject);
begin
  reEditor.Modified := false;
end;

procedure TfrmTextViewer.mnuEdFindClick(Sender: TObject);
begin
  FindDialog1.Execute;
end;

procedure TfrmTextViewer.mnuFiExitClick(Sender: TObject);
begin
  Close;                               // close the form
end;

procedure TfrmTextViewer.mnuFiSaveAsClick(Sender: TObject);
var
  loSaveDialog: TSaveDialog;
begin
  loSaveDialog := nil;
  try
  begin
     // create and show save dialog box
    loSaveDialog := TSaveDialog.Create(Self);
    loSaveDialog.Filter := getlangtext('Text File')+' (*.txt)|*.TXT|'+getlangtext('SQL files')+' (*.sql)|*.SQL|'+getlangtext('All files')+' (*.*)|*.*';
    if loSaveDialog.Execute then
    begin
      // if the specified file already exists the show overwrite message
      // if the user does not wish to overwrite the file then exit
      if FileExists(loSaveDialog.FileName) then
        if LMMessageDlg(0, [loSaveDialog.FileName], mtConfirmation, mbYesNoCancel, 0) <> mrYes then Exit;

      // if the file doesn't exist of the user wishes to overwrite it then
      // save the contents of the richedit component to the specified file
      reEditor.Lines.SaveToFile(loSaveDialog.FileName);
//      SetFileName(loSaveDialog.FileName);
      reEditor.Modified := False;      // set modified flag to false
    end;
  end
  finally
    // deallocate memory
    loSaveDialog.free;
  end;
end;

{procedure TfrmTextViewer.SetFileName(const sFileName: String);
begin
  FFileName := sFileName;              // set filename
end; }

function TfrmTextViewer.ShowStrings(const Data: TStrings;
  const Title: String; const readonly: boolean = true): integer;
begin
  Caption := Title;
  reEditor.Lines.BeginUpdate;
  reEditor.Lines.Clear;
  reEditor.ReadOnly := readonly;
  reEditor.Lines.Text := Data.Text;
  reEditor.Modified := false;
  reEditor.Lines.EndUpdate;
  EditPaste1.Enabled := not Readonly;
  Show;
  // kris Kylix
  if Assigned(OpenWindows) then
    OpenWindows.Add(Title, Self);
  result := SUCCESS;
  reEditor.Modified := false;
end;

procedure TfrmTextViewer.EditFontExecute(Sender: TObject);
begin
  FontDialog1.Font.Assign(reEditor.Font);
  if FontDialog1.Execute then
  begin
    reEditor.Font.Assign(FontDialog1.Font);
    reEditor.SetFocus;
  end;
end;

procedure TfrmTextViewer.mnuFiPrintClick(Sender: TObject);
begin
{$IFDEF MSWINDOWS}
  if PrintDialog.Execute then
  begin
    SynEditPrint.SynEdit := reEditor;
    SynEditPrint.Print;
  end;
{$ENDIF}
end;

procedure TfrmTextViewer.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  // kris Kylix
  if Assigned(OpenWindows) then
    OpenWindows.Remove(Self);
  FCancelled := true;
{$IFDEF MSWINDOWS}
  Action := caFree;
{$ENDIF}
end;

procedure TfrmTextViewer.EditUndo1Update(Sender: TObject);
begin
  (Sender as TAction).Enabled := reEditor.Modified;
  if reEditor.Modified then
    stbStatusBar.Panels[1].Text := GetLangText('Modified');
end;

procedure TfrmTextViewer.FindDialog1Find(Sender: TObject);
var
  Options: TSynSearchOptions;
begin
  Options := [];
  if not (frDown in FindDialog1.Options) then
    Include(Options, ssoBackwards);
  if frMatchCase in FindDialog1.Options then
    Include(Options, ssoMatchCase);
  if frWholeWord in FindDialog1.Options then
    Include(Options, ssoWholeWord);
  if reEditor.SearchReplace(FindDialog1.FindText, '', Options) = 0 then
  begin
    LMMessageDlg(67,[FindDialog1.FindText],mtInformation,[mbOK],0);
    if ssoBackwards in Options then
      reEditor.BlockEnd := reEditor.BlockBegin
    else
      reEditor.BlockBegin := reEditor.BlockEnd;
    reEditor.CaretXY := reEditor.BlockBegin;
  end;
end;

procedure TfrmTextViewer.reEditorStatusChange(Sender: TObject;
  Changes: TSynStatusChanges);
var
  p: TBufferCoord;
begin
  p := reEditor.CaretXY;
  stbStatusBar.Panels[0].Text := Format('%6d:%3d', [p.Line, p.Char]);
end;

procedure TfrmTextViewer.FormCreate(Sender: TObject);
begin
  HelpContext := FEATURES_INTERNAL_TEXT_VIEWER;
  f_Running:=false;
end;

procedure TfrmTextViewer.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  Canclose := not Running;
end;

procedure TfrmTextViewer.SetRunning(const Value: boolean);
begin
  F_Running := Value;
  if F_Running then
    stbStatusBar.Panels[2].Text := GetLangText('Running')
  else
    stbStatusBar.Panels[2].Text := '';
end;

end.
