{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 *
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 *
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 *
 * Contributor(s): Krzysztof Golko.
}

unit frmuTools;

interface

uses
{$IFDEF LINUX}
  QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QButtons, QActnList, Types,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ActnList,
{$ENDIF}
  SysUtils, Classes, IBOConsoleForm;

type

  TfrmTools = class(TIBOConsoleForm)
    btnAdd: TButton;
    btnDelete: TButton;
    btnEdit: TButton;
    Label1: TLabel;
    lbTools: TListBox;
    btnActions: TActionList;
    ToolAdd: TAction;
    ToolDelete: TAction;
    JvToolEdit: TAction;
    ToolBtnUp: TAction;
    ToolBtnDown: TAction;
    Button1: TButton;
    ToolBtnClose: TAction;
    procedure ToolDeleteUpdate(Sender: TObject);
    procedure lbToolsDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure ToolAddExecute(Sender: TObject);
    procedure ToolEditExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ToolDeleteExecute(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

implementation

{$IF Declared(Forms)}
{$R *.dfm}
{$IFEND}

{$IF Declared(QForms)}
{$R *.xfm}
{$IFEND}

uses
  IBOCGlobal, frmuAddTool, IBoCPersistent;

procedure TfrmTools.ToolDeleteUpdate(Sender: TObject);
var
  lcnt: integer;
begin
  inherited;

  for lCnt := 0 to lbTools.Items.Count - 1 do
    if lbTools.Selected[lCnt] then
    begin
      (Sender as TAction).Enabled := true;
      exit;
    end;
  (Sender as TAction).Enabled := false;
end;

procedure TfrmTools.lbToolsDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
  FontHeight: integer;
  Flags: LongInt;
begin
{$IFDEF MSWINDOWS}
  inherited;
  with lbTools.Canvas do
  begin
    FillRect(Rect);
    Font := Self.Font;

    if odSelected in State then
      Font.Color := clHighlightText;

    FontHeight := TextHeight('W');
    with Rect do
    begin
      Top := ((Bottom + Top) - FontHeight) div 2;
      Bottom := Top + FontHeight;
      Left := Left+TextWidth('W');
    end;

    Flags := DT_EXPANDTABS or DT_VCENTER;
    Flags := DrawTextBiDiModeFlags(Flags);
    DrawText(Handle, PChar(lbTools.Items[Index]), -1, Rect, Flags);
  end;
{$ENDIF}
end;

procedure TfrmTools.ToolAddExecute(Sender: TObject);
var
  toolDlg: TfrmAddTools;
begin
  inherited;
  toolDlg := TfrmAddTools.Create (self);
  with toolDlg do
  begin
    if ShowModal = mrOK then
    begin
      gPersistentInfo.AddExternalApp(edtTitle.Text, edtProgram.Text, edtWorkingDir.Text, edtParams.Text);

      lbTools.Items.Add(edtTitle.text);
      gExternalApps.Add(edtTitle.text);
    end;
  end;
end;

procedure TfrmTools.ToolEditExecute(Sender: TObject);
var
  toolDlg: TfrmAddTools;
  lTitle, lAppPath, lWorkDir, lParams: string;
begin
  inherited;
  toolDlg := TfrmAddTools.Create (self);
  with toolDlg do
  begin
    gPersistentInfo.GetExternalApp(lbTools.ItemIndex, lTitle, lAppPath, lWorkDir, lParams);
    edtTitle.Text := lTitle;
    edtProgram.Text := lAppPath;
    edtWorkingDir.Text := lWorkDir;
    edtParams.Text := lParams;

    if ShowModal = mrOK then
    begin
      lTitle := edtTitle.Text;
      lAppPath := edtProgram.Text;
      lWorkDir := edtWorkingDir.Text;
      lParams := edtParams.Text;
      gPersistentInfo.StoreExternalApp(lbTools.ItemIndex, lTitle, lAppPath, lWorkDir, lParams);
      lbTools.Items.BeginUpdate;
      lbTools.Items.Strings[lbTools.ItemIndex] := edtTitle.Text;
      lbTools.Items.EndUpdate;
      gExternalApps.Strings[lbTools.ItemIndex] :=  edtTitle.text;
    end;
  end;
end;

procedure TfrmTools.FormShow(Sender: TObject);
begin
  inherited;
  lbTools.Items.AddStrings(gExternalApps);
end;

procedure TfrmTools.ToolDeleteExecute(Sender: TObject);
begin
  inherited;
  gPersistentInfo.DeleteExternalApp(lbTools.ItemIndex);
  gPersistentInfo.GetExternalApps(gExternalApps);
  lbTools.Items.BeginUpdate;
  lbTools.Items.Clear;
  lbTools.Items.AddStrings(gExternalApps);
  lbTools.Items.EndUpdate;
  lbTools.SetFocus;
end;

end.
