{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 * 
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 * 
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 * 
 * Contributor(s): Jeff Overcash, Krzysztof Golko.
}

{****************************************************************
*
*  f r m u U s e r
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Description:  This unit provides an interface for managing
*                user accounts
*
*****************************************************************
* Revisions:
*

30.November.2004 by Lorenzo
frmuUser
- simplified data assignment to connection to avoid a bug in protocol value
Thanks a lot to Akim Kalwa for let know about.
01.December.2004 by Lorenzo
- as suggested by Akim Kalwa I changed the buttons in delete confirmation to yes no cancel

*****************************************************************}

unit frmuUser;

interface

uses
{$IFDEF LINUX}
  Qt, QGraphics, QForms, QActnList, QStdCtrls, QControls,
  QDialogs,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Graphics, Forms, ExtCtrls, StdCtrls, Controls, Dialogs,
  ActnList,
{$ENDIF}
  SysUtils, Classes, IBOServices, zluibcClasses, IB_Components, IB_Session,
  IBOConsoleForm;

type
  TfrmUserInfo = class(TIBOConsoleForm)
    btnNew: TButton;
    btnApply: TButton;
    btnDelete: TButton;
    btnClose: TButton;
    btnCancel: TButton;
    gbOptionalInfo: TGroupBox;
    lblFName: TLabel;
    lblMName: TLabel;
    lblLName: TLabel;
    edtLName: TEdit;
    edtFName: TEdit;
    edtMName: TEdit;
    gbRequiredInfo: TGroupBox;
    lblPassword: TLabel;
    lblConfirmPassword: TLabel;
    lblUserName: TLabel;
    edtUsername: TEdit;
    cbUsername: TComboBox;
    edtPassword: TEdit;
    edtConfirmPassword: TEdit;
    ActionList1: TActionList;
    NewUser: TAction;
    ModifyUser: TAction;
    DeleteUser: TAction;
    IB_Connection1: TIB_Connection;
    IB_Transaction1: TIB_Transaction;
    procedure btnApplyClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnNewClick(Sender: TObject);
    procedure cbUsernameChange(Sender: TObject);
    procedure cbUsernameClick(Sender: TObject);
    procedure edtConfirmPasswordChange(Sender: TObject);
    procedure edtFNameChange(Sender: TObject);
    procedure edtLNameChange(Sender: TObject);
    procedure edtMNameChange(Sender: TObject);
    procedure edtPasswordChange(Sender: TObject);
    procedure edtUsernameChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DeleteUserUpdate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FConfirmPassword: string;
    FCurrSelServer: TibcServerNode;
    FPassword: string;
    //FSecurityService: TIBOSecurityService;
    function GetUserInfo(UserName:String): boolean;
    function VerifyInputData(): boolean;
    procedure GetUsers(const idx: integer = -1);
  public
    { Public declarations }
    AddNew: boolean;
  end;

  function UserInfo(var CurrSelServer: TibcServerNode;
    const CurrSelUser: string;
    const AddNew: boolean = false): boolean;

implementation

uses
  frmuMessage, IBOCGlobal, zluContextHelp, zluUtility, frmuMain, IB_Header,
  LMMessageDialog;

{$IF Declared(Forms)}
{$R *.DFM}
{$IFEND}
{$IF Declared(QForms)}
{$R *.xfm}
{$IFEND}

{****************************************************************
*
*  U s e r I n f o ( )
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  CurrSelServer - The currently selected server
*          CurrSelUser   - The currently selected username if any,
*                          else ''
*
*  Return: Boolean - Indicates the success/failure of the operation
*
*  Description:  Creates an instance of TIBSecurityService, and establishes
*                a connection to the server. If it is successful it also
*                creates an instance of the User Information form
*                Once the form is closed it returns control to the
*                calling function.
*
*****************************************************************
* Revisions:
*
*****************************************************************}
function UserInfo(var CurrSelServer: TibcServerNode;
  const CurrSelUser: string;
  const AddNew: boolean = false): boolean;
var
  frmUserInfo: TfrmUserInfo;
begin
      // create an instance of the user info form
      frmUserInfo := TfrmUserInfo.Create(Application.MainForm);
      try
        frmUserInfo.FCurrSelServer := CurrSelServer;
        frmUserInfo.GetUsers;
        frmUserInfo.cbUsername.Hint := frmUserInfo.cbUsername.Text;

        // if there is a valid user
        if CurrSelUser <> '' then
        begin
          // show user along with user info
          frmUserInfo.cbUsername.ItemIndex := frmUserInfo.cbUsername.Items.IndexOf(CurrSelUser);
          frmUserInfo.cbUsernameClick(frmUserInfo);

          if UpperCase(CurrSelUser) = 'SYSDBA' then
            frmUserInfo.btnDelete.Enabled := false;
        end;
        frmUserInfo.AddNew := AddNew;
{$ifdef initlangfile} WriteAllCaptionAndText('',frmUserInfo,gTransIniFile); {$endif}
        frmUserInfo.ShowModal;
        Application.ProcessMessages;
      finally
        // deallocate memory
        frmUserInfo.Release;
      end;
      result := true;
end;

{****************************************************************
*
*  b t n A p p l y C l i c k ( )
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  Sender - The object initiating the event
*
*  Return: None
*
*  Description:  Adds/Modifies a user in the security database
*                depending on the current mode (insert/edit)
*                which is determined by the visibility of edtUsername
*
*****************************************************************
* Revisions:
*
*****************************************************************
}
procedure TfrmUserInfo.btnApplyClick(Sender: TObject);
var
  lUserCount : Integer;
  op_result:boolean;
  lCurrUser:String;
begin
  { Get the information first }
  FPassword := edtPassword.Text;
  FConfirmPassword := edtConfirmPassword.Text;
  lUserCount := -1;
  // if all fields pass validity checks
  if VerifyInputData() then
  try
    Screen.Cursor := crHourGlass;     // change cursor to hourglass
    try
      // fill in security service details regarding user
      IB_Connection1.Server := FCurrSelServer.Server.ServerName;
      IB_Connection1.Password := FCurrSelServer.Password;
      IB_Connection1.Protocol := FCurrSelServer.Server.Protocol;
      IB_Connection1.Username := FCurrSelServer.UserName;
      // if the username edit box is visible
      // then this is a new user being added to the security database
      if edtUsername.Visible then
      begin
          lCurrUser:= edtUsername.Text;
          IB_Connection1.AlterUser(uaAddUser,lCurrUser,edtPassword.Text,'',edtFName.Text,
                                 edtMName.Text,
                                 edtLName.Text);
      end
      else
      begin
          // fill in security service details regarding user
          lCurrUser:= cbUsername.Text;
          if edtPassword.Text = DUMMY_PASSWORD then
          begin
              IB_Connection1.AlterUser(uaModifyUser,cbUsername.Text,'','',edtFName.Text,
                                     edtMName.Text,
                                     edtLName.Text)
          end
          else
            IB_Connection1.AlterUser(uaModifyUser,cbUsername.Text,edtPassword.Text,'',edtFName.Text,
                                   edtMName.Text,
                                   edtLName.Text)
      end;
      op_result := true;
      frmMain.RefreshUsers(fCurrSelServer);
    except
      on E:EIB_ISCError do           // and show error message
      begin
        op_result := false;
        Screen.Cursor := crDefault;    // change cursor back to default
        if edtUsername.Visible then
          DisplayMsg(ERR_ADD_USER, E.Message)
        else
          DisplayMsg(ERR_MODIFY_USER, E.Message);
      end;
    end;
    frmMain.ErrorState:= not op_result;
    if op_result then
    frmMain.CheckUserList(FCurrSelServer);
  finally
    // change form back to the display user state
    cbUserName.Visible := true;
    edtUsername.Visible := false;
    btnNew.Enabled := true;
    btnApply.Enabled := false;
    btnDelete.Enabled := true;
    btnClose.Visible := true;
    btnCancel.Visible := false;
    lblUserName.FocusControl := cbUserName;
    cbUserName.SetFocus;
    GetUsers(lUserCount);
    GetUserInfo(lCurrUser);
    Screen.Cursor := crDefault;    // change cursor back to default
  end;
end;

procedure TfrmUserInfo.btnCancelClick(Sender: TObject);
var lCurrUser:String;
begin
  // change form back to the display user state
  cbUserName.Visible := true;
  edtUsername.Visible := false;
  btnNew.Enabled := true;
  btnApply.Enabled := false;
  btnDelete.Enabled := true;
  btnClose.Visible := true;
  btnCancel.Visible := false;
  lblUserName.FocusControl := cbUserName;
  cbUserName.SetFocus;
  lCurrUser:= cbUsername.Text;
  GetUsers;
  GetUserInfo(cbUsername.Text);
end;

procedure TfrmUserInfo.btnCloseClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

{****************************************************************
*
*  b t n D e l e t e C l i c k ( )
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  Sender - The object initiating the event
*
*  Return: None
*
*  Description:  Deletes a user from the security database
*                after confirmation from the user
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmUserInfo.btnDeleteClick(Sender: TObject);
var
  lConfirmed : Boolean;
begin
  lConfirmed := False;

  if not btnDelete.Enabled then
    exit;
  // show confirmation dialog
  if not SameText(cbUsername.Text,FCurrSelServer.UserName) then
  begin
    if LMMessageDlg(Format(GetPrompt(71),[cbUsername.Text]),mtConfirmation, mbYesNoCancel, 0) = mrYes then
      lConfirmed := True;
  end
  else
  begin
    if LMMessageDlg(Format(GetPrompt(83),[cbUsername.Text]),mtConfirmation, mbYesNoCancel, 0) = mrYes then
      lConfirmed := True;
  end;

  if lConfirmed then
  try
    Screen.Cursor := crHourGlass;    // change cursor to hourglass
    try
      // fill in security service details regarding user
      IB_Connection1.Server := FCurrSelServer.Server.ServerName;
      IB_Connection1.Protocol := FCurrSelServer.Server.Protocol;
      IB_Connection1.Username := FCurrSelServer.UserName;
      IB_Connection1.Password := FCurrSelServer.Password;
      IB_Connection1.AlterUser(uaDeleteUser,cbUsername.Text,edtPassword.Text,'',edtFName.Text,
                               edtMName.Text,
                               edtLName.Text);
      GetUsers;
      frmMain.RefreshUsers(fCurrSelServer);
      frmMain.tvMainChange(nil, nil);
    except                       // if an exception occurs then trap it
      on E:EIB_ISCError do           // and show error message
      begin
        Screen.Cursor := crDefault;    // change cursor back to default
        DisplayMsg(ERR_DELETE_USER, E.Message);
        frmMain.ErrorState:= True;
      end;
    end;
  finally
    GetUserInfo(cbUsername.Text);
    Screen.Cursor := crDefault;    // change hour glass to default
  end;
end;

{****************************************************************
*
*  b t n N e w C l i c k ( )
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  Sender - The object initiating the event
*
*  Return: None
*
*  Description:  Initializes the forms controls in order to receive
*                new user information
*
*****************************************************************
* Revisions:
*
*****************************************************************}
procedure TfrmUserInfo.btnNewClick(Sender: TObject);
begin
  // change form to the add user state
  cbUsername.Visible := false;
  edtUsername.Visible := true;
  edtUserName.Text := '';
  edtPassword.Text := '';
  edtConfirmPassword.Text := '';
  edtFName.Text := '';
  edtMName.Text := '';
  edtLName.Text := '';
  cbUsername.Enabled := true;
  cbUserName.Color := clWindow;
  edtPassword.Enabled := true;
  edtPassword.Color := clWindow;
  edtConfirmPassword.Enabled := true;
  edtConfirmPassword.Color := clWindow;
  edtFName.Enabled := true;
  edtFName.Color := clWindow;
  edtMName.Enabled := true;
  edtMName.Color := clWindow;
  edtLName.Enabled := true;
  edtLName.Color := clWindow;
  btnNew.Enabled := false;
  btnApply.Enabled := false;
  btnDelete.Enabled := false;
  btnCancel.Visible := true;
  btnClose.Visible := false;
  lblUserName.FocusControl := edtUserName;
  edtUserName.SetFocus;
end;

procedure TfrmUserInfo.cbUsernameChange(Sender: TObject);
begin
  if UpperCase (cbUserName.Text) = 'SYSDBA' then
    btnDelete.Enabled := false
  else
    btnDelete.Enabled := true;
  cbUsername.Hint := cbUsername.Text;  // assign current name in combo to hint
end;

procedure TfrmUserInfo.cbUsernameClick(Sender: TObject);
begin
  if GetUserInfo(cbUsername.Text) then // get user info
  begin                                // if info was returned
    btnNew.Enabled := true;            // change form to display user state
    btnApply.Enabled := false;
    btnDelete.Enabled := true;
    btnCancel.Visible := false;
    btnClose.Visible := true;
  end
  else                                 // otherwise change form to display
  begin                                // user state and
    btnNew.Enabled := true;
    btnApply.Enabled := false;
    btnDelete.Enabled := false;        // disable the delete buttom
    btnCancel.Visible := false;
    btnClose.Visible := true;
  end;
end;

procedure TfrmUserInfo.edtConfirmPasswordChange(Sender: TObject);
begin
  if edtConfirmPassword.Text = DUMMY_PASSWORD then
    exit;                              // set confirm password
  FConfirmPassword := edtConfirmPassword.Text;
  if (edtUsername.Text <> '') and (edtPassword.Text <> '') and
    (edtConfirmPassword.Text <> '') then
    btnApply.Enabled := true;          // enable the password if username changes
end;

procedure TfrmUserInfo.edtFNameChange(Sender: TObject);
begin
  btnApply.Enabled := true;            // enable the apply button if first name changes
end;

procedure TfrmUserInfo.edtLNameChange(Sender: TObject);
begin
  btnApply.Enabled := true;            // enable the apply button if last name changes
end;

procedure TfrmUserInfo.edtMNameChange(Sender: TObject);
begin
  btnApply.Enabled := true;            // enable the apply button if middle name changes
end;

procedure TfrmUserInfo.edtPasswordChange(Sender: TObject);
begin
  if Trim(edtPassword.Text) <> '' then
    btnApply.Enabled := true;
end;

procedure TfrmUserInfo.edtUsernameChange(Sender: TObject);
begin
  if Trim(edtUserName.Text) <> '' then
    btnApply.Enabled := true;
end;

{****************************************************************
*
*  G e t U s e r I n f o ( )
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  None
*
*  Return: Boolean - Indicates the success/failure of the operation
*
*  Description:  Retrieves account information for a single user
*
*****************************************************************
* Revisions:
*
*****************************************************************}
function TfrmUserInfo.GetUserInfo(UserName:String): boolean;
var
  lUserInfo: TUserInfo;
begin
  Result := True;
  try
    Screen.Cursor := crHourGlass;
    lUserInfo := FCurrSelServer.UserInfoByName[UserName];
    if Assigned(lUserInfo) then
    begin
      if cbUsername.Text <> UserName then
        cbUserName.ItemIndex := cbUserName.Items.IndexOf(UserName);
      edtPassword.Text := DUMMY_PASSWORD;
      FPassword := '';
      edtConfirmPassword.text := DUMMY_PASSWORD;
      FConfirmPassword := '';
      edtFName.Text := lUserInfo.FirstName;
      edtMName.Text := lUserInfo.MiddleName;
      edtLName.Text := lUserInfo.LastName;
      if UserName = 'SYSDBA' then
        btnDelete.Enabled := False;
      btnApply.Enabled := False;  
    end
    else
      Result := False;
  finally
    Screen.Cursor := crDefault;      // change cursor to default
  end;
end;

{****************************************************************
*
*  G e t U s e r L i s t ( )
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  None
*
*  Return: Boolean - Indicates the success/failure of the operation
*
*  Description:  Retrieves a list of existing users in the security
*                database and enables/disables the appropriate
*                controls based on the success/failure of the operation
*
*****************************************************************
* Revisions:
*
*****************************************************************}
{****************************************************************
*
*  V e r i f y I n p u t D a t a ( )
*
****************************************************************
*  Author: The Client Server Factory Inc.
*  Date:   March 1, 1999
*
*  Input:  None
*
*  Return: Boolean - Indicates the success/failure of the operation
*
*  Description:  Performs some basic validation on data entered by
*                the user
*
*****************************************************************
* Revisions:
*
*****************************************************************}
function TfrmUserInfo.VerifyInputData(): boolean;
var
  i: integer;
begin
  result := true;

  // if the form is in the add user state
  if edtUsername.Visible then
  begin
    // check if any data was entered in the username edit box
    if (Trim(edtUsername.Text) = '') then
    begin                              // if not then show
      DisplayMsg(ERR_USERNAME,'');     // error message
      edtUsername.SetFocus;            // and give the control focus
      result := false;
      btnApply.Enabled := false;      
      Exit;
    end;

    // check if a password was supplied or if it's been changed from the dummy password
    if (Trim(FPassword) = '') or (FPassword = DUMMY_PASSWORD) then
    begin                              // if not then show
      DisplayMsg(ERR_PASSWORD,'');     // error message
      edtPassword.SetFocus;            // and give the control focus
      result := false;
      btnApply.Enabled := false;
      Exit;
    end;
  end;

  // check if the password and confirmation password are the same
  i := CompareStr(FPassword,FConfirmPassword);
  if i <> 0 then
  begin                                // if not then show error message
    DisplayMsg(ERR_PASSWORD_MISSMATCH,'');
    edtPassword.Text := DUMMY_PASSWORD;
    edtConfirmPassword.Text := DUMMY_PASSWORD;
    edtPassword.SetFocus;       // and give the control focus
    btnApply.Enabled := false;
    result := false;
  end;
end;

procedure TfrmUserInfo.GetUsers(const idx: integer = -1);
begin
  try
    FCurrSelServer.GetUserNameList(cbUsername.Items);

    if cbUsername.Items.Count>16 then
      cbUsername.DropDownCount := 16
    else
      cbUsername.DropDownCount := cbUsername.Items.Count;

    if cbUsername.Items.Count > 0 then // if there are users then
    begin                              // change form to display user state
      btnDelete.Enabled := true;
      cbUsername.Enabled := true;
      cbUserName.Color := clWindow;
      edtPassword.Enabled := true;
      edtPassword.Color := clWindow;
      edtConfirmPassword.Enabled := true;
      edtConfirmPassword.Color := clWindow;
      edtFName.Enabled := true;
      edtFName.Color := clWindow;
      edtMName.Enabled := true;
      edtMName.Color := clWindow;
      edtLName.Enabled := true;
      edtLName.Color := clWindow;
      if idx >= 0 then
        cbUsername.ItemIndex := idx
      else
        cbUsername.ItemIndex := 0;
//      cbUsernameClick(Self);
    end
    else                               // otherwise change form to display
    begin                              // user state and disable the
      btnDelete.Enabled := false;      // delete button
      cbUsername.Enabled := false;     // also grey out user info fields
      cbUserName.Color := clSilver;
      edtPassword.Enabled := false;
      edtPassword.Color := clSilver;
      edtConfirmPassword.Enabled := false;
      edtConfirmPassword.Color := clSilver;
      edtFName.Enabled := false;
      edtFName.Color := clSilver;
      edtMName.Enabled := false;
      edtMName.Color := clSilver;
      edtLName.Enabled := false;
      edtLName.Color := clSilver;
//      cbUsernameClick(Self);
    end;
  finally

  end;
end;

procedure TfrmUserInfo.FormCreate(Sender: TObject);
begin
  inherited;
  HelpContext := SERVER_SECURITY;
end;


procedure TfrmUserInfo.DeleteUserUpdate(Sender: TObject);
begin
  inherited;
  (Sender as TAction).Enabled := (UpperCase(CbUserName.Text) = 'SYSDBA');
end;

procedure TfrmUserInfo.FormShow(Sender: TObject);
begin
  inherited;
  if AddNew then
    btnNew.Click;
  cbUserNameChange(Sender);
end;

end.
