{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 *
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 *
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 *
 * Contributor(s): Krzysztof Golko.
}

unit frmuWindowList;

interface

uses
{$TYPEINFO ON}
{$IFDEF LINUX}
  QControls, QForms, QDialogs, QStdCtrls, QActnList,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages, Controls, Forms, Dialogs, StdCtrls, ActnList, ComCtrls,
{$ENDIF}
  Classes, SysUtils, dmuNode, TypInfo;

type
  TFormArray=array of TForm;

  TIBOCWindowList = class(TComponent)
  private
    FList: TStringList;
    fDataModule:TDmN;
    function GetCount: integer;
    function GetForms(Index: integer): TForm;
  protected
  public
    constructor Create(aOwner:TComponent); override;
    destructor Destroy; override;
    procedure UpdateWindowList(const Caption: String; const Window: TObject; const Remove: boolean = false);
    property Count: integer read GetCount;
    property Forms[Index: integer]: TForm read GetForms; default;
    procedure Add(const Caption: string; const Window: TForm);
    procedure Remove(const Window: TForm);
    procedure UpdateAll(const DataModule: TDmN = Nil);
    procedure RefreshObject(const aDataModule: TDmN; const aObjType: integer; const aObjName: String);
    procedure Show(const DataModule:TDmN);overload;
    function FindWindow(const DataModule:TDmN; ObjectType:integer; ObjectName:String):TForm;
    function FindISQLWindow(const DataModule:TDmN):TForm;
    procedure CloseObjectWindow(const DataModule:TDmN; ObjectType:integer; ObjectName:String);
    function CanClose:boolean;
    function ImageIndex(Index: integer):integer;
    function CanCloseAll(const DataModule:TDmN=Nil): boolean; overload;
    function CanCloseAll(var aForms:TFormArray): boolean; overload;
    function CloseAll(DataModule:TDmN=Nil):boolean; overload;
    procedure GetFormList(var aForms:TFormArray; DataModule:TDmN=Nil);
    function HasForms(DataModule:TDmN):boolean; overload;
  end;

  TdlgWindowList = class(TForm)
    btnSwitch: TButton;
    btnCloseWindow: TButton;
    actWindows: TActionList;
    SwitchTo: TAction;
    Closewindow: TAction;
    ListView1: TListView;
    ActionCloseAll: TAction;
    Button1: TButton;
    procedure SwitchClick(Sender: TObject);
    procedure btnCloseWindowClick(Sender: TObject);
    procedure lbWindowsClick(Sender: TObject);
    procedure SwitchToUpdate(Sender: TObject);
    procedure ListView1Click(Sender: TObject);
    procedure ListView1GetImageIndex(Sender: TObject; Item: TListItem);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ActionCloseAllUpdate(Sender: TObject);
    procedure ActionCloseAllExecute(Sender: TObject);
    procedure ListView1KeyPress(Sender: TObject; var Key: Char);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FWindowList: TIBOCWindowList;
    fDataModule:TDmN;
    procedure GenerateList(AWindowList: TIBOCWindowList; DataModule: TDmN=Nil);
  public
    { Public declarations }
    //constructor Create(aOwner: TComponent); override;
    constructor CreateList(AOwner: TComponent; AWindowList: TIBOCWindowList; DataModule:TDmN=Nil);
  end;

var
  OpenWindows: TIBOCWindowList;

implementation

uses IBOCGlobal, IBOConsoleForm, frmuEditorUDF,
  frmuEditorFilter, frmuMain, dmuMain, wisql,
  frmuEditorRole, frmuEditorTable;

{$IF Declared(Forms)}
{$R *.DFM}
{$IFEND}
{$IF Declared(QForms)}
{$R *.xfm}
{$IFEND}

function GetImageIndex(form:TObject):integer;

begin
  result := -1;
  if Assigned(Form) then
  try
    if (Form is TIBOConsoleForm)     then Result := TIBOConsoleForm(Form).ImageIndex
    else
    if (Form is TfrmUDFEditor)       then Result := NODE_FUNCTIONS_IMG
    else
    if (Form is TfrmEditorTable)     then Result := NODE_TABLES_IMG
    else
    if (Form is TFrmEditorFilter)    then Result := NODE_blob_FILTERS_IMG
  except
  end;
end;

function GetDataModule(f:TComponent): TDmN;
begin
  result := nil;
  begin
    if (F is TIBOConsoleDatabaseForm)  then Result := TIBOConsoleDatabaseForm(F).DataModule
    else
    if (f is TfrmUDFEditor)       then Result := TfrmUDFEditor(f).DataModule
    else
    if (f is TfrmEditorTable)     then Result := TfrmEditorTable(f).DataModule
    else
    if (f is TFrmEditorFilter)    then Result := TFrmEditorFilter(f).DataModule
  end;
end;

function GetDbPath(form:TComponent):String;
var x:TDmN;
begin
  result := '';
  x:= GetDataModule(Form);
  if Assigned(x) then
  try
    Result := X.Connection.DatabaseName;
  except
  end;
end;

procedure TdlgWindowList.GenerateList(AWindowList: TIBOCWindowList; DataModule:TDmN=Nil);
var
  i: integer;
  db:String;
  x:TDmN;
begin
  FWindowList := AWindowList;
  fDataModule := DataModule;
  ListView1.Clear;
  if Assigned(fDataModule) then
  begin
    for i := 0 to FWindowList.Flist.Count - 1 do
    begin
      x:= GetDataModule(TComponent(FWindowList.FList.Objects[i]));
      if x = fDataModule then
        ListView1.AddItem(FWindowList.Flist.Strings[i],FWindowList.FList.Objects[i])
    end;
    Caption := GetLangText('Active Windows on')+' '+ fDataModule.Connection.DatabaseName;
  end
  else
    for i := 0 to FWindowList.Flist.Count - 1 do
    begin
      db := GetDbPath(TComponent(FWindowList.FList.Objects[i]));
      if db <> '' then
        ListView1.AddItem(FWindowList.Flist.Strings[i]+' - '+db,FWindowList.FList.Objects[i])
      else
        ListView1.AddItem(FWindowList.Flist.Strings[i],FWindowList.FList.Objects[i]);
    end;
end;


constructor TdlgWindowList.CreateList(AOwner: TComponent; AWindowList: TIBOCWindowList; DataModule:TDmN=Nil);
begin
  inherited Create(AOwner);
  SetAllFontsNameCharset(self,gAppSettings[FONT_NAME],gAppSettings[FONT_CHARSET]);
  GenerateList(AWindowList,DataModule);
end;

procedure TdlgWindowList.SwitchClick(Sender: TObject);
var
  frm: TForm;
begin
  inherited;
  if ListView1.ItemIndex <> -1 then
  try
    frm := TForm(ListView1.Items[ListView1.ItemIndex].Data);
    frm.Show;
  except
    ListView1.Items.Delete(ListView1.ItemIndex);
  end;
  Close;
end;

procedure TdlgWindowList.btnCloseWindowClick(Sender: TObject);
var
  frm: TForm;
begin
  inherited;
  if ListView1.ItemIndex <> -1 then
  begin
    frm := TForm(ListView1.Items[ListView1.ItemIndex].Data);
    frm.Close;
    ListView1.Items.Delete(ListView1.ItemIndex);
  end;
end;

procedure TdlgWindowList.lbWindowsClick(Sender: TObject);
begin
  inherited;
  btnSwitch.Enabled := true;
  btnCloseWindow.Enabled := true;
end;

procedure TdlgWindowList.SwitchToUpdate(Sender: TObject);
begin
  TAction(Sender).Enabled := (ListView1.ItemIndex <> -1);
end;

{ TIBOCWindowList }
constructor TIBOCWindowList.Create;
begin
  FList := TStringList.Create;
end;

destructor TIBOCWindowList.Destroy;
begin
  FList.Free;
end;

function TIBOCWindowList.GetCount: integer;
begin
  Result := FList.Count;
end;

function TIBOCWindowList.GetForms(Index: integer): TForm;
begin
  Result := FList.Objects[Index] as TForm;
end;

procedure TIBOCWindowList.UpdateWindowList(const Caption: String;
  const Window: TObject; const Remove: boolean = false);
var
  idx: integer;
begin
  idx := FList.IndexOfObject(Window);
  if idx <> -1 then FList.Delete(idx);
  if not Remove then FList.AddObject(Caption, Window);
end;

procedure TIBOCWindowList.Add(const Caption: string; const Window: TForm);
begin
  if FList.IndexOfObject(Window) = -1 then
    FList.AddObject(Caption, Window);
end;

procedure TIBOCWindowList.Remove(const Window: TForm);
var
  idx: integer;
begin
  idx := FList.IndexOfObject(Window);
  if idx <> -1 then
    FList.Delete(idx);
end;

procedure TIBOCWindowList.UpdateAll(const DataModule: TDmN);
var
  i: integer;
  f: TForm;
  fdb: TIBOConsoleDatabaseForm;
begin
  if Assigned(DataModule) then
    for i :=  FList.Count - 1 downto 0 do
    try
      if FList.Objects[i] is TIBOConsoleDatabaseForm then
      begin
        fdb := TIBOConsoleDatabaseForm(FList.Objects[i]);
        if fdb.DataModule = DataModule then
          fdb.Update;
      end;
    except
    end
  else
    for i :=  FList.Count - 1 downto 0 do
    try
      f := TForm(FList.Objects[i]);
      f.Update;
    except
    end;
end;

function TIBOCWindowList.FindWindow(const DataModule: TDmN; ObjectType: integer;
  ObjectName: String): TForm;
var n:integer;
begin
  result := nil;
  for n := 0 to FList.Count - 1 do
  begin
    if (FList.Objects[n] is TIBOConsoleEditor)
       and (TIBOConsoleEditor(FList.Objects[n]).DataModule = DataModule)
       and (TIBOConsoleEditor(FList.Objects[n]).ObjectName = ObjectName)
       and NodeMatch(TIBOConsoleEditor(FList.Objects[n]).ObjectType,ObjectType)
    then
      Result := TForm(FList.Objects[n])
    else
    if
       (ObjectType = NODE_FUNCTIONS) and
       (FList.Objects[n] is TfrmUDFEditor) and
       (DataModule = TfrmUDFEditor(FList.Objects[n] ).DataModule) and
       (ObjectName = TfrmUDFEditor(FList.Objects[n] ).ObjectName)
    then
      Result := TForm(FList.Objects[n])
    else
    if
       (ObjectType = NODE_TABLES) and
       (FList.Objects[n] is TfrmEditorTable) and
       (DataModule = TfrmEditorTable(FList.Objects[n] ).DataModule) and
       (ObjectName = TfrmEditorTable(FList.Objects[n] ).ObjectName)
    then
      Result := TForm(FList.Objects[n])
    else
    if
       (ObjectType = NODE_blob_FILTERS) and
       (FList.Objects[n] is TFrmEditorFilter) and
       (DataModule = TFrmEditorFilter(FList.Objects[n] ).DataModule) and
       (ObjectName = TFrmEditorFilter(FList.Objects[n] ).ObjectName)
    then
      Result := TForm(FList.Objects[n])
  end;
end;


function TIBOCWindowList.CanClose: boolean;
var n:integer;
    f:TForm;
begin
  result := True;
  for n := 0 to FList.Count - 1 do
  begin
    f := TForm(FList.Objects[n]);
    if f <> frmMain then
    if Assigned(f.OnCloseQuery) then
    begin
      f.OnCloseQuery(Self,Result);
      if not result then
        exit;
    end;
  end;
end;

procedure TdlgWindowList.ListView1Click(Sender: TObject);
begin
  inherited;
  btnSwitch.Enabled := true;
  btnCloseWindow.Enabled := true;
end;

procedure TdlgWindowList.ListView1GetImageIndex(Sender: TObject;
  Item: TListItem);
begin
  inherited;
  Item.ImageIndex := GetImageIndex(Item.Data);
end;

function TIBOCWindowList.ImageIndex(Index: integer): integer;
begin
  Result := GetImageIndex(FList.Objects[Index] as TForm);
end;

function TIBOCWindowList.CanCloseAll(const DataModule: TDmN): boolean;
var
  aForms:TFormArray;
begin
  GetFormList(aForms,DataModule);
  result := CanCloseAll(aForms);
end;

function TIBOCWindowList.CanCloseAll(var aForms: TFormArray): boolean;
var n:integer;
begin
  result := True;
  for n := Low(aForms) to High(aForms) do
  begin
    if Assigned(aForms[n].OnCloseQuery) then
    begin
      aForms[n].OnCloseQuery(Self,Result);
      if not result then exit;
    end;
  end;
end;

function TIBOCWindowList.CloseAll(DataModule: TDmN):boolean;
var n:integer;
    aForms:TFormArray;
begin
  GetFormList(aForms,DataModule);
  Result := CanCloseAll(aForms);
  if Result then
  begin
    for n := Low(aForms) to High(aForms) do
    try
      aForms[n].Close;
    except
    end;
    SetLength(aForms,0);
  end;
end;

procedure TIBOCWindowList.Show(const DataModule: TDmN);
var
  dlgWindowList: TdlgWindowList;
begin
  fDataModule := DataModule;
  dlgWindowList := TdlgWindowList.CreateList(Application, Self, fDataModule);
{$ifdef initlangfile} WriteAllCaptionAndText('',dlgWindowList,gTransIniFile); {$endif}
  dlgWindowList.ShowModal;
  dlgWindowList.Release;
end;

procedure TdlgWindowList.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then ModalResult := mrCancel;
end;

procedure TdlgWindowList.ActionCloseAllUpdate(Sender: TObject);
begin
  TAction(Sender).Enabled := ((ListView1.Items.Count > 1) and (fDataModule=Nil)) or
                             ((fDataModule<>Nil) and (ListView1.Items.Count > 0));
end;

procedure TdlgWindowList.ActionCloseAllExecute(Sender: TObject);
begin
  OpenWindows.CloseAll;
  GenerateList(FWindowList,fDataModule);
end;

procedure TIBOCWindowList.GetFormList(var aForms: TFormArray;
  DataModule: TDmN);
var n,i:integer;
    f:TForm;
begin
  i:=0;
  SetLength(aForms,FList.Count-1);
  for n := 0 to FList.Count - 1 do
  begin
    f := FList.Objects[n] as TForm;
    if (f <> frmMain)
       and ( (Datamodule = Nil) or (GetDataModule(f)=DataModule) )
    then
    begin
      aForms[i]:= f;
      inc(i);
    end;
  end;
  SetLength(aForms,i);
end;

function TIBOCWindowList.HasForms(DataModule: TDmN): boolean;
var n:integer;
    f:TForm;
begin
  Result := False;
  for n := 0 to FList.Count - 1 do
  begin
    f := TForm(FList.Objects[n]);
    if (f <> frmMain) and (GetDataModule(f)=DataModule) then
    begin
      Result := True;
      Break;
    end;
  end;
end;

procedure TdlgWindowList.ListView1KeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #27 then ModalResult := mrCancel;
end;

procedure TIBOCWindowList.RefreshObject(const aDataModule: TDmN;
  const aObjType: integer; const aObjName: String);
var n:integer;
begin
  for n := 0 to FList.Count - 1 do
  begin
    if FList.Objects[n] is TIBOConsoleObjectForm then
    begin
      TIBOConsoleObjectForm(FList.Objects[n]).SignalObjectRefresh(aDataModule,aObjType,aObjName);
    end
    else
    if FList.Objects[n] is TfrmEditorTable then
    begin
      TfrmEditorTable(FList.Objects[n]).SignalObjectRefresh(aDataModule,aObjType,aObjName);
    end;
  end;

end;

procedure TIBOCWindowList.CloseObjectWindow(const DataModule: TDmN;
  ObjectType: integer; ObjectName: String);
var f:TForm;
begin
  f := FindWindow(DataModule,ObjectType,ObjectName);
  if Assigned(f) then
  begin
    f.close;
  end;
end;

procedure TdlgWindowList.FormShow(Sender: TObject);
begin
  ListView1.SetFocus;
end;

//constructor TdlgWindowList.Create(aOwner: TComponent);
//begin
//  inherited;
//  SetAllFontsNameCharset(self,gAppSettings[FONT_NAME],gAppSettings[FONT_CHARSET]);
//end;

procedure TdlgWindowList.FormCreate(Sender: TObject);
begin
{$ifdef initlangfile} WriteAllCaptionAndText('',Self,gTransIniFile); {$endif}
  if Assigned(gGUILanguageFile) then
    GetAllCaptionAndText('',self,gGUILanguageFile);
end;

function TIBOCWindowList.FindISQLWindow(const DataModule: TDmN): TForm;
var n:integer;
begin
  result := nil;
  for n := 0 to FList.Count - 1 do
  begin
    if (FList.Objects[n] is TdlgWisql)
       and (TdlgWisql(FList.Objects[n]).DataModule = DataModule)
    then
    begin
      Result := TForm(FList.Objects[n]);
      break;
    end;
  end;
end;

end.
