{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 * 
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 * 
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 * 
 * Contributor(s): ______________________________________.
}

unit zluCommDiag;

interface

{$IFDEF MSWINDOWS}
uses
  Windows, SysUtils, Classes;

type
  HANDLE     = Cardinal;               // handle

  // Novell Netware function types
  TNWCallsInit        = function(Reserved1, Reserved2 : Pointer) : Cardinal;  StdCall;
  TNWCLXInit          = function(Reserved1, Reserved2 : Pointer) : Cardinal;  StdCall;
  TNWCCOpenConnByName = function(StartConnHandle : Cardinal; Name : PChar; NameFormat, OpenState, TranType : Cardinal; var pConnHandle : Cardinal) : Cardinal;  StdCall;
  TNWCCCloseConn      = function(ConnHandle : Cardinal) : Cardinal;  StdCall;

  {****************************************************************
  *
  *  T i b c S P X
  *
  ****************************************************************
  *  Author: The Client Server Factory Inc.
  *  Date:   March 1, 1999
  *
  *  Description:  This class implements the interface to
  *                test an unlicensed connection to a Netware
  *                server using IPX/SPX.  The client machine
  *                must have the proper Netware client installed.
  *
  *****************************************************************
  * Revisions:
  *
  *****************************************************************}
  TibcSPX = class
  private
    { private declarations }
    FServer : String;
    function GetServer : String;
    procedure SetServer(Server : String);
  public
    { public declarations }
    procedure TestSPX(var sResult : String);
  published
    { published declarations }
    property ServerName : String read GetServer write SetServer;

  end;

  {****************************************************************
  *
  *  T i b c P i p e s
  *
  ****************************************************************
  *  Author: The Client Server Factory Inc.
  *  Date:   March 1, 1999
  *
  *  Description:  This class implements the interface to
  *                test a connection to an NT server using a named
  *                pipe.
  *
  *****************************************************************
  * Revisions:
  *
  *****************************************************************}
  TibcPipes = class
  private
    FDefault : String;  
    FPath    : String;
    FPipe    : String;
    FSA      : SECURITY_ATTRIBUTES;
    FServer  : String;
    FTotal   : Integer;
    hPipe    : HANDLE;
    function GetPath : String;
    function GetServer : String;
    function GetTries : Integer;
    procedure SetPath(Path : String);
    procedure SetServer(Server : String);
    procedure SetTries(Tries : Integer);
  public
    function TestPipe(var sResult : String; SilentTest : Boolean) : Boolean;
  published
    property Path   : String read GetPath write SetPath;
    property Server : String read GetServer write SetServer;
    property Tries  : Integer read GetTries write SetTries;
  end;

const
  CALWIN32      = 'calwin32.dll';
  CLXWIN32      = 'clxwin32.dll';
	NSPROTO_IPX   = 1000;
	NSPROTO_SPX   = 1256;
	NSPROTO_SPXII = 1257;
  SOCK_DIAG_SPX = $0456;
  NWCC_NAME_FORMAT_BIND = $0002;
  NWCC_OPEN_LICENSED    = $0001;
  NWCC_TRAN_TYPE_WILD  = $00008000;

{$ENDIF}
implementation

uses IBOCGlobal;
{$IFDEF MSWINDOWS}
// accessor to get server name
function TibcPipes.GetServer : String;
begin
  Result := FServer;
end;

// accessor to get path name
function TibcPipes.GetPath : String;
begin
  Result := FPath;
end;

// accessor to get max number of attempts
function TibcPipes.GetTries : Integer;
begin
  Result := FTotal;
end;

// accessor to set the server name
procedure TibcPipes.SetServer(Server : String);
begin
  FServer := Server;
end;

// accessor to set path
procedure TibcPipes.SetPath(Path : String);
begin
  FPath := Path;
end;

// accessor to set max number of attempts
procedure TibcPipes.SetTries(Tries : Integer);
begin
  FTotal := Tries;
end;

function TibcPipes.TestPipe(var sResult : String; SilentTest : Boolean) : Boolean;
var
  pPipe    : PChar;                    // pipe name
  iSuccess : Boolean;                  // success flag
  iNum     : Integer;                  // number of tries so far

begin
  iSuccess := False;                 // initialize variables
  iNum := 0;
  try

    if Server = '' then
    begin                               // if no server name is supplied
      result := false;
      Exit;                            // then exit function
    end;

    // set pipe name
    FPipe := Format('\\%s%s', [FServer, FPath]);

    // set security attributes
    FDefault:='CSDiagDefault';
    FSA.nLength := SizeOf(FSA);
    FSA.lpSecurityDescriptor := Nil;
    FSA.bInheritHandle := FALSE;

    pPipe:=PChar(FPipe);

    if not SilentTest then
      sResult:=Format(getprompt(75), [Server, CRLF, CRLF]);
    // poll until a connection is established or the max number of tries have exceeded
    while (not iSuccess) and (iNum < FTotal) do
    begin
      hPipe := CreateFile(pPipe, GENERIC_READ or GENERIC_WRITE,
			  FILE_SHARE_READ or FILE_SHARE_WRITE, @FSA,
			  OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

      if hPipe <> INVALID_HANDLE_VALUE then
      begin
        // if a connection is established then set success flag
        iSuccess:=True;
      end
      else
        // if no connection yet then increment the number of tries so far
        Inc(iNum);
    end;
    // build result message in buffer
    if not SilentTest then
    begin
      if hPipe <> INVALID_HANDLE_VALUE then
      begin
        sResult:= sResult + Format(GetPrompt(76), [Server, CRLF, CRLF, FPipe, CRLF, CRLF]);
      end
      else
      begin
        sResult:=sResult+Format(getprompt(77), [Server, CRLF,CRLF,FPipe, CRLF, CRLF]);
      end;
    end;
  finally
    if iSuccess then
      CloseHandle(hPipe);

    Result := iSuccess;
  end;
end;

procedure TibcSPX.TestSPX(var sResult : String);
var
  ccode         : Cardinal;            // return code
  ConnHandle    : Cardinal;            // connection handle
  lServer       : array [0..7] of char;// servername
  hInstCalWin   : THandle;
  hInstCLXWin   : THandle;
  lError        : Boolean;
  NWCallsInit   : TNWCallsInit;
  NWCLXInit     : TNWCLXInit;
  NWCCCloseConn : TNWCCCloseConn;
  NWCCOpenConnByName : TNWCCOpenConnByName;
begin
  lError := False;
  ccode := 0;
  hInstCalWin := 0;
//  hInstCLXWin := 0;
  try
    // load libraries
    hInstCalWin := LoadLibrary(CALWIN32);
    hInstCLXWin := LoadLibrary(CLXWIN32);

    // determine if libraries where successfully loaded
    if (hInstCalWin > 0) and (hInstCLXWIN > 0) then
    begin
      // get function pointer
      @NWCallsInit := GetProcAddress(hInstCalWin, PChar('NWCallsInit'));

      // if a valid function pointer is returned then call function
      if @NWCallsInit <> Nil then
        ccode := NWCallsInit(Nil, Nil)   // initialize
      else
        lError := True;                  // if no valid pointer then set error flag

      // if function call successful and no error
      if (ccode = 0) and (not lError) then
      begin                              // check return code
        // get function pointer
        @NWCLXInit := GetProcAddress(hInstCLXWin, PChar('NWCLXInit'));

        // if a valid function pointer is returned then call function
        if @NWCLXInit <> Nil then
          ccode := NWCLXInit(Nil, Nil)   // initialize
        else
          lError := True;                // if no valid pointer then set error flag

      end;

      // if either of the calls failed
      if (ccode <> 0) or (lError) then
      begin                              // show error message
        sResult:=Format(GetPrompt(78), [CRLF, CRLF]);
      end
      else
      begin
        // get function pointer
        @NWCCOpenConnByName := GetProcAddress(hInstCLXWin, PChar('NWCCOpenConnByName'));

        StrCopy(lServer, PChar(FServer));

        sResult:=Format(getprompt(79), [FServer, CRLF, CRLF]);

        // if a valid function pointer is returned then call function
        if @NWCCOpenConnByName <> Nil then
        begin
          // attempt to connect to the Netware server
          ccode := NWCCOpenConnByName(0, @lServer, NWCC_NAME_FORMAT_BIND,
            NWCC_OPEN_LICENSED, NWCC_TRAN_TYPE_WILD, ConnHandle);
        end
        else
          lError := True;              // if no valid pointer then set error flag

        // check return code
        if (ccode <> 0) or (lError) then
        begin                            // show error message if return code not 0
          sResult:=sResult+Format(getprompt(80), [sResult, FServer, CRLF, CRLF, CRLF]);
        end
        else
        begin                            // show success message if return node is 0
          sResult:=sResult+Format(GetPrompt(81), [sResult, FServer, CRLF, CRLF, CRLF]);
        end;
      end;
    end
    else
    begin
      // show error message is one of the dlls failed to load
      sResult:=sResult+Format(getprompt(82), [FServer, CRLF, CRLF, CRLF, CRLF, CRLF]);
    end;
  finally
    // close connection if active
    hInstCLXWin := LoadLibrary(CLXWIN32);
    if hInstCLXWin > 0 then
    begin
      @NWCCCloseConn := GetProcAddress(hInstCLXWin, PChar('NWCCCloseConn'));
      if (@NWCCCloseConn <> Nil) and (ccode = 0) then
      begin                              // if a connection is open
        NWCCCloseConn(ConnHandle);       // then close the connection
      end;
    end;

    // free libraries
    FreeLibrary(hInstCalWin);
    FreeLibrary(hInstCLXWin);
  end;
end;

function TibcSPX.GetServer : String;
begin
  Result := FServer;
end;

procedure TibcSPX.SetServer(Server : String);
begin
  FServer:=Server;
end;

{$ENDIF}
end.

