{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.
                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 *
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 *
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 *
 * Contributor(s): Jeff Overcash, Krzysztof Golko.

23.August.2002, Lorenzo:
- Modified TIBOSqlObj.GetEvent for handling the name of the object involved

15.January.2005 , Lorenzo:
- fixed create database behaviour
  putted database name and params in the right order in _create_database_schema


}

unit zluSQL;

interface

uses
{$IFDEF LINUX}
  QForms, QStdCtrls,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Dialogs, Forms, Messages, Controls,
{$ENDIF}
  IB_Components, IBODataset, IBODatabaseInfo, IBOConnectionInfo, Classes, SysUtils, dmuNode,
  IB_Schema, IB_session;

type
  TISQLExceptionCode = (eeInitialization, eeInvDialect, eeFOpen, eeParse,
                        eeCreate, eeConnect, eeStatement,
                        eeCommit, eeRollback, eeDDL, eeDML, eeQuery, eeFree,
                        eeUnsupt);

  EISQLException = class(Exception)
  private
    FExceptionCode: TISQLExceptionCode;
    FErrorCode: Integer;
    FExceptionData: String;
  public
    constructor Create (ErrorCode: Integer;
                        ExceptionData: String;
                        ExceptionCode: TISQLExceptionCode;
                        Msg: String);
  published
    property ExceptionCode: TISQLExceptionCode read FExceptionCode;
    property ExceptionData: String read FExceptionData;
    property ErrorCode: Integer read FErrorCode;
  end;

  TISQLAction = (actInput, actOutput, actCount, actEcho, actList, actNames, actPlan,
                 actStats, actTime, actUnk, actUnSup, actDialect, actAutoDDL);

  TSQLEvent = (evntDrop, evntAlter, evntConnect, evntCreate, evntTransaction,
               evntUnk, evntDialect, evntRows, evntISQL);

  TSQLSubEvent = (seTable, seTrigger, seProcedure, seDomain, seFilter, seFunction,
                  seException, seDatabase, seUnk, seView, seRole, seGenerator,
                  seDialect1, seDialect2, seDialect3, seAutoDDL);

  TDataOutput = procedure (const Info: String) of object;

  TISQLEvent = procedure (const ISQLEvent: TSQLEvent; const ISQLSubEvent: TSQLSubEvent;
                          const Data: variant; const DataModule: TDmN; const ObjectName:string) of object;

  TQueryStats = record
    Plan,
    Query: String;
    TimePrepare,
    TimeExecute: TDateTime;
    Rows: Integer;
    Buffers,
    Reads,
    Writes,
    Fetches,
    MaxMem,
    StartMem,
    EndMem: longint;
  end;

  TQueryParam = record
    Name:String;
    Value:Variant;
  end;

  TQueryParams = array of TQueryParam;

  TIBOSqlObj = class (TComponent)
  private
    FStatistics: TStringList;
    FQuery: TStrings;
    FDataSet: TIBOQuery;
    FDataOutput: TDataOutput;
    FISQLEvent: TISQLEvent;
    FProgressEvent: TNotifyEvent;
    FStatements: integer;
    FStatsOn,
    FCanceled,
    FAutoDDL: boolean;
    FQueryStats: TQueryStats;
    FDBInfo: TIB_ConnectionInfo;
    FDataModule: TDmN;
    FHasParams: boolean;
    fParams:TQueryParams;
    FWaitForParams: boolean;
    fStoredQuery:String;

    procedure GetEvent(const Query: String; out event: TSQLEvent; out sEvent: TSQLSubEvent; out sObjName: String);
    function ParseSQL(const InputSQL: TStringList; out Data: TStringList; const TermChar: String): boolean;
    function ParseDBConnect (const InputSQL: String; out DBName: String; out Params: TStringList): boolean;
    function ParseClientDialect (const InputSQL: String): Integer;
    function IsISQLCommand (const InputSQL: String; out Data: Variant): TISQLAction;
    function ParseOnOff (const InputSQL, Item: String): Boolean;
    procedure SetDataModule(const Value: TDmN);
//    procedure SetHasParams(const Value: boolean);
    procedure SetWaitForParams(const Value: boolean);
    procedure DoGetParams(q: TIB_Cursor); overload;
    procedure DoGetParams(q: TIBOQuery); overload;
    function ParseDBCreate (const InputSQL: String; out DBName: String; out Params: String): boolean;

  published
    property AutoDDL: boolean read FAutoDDL write FAutoDDL;
    property Query: TStrings read FQuery write FQuery;
    property Dataset: TIBOQuery read FDataset write FDataset;
    property Statements: Integer read FStatements;
    property Statistics: boolean read FStatsOn write FStatsOn;
    property StatisticsList: TStringList read FStatistics;
    property OnDataOutput: TDataOutput read FDataOutput write FDataOutput;
    property OnISQLEvent: TISQLEvent read FISQLEvent write FISQLEvent;
    property OnQueryProgress: TNotifyEvent read FProgressEvent write FProgressEvent;

  public
    constructor Create (AComponent: TComponent); override;
    destructor Destroy; override;
    procedure DoISQL;
    procedure DoISQLWithParams;
    procedure DoPrepare;
    procedure Cancel;
    property DataModule:TDmN read FDataModule write SetDataModule;
    property HasParams:boolean read FHasParams; // write SetHasParams;
    property Params:TQueryParams read FParams;
    property WaitForParams:boolean read FWaitForParams write SetWaitForParams;
  end;

  function TransIndex(cn:TIB_Connection;tr:TIB_Transaction):integer;

implementation

uses
  IBOCGlobal, dmuMain, IB_Header, frmuMessage, IBOAdminUtils, IBOServices,
  Variants, DB, zluibcClasses;

function TransIndex(cn:TIB_Connection;tr:TIB_Transaction):integer;
var n:integer;
begin
  result := -1;
  for n := 0 to cn.TransactionCount-1 do
  begin
    if cn.Transactions[n] = tr then
    begin
      Result := n;
      break;
    end;
  end;

end;


procedure TIBOSqlObj.GetEvent (const Query: String; out event: TSQLEvent; out sEvent: TSQLSubEvent; out sObjName: String);
var
  str: String;
  ostr: String;

  function GetName(keyWord:String; keyWord2:String=''):String;
  var n:integer;
      go:boolean;
      quote:boolean;
  begin
    Delete (ostr, 1, Length(keyWord)+1);
    if keyWord2<>'' then
    begin
      n:= pos(keyWord2,UpperCase(ostr));
      if n>0 then
        Delete(ostr, n, Length(keyWord)+1);
    end;
    n:=1;
    go:=true;
    quote:=false;
    Result := '';
    while go and (n<=Length(ostr))do
    begin
      case ostr[n] of
        '"':
        begin
          if (DataModule.Connection.SQLDialect = 3) then
          begin
            if quote then
            begin
              inc(n);
              if (n<=Length(ostr)) then
              begin
                if (ostr[n]='"') then
                begin
                  Result := Result + '"';
                  inc(n);
                end
                else
                  go := False;
              end;
            end
            else
            begin
              quote := true;
              inc(n);
            end;
          end;
        end;
        ' ':
        begin
          if quote then
          begin
            Result := Result + ' ';
            inc(n);
          end
          else
          begin
            if Result <> '' then
              go := False;
          end;
        end;
        else
        begin
          Result := Result + ostr[n];
          inc(n);
        end;
      end;
    end;
  end;

begin
  sObjName := '';
  ostr := StringReplace(Query,#9,' ',[rfReplaceAll]);
  str := AnsiUpperCase(ostr);
  if Pos ('CREATE', Str) = 1 then
  begin
    event := evntCreate;
    Delete (str, 1, Length('CREATE')+1);
    Delete (ostr, 1, Length('CREATE')+1);
  end
  else
  if Pos ('DECLARE', Str) = 1 then
  begin
    event := evntCreate;
    Delete (str, 1, Length('DECLARE')+1);
    Delete (ostr, 1, Length('DECLARE')+1);
  end
  else
  begin
    if Pos ('DROP', str) = 1 then
    begin
      event := evntDrop;
      Delete (str, 1, Length ('DROP')+1);
      Delete (ostr, 1, Length('DROP')+1);
    end
    else
    begin
      if Pos ('ALTER', str) = 1 then
      begin
        event := evntALTER;
        Delete (str, 1, Length ('ALTER')+1);
        Delete (ostr, 1, Length('ALTER')+1);
      end
      else
      begin
        event := evntUnk;
        sEvent := seUnk;
        exit;
      end;
    end
  end;

  if Pos ('VIEW', str) = 1 then
  begin
    sEvent := seView;
    sObjName := GetName('VIEW');
    exit;
  end;
  if Pos ('TABLE', str) = 1 then
  begin
    sEvent := seTable;
    sObjName := GetName('TABLE');
    exit;
  end;
  if Pos ('PROCEDURE', str) = 1 then
  begin
    sEvent := seProcedure;
    sObjName := GetName('PROCEDURE');
    exit;
  end;
  if Pos ('DOMAIN', str) = 1 then
  begin
    sEvent := seDomain;
    sObjName := GetName('DOMAIN');
    exit;
  end;
  if Pos ('EXTERNAL', str) = 1 then
  begin
    sEvent := seFunction;
    sObjName := GetName('EXTERNAL','FUNCTION');
    exit;
  end;
  if Pos ('FILTER', str) = 1 then
  begin
    sEvent := seFilter;
    sObjName := GetName('FILTER');
    exit;
  end;
  if Pos ('EXCEPTION', str) = 1 then
  begin
    sEvent := seException;
    SObjName := GetName('EXCEPTION');
    exit;
  end;
  if Pos ('ROLE', str) = 1 then
  begin
    sEvent := seRole;
    SObjName := GetName('ROLE');
    exit;
  end;
  if Pos ('GENERATOR', str) = 1 then
  begin
    sEvent := seGenerator;
    sObjName := GetName('GENERATOR');
    exit;
  end;
  if Pos ('TRIGGER', str) = 1 then
  begin
    sEvent := seTrigger;
    sObjName := GetName('TRIGGER');
    exit;
  end;
  sEvent := seUnk;
end;


function TIBOSqlObj.ParseSQL(const InputSQL: TStringList; out Data: TStringList;
  const TermChar: string): boolean;

var
  lTermLen,
  lDelimPos,
  lLineLen,
  lSrc_Cnt: Integer;

  done,
  inStatement,
  inComment: boolean;

  lTmp,
  lLine,
  lTermChar: String;


begin
  lTermChar := TermChar;
  inComment := false;
  lSrc_Cnt := 0;
  result := true;
  done := false;
  while not done and (lSrc_Cnt < InputSQL.Count) do
  begin
    Application.ProcessMessages;
    { If the line is blank, skip it }
    if Length (Trim (InputSQL.Strings[lSrc_Cnt])) = 0 then
    begin
      Inc (lSrc_Cnt);
      Continue;
    end;

    { If the current line contains only a close comment, append it to the last
      line }
    if Trim(InputSQL.Strings[lSrc_Cnt]) = '*/' then
    begin
      Data.Strings[Data.Count-1] := Format('%s%s', [Data.Strings[Data.Count-1],
        InputSQL.Strings[lSrc_Cnt]]);
      Inc(lSrc_Cnt);
      Continue;
    end;

    { Next, check to see if the line is a comment or starts one.
      If it does, then remove it }
    lDelimPos := AnsiPos('/*', Trim(InputSQL.Strings[lSrc_Cnt]));
    if  lDelimPos = 1 then
    begin
      { since this line contains a comment character, save anything to the
        left of the comment (if applicable) }
      if AnsiPos('*/', InputSQL.Strings[lSrc_Cnt]) = 0 then
        inComment := true;

      { If the line above started a comment, then keep going until the
        comment is completed }
      while inComment do
      begin
        repeat
          Application.ProcessMessages;
          Inc (lSrc_Cnt);
          if (lSrc_Cnt = InputSQL.Count) then
          begin
            result := false;
            done := true;
            continue;
          end;
        until (AnsiPos('*/', InputSQL.Strings[lSrc_Cnt]) <> 0);

        { if there is something after the comment, save it }
        lDelimPos := AnsiPos('*/', InputSQL.Strings[lSrc_Cnt]);
        lLine := Format('%s %s', [lLine, Copy (InputSQL.Strings[lSrc_cnt],
          lDelimPos+2, Length(InputSQL.Strings[lSrc_Cnt]))]);
        inComment := false;
      end;
      Inc (lSrc_Cnt);
      Continue;
    end;

    { Is the delimiter being changed? }
    if AnsiPos ('SET TERM', AnsiUpperCase(Trim(InputSQL.Strings[lSrc_Cnt]))) = 1 then
    begin
      lTmp := Trim(InputSQL.Strings[lSrc_Cnt]);
      Delete (lTmp, 1, Length('SET TERM'));
      lTmp := Trim(lTmp);
      lDelimPos := Pos (lTermChar, lTmp);
      Delete (lTmp, lDelimPos, Length(lTermChar));
      lTermChar := Trim(lTmp);
      lTmp := '';
      Inc (lSrc_Cnt);
      Continue;
    end;

    { If the delimiter is at the end of the line, or if a comment exists
       after the delimiter, then this is an entire statement.  If there is
       a comment after the delimiter, remove it }
    lDelimPos := AnsiPos (lTermChar, Trim(InputSQL.Strings[lSrc_Cnt]));
    lTermLen := Length (lTermChar);
    lLineLen := Length (Trim(InputSQL.Strings[lSrc_Cnt]));
    
    if (lDelimPos = (lLineLen - (lTermLen - 1))) or
       (AnsiPos ('/*', InputSQL.Strings[lSrc_Cnt]) > lDelimPos) then
    begin
      lLine := Trim(Format('%s %s',[lLine, InputSQL.Strings[lSrc_Cnt]]));
      Delete (lLine, lDelimPos, lTermLen);

      { Make sure that the line isn't blank after removing the terminator.
        Some case tools print too many termination characters }
      if Length(Trim(lLine)) <> 0 then
         Data.Append (Trim(lLine));
      lLine := '';
      Inc(lSrc_Cnt);
    end
    else
    begin
      { This statement spans multiple lines, so concatenate the lines into 1
        adding a CR/LF between the lines to ensure that the source looks
        as it was added }
      inStatement := true;
      lLine := Format('%s %s',[lLine, InputSQL.Strings[lSrc_Cnt]]);
      repeat
        Application.ProcessMessages;
        Inc (lSrc_Cnt);
        if (lSrc_Cnt = InputSQL.Count) then
        begin
          result := true;
          done := true;
          instatement := false;
          continue;
        end;
        { Blank line }
        if Length (Trim (InputSQL.Strings[lSrc_Cnt])) = 0 then
          Continue;

        lDelimPos := AnsiPos (lTermChar, InputSQL.Strings[lSrc_Cnt]);
        lLineLen := Length (InputSQL.Strings[lSrc_Cnt]);
        if (lDelimPos = (lLineLen - (lTermLen - 1))) or
           ((lDelimPos > 0) and
            (AnsiPos ('/*', InputSQL.Strings[lSrc_Cnt]) > lDelimPos)) then
          inStatement := false;
        lLine := lLine + CRLF + InputSQL.Strings[lSrc_Cnt];
      until not inStatement;

      { Remove the termination character }
      lDelimPos := AnsiPos (lTermChar, lLine);
      Delete (lLine, lDelimPos, lTermLen);

      { Make sure that the line isn't blank after removing the terminator.
        Some case tools print too many termination characters }
      if Length(Trim(lLine)) <> 0 then
        Data.Append (Trim(lLine));

      lLine := '';
      Inc (lSrc_Cnt);
    end;
  end; { while }
end;

function TIBOSqlObj.ParseDBCreate (const InputSQL: String; out DBName: String; out Params: String): boolean;
var
  lCnt,
  lDelimPos,
  lStartPos: Integer;

  lLine,
  lTemp,
  QuoteChar: String;

begin

  lLine := Trim(InputSQL);

  (* Get the database name *)
  (* A database can be created using CREATE DATABASE or CREATE SCHEMA *)
  if Pos ('CREATE DATABASE', AnsiUpperCase(lLine)) = 1 then
    Delete (lLine, 1, Length ('CREATE DATABASE '))
  else
    Delete (lLine, 1, Length ('CREATE SCHEMA '));

  QuoteChar := Copy (lLine, 1, 1);

  lStartPos := 1;
  lDelimPos := 1;

  if lLine <> '' then 
  repeat
    Application.ProcessMessages;
    Delete (lLine, lStartPos, lDelimPos);
    lDelimPos := Pos (QuoteChar, lLine);
    lTemp := Copy (lLine, lStartPos, lDelimPos-1);
    DBName := DBName + lTemp;
  until (lLine = '')
        or (lLine[lDelimPos+1] in [#0, #13, #10, ' '])
        or (lDelimPos = 0);

  if lDelimPos = 0 then
  begin
    result := false;
    exit;
  end;

  (* Remove the database name from the line *)
  Delete (lLine, lStartPos, lDelimPos);

  Params := ' '+lLine;

  for lCnt := 1 to Length(Params) do
    if Params[lCnt] = '"' then Params[lCnt] := '''';

  result := true;
end;

function TIBOSqlObj.ParseDBConnect (const InputSQL: String; out DBName: String; out Params: TStringList): boolean;
var
  lDelimPos: Integer;

  lLine,
  lTemp,
  lParam,
  QuoteChar: String;

  isaRole: boolean;
begin

  lLine := Trim(InputSQL);
  isaRole := false;
  (* Get the database name *)
  Delete (lLine, 1, Length ('CONNECT '));
  QuoteChar := Copy (lLine, 1, 1);

  (* Connect statements don't require a quoted database name *)
  if (QuoteChar = '''') or (QuoteChar = '"') then
  begin
    lDelimPos := 1;

    repeat
      Application.ProcessMessages;
      lLine := Trim(lLine);
      Delete (lLine, 1, lDelimPos);
      lDelimPos := Pos (QuoteChar, lLine);
      lTemp := Copy (lLine, 1, lDelimPos-1);
      DBName := DBName + lTemp;
    until (lLine[lDelimPos+1] in [#0, #13, #10, ' ']) or
          (lDelimPos = 0);

    if lDelimPos = 0 then
    begin
      result := false;
      exit;
    end;
    (* Remove the database name from the line *)
    Delete (lLine, 1, lDelimPos);
  end
  else
  begin
    lDelimPos := Pos (' ', lLine);

    (* If there are no spaces on the line, then the only thing is the database
     * name
     *)
    if lDelimPos = 0 then
    begin
      DBName := lLine;
      result := true;
      exit;
    end;
    DBName := Copy (lLine, 1, lDelimPos-1);
    Delete (lLine, 1, lDelimPos);
  end;

  (* Get the parameters for connecting *)
  lLine := Trim(lLine);
  while Length (lLine) > 0 do
  begin
    Application.ProcessMessages;
    lDelimPos := Pos (' ', lLine);
    if lDelimPos = 0 then
    begin
      lTemp := Copy (lLine, 1, Length(lLine));
      lParam := lParam + ' ' + lTemp;
      Params.Append (lParam);
      break;
    end;
    lTemp := Copy (lLine, 1, lDelimPos-1);

    if AnsiSameText (lTemp, 'USER') or AnsiSameText (lTemp, 'USERNAME') then
      lParam := 'USER_NAME='
    else
    begin
      if AnsiSameText (lTemp, 'PASS') or AnsiSameText (lTemp, 'PASSWORD') then
        lParam := 'PASSWORD='
      else
      begin
        if AnsiSameText (lTemp, 'ROLE') then
        begin
          lParam := 'SQL_ROLE_NAME=';
          isaRole := true;
        end
        else
          if AnsiSameText (lTemp, 'CACHE') then
            lParam := 'NUM_BUFFERS='
      end;
    end;

    Delete (lLine, 1, lDelimPos);
    lLine := Trim(lLine);
    lDelimPos := Pos (' ', lLine);
    if lDelimPos = 0 then
      lDelimPos := Length(lLine)+1;

    lTemp := Copy (lLine, 1, lDelimPos-1);
    QuoteChar := Copy (lTemp, 1, 1);
    if (not isaRole) and
       ((QuoteChar = '''') or
        (QuoteChar = '"')) then
    begin
      { Delete the first quote }
      Delete (lTemp, 1, 1);
      { Delete the second }
      Delete (lTemp, Pos(QuoteChar, lTemp), 1);
    end;

    Delete (lLine, 1, lDelimPos);
    lLine := Trim(lLine);
    lParam := lParam + lTemp;
    Params.Append (lParam);
    Continue;
  end;

  result := true;
end;

function TIBOSqlObj.ParseClientDialect (const InputSQL: String): Integer;
var
  lLine: String;
begin
  lLine := Trim(InputSQL);
  Delete (lLine, 1, Length ('SET SQL DIALECT '));
  result := StrToInt(lLine);
end;

function TIBOSqlObj.ParseOnOff (const InputSQL, Item: String): Boolean;
var
  lLine: String;
begin
  lLine := Trim(AnsiUpperCase(InputSQL));
  Delete (lLine, 1, Length (Item+' '));

  if Pos ('ON', lLine) = 1 then
    result := true
  else
    if Pos ('OFF', lLine) = 1 then
      result := false
    else
      result := true;  (* default *)
end;

function TIBOSqlObj.IsISQLCommand (const InputSQL: String; out Data: Variant): TISQLAction;
var
  lLine: String;

begin

  lLine := Trim(AnsiUpperCase(InputSQL));

  if (Pos ('BLOBDUMP', lLine) = 1) or
     (Pos ('EDIT', lLine) = 1) or
     (Pos ('EXIT', lLine) = 1) or
     (Pos ('HELP', lLine) = 1) or
     (Pos ('QUIT', lLine) = 1) or
     (Pos ('SHOW ',lLine) = 1) then
  begin
     result := actUnSup;
     exit;
  end;

  if (Pos ('IN ', lLine) = 1) or
     (Pos ('INPUT', lLine) = 1) or
     (Pos ('OUT ', lLine) = 1) or
     (Pos ('OUTPUT', lLine) = 1) then
  begin
    if (Pos ('I', lLine) = 1) then
      result := actInput
    else
      result := actOutput;

    (* grab the filename to read or output to *)
    Data := Copy (lLine, Pos (' ', lLine), Length(lLine));
    Data := Trim(Data);
    exit;
  end;

  if Pos('SET COUNT', lLine) = 1 then
  begin
    Data := ParseOnOff (lLine, 'SET COUNT');
    result := actCount;
    exit;
  end;

  if Pos('SET ECHO', lLine) = 1 then
  begin
    Data := ParseOnOff (lLine, 'SET ECHO');
    result := actEcho;
    exit;
  end;

  if Pos('SET LIST', lLine) = 1 then
  begin
    Data := ParseOnOff (lLine, 'SET LIST');
    result := actList;
    exit;
  end;

  if Pos('SET NAMES', lLine) = 1 then
  begin
    result := actNames;

    (* Grab the character set name *)
    Data := Copy (lLine, Length('SET NAMES '), Length(lLine));
    Data := Trim(Data);
    exit;
  end;

  if Pos('SET PLAN', lLine) = 1 then
  begin
    Data := ParseOnOff (lLine, 'SET PLAN');
    result := actPlan;
    exit;
  end;

  if Pos('SET STATS', lLine) = 1 then
  begin
    Data := ParseOnOff (lLine, 'SET STATS');
    result := actStats;
    exit;
  end;

  if Pos('SET TIME', lLine) = 1 then
  begin
    Data := ParseOnOff (lLine, 'SET TIME');
    result := actTime;
    exit;
  end;

    (* Setting the Client dialect *)
  if Pos ('SET SQL DIALECT', lLine) = 1 then
  begin
    Data := ParseClientDialect (lLine);
    Data := Trim(Data);
    result := actDialect;
    exit;
  end;

  if (Pos ('SET AUTODDL', lLine) = 1) or
     (Pos ('SET AUTO', lLine) = 1) then
  begin
    if Pos ('DDL', lLine) <> 0 then
      Data := ParseOnOff (lLine, 'SET AUTODDL')
    else
      Data := ParseOnOff (lLine, 'SET AUTO');
    result := actAutoDDL;
    exit;
  end;

  result := actUnk;
end;


function Delta(StartSecs:TDateTime):TDateTime;
var EndSecs,TmpTime:TDateTime;
begin
  EndSecs := IBTimeStampToMSecs(DateTimeToTimeStamp(Time));
  TmpTime := TimeStampToDateTime(MSecsToTimeStamp(EndSecs - StartSecs));
  Result := TmpTime;
end;


procedure TIBOSqlObj.DoISQL;
var
  ISQLObj: TIBOSqlObj;
  stIdx:  integer;

  evnt: TSQLEvent;
  sEvent: TSQLSubEvent;

  Hour,
  Min,
  Sec,
  MSec: Word;

  Tmp,
  Params,
  DBName: String;

  NewSource : TStringList;

  Data,
  ParamList,
  Source: TStringList;

  TmpTransaction: TIB_Transaction;

  TmpQuery,
  DDLQuery,
  IBQuery: TIB_Cursor;

  ISQLValue: Variant;
  ISQLAction: TISQLAction;

  show_Plan,
  show_Stats,
  show_List,
  show_Count,
  show_Echo,
  show_time,
  Output: boolean;

  ClientDialect: Integer;
  CharSet: String;

  InputFile: TextFile;
  TmpTime: TDateTime;

  StartSecs,
  EndSecs : TDateTime;

  TrIdx:integer;

  ObjectName:String;


    procedure _actInput;
    begin
      try
        AssignFile(InputFile, IsqlValue);
        Reset (InputFile);
        NewSource := TStringList.Create;
        while not SeekEof(InputFile) do
        begin
          Readln(InputFile, Tmp);
          NewSource.Append(Tmp);
        end;
      except on E: Exception do
        raise EISQLException.Create (ERR_FOPEN, ISQLValue, eeFopen, E.Message);
      end;
      ISQLObj := TIBOSqlObj.Create (self);
      try
        ISQLObj.AutoDDL := FAutoDDL;
        ISQLObj.Query := NewSource;
        ISQLObj.DataModule := DataModule;
        ISQLObj.DataSet := FDataSet;
        ISQLObj.OnDataOutput := Self.FDataOutput;
        ISQLObj.OnISQLEvent := Self.OnISQLEvent;
        ISQLObj.DoIsql;
      finally
        ISQLObj.Free;
        NewSource.Free;
      end;
    end;

  procedure _actDialect;
  begin
    ClientDialect := ISQLValue;
    if Assigned (DataModule) then
    try
      DataModule.Connection.SQLDialect := ClientDialect;
      if Assigned (OnISQLEvent) then
        case ClientDialect of
          1: OnISQLEvent (evntDialect, seDialect1, ClientDialect, DataModule, '');
          2: OnISQLEvent (evntDialect, seDialect2, ClientDialect, DataModule, '');
          3: OnISQLEvent (evntDialect, seDialect3, ClientDialect, DataModule, '');
        end;
    except
      on E: Exception do
      begin
        raise EISQLException.Create (ERR_ISQL_ERROR, IntToStr(ClientDialect), eeInvDialect, E.Message);
      end;
    end;
  end;

  procedure _drop_database;
  begin
    DataModule.Connection.DropDatabase;
    if Assigned (OnISQLEvent) then
      OnISQLEvent (evntDrop, seDatabase, DataModule.Connection.DatabaseName, DataModule, '');
  end;

  procedure _create_database_schema;

  begin
    if not ParseDBCreate (Data.Strings[stIdx], DBName, Params) then
      raise EISQLException.Create (ERR_ISQL_ERROR, '', eeParse, GetPrompt(61))
    else
    begin
      try
        try
           if DataModule.Connection.dbHandle <> nil then
             IBOError(iboeDatabaseOpen, [nil]);
        except
          on E: Exception do
          begin
            if DataModule.Transaction.InTransaction then
              raise EISQLException.Create (ERR_ISQL_ERROR, DBName, eeConnect,
                E.Message+CRLF+GetPrompt(60))
            else
              DataModule.Connection.Close;
          end;
        end;

        DataModule.Connection.Params.Text := Params;
        DataModule.Connection.DatabaseName := DBName;
        DataModule.Connection.SQLDialect := ClientDialect;
        DataModule.Connection.Username := DataModule.ServerNode.UserName;
        DataModule.Connection.Password := DataModule.ServerNode.Password;

        DataModule.Connection.LoginPrompt := false;
        DataModule.Connection.CreateDatabase;

        TmpTransaction := TIB_Transaction.Create (nil);

        TmpTransaction.IB_Connection := DataModule.Connection;

        IBQuery := TIB_Cursor.Create (nil);
        DDLQuery:= TIB_Cursor.Create (nil);
        TmpQuery:= TIB_Cursor.Create (nil);

        IBQuery.IB_Connection := DataModule.Connection;
        IBQuery.IB_Transaction := Datamodule.Transaction;
        IBQuery.ParamCheck := false;

        TmpQuery.IB_Connection := DataModule.Connection;
        TmpQuery.IB_Transaction := TmpTransaction;
        TmpQuery.ParamCheck := false;

        DDLQuery.IB_Connection := DataModule.Connection;
        DDLQuery.IB_Transaction := DataModule.DDLTransaction;
        DDLQuery.ParamCheck := false;

        DataModule.Connection.Open;
        if Assigned (OnISQLEvent) then
          OnISQLEvent (evntCreate, seDatabase, DBName, DataModule, '');
      except
        on E: Exception do
        raise EISQLException.Create (ERR_ISQL_ERROR, DBName, eeCreate, E.Message);
      end;
    end;
  end;

          procedure _stRollback;
          begin
            try
              if Assigned(Dataset) then
                IBQuery.IB_Transaction := Dataset.IB_Transaction;

              if IBQuery.IB_Transaction.InTransaction then
                IBQuery.IB_Transaction.Rollback;

              if DataModule.DDLTransaction.InTransaction then
                DataModule.DDLTransaction.Rollback;

              if Assigned(OnISQLEvent) then
                OnISQLEvent (evntTransaction, seUnk, false, DataModule, '');

            except on E: Exception do
              raise EISQLException.Create (ERR_ISQL_ERROR, Data.Strings[stIdx], eeRollback, E.Message);
            end;
          end;
          
          procedure _stDDL;
          begin
            (* Use a different IBQuery since DDL can be set to autocommit *)
            DDLQuery.SQL.Clear;
            DDLQuery.SQL := IBQuery.SQL;
            with DDLQuery do begin
              try
                Prepare;
                Execute;
                if FAutoDDL then
                  DataModule.DDLTransaction.Commit;

                if Assigned (OnISQLEvent) then
                begin
                  GetEvent (DDLQuery.SQL.Strings[0], evnt, sevent, ObjectName);
                  OnISQLEvent (evnt, sEvent, DBName, DataModule, ObjectName);
                end;
                 Close;
              except on E: Exception do
              begin
                Close;
                if FAutoDDL then
                  DataModule.DDLTransaction.Rollback;
                raise EISQLException.Create (ERR_ISQL_ERROR, Data.Strings[stIdx], eeDDL, E.Message);
              end;
              end;
            end;
          end;

          procedure _stCommit;
          begin
            try
              if Assigned(Dataset) then
                IBQuery.IB_Transaction := Dataset.IB_Transaction;

              if IBQuery.IB_Transaction.InTransaction then
                IBQuery.IB_Transaction.Commit;

              if DataModule.DDLTransaction.InTransaction then
                DataModule.DDLTransaction.Commit;

              if Assigned(OnISQLEvent) then
                OnISQLEvent (evntTransaction, seUnk, false, DataModule, '');

            except on E: Exception do
              raise EISQLException.Create (ERR_ISQL_ERROR, Data.Strings[stIdx], eeCommit, E.Message);
            end;
          end;
          procedure _stDelete;
          begin
            try
              if Assigned(Dataset) then
                IBQuery.IB_Transaction := Dataset.IB_Transaction;

              StartSecs := 0;
              if FStatsOn then
              try
                FDBInfo.Database := DataModule.Connection;
                FQueryStats.StartMem := FDBInfo.CurrentMemory;
                FQueryStats.Query := IBQuery.SQL.Text;
                StartSecs := Time;
              except
              end;

              IBQuery.Prepare;
              DoGetParams(IBQuery);

              if FStatsOn then
              try
                FQueryStats.TimePrepare := Time - StartSecs;
                StartSecs := Time;
              except
              end;

              if not FHasParams then
              begin
                IBQuery.Execute;
              end
              else
              begin
                fStoredQuery := FQuery.Text;
                FWaitForParams := true;
              end;

              try
                FQueryStats.Plan := IBQuery.SQLPlan.Text;
              except
                FQueryStats.Plan := GetPrompt(56);
              end;
              FQueryStats.Rows := IBQuery.RowsAffected;

              if FStatsOn then
              try
                FQueryStats.TimeExecute := Time - StartSecs;
                FQueryStats.EndMem := FDBInfo.CurrentMemory;
                FQueryStats.MaxMem := FDBInfo.MaxMemory;
                FQueryStats.Buffers := FDBInfo.NumBuffers;
                FQueryStats.Reads := FDBInfo.Reads;
                FQueryStats.Writes := FDBInfo.Writes;
              except
              end;

              if Assigned (OnISQLEvent) then
                OnISQLEvent(evntRows, seUnk, IBQuery.RowsAffected, DataModule, '');
              //shold be fine to get table names - Lorenzo

              IBQuery.Close;
            except on E: Exception do
            begin
              IBQuery.Close;
              raise EISQLException.Create (ERR_ISQL_ERROR, Data.Strings[stIdx], eeDML, E.Message);
            end;
            end;
          end;

          procedure _stSelect;
          begin
            try
              if Assigned (DataSet) then
              begin
                DataSet.Close;
                DataSet.SQL.Text := IBQuery.SQL.Text;

                StartSecs := 0;
                if FStatsOn then
                try
                  FDBInfo.Database := DataModule.Connection;
                  FQueryStats.StartMem := FDBInfo.CurrentMemory;
                  FQueryStats.Query := IBQuery.SQL.Text;
                  StartSecs := Time;
                except
                end;

                DataSet.Prepare;
                DoGetParams(DataSet);

                if FStatsOn then
                try
                  EndSecs := Time;
                  TmpTime := EndSecs - StartSecs;
                  FQueryStats.TimePrepare := TmpTime;
                  StartSecs := Time;
                except
                end;
                if not FHasParams then
                begin
                  DataSet.Open;
                end
                else
                begin
                  fStoredQuery := FQuery.Text;
                  FWaitForParams := true;
                end;
                try
                  FQueryStats.Plan := Dataset.Plan;
                except
                  FQueryStats.Plan := GetPrompt(56);
                end;

                try
                  FQueryStats.Rows := Dataset.RecordCount;
                except
                  FQueryStats.Rows := 0;
                end;

                if FStatsOn then
                try
                  EndSecs := Time;
                  TmpTime := EndSecs - StartSecs;
                  FQueryStats.TimeExecute := TmpTime;
                  FQueryStats.EndMem := FDBInfo.CurrentMemory;
                  FQueryStats.MaxMem := FDBInfo.MaxMemory;
                  FQueryStats.Buffers := FDBInfo.NumBuffers;
                  FQueryStats.Reads := FDBInfo.Reads;
                  FQueryStats.Writes := FDBInfo.Writes;
                except
                end;
              end
              else
              begin
                IBQuery.Prepare;
                DoGetParams(IBQuery);
                if not FHasParams then
                begin
                  fStoredQuery := FQuery.Text;
                  IBQuery.Execute;
                  IBQuery.Close;
                end
                else
                  FWaitForParams := true;

              end;
            except
              on E: Exception do
                begin
                  IBQuery.Close;
                  raise EISQLException.Create (ERR_ISQL_ERROR, Data.Strings[stIdx], eeQuery, E.Message);
                end;
            end;
          end;

      procedure _Connect;
      begin
        ParamList := TStringList.Create;
        if not ParseDBConnect (Data.Strings[stIdx], DBName, ParamList) then
          raise EISQLException.Create (ERR_ISQL_ERROR, '', eeParse, GetPrompt(59))
        else
        begin
          try
            try
              if DataModule.Connection.dbHandle <> nil then
                IBOError(iboeDatabaseOpen, [nil]);
            except
              on E: Exception do
              begin
                if DataModule.Transaction.InTransaction then
                  raise EISQLException.Create (ERR_ISQL_ERROR, DBName, eeConnect,
                    E.Message+CRLF+GetPrompt(60))
                else
                  DataModule.Connection.Close;
              end;
            end;
            DataModule.Connection.DatabaseName := DBName;
            if (charset <> '') and (charset <> 'NONE') then
              ParamList.Add ('lc_ctype='+charset);
            DataModule.Connection.Params.Text := ParamList.Text;
            DataModule.Connection.Params.Add(Format('isc_dpb_sql_dialect=%d',[ClientDialect]));
            ParamList.Free;
            DataModule.Connection.LoginPrompt := false;
            DataModule.Connection.SQLDialect := ClientDialect;
            DataModule.Connection.Open;

            TmpTransaction := TIB_Transaction.Create (nil);
            TmpTransaction.IB_Connection := DataModule.Connection;

            IBQuery := TIB_Cursor.Create (nil);
            DDLQuery:= TIB_Cursor.Create (nil);
            TmpQuery:= TIB_Cursor.Create (nil);

            IBQuery.IB_Connection := DataModule.Connection;
            if Assigned (FDataset) then
              IBQuery.IB_Transaction := FDataset.IB_Transaction
            else
              IBQuery.IB_Transaction := DataModule.Transaction;
            IBQuery.ParamCheck := false;

            TmpQuery.IB_Connection := DataModule.Connection;
            TmpQuery.IB_Transaction := TmpTransaction;
            TmpQuery.ParamCheck := false;

            DDLQuery.IB_Connection := DataModule.Connection;
            DDLQuery.IB_Transaction := DataModule.DDLTransaction;
            DDLQuery.ParamCheck := false;

            if Assigned (OnISQLEvent) then
              OnISQLEvent (evntConnect, seDatabase, DBName, DataModule, '');
          except
            on E: Exception do
              raise EISQLException.Create (ERR_ISQL_ERROR, DBName, eeConnect, E.Message);
          end;
        end;
      end;

      procedure RecordStats;
      begin
        try
          with FQueryStats do
          begin
            DecodeTime(TimeExecute, Hour, Min, Sec, MSec);
            FStatistics.Add(Format(GetLangText('Execution Time')+' (hh:mm:ss.ssss)%s%.2d:%.2d:%.2d.%.4d',[DEL, Hour, Min, Sec, MSec]));
            DecodeTime(TimePrepare, Hour, Min, Sec, MSec);
            FStatistics.Add(Format(GetLangText('Prepare Time')+' (hh:mm:ss.ssss)%s%.2d:%.2d:%.2d:%.4d',[DEL,Hour, Min, Sec, MSec]));
            FStatistics.Add(Format(GetLangText('Starting Memory')+'%s%d',[DEL,StartMem]));
            FStatistics.Add(Format(GetLangText('Current Memory')+'%s%d',[DEL,EndMem]));
            FStatistics.Add(Format(GetLangText('Delta Memory')+'%s%d',[DEL,EndMem-StartMem]));
            FStatistics.Add(Format(GetLangText('Number of Buffers')+'%s%d',[DEL, Buffers]));
            FStatistics.Add(Format(GetLangText('Reads')+'%s%d',[DEL,Reads]));
            FStatistics.Add(Format(GetLangText('Writes')+'%s%d',[DEL,Writes]));
            if Length(Plan) > 0 then
              FStatistics.Add(Format(GetLangText('Plan')+'%s%s',[DEL, Plan]))
            else
              FStatistics.Add(Format(GetLangText('Plan')+'%s%s',[DEL, GetLangText('Not Available')]));
            if Rows > -1 then
              FStatistics.Add(Format(GetLangText('Rows Affected')+'%s%d',[DEL, Rows]));
          end;
          TrIdx := TransIndex(DataModule.Connection,TmpTransaction);
          if trIDx <> -1 then
            TmpTransaction.IB_Connection := Nil;
        except
          on E: Exception do
            raise EISQLException.Create (ERR_ISQL_ERROR, '', eeFree, E.Message);
        end;
      end;

      procedure _CreateComponents;
      begin
        (* If the database was already assigned and attached to,
           then create the components for the database *)
        if Assigned(DataModule.Connection.dbHandle) then
        begin
          TmpTransaction := TIB_Transaction.Create (nil);
          IBQuery := TIB_Cursor.Create (nil);
          DDLQuery:= TIB_Cursor.Create (nil);
          TmpQuery:= TIB_Cursor.Create (nil);
          TmpTransaction.IB_Connection := DataModule.Connection;
          IBQuery.IB_Connection := DataModule.Connection;
          IBQuery.IB_Transaction := DataModule.Transaction;
          IBQuery.ParamCheck := false;
          TmpQuery.IB_Connection := DataModule.Connection;
          TmpQuery.IB_Transaction := TmpTransaction;
          TmpQuery.ParamCheck := false;
          DDLQuery.IB_Connection := DataModule.Connection;
          DDLQuery.IB_Transaction := DataModule.DDLTransaction;
          DDLQuery.ParamCheck := false;
        end;
      end;

      procedure DoStatement;
      begin
          Application.ProcessMessages;
          if Assigned(FProgressEvent) then OnQueryProgress(self);
          (* Is this an ISQL Command? *)
          IsqlAction := IsISQLCommand (Data.Strings[stIdx], IsqlValue);
          case ISQlAction of
            actInput:   begin _actInput; end;
            actOutput:  Output := True;
            actCount:   begin show_Count := ISQLValue; end;
            actEcho:    begin show_Echo := ISQLValue; end;
            actList:    begin show_List := ISQLValue; end;
            actNames:   begin Charset := ISQLValue;   end;
            actPlan:    begin show_Plan := ISQLValue;  end;
            actStats:   begin show_Stats := ISQLValue; end;
            actTime:    begin show_time := ISQLValue;  end;
            actDialect: begin _actDialect;  end;
            actAutoDDL:
            begin
              FAutoDDL := ISQLValue;
              if Assigned (OnISQLEvent) then OnISQLEvent (evntISQL, seAutoDDL, FAutoDDL, DataModule, '');
            end;
          end;
          if ISQLAction <> actUnk then exit;
          (* Does this line drop a database *)
          if (Pos ('DROP DATABASE', AnsiUpperCase(Data.Strings[stIdx])) = 1) then
          begin
            _drop_database;
          end
          else
          (* Does this line create a database *)
          if (Pos ('CREATE DATABASE', AnsiUpperCase(Data.Strings[stIdx])) = 1) or
             (Pos ('CREATE SCHEMA', AnsiUpperCase(Data.Strings[stIdx])) = 1) then
          begin
            _create_database_schema;
          end
          else
          (* Does this line connect to a database *)
          if Pos ('CONNECT', AnsiUpperCase(Data.Strings[stIdx])) = 1 then
          begin
            _Connect;
          end
          else
          begin
            (* If it isn't any of the above, then execute the statement *)
            if not Assigned (IBQuery) then
              raise EISQLException.Create(ERR_ISQL_ERROR, Data.Strings[stIdx], eeStatement, GetPrompt(58));
            IBQuery.Close;
            IBQuery.SQL.Clear;
            IBQuery.SQL.Add (Data.Strings[stIdx]);
            TmpQuery.SQL.Clear;
            TmpQuery.SQL.Add (Data.Strings[stIdx]);
            (* See if the statement is valid *)
            try
              if TmpTransaction.InTransaction then
                TmpTransaction.Commit;
              TmpQuery.Prepare;
              TmpQuery.Close;
            except on E: Exception do
              begin
                TmpTransaction.Commit;
                TmpQuery.Close;
                raise EISQLException.Create (ERR_ISQL_ERROR, Data.Strings[stIdx], eeStatement, E.Message);
              end;
            end;
            TmpTransaction.Commit;
            case TmpQuery.StatementType of
              stCommit:   _stCommit;
              stRollback: _stRollback;
              stDDL, stSetGenerator:  _stDDL;
              stDelete, stInsert, stUpdate: _stDelete;
              stSelect, stSelectForUpdate, stExecProcedure: _stSelect;
            end;
          end;
      end;

var
  lCnt: integer;

begin
  // no way to run whitout it
  if not Assigned(DataModule) then exit;
  TmpQuery := nil;
  DDLQuery := nil;
  IBQuery := nil;
  TmpTransaction := Nil;
  Data := TStringList.Create;
  Source := TStringList.Create;

  (* Defaults *)
  try
    ClientDialect := gAppSettings[DEFAULT_DIALECT];
    CharSet := gAppSettings[CHARACTER_SET];
    Source.AddStrings(FQuery);
    if not ParseSQL (Source, Data, gAppSettings[ISQL_TERMINATOR]) then
      raise EISQLException.Create (ERR_ISQL_ERROR, '', eeParse, GetPrompt(57));
    FStatements := Data.Count - 1;
    try
      _CreateComponents;
    except on E: Exception do
      raise EISQLException.Create (ERR_ISQL_ERROR, '', eeInitialization, E.Message);
    end;
    (* Go through the String list excecuting the information line by line.
     * Each statement is executed against the currently connected database
     * and server (if there is one).   *)
    if Data.Count = 1 then
    begin
      stIdx := 0;
      DoStatement;
    end
    else
    for lCnt := 0 to Data.Count-1 do
    begin
      stIdx := lCnt;
      if FCanceled then break;
      DoStatement;
    end;

    RecordStats;
  finally
    Data.Free;
    Source.Free;
    TmpQuery.Free;
    DDLQuery.Free;
    IBQuery.Free;
  end;
end;

procedure TIBOSqlObj.DoISQLWithParams;
var
  Data, Source: TStringList;

  TmpTransaction: TIB_Transaction;

  TmpQuery,
  DDLQuery,
  IBQuery: TIB_Cursor;
  CharSet: String;

  stIdx:Integer;

  ISQLAction: TISQLAction;
  IsqlValue:Variant;
  evnt: TSQLEvent;
  sEvent: TSQLSubEvent;
  ObjectName:String;
  DBName: String;
  StartSecs,
  EndSecs : TDateTime;
  TmpTime: TDateTime;
  Hour,
  Min,
  Sec,
  MSec: Word;
  TrIdx :integer;

  procedure AssignDDLQueryParams;
  var n:integer;
  begin
    if FHasParams then
      for n:=Low(fParams) to High(fParams) do
      begin
        DDLQuery.ParamByName(fParams[n].Name).Value := fParams[n].Value;
      end
  end;

  procedure _stDDL;
  begin
    (* Use a different IBQuery since DDL can be set to autocommit *)
    DDLQuery.SQL.Clear;
    DDLQuery.SQL := IBQuery.SQL;
      try
        DDLQuery.Prepare;
        AssignDDLQueryParams;
        DDLQuery.Execute;
        if FAutoDDL then
          DataModule.DDLTransaction.Commit;

        if Assigned (OnISQLEvent) then
        begin
          GetEvent (DDLQuery.SQL.Strings[0], evnt, sevent, ObjectName);
          OnISQLEvent (evnt, sEvent, DBName, DataModule, ObjectName);
        end;
        DDLQuery.Close;
      except on E: Exception do
      begin
        DDLQuery.Close;
        if FAutoDDL then
          DataModule.DDLTransaction.Rollback;
        raise EISQLException.Create (ERR_ISQL_ERROR, Data.Strings[stIdx], eeDDL, E.Message);
      end;
    end;
  end;

  procedure AssignQueryParams;
  var n:integer;
  begin
    if FHasParams then
      for n:=Low(fParams) to High(fParams) do
      begin
        IBQuery.ParamByName(fParams[n].Name).Value := fParams[n].Value;
      end
  end;

  procedure AssignDatasetParams;
  var n:integer;
  begin
    if FHasParams then
      for n:=Low(fParams) to High(fParams) do
      begin
        Dataset.ParamByName(fParams[n].Name).Value := fParams[n].Value;
      end
  end;


  procedure _stDelete;
  begin
        try
          if Assigned(Dataset) then
            IBQuery.IB_Transaction := Dataset.IB_Transaction;
          StartSecs := 0;
          if FStatsOn then
          try
            FDBInfo.Database := DataModule.Connection;
            FQueryStats.StartMem := FDBInfo.CurrentMemory;
            FQueryStats.Query := IBQuery.SQL.Text;
            StartSecs := Time;
          except
          end;
          IBQuery.Prepare;
          if FStatsOn then
          try
            FQueryStats.TimePrepare := Time - StartSecs;
            StartSecs := Time;
          except
          end;
          AssignQueryParams;
          IBQuery.Execute;
          try
            FQueryStats.Plan := IBQuery.SQLPlan.Text;
          except
            FQueryStats.Plan := GetPrompt(56);
          end;
          FQueryStats.Rows := IBQuery.RowsAffected;

          if FStatsOn then
          try
            FQueryStats.TimeExecute := Time - StartSecs;
            FQueryStats.EndMem := FDBInfo.CurrentMemory;
            FQueryStats.MaxMem := FDBInfo.MaxMemory;
            FQueryStats.Buffers := FDBInfo.NumBuffers;
            FQueryStats.Reads := FDBInfo.Reads;
            FQueryStats.Writes := FDBInfo.Writes;
          except
          end;

          if Assigned (OnISQLEvent) then
            OnISQLEvent(evntRows, seUnk, IBQuery.RowsAffected, DataModule, '');
          //shold be fine to get table names - Lorenzo

          IBQuery.Close;
        except
          on E: Exception do
          begin
            IBQuery.Close;
            raise EISQLException.Create (ERR_ISQL_ERROR, Data.Strings[stIdx], eeDML, E.Message);
          end;
        end;
  end;

  procedure _stSelect;
  begin
    try
      if Assigned (DataSet) then
      begin
        DataSet.Close;
        DataSet.SQL.Text := IBQuery.SQL.Text;

        StartSecs := 0;
        if FStatsOn then
        try
          FDBInfo.Database := DataModule.Connection;
          FQueryStats.StartMem := FDBInfo.CurrentMemory;
          FQueryStats.Query := IBQuery.SQL.Text;
          StartSecs := Time;
        except
        end;

        DataSet.Prepare;
        AssignDatasetParams;
        if FStatsOn then
        try
          EndSecs := Time;
          TmpTime := EndSecs - StartSecs;
          FQueryStats.TimePrepare := TmpTime;
          StartSecs := Time;
        except
        end;
        DataSet.Open;
        try
          FQueryStats.Plan := Dataset.Plan;
        except
          FQueryStats.Plan := GetPrompt(56);
        end;

        FQueryStats.Rows := Dataset.RecordCount;

        if FStatsOn then
        try
          EndSecs := Time;
          TmpTime := EndSecs - StartSecs;
          FQueryStats.TimeExecute := TmpTime;
          FQueryStats.EndMem := FDBInfo.CurrentMemory;
          FQueryStats.MaxMem := FDBInfo.MaxMemory;
          FQueryStats.Buffers := FDBInfo.NumBuffers;
          FQueryStats.Reads := FDBInfo.Reads;
          FQueryStats.Writes := FDBInfo.Writes;
        except
        end;
      end
      else
      begin
        IBQuery.Prepare;
        AssignQueryParams;
        IBQuery.Execute;
        IBQuery.Close;
      end;
    except
      on E: Exception do
        begin
          IBQuery.Close;
          raise EISQLException.Create (ERR_ISQL_ERROR, Data.Strings[stIdx], eeQuery, E.Message);
        end;
    end;
  end;

      procedure DoStatement;
      begin
          Application.ProcessMessages;
          if Assigned(FProgressEvent) then OnQueryProgress(self);
          (* Is this an ISQL Command? *)
          IsqlAction := IsISQLCommand (Data.Strings[stIdx], IsqlValue);
          if ISQLAction <> actUnk then exit;
          (* If it isn't any of the above, then execute the statement *)
          if not Assigned (IBQuery) then
            raise EISQLException.Create(ERR_ISQL_ERROR, Data.Strings[stIdx], eeStatement, GetPrompt(58));
          IBQuery.Close;
          IBQuery.SQL.Clear;
          IBQuery.SQL.Add (Data.Strings[stIdx]);
          TmpQuery.SQL.Clear;
          TmpQuery.SQL.Add (Data.Strings[stIdx]);
          (* See if the statement is valid *)
          try
            if TmpTransaction.InTransaction then
              TmpTransaction.Commit;
            TmpQuery.Prepare;
            TmpQuery.Close;
          except on E: Exception do
            begin
              TmpTransaction.Commit;
              TmpQuery.Close;
              raise EISQLException.Create (ERR_ISQL_ERROR, Data.Strings[stIdx], eeStatement, E.Message);
            end;
          end;
          TmpTransaction.Commit;
          case TmpQuery.StatementType of
            stDDL, stSetGenerator:  _stDDL;
            stDelete, stInsert, stUpdate: _stDelete;
            stSelect, stSelectForUpdate, stExecProcedure: _stSelect;
          end;
      end;

      procedure _CreateComponents;
      begin
        (* If the database was already assigned and attached to,
           then create the components for the database *)
        if Assigned(DataModule.Connection.dbHandle) then
        begin
          TmpTransaction := TIB_Transaction.Create (nil);
          IBQuery := TIB_Cursor.Create (nil);
          DDLQuery:= TIB_Cursor.Create (nil);
          TmpQuery:= TIB_Cursor.Create (nil);
          TmpTransaction.IB_Connection := DataModule.Connection;
          IBQuery.IB_Connection := DataModule.Connection;
          IBQuery.IB_Transaction := DataModule.Transaction;
          IBQuery.ParamCheck := false;
          TmpQuery.IB_Connection := DataModule.Connection;
          TmpQuery.IB_Transaction := TmpTransaction;
          TmpQuery.ParamCheck := false;
          DDLQuery.IB_Connection := DataModule.Connection;
          DDLQuery.IB_Transaction := DataModule.DDLTransaction;
          DDLQuery.ParamCheck := false;
        end;
      end;

      procedure RecordStats;
      begin
        try
          with FQueryStats do
          begin
            DecodeTime(TimeExecute, Hour, Min, Sec, MSec);
            FStatistics.Add(Format(GetLangText('Execution Time')+' (hh:mm:ss.ssss)%s%.2d:%.2d:%.2d.%.4d',[DEL, Hour, Min, Sec, MSec]));
            DecodeTime(TimePrepare, Hour, Min, Sec, MSec);
            FStatistics.Add(Format(GetLangText('Prepare Time')+' (hh:mm:ss.ssss)%s%.2d:%.2d:%.2d:%.4d',[DEL,Hour, Min, Sec, MSec]));
            FStatistics.Add(Format(GetLangText('Starting Memory')+'%s%d',[DEL,StartMem]));
            FStatistics.Add(Format(GetLangText('Current Memory')+'%s%d',[DEL,EndMem]));
            FStatistics.Add(Format(GetLangText('Delta Memory')+'%s%d',[DEL,EndMem-StartMem]));
            FStatistics.Add(Format(GetLangText('Number of Buffers')+'%s%d',[DEL, Buffers]));
            FStatistics.Add(Format(GetLangText('Reads')+'%s%d',[DEL,Reads]));
            FStatistics.Add(Format(GetLangText('Writes')+'%s%d',[DEL,Writes]));
            if Length(Plan) > 0 then
              FStatistics.Add(Format(GetLangText('Plan')+'%s%s',[DEL, Plan]))
            else
              FStatistics.Add(Format(GetLangText('Plan')+'%s%s',[DEL, GetLangText('Not Available')]));
            if Rows > -1 then
              FStatistics.Add(Format(GetLangText('Rows Affected')+'%s%d',[DEL, Rows]));
          end;
          TrIdx := TransIndex(DataModule.Connection,TmpTransaction);
          if trIDx <> -1 then
            TmpTransaction.IB_Connection := Nil;
        except
          on E: Exception do
            raise EISQLException.Create (ERR_ISQL_ERROR, '', eeFree, E.Message);
        end;
      end;

begin
  if WaitForParams then
  begin
    if not Assigned(DataModule) then exit;
    TmpQuery := nil;
    DDLQuery := nil;
    IBQuery := nil;
    TmpTransaction := Nil;
    Data := TStringList.Create;
    Source := TStringList.Create;

    (* Defaults *)
    try
      CharSet := gAppSettings[CHARACTER_SET];
      Source.Text := fStoredQuery;
      if not ParseSQL (Source, Data, gAppSettings[ISQL_TERMINATOR]) then
        raise EISQLException.Create (ERR_ISQL_ERROR, '', eeParse, GetPrompt(68));
      FStatements := Data.Count - 1;
      try
        _CreateComponents;
      except on E: Exception do
        raise EISQLException.Create (ERR_ISQL_ERROR, '', eeInitialization, E.Message);
      end;
      (* Go through the String list excecuting the information line by line.
       * Each statement is executed against the currently connected database
       * and server (if there is one).   *)
      if Data.Count = 1 then
      begin
        stIdx := 0;
        DoStatement;
      end;

      RecordStats;
    finally
      Data.Free;
      Source.Free;
      TmpQuery.Free;
      DDLQuery.Free;
      IBQuery.Free;
    end;
  end;
end;


procedure TIBOSqlObj.SetWaitForParams(const Value: boolean);
begin
  FWaitForParams := Value;
end;

procedure TIBOSqlObj.Cancel;
begin
  FCanceled := true;
end;

constructor TIBOSqlObj.Create(AComponent: TComponent);
begin
  inherited;
  FAutoDDL := true;
  FDBInfo := TIB_ConnectionInfo.Create(nil);
  FStatistics := TStringList.create;
  FHasParams := False;
end;

procedure TIBOSqlObj.DoGetParams(q:TIB_Cursor);
var
  n:integer;
begin
  SetLength(fParams,q.ParamCount);
  FHasParams := q.ParamCount>0;
  if FHasParams then
    for n:=0 to Pred(q.ParamCount) do
    begin
      fParams[n].Name := q.Params[0].FieldName;
      fParams[n].Value := Null;
    end;
end;

procedure TIBOSqlObj.DoPrepare;
var
  Data,
  Source: TStringList;

  lQry: TIB_Cursor;
  lTrans: TIB_Transaction;

  lCnt: integer;

begin
  Data := TStringList.Create;
  Source := TStringList.Create;
  Source.AddStrings (FQuery);

  (* Defaults *)
  if not ParseSQL (Source, Data,  gAppSettings[ISQL_TERMINATOR]) then
    raise EISQLException.Create (ERR_ISQL_ERROR, '', eeParse, GetPrompt(68));
  Source.Free;

  SetLength(fParams,0);
  FHasParams := False;

  { Prepare each line and post results }
  lQry := TIB_Cursor.Create(self);
  lTrans := TIB_Transaction.Create(self);
  try
    lTrans.IB_Connection := DataModule.Connection;
    lQry.IB_Connection := DataModule.Connection;
    lQry.IB_Transaction := lTrans;
    if Data.Count = 1 then
    begin
        lQry.SQL.Text := Data.Strings[0];
        try
          lQry.Prepare;
          DoGetParams(lqry);
        except on E: Exception do
          raise EISQLException.Create (ERR_ISQL_ERROR, Data.Strings[0], eeStatement, E.Message);
        end;
        if Assigned (OnDataOutput) then
        begin
          OnDataOutput (Format('%s: %s',[GetLangText('Statement'), Data.Strings[0]]));
          if Length(lQry.SQLPlan.Text) > 0 then
            OnDataOutput (Format('%s', [lQry.SqlPlan.Text]))
          else
            OnDataOutput (GetLangText('Not Available'))
        end;
        //lQry.IB_Transaction.Commit;
        lQry.Close;
    end
    else
    for lCnt := 0 to Data.Count-1 do
    begin
        lQry.SQL.Text := Data.Strings[lCnt];
        try
          lQry.Prepare;
        except on E: Exception do
          raise EISQLException.Create (ERR_ISQL_ERROR, Data.Strings[lCnt], eeStatement, E.Message);
        end;

        if Assigned (OnDataOutput) then
        begin
          OnDataOutput (Format('%s: %s',[GetLangText('Statement'), Data.Strings[lCnt]]));
          if Length(lQry.SQLPlan.Text) > 0 then
            OnDataOutput (Format('%s', [lQry.SqlPlan.Text]))
          else
            OnDataOutput (GetLangText('Not Available'))
        end;

        //lQry.IB_Transaction.Commit;
        lQry.Close;
    end;
  finally
    lQry.Free;
    lTrans.Free;
  end;
end;

{ EISQLException }
constructor EISQLException.Create(ErrorCode: Integer;
  ExceptionData: String; ExceptionCode: TISQLExceptionCode; Msg: String);
begin
  inherited Create(Msg);
  FExceptionData := ExceptionData;
  FExceptionCode := ExceptionCode;
  FErrorCode:= ErrorCode;
end;

destructor TIBOSqlObj.Destroy;
begin
  FDBInfo.Free;
  FStatistics.Free;
  inherited;
end;

procedure TIBOSqlObj.SetDataModule(const Value: TDmN);
begin
  FDataModule := Value;
end;

procedure TIBOSqlObj.DoGetParams(q: TIBOQuery);
var
  n:integer;
begin
  SetLength(fParams,q.ParamCount);
  FHasParams := q.ParamCount>0;
  if FHasParams then
    for n:=0 to Pred(q.ParamCount) do
    begin
      fParams[n].Name := q.Params[n].Name;
      fParams[n].Value := Null;
    end;
end;

end.

