{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{ 
 * The contents of this file are subject to the InterBase Public License 
 * Version 1.0 (the "License"); you may not use this file except in 
 * compliance with the License. 
 * 
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html. 
 * 
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific language governing rights and limitations 
 * under the License.  The Original Code was created by Inprise 
 * Corporation and its predecessors. 
 * 
 * Portions created by Inprise Corporation are Copyright (C) Inprise 
 * Corporation. All Rights Reserved. 
 * 
 * Contributor(s): Krzysztof Golko. 
} 
 
 
unit zluUtility; 
 
interface 
 
uses 
  SysUtils, Classes, 
{$IFDEF MSWINDOWS} 
  Windows, Messages, Controls, Forms, Graphics, Dialogs, StdCtrls, Registry; 
{$ENDIF} 
{$IFDEF LINUX} 
  Libc, QControls, QForms, QDialogs, QStdCtrls, QGraphics; 
{$ENDIF} 
 
type 
  TOrderedStringList = class(TStringList) 
  public 
    procedure Sort; override; 
    function NextAvail: integer; 
  end; 
 
function CheckDirectory(Directory: string): boolean; 
function GetNewFileName(Directory: string; FileExtension: string): string; 
function GetNextField(var InputStr: string; const FieldDelimiter: string): string; 
function OSVerInfo(): string; 
 
function RemoveControlChars(const InputStr: string): string; 
//Lorenzo procedure CheckLocalIBRunning; 
function ParseConnectStr(Str, Tok, Del : String) : String; 
function ConvertStr(lStr : String) : String; 
function ObjTypeToStr(const objType: integer): string; 
function StripMenuChars(const Caption: String): String; 
function GetImageIndex (const ObjType: integer): integer; 
function IsValidDBName(const DBName: String): boolean; 
function MinimizeName(const Filename: TFileName; Canvas: TCanvas; 
  MaxLen: Integer): TFileName; 
 
 
implementation 
 
uses 
  IBOCGlobal(*Lorenzo , IBOManager*), LMMessageDialog;
 
function MinimizeName(const Filename: TFileName; Canvas: TCanvas; 
  MaxLen: Integer): TFileName; 
var 
  Drive: TFileName; 
  Dir: TFileName; 
  Name: TFileName; 
  PathDelim : String; 
 
  procedure CutFirstDirectory(var S: TFileName); 
  var 
    Root: Boolean; 
    P: Integer; 
  begin 
    if S = PathDelim then 
      S := '' 
    else 
    begin 
      if S[1] = PathDelim then 
      begin 
        Root := True; 
        Delete(S, 1, 1); 
      end 
      else 
        Root := False; 
      if S[1] = '.' then 
        Delete(S, 1, 4); 
      P := AnsiPos(PathDelim,S); 
      if P <> 0 then 
      begin 
        Delete(S, 1, P); 
        S := '...' + PathDelim + S; 
      end 
      else 
        S := ''; 
      if Root then 
        S := PathDelim + S; 
    end; 
  end; 
 
begin 
  Result := FileName; 
  Dir := ExtractFilePath(Result); 
  Name := ExtractFileName(Result); 
  if Pos('\', Dir) > 0 then 
    PathDelim := '\' 
  else 
    PathDelim := '/'; 
  {$IFDEF MSWINDOWS} 
  if (Length(Dir) >= 2) and (Dir[2] = ':') then 
  begin 
    Drive := Copy(Dir, 1, 2); 
    Delete(Dir, 1, 2); 
  end 
  else 
    Drive := ''; 
  {$ENDIF} 
  {$IFDEF LINUX} 
  Drive := ''; 
  {$ENDIF} 
  while ((Dir <> '') or (Drive <> '')) and (Canvas.TextWidth(Result) > MaxLen) do 
  begin 
    if Dir = PathDelim + '...' + PathDelim then 
    begin 
      Drive := ''; 
      Dir := '...' + PathDelim; 
    end 
    else if Dir = '' then 
      Drive := '' 
    else 
      CutFirstDirectory(Dir); 
    Result := Drive + Dir + Name; 
  end; 
end; 
 
function OrderedListCompare(List: TStringList; Index1, Index2: Integer): Integer; 
begin 
  Result := StrToInt(List.Names[Index1]) - StrToInt(List.Names[Index2]); 
end; 
 
  { TOrderedStringList } 
procedure TOrderedStringList.Sort; 
begin 
  CustomSort(OrderedListCompare); 
end; 
 
function TOrderedStringList.NextAvail: integer; 
begin 
  if Count > 0 then 
    Result := StrToInt(Names[Count - 1]) + 1 
  else 
    Result := 1; 
end; 
 
function CheckDirectory(Directory: string): boolean; 
begin 
  if (Directory <> '') and not (DirectoryExists(Directory)) then 
  begin 
    if LMMessageDlg(73,[Directory], mtConfirmation, [mbYes,mbNo], 0) = mrYes then 
    begin 
      if not CreateDir(Directory) then 
      begin 
        LMMessageDlg(72, [Directory], mtInformation, [mbOk], 0); 
        result := false; 
      end 
      else 
        result := true; 
    end 
    else 
      result := false; 
  end 
  else 
    result := true; 
end; 
 
function GetNewFileName(Directory: string; FileExtension: string): string; 
var 
  lFileName: string; 
begin 
  Randomize; 
  lFileName := Format('%s%s%s',[Directory,Format('%-8.8d',[Random(99999999)]),FileExtension]); 
  while FileExists(lFileName) do 
  begin 
    lFileName := Format('%s%s%s',[Directory,Format('%-8.8d',[Random(99999999)]),FileExtension]); 
  end; 
  result := lFileName; 
end; 
 
{ Receives a delimited string and extracts the first field 
  in the string based on the given delimiter } 
function GetNextField(var InputStr: string; const FieldDelimiter: string): string; 
var 
  lFieldDelPos: integer; 
  lRetVal: string; 
begin 
  // get position of the first Delimiter found 
  lFieldDelPos := Pos(FieldDelimiter, InputStr); 
  if lFieldDelPos > 0 then 
  begin 
    // copy the field to a new string 
    lRetVal := Copy(InputStr, 1, lFieldDelPos - 1); 
 
    // delete field from our incoming string 
    Delete(InputStr, 1, lFieldDelPos + Length(FieldDelimiter)-1); 
  end 
  else 
  begin 
    // last field 
    lRetVal := InputStr; 
    InputStr := ''; 
  end; 
  result := TrimRight(lRetVal); 
end; 
 
function OSVerInfo(): string;
{$IFDEF LINUX}
var
  VersionInfo: TUTSName;
{$ENDIF}
begin
{$IFDEF LINUX}
  uname(VersionInfo); 
  Result := VersionInfo.sysname + ' ' + VersionInfo.release; 
{$ENDIF} 
end; 

{ Receives a string and removes any control characters from it } 
function RemoveControlChars(const InputStr: string): string; 
var 
  i: integer; 
  lStrVal: string; 
begin 
  lStrVal := Trim(InputStr); 
 
  for i := 1 to Length(lStrVal) do 
  begin 
    if (Ord(lStrVal[i]) in [32..126]) then 
    begin 
      // The lStrVal[i+1] is safe in this case because with the use 
      // of the above Trim() function the if condition will never be true for 
      // the last character. 
      if (lStrVal[i] = Chr(32)) and (lStrVal[i+1] = Chr(32)) then 
        Delete(lStrVal,i,1); 
    end; 
  end; 
  { Make sure that the first character is capitalized } 
  if Length(lStrVal) > 0 then 
    lStrVal[1] := UpCase(lStrVal[1]); 
  result := lStrVal; 
end; 

// extracts param values for a connect statememt
function ParseConnectStr(Str, Tok, Del : String) : String;
  var
    iStart, iEnd : Integer; 
    lReturn : String; 
  begin 
    iStart := Pos(Tok, Str); 
    // found parameter - must find parameter value 
    if iStart <> 0 then 
    begin 
      Delete(Str, 1, iStart); 
      iEnd := Pos(Del, Str); 
      if iEnd <> 0 then 
      begin 
        // delete the parameter - next is the parameter value 
        Delete(Str, 1, iEnd); 
 
        iEnd := Pos(Del, Str); 
 
        if iEnd <> 0 then 
          lReturn := Trim(Copy(Str, 0, iEnd - 1)) 
        else 
          lReturn := ''; 
      end 
      else 
        lReturn := ''; 
    end 
    else 
      lReturn := ''; 
    Result := lReturn; 
  end; 
 
  // this function removes ' and replaces them with " 
  // and converts all non-quoted characters to uppercase 
function ConvertStr(lStr : String) : String; 
  var 
    lQuote : Boolean; 
    i : Integer; 
  begin 
    lStr := Trim(lStr); 
    lQuote := False; 
    i := 1; 
    while i <= Length(lStr) do 
    begin 
      if (Ord(lStr[i]) = 10) or (Ord(lStr[i]) = 13) then 
        lStr[i] := ' ' 
      else  if (lStr[i] = '''') or (lStr[i] = '"') then 
      begin 
        if (lStr[i] = '''') then 
          lStr[i] := '"'; 
        if lQuote then 
          lQuote := False 
        else 
          lQuote := True; 
      end 
      else 
      begin 
        if (not lQuote) and (Ord(lStr[i]) > 96) and (Ord(lStr[i]) < 123) then 
        begin 
          lStr[i] := Chr(Ord(lStr[i]) - 32) 
        end; 
      end; 
      Inc(i); 
    end; 
    Result := lStr; 
  end; 
 
function ObjTypeToStr(const objType: integer): string; 
begin 
  case objType of 
    0: result := 'Table'; 
    1: result := 'View'; 
    2: result := 'Trigger'; 
    3: result := 'Computed Field'; 
    4: result := 'Validation'; 
    5: result := 'Procedure'; 
    6: result := 'Expression Index'; 
    7: result := 'Exception'; 
    8: result := 'User'; 
    9: result := 'Field'; 
    10: result := 'Index'; 
    13: result := 'Role'; 
    else 
      result := 'Unknown'; 
  end; 
end; 
 
function StripMenuChars(const Caption: String): String; 
var 
  Psrc, Pdest: PChar; 
  dest_str: String; 
{$IFDEF LINUX} 
  i: integer; 
{$ENDIF} 
begin 
{$IFDEF MSWINDOWS} 
  Psrc := PChar(Caption); 
  SetLength(dest_str, Length(Caption)); 
  Pdest := PChar(dest_str); 
  while Psrc^ <> #0 do 
  begin 
    if not (Psrc^ in ['&', '.']) then 
    begin 
      Pdest^ := Psrc^; 
      Inc(Pdest); 
    end; 
    Inc(Psrc); 
  end; 
  Pdest^ := PSrc^; 
  result := dest_str; 
{$ENDIF} 
{$IFDEF LINUX} 
  Result := ''; 
  for i := 1 to Length(Caption) do 
    if not (Caption[i] in ['&', '.']) then 
      Result := Result + Caption[i]; 
{$ENDIF} 
end; 
 
function GetImageIndex (const ObjType: integer): integer; 
begin 
  case ObjType of 
    DEP_TABLE: result := NODE_TABLES_IMG; 
    DEP_VIEW:  result := NODE_VIEWS_IMG; 
    DEP_TRIGGER: result := NODE_TRIGGERS_IMG; 
    DEP_COMPUTED_FIELD: result := NODE_UNK_IMG; 
    DEP_VALIDATION: result := NODE_CHECK_CONSTRAINTS_IMG; 
    DEP_PROCEDURE: result := NODE_PROCEDURES_IMG; 
    DEP_EXPRESSION_INDEX: result := NODE_UNK_IMG; 
    DEP_EXCEPTION: result := NODE_EXCEPTIONS_IMG; 
    DEP_USER: result := NODE_USERS_IMG; 
    DEP_FIELD: result := NODE_COLUMNS_IMG; 
    DEP_INDEX: result := NODE_INDEXES_IMG; 
    else 
      result := NODE_UNK_IMG; 
  end; 
end; 
 
function IsValidDBName(const DBName: String): boolean; 
(* NOTE:  If this function returns FALSE, all areas of the code 
   return a warning and DO NOT ABORT *) 
begin 
  result := true; 
  { Make sure it is not a UNC name } 
  if (Pos('//', DBName) = 1) or 
     (Pos ('\\', DBName) = 1) or 
  { Make sure it is not TCP/IP } 
     (Pos(':', DBName) > 2) or 
  { Make sure it is not SPX.} 
     (Pos('@', DBName) <> 0) then 
    result := false; 
end; 
 
var 
  OL: TOrderedStringList; 
 
begin 
  OL := TOrderedStringList.Create; 
  OL.Add('11=xxxx'); 
  OL.Add('3=zzzx'); 
  OL.Add('7=xxxx'); 
  OL.Sort; 
  OL.NextAvail; 
  OL.Free; 
end. 
