{************************************************************************
  This Unit was modified to be interfaced in IBObjects
  by Lorenzo Mengoni - (Ancona, Italy)

  for suggestion and contributions email to lorenzo@mengoni.com

  This software is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied.
  If you live or are intended to use this software in countries
  where this is not applicable ("AS IS" basis), DO NOT USE IT.

                                                                         }
{
 * The contents of this file are subject to the InterBase Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License.
 *
 * You may obtain a copy of the License at http://www.Inprise.com/IPL.html.
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.  The Original Code was created by Inprise
 * Corporation and its predecessors.
 *
 * Portions created by Inprise Corporation are Copyright (C) Inprise
 * Corporation. All Rights Reserved.
 *
 * Contributor(s): Krzysztof Golko.
}


unit zluibcClasses;

interface

uses
{$IFDEF LINUX}
  QForms, QT, QControls,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Forms, CommCtrl, Controls,
{$ENDIF}
  IBOServices, IBOCGlobal, IBOCPersistent, IB_SESSION, IB_Header,
  DB, SysUtils, Classes, IB_Components, IBODataset;

type

  TibcTreeNode = class(TComponent)
  private
{$IFDEF LINUX}
    FNodeID: QListViewItemH;
{$ENDIF}
{$IFDEF MSWINDOWS}
    FNodeID: HTreeItem;
{$ENDIF}
    FNodeName: string;
    FNodeType: word;
    FShowSystem: boolean;
    FObjectList: TStringList;
    FIsPersistent: boolean;
    FLastAccessed: TDateTime;
    FCreated: TDateTime;
  public
{$IFDEF LINUX}
    property NodeID: QListViewItemH read FNodeID;
{$ENDIF}
{$IFDEF MSWINDOWS}
    property NodeID: HTreeItem read FNodeID;
{$ENDIF}
    property NodeName: string read FNodeName write FNodeName;
    property NodeType: word read FNodeType;
    property ShowSystem: boolean read FShowSystem write FShowSystem;
    property ObjectList: TStringList read FObjectList write FObjectList;

    property IsPersistent: boolean read FIsPersistent write FIsPersistent;
    property LastAccessed: TDateTime read FLastAccessed write FLastAccessed;
    property Created: TDateTime read FCreated write FCreated;
{$IFDEF LINUX}
    constructor Create(AOwner: TComponent; const NodeID: QListViewItemH; const NodeName: string; const NodeType: word); reintroduce;
{$ENDIF}
{$IFDEF MSWINDOWS}
    constructor Create(AOwner: TComponent; const NodeID: HTreeItem; const NodeName: string; const NodeType: word); reintroduce;
{$ENDIF}
    destructor Destroy(); override;
  end;

const
  DUMMY_PASSWORD = 'TheyKilledKennyAgain';

type

  TibcServerNode = class(TibcTreeNode)
  private
    FServerName: string;
    FUserName: string;
    FPassword: string;
    FSqlRole: string; //used in db connection
    FVersion : Integer;
{$IFDEF LINUX}
    FDatabasesID: QListViewItemH;
    FBackupFilesID: QListViewItemH;
{$ENDIF}
{$IFDEF MSWINDOWS}
    FDatabasesID: HTreeItem;
    FBackupFilesID: HTreeItem;
{$ENDIF}
    FServer: TIBOServerProperties;
    FDescription: string;
    FCharacterSet: string;
    FUserList: TStrings;
    fUserCount:integer;
    fUserInfo: TUserInfoArray;
    fSecurityService: TIBOSecurityService;
    FAlive:boolean;
    procedure SetUserList(const Value: TStrings);
    function GetUserInfo(const UserName: string): TUserInfo;
    function GetUserInfoIDX(const Idx: integer): TUserInfo;
    procedure GetServerUsersList;
    procedure CreateSecurityService;
  public
{$IFDEF LINUX}
    constructor Create(AOwner: TComponent; const NodeID: QListViewItemH; const NodeName: string; const NodeType: word; const ServerProps: TibcServerProps; const Password: string);
{$ENDIF}
{$IFDEF MSWINDOWS}
    constructor Create(AOwner: TComponent; const NodeID: HTreeItem; const NodeName: string; const NodeType: word; const ServerProps: TibcServerProps; const Password: string);
{$ENDIF}
    destructor Destroy(); override;
    procedure AttachServer;
    property Servername: string read FServername write FServername;
    property UserName: string read FUserName write FUserName;
    property Password: string read FPassword write FPassword;
    property SqlRole: string read FSqlRole write FSqlRole;
{$IFDEF LINUX}
    property DatabasesID: QListViewItemH read FDatabasesID write FDatabasesID;
    property BackupFilesID: QListViewItemH read FBackupFilesID write FBackupFilesID;
{$ENDIF}
{$IFDEF MSWINDOWS}
    property DatabasesID: HTreeItem read FDatabasesID write FDatabasesID;
    property BackupFilesID: HTreeItem read FBackupFilesID write FBackupFilesID;
{$ENDIF}
    function ModifyUser(UserName: string;
                         FirstName: string;
                         MiddleName: string;
                         LastName: string;
                         Password:String;
                         GroupID: Integer;
                         UserID: Integer):boolean;
    function NewUser(UserName: string;
                      FirstName: string;
                      MiddleName: string;
                      LastName: string;
                      Password:String;
                      GroupID: Integer;
                      UserID: Integer):boolean;
    function DeleteUser(UserName: string):boolean;
    property Version : Integer read FVersion write FVersion;
    property Description: string read FDescription write FDescription;
    property Server: TIBOServerProperties read FServer write FServer;
    property CharacterSet: string read FCharacterSet write FCharacterSet;
    property UserList:TStrings read FUserList write SetUserList;
    property UserInfoByName[const UserName: string]:TUserInfo read GetUserInfo;
    property User_Info[const Idx: integer]:TUserInfo read GetUserInfoIdx;
    property UserCount:integer read fuserCount;
    property Alive: boolean read FAlive;
    procedure GetUsersList(lObjectList:TStringList);
    procedure GetUserNameList(List:TStrings);
    procedure Reload;
  end;


  TibcBackupAliasNode = class(TibcTreeNode)
  private
    FServerAlias: string;
    FDatabaseAlias: string;
    FBackupFiles: TStringList;
  protected
  public
    property SourceDBServer: string read FServerAlias write FServerAlias;
    property SourceDBAlias: string read FDatabaseAlias write FDatabaseAlias;
    property BackupFiles: TStringList read FBackupFiles write FBackupFiles;
{$IFDEF LINUX}
    constructor Create(AOwner: TComponent; const NodeID: QListViewItemH; const NodeName: string; const Created, LastAccessed: TDateTime; const NodeType: word);
{$ENDIF}
{$IFDEF MSWINDOWS}
    constructor Create(AOwner: TComponent; const NodeID: HTreeItem; const NodeName: string; const Created, LastAccessed: TDateTime; const NodeType: word);
{$ENDIF}
    destructor Destroy(); override;
  published
  end;

  EibcCriticalError = class(Exception)
    constructor Create;
  end;

implementation

uses frmuMessage;

{$IFDEF LINUX}
constructor TibcTreeNode.Create(AOwner: TComponent; const NodeID: QListViewItemH; const NodeName: string; const NodeType: word);
{$ENDIF}
{$IFDEF MSWINDOWS}
constructor TibcTreeNode.Create(AOwner: TComponent; const NodeID: HTreeItem; const NodeName: string; const NodeType: word);
{$ENDIF}
begin
  inherited Create(AOwner);
  FNodeID := NodeID;
  FNodeName := NodeName;
  FNodeType := NodeType;
  FObjectList := TStringList.Create;
  FShowSystem := false;
  FIsPersistent := true;
end;

destructor TibcTreeNode.Destroy();
begin
  FObjectList.Free;
  inherited Destroy();
end;

function TibcServerNode.GetUserInfo(const UserName: string): TUserInfo;
var n:integer;
begin
  Result := Nil;
  for n:=0 to High(FUserInfo) do
  if FUserInfo[n].UserName = UserName then
  begin
    result := FUserInfo[n];
    exit;
  end;
end;

function TibcServerNode.GetUserInfoIDX(const Idx: integer): TUserInfo;
begin
  Result := Nil;
  if Idx in [low(FUserInfo)..High(FUserInfo)] then
  begin
    result := FUserInfo[Idx];
  end;
end;

procedure TibcServerNode.CreateSecurityService;
begin
  fSecurityService:= TIBOSecurityService.Create(self);
  fSecurityService.LoginPrompt := false;
  fSecurityService.ServerName := Self.Servername;
  fSecurityService.Protocol := Self.Server.Protocol;
  fSecurityService.Params.Assign(Self.Server.Params);
end;

procedure TibcServerNode.AttachServer;
begin
  Version := 6;
//  FServer.Params.Clear;
//  FServer.Params.Add(Format('isc_spb_user_name=%s',[UserName]));
//  FServer.Params.Add(Format('isc_spb_password=%s',[Password]));
  FServer.Params.Values['isc_spb_user_name']:=UserName;
  FServer.Params.Values['isc_spb_password']:=Password;
  FServer.Attach();
  FAlive:=True;
  try
    if not Assigned(fSecurityService) then
      CreateSecurityService;
    GetServerUsersList;
  except
  end;
end;

{$IFDEF LINUX}
constructor TibcServerNode.Create(AOwner: TComponent; const NodeID: QListViewItemH; const NodeName: string; const NodeType: word; const ServerProps: TibcServerProps; const Password: string);
{$ENDIF}
{$IFDEF MSWINDOWS}
//constructor TibcServerNode.Create(AOwner: TComponent; const NodeID: HTreeItem; const NodeName,Servername,UserName,Password, Description: string; const Protocol: TProtocol; const LastAccessed: TDateTime; const NodeType: word);
constructor TibcServerNode.Create(AOwner: TComponent; const NodeID: HTreeItem; const NodeName: string; const NodeType: word; const ServerProps: TibcServerProps; const Password: string);
{$ENDIF}
begin
  inherited Create(AOwner, NodeID, NodeName, NodeType);
  FServername := ServerProps.Servername;
  FUserName := ServerProps.UserName;
  FPassword := Password;
  FVersion := 6;
  FServer := TIBOServerProperties.Create(nil);
  FServer.ServerName := ServerProps.ServerName;
  FServer.Protocol := ServerProps.Protocol;
  FServer.LoginPrompt := false;
  FDescription := ServerProps.Description;
  FLastAccessed := ServerProps.Accessed;
  FCreated := ServerProps.Created;
  FUserList := TStringList.Create;
  FAlive := False;
  FSqlRole := '';
end;

destructor TibcServerNode.Destroy();
begin
  try
    if Self.Server.Active then
      FServer.Detach;
    FServer.Free;
  except on E: Exception do
    FServer.Free;
  end;
  try
    FUserList.Free;
  except
  end;
  try
    fSecurityService.Free;
  except
  end;
  inherited Destroy();
end;

procedure TibcServerNode.SetUserList(const Value: TStrings);
begin
  if Assigned(Value) then
    FUserList.Assign(Value);
end;

procedure TibcServerNode.GetServerUsersList;
var
  n: integer;

  procedure AddUserInfo;
  begin
    if High(fUserInfo)<fUserCount then
    begin
       SetLength(fUserInfo,fUserCount+1);
    end;
    if not Assigned(fUserInfo[fUserCount]) then
      fUserInfo[fUserCount] := TUserInfo.Create;
    fUserInfo[fUserCount].UserName   := fSecurityService.UserInfo[fUserCount].UserName   ;
    fUserInfo[fUserCount].FirstName  := fSecurityService.UserInfo[fUserCount].FirstName  ;
    fUserInfo[fUserCount].MiddleName := fSecurityService.UserInfo[fUserCount].MiddleName ;
    fUserInfo[fUserCount].LastName   := fSecurityService.UserInfo[fUserCount].LastName   ;
    fUserInfo[fUserCount].GroupID    := fSecurityService.UserInfo[fUserCount].GroupID    ;
    fUserInfo[fUserCount].UserID     := fSecurityService.UserInfo[fUserCount].UserID     ;
    FUserList.Add(fUserInfo[fUserCount].UserName);
  end;

begin
  fUserCount := 0;
  FUserList.Clear;
  if not Assigned(fSecurityService) then
    CreateSecurityService;
  try
    Application.ProcessMessages;
    Screen.Cursor := crHourGlass;
    try
      if not fSecurityService.Active then
        fSecurityService.Attach;
      if fSecurityService.Active then
      begin
        fSecurityService.DisplayUsers;
        while (fSecurityService.IsServiceRunning) and (not gApplShutdown) do
          Application.ProcessMessages;
      end;
    except
      on E:EIB_ISCError do
      begin
        DisplayMsg(ERR_GET_USERS, E.Message);
        exit;
      end;
    end;
    while Assigned(fSecurityService.UserInfo[fUserCount]) do
    begin
      AddUserInfo;
      inc(fUserCount);
    end;
    if Length(fUserInfo) > fUserCount then
    begin
      for n:= High(fUserInfo) downto fUserCount+1 do
      try
        fUserInfo[n].Free;
      except
      end;
      SetLength(fUserInfo,fUserCount+1);
    end;
  finally
    if fSecurityService.Active then
      fSecurityService.Detach;
    Screen.Cursor := crDefault;
  end;
end;

procedure TibcServerNode.GetUsersList(lObjectList:TStringList);
var  n: integer;
begin
  lObjectList.Clear;
  lObjectList.Add(Format('User Name%sFirst Name%sMiddle Name%sLast Name',[DEL,DEL,DEL]));
  for n:=0 to fUserCount-1 do
  begin
    lObjectList.Add(Format('%s%s%s%s%s%s%s',
                           [fUserInfo[n].UserName,DEL,fUserInfo[n].FirstName,DEL,
                           fUserInfo[n].MiddleName,DEL,fUserInfo[n].LastName]));
  end;
end;

procedure TibcServerNode.GetUserNameList(List:TStrings);
var  n: integer;
begin
  List.Clear;
  for n:=0 to fUserCount-1 do
  begin
    List.Add(fUserInfo[n].UserName);
  end;
end;

function TibcServerNode.ModifyUser(UserName: string;
                                    FirstName: string;
                                    MiddleName: string;
                                    LastName: string;
                                    Password:String;
                                    GroupID: Integer;
                                    UserID: Integer):boolean;
begin
  result := false;
  Screen.Cursor := crHourGlass;     // change cursor to hourglass
  if not Assigned(fSecurityService) then
    CreateSecurityService;
  try
    // if not attached to server then attach
    if not fSecurityService.Active then
      fSecurityService.Attach();
    // if attached to server
    if fSecurityService.Active then
    begin
        fSecurityService.UserName := Username;

        // IBX cannot modify an existing field to blank so
        // if field is blank make it a space

        if FirstName = '' then
          fSecurityService.FirstName := ' '
        else
          fSecurityService.FirstName := FirstName;
        if MiddleName = '' then
          fSecurityService.MiddleName := ' '
        else
          fSecurityService.MiddleName := MiddleName;
        if LastName = '' then
          fSecurityService.LastName := ' '
        else
          fSecurityService.LastName := LastName;

        //fSecurityService.GroupID := GroupID;
        //fSecurityService.UserID := UserID;
        fSecurityService.ModifyParams := fSecurityService.ModifyParams - [ ModifyUserId,ModifyGroupId];

        if Password <> DUMMY_PASSWORD then
          fSecurityService.Password := Password
        else
          fSecurityService.ModifyParams := fSecurityService.ModifyParams - [ModifyPassword];

        try
          // try modifying the user in the security database
          fSecurityService.ModifyUser();
          while (fSecurityService.IsServiceRunning) and (not gApplShutdown) do
          begin
            Application.ProcessMessages;
          end;
          result := true;
        except                       // if an exception occurs then trap it
          on E:EIB_ISCError do           // and show error message
          begin
            DisplayMsg(ERR_MODIFY_USER, E.Message);
            //ErrorState:= True;
            exit;
          end;
        end;
        GetServerUsersList;
        if fSecurityService.Active then
          fSecurityService.Detach();
      end;
  finally
    Screen.Cursor := crDefault;    // change cursor back to default
  end;
end;

function TibcServerNode.NewUser(UserName: string;
                                 FirstName: string;
                                 MiddleName: string;
                                 LastName: string;
                                 Password:String;
                                 GroupID: Integer;
                                 UserID: Integer):boolean;

begin
  Screen.Cursor := crHourGlass;     // change cursor to hourglass
  if not Assigned(fSecurityService) then
    CreateSecurityService;
  result := false;
  try
    if not fSecurityService.Active then
      fSecurityService.Attach();
    // if attached to server
    if fSecurityService.Active then
    begin
        // fill in security service details regarding user
        fSecurityService.UserName := Username;
        fSecurityService.Password := Password;
        fSecurityService.FirstName := FirstName;
        fSecurityService.MiddleName := MiddleName;
        fSecurityService.LastName := LastName;
        fSecurityService.GroupID := GroupID;
        fSecurityService.UserID := UserID;
        try
          // try adding user to security database
          fSecurityService.AddUser();
          while (fSecurityService.IsServiceRunning) and (not gApplShutdown) do
            Application.ProcessMessages;
          if fSecurityService.Active then
            fSecurityService.Detach();
          result := true;
        except                       // if an exception occurs then trap it
          on E:EIB_ISCError do           // and show error message
          begin
            DisplayMsg(ERR_ADD_USER, E.Message);
            //ErrorState:= True;
          end;
        end;
        try
          if not fSecurityService.Active then
            fSecurityService.Attach();
          GetServerUsersList;
        except
        end;
        try
          if fSecurityService.Active then
            fSecurityService.Detach();
        except
        end;
    end
  finally
    Screen.Cursor := crDefault;    // change cursor back to default
  end;
end;

function TibcServerNode.DeleteUser(UserName: string):boolean;
begin
  if not Assigned(fSecurityService) then
    CreateSecurityService;
  Screen.Cursor := crHourGlass;     // change cursor to hourglass
  result := False;
  try
    // if not attached to server then attach
    if not fSecurityService.Active then
      fSecurityService.Attach();
    // if attached to server
    if fSecurityService.Active then
    begin
      // fill in security service details regarding user
      fSecurityService.UserName := Username;
      try
        // try adding user to security database
        fSecurityService.DeleteUser();
        while (fSecurityService.IsServiceRunning) and (not gApplShutdown) do
          Application.ProcessMessages;
        result := true;
      except                       // if an exception occurs then trap it
        on E:EIB_ISCError do           // and show error message
        begin
          DisplayMsg(ERR_ADD_USER, E.Message);
        end;
      end;
      GetServerUsersList;
      if fSecurityService.Active then
        fSecurityService.Detach();
    end
  finally
    Screen.Cursor := crDefault;    // change cursor back to default
  end;
end;

{$IFDEF LINUX}
constructor TibcBackupAliasNode.Create(AOwner: TComponent; const NodeID: QListViewItemH; const NodeName: string; const Created, LastAccessed: TDateTime; const NodeType: word);
{$ENDIF}
{$IFDEF MSWINDOWS}
constructor TibcBackupAliasNode.Create(AOwner: TComponent; const NodeID: HTreeItem; const NodeName: string; const Created, LastAccessed: TDateTime; const NodeType: word);
{$ENDIF}
begin
  inherited Create(AOwner,NodeID,NodeName,NodeType);
  FBackupFiles := TStringList.Create;
  FCreated := Created;
  FLastAccessed := LastAccessed
end;


destructor TibcBackupAliasNode.Destroy();
begin
  FBackupFiles.Free;
  inherited Destroy();
end;

  { EibcCriticalError }
constructor EibcCriticalError.Create;
begin
  inherited Create(GetPrompt(104));
end;

procedure TibcServerNode.Reload;
begin
  GetServerUsersList;
end;


end.
